#include "..\wincrypt\certstore.h"
#include <stdio.h>
#include <stdlib.h>
#include "..\x509pkts\oidfuncs.h"
#include "..\x509pkts\x509_structs.h"
#include "..\x509pkts\pkcs12_funcs.h"
#include "..\PGPPkts\pktdefs.h."
#include "..\publicKeyAlgs\publicKeyAlgs.h"
#include "..\RNG\cRNG.h"
//#include "..\SMemory\SBlocks.h"
#include "..\SMemory\cPassword.h"
#include "V:\Includes\FileView.h"
#include <time.h>
#include "..\x509pkts\base64funcs.h"
#include "..\publicKeyAlgs\power\power.h"

typedef WINBASEAPI
			FARPROC
			WINAPI
			typeProc( int, int,  const char *, int ); // , const char *, const char *, const char *, int );

int viewCertificate( const char *fileName )
{
	HINSTANCE hInst = LoadLibrary( "cryptext.dll" );
	if ( hInst == NULL ) return 1;

	int res = 1;

	typeProc *proc = ( typeProc * )GetProcAddress( hInst, "CryptExtOpenCER" );

	if ( proc )
		proc( 0, 0, fileName, 0 );

	FreeLibrary( hInst );

	return res;
}

int addCertificate( BYTE *ptr, int size )
{
	cCertStore sto( cCertStore::estRoot );

	return sto.addCertificateToStore( ptr, size );
}

int verifyCertificate( BYTE *ptr, int size )
{
	x509_certificate crt;
	int res = crt.parse( ptr, size );

	if ( res != x509e_ok ) return res;

	BYTE *subjectKeyID;
	int subjectKeyIDSize;
	res = crt.tbsCertificate.getSubjectKeyID( &subjectKeyID, &subjectKeyIDSize );
	if ( res != x509e_ok ) return res;

	BYTE *issuerKeyID;
	int issuerKeyIDSize;
	res = crt.tbsCertificate.getIssuerKeyID( &issuerKeyID, &issuerKeyIDSize );
	if ( res != x509e_ok ) return res;

	if ( ( subjectKeyIDSize == issuerKeyIDSize ) &&
		 ( memcmp( subjectKeyID, issuerKeyID, subjectKeyIDSize ) == 0 ) )
	{
		res = crt.verify( &(crt.tbsCertificate.subjectPKInfo) );
		if ( res != x509e_ok ) return res;

		return x509e_ok;
	}


	cCertStore sto( cCertStore::estRoot );

	BYTE *issuerCrtPtr;
	int issuerCrtSize;
/*
	const char *szAuthorityKeyOID = "2.5.29.35";
	BYTE *authorityKeyID;
	int authorityKeyIDSize;

	res = crt.tbsCertificate.extension.getExtensionValue( szAuthorityKreyOID, &authorityKeyID, &authorityKeyIDSize );

	if ( res == x509e_ok )
	{
		res = sto.findCertificateByAthorityKeyID( &issuerCrtPtr, &issuerCrtSize, authorityKeyID, authorityKeyIDSize );
	}
	else
*/	{

		BYTE *issuerDN = crt.tbsCertificate.issuerName.H;
		int issuerDNSize = crt.tbsCertificate.issuerName.TL;

		res = sto.findCertificateByDN( &issuerCrtPtr, &issuerCrtSize, issuerDN, issuerDNSize );
	}

	if ( res != sto.errOK ) return res;

	x509_certificate issuerCrt;

	res = issuerCrt.parse( issuerCrtPtr, issuerCrtSize );
	if ( res != x509e_ok ) return res;

	res = crt.verify( &(issuerCrt.tbsCertificate.subjectPKInfo) );
	if ( res != x509e_ok ) return res;

	return res;
}



#include <windows.h>
#include <stdio.h>
#include <io.h>

//#include "publicKeyAlgs\publicKeyAlgs.h"
#include "..\symmetricAlgs\symmetricAlgs.h"

#include "..\PGPPkts\PKTSignature.h"
#include "..\PGPPkts\PGPPkts.h"
#include "..\PGPPkts\Packets.h"

void Usage()
{
	printf(
		"Certificate Parser application v.0.1.b\n"
		">CrtParser <fileName>\n"
		);

}

static void printfProbels( int num )
{
	while ( num-- )
	{
		printf( "\t" );
	}
}

static void bytesOut( int rec, const char *str, BYTE *bf, int bfSize, bool out = true )
{
	printfProbels( rec );
	printf( "%s\n", str );

	if ( !out || !bfSize )
		return;

	printfProbels( rec + 1 );
	printf( "Bytes %d  - ", bfSize );
	for( int i = 0; i < bfSize; i++ )
	{
		printf( "%02X ", bf[i] );
	}
	printf("\n");
}

static void mpiBufOut( int rec, const char *str, BYTE *bf, int cnt, bool bOut = false )
{
	printfProbels( rec );
	printf( "%s\n", str );

	for( int i = 0; i < cnt; i++ )
	{
		char t[ 100 ];
		int bits = ((int)bf[0] << 8 ) | bf[1];
		sprintf( t, "%d. MPI length - %d ", i, bits );
		int bytes = (bits + 7) / 8;

		bytesOut( rec + 1, t, bf + 2, bytes, bOut );

		bf += 2 + bytes;
	}
}

int ParsePacketBody( BYTE tag, BYTE *pkt, int pktSize, int recCount )
{
	int res;

	switch ( tag )
	{
		case pkt_PublicKeyEncryptedSessionKey :
		{
			sPublicKeyEncryptedSessionKey *pkesk = ( sPublicKeyEncryptedSessionKey * )pkt;

			int keyMpiSize;

			res = PKT_PublicKeyEncryptedSessionKey_Parse( pkesk, pktSize, &keyMpiSize );

			if ( res != ppe_ok )
				return res;

			bytesOut( recCount, "Key ID", pkesk->keyID, 8 );
			bytesOut( recCount, "Alg ID", &(pkesk->algID), 1 );

			switch ( pkesk->algID )
			{
				case 1:
				case 2:
				case 3:		// RSA ID.
					mpiBufOut( recCount, "RSA  MPI = key^e mod p", pkesk->mpi, 1 );
					break;
				case 16:
				case 20:	// ElGamal
					mpiBufOut( recCount, "Elgamal	two MPIs: 1) g^k mod p, 2) m * (y^k) mod p", pkesk->mpi, 2 );
					break;
				default:
					return ppe_invalidAlgorithmID;

			}

			break;
		}
		case pkt_SecretKey :
		case pkt_PublicKey :
		case pkt_SecretSubkey:
//		case pkt_PublicSubkey:
		{
			sPublicKey *pk = ( sPublicKey * )pkt;

			res = PKT_PublicKey_Parse( pk, pktSize );

			if ( res != ppe_ok )
				return res;

			if ( pk->version < 4 )
			{
				sPublicKeyV3 *pk = ( sPublicKeyV3 * )pkt;

				bytesOut( recCount, "Time      ", (BYTE *)(&pk->time), 4 );

				const char *algName = PKT_GetPublicKeyAlgName( pk->algID );

				if ( algName == NULL )
					return ppe_invalidAlgorithmID;

				int mpiCnt = PKT_GetPublicKeyMPICount( pk->algID );

				char str[ 80 ];
				sprintf( str, "%s - %d mpis", algName, mpiCnt );

				mpiBufOut( recCount, str, pk->mpi, mpiCnt );
			}
			else
			{
				bytesOut( recCount, "Time      ", (BYTE *)(&pk->time), 4 );

				const char *algName = PKT_GetPublicKeyAlgName( pk->algID );

				if ( algName == NULL )
					return ppe_invalidAlgorithmID;

				int mpiCnt = PKT_GetPublicKeyMPICount( pk->algID );

				char str[ 80 ];
				sprintf( str, "%s - %d mpis", algName, mpiCnt );

				mpiBufOut( recCount, str, pk->mpi, mpiCnt );
			}

			BYTE keyID[8];

			int hdrSize = 3; // Publick key packet version 4 has header size 3

			res = PKT_GetPublicKeyIDInternal( pk, pktSize, keyID );
			if ( res != ppe_ok )
				return res;

			bytesOut( recCount, "Key ID", keyID, sizeof( keyID ) );

			break;
		}

		case pkt_0x3d:
		{
			int len = strlen( (char *)pkt ) + 1;
			if ( PKT_CheckMPIBuffer( pkt + len , 1, pktSize - len  ) != pktSize - len )
				bytesOut( recCount, (char *)pkt, pkt + len, pktSize - len );
			else
				mpiBufOut( recCount, (char *)pkt, pkt + len, 1, true );

			break;
		}
		case pkt_UserID :
		{
			char *str = new char [ pktSize + 1 ];
			memcpy( str, pkt, pktSize );
			str[ pktSize ] = 0;
			printfProbels( recCount );
			printf( "%s\n", str );
			delete str;

			break;
		}

		case pkt_Trust:
		{
			bytesOut( recCount, "Value", pkt, pktSize );
			break;
		}

		case pkt_Signature:
		{
			if ( *pkt < 4 )
			{				
				sSignatureV3 *sig = (sSignatureV3 *)pkt;

				res = PKT_SignatureV3_Parse( sig, pktSize );
				if ( res != ppe_ok )
					return res;

				bytesOut( recCount, PKT_Signature_GetTypeName( sig->type ), 0 , 0 );
				bytesOut( recCount, "Time:", (BYTE *)&(sig->time) , sizeof( sig->time) );
				bytesOut( recCount, "Alder hash bytes", (BYTE *)sig->hashAlders , sizeof( sig->hashAlders ) );
				bytesOut( recCount, "Key ID", sig->keyID , sizeof( sig->keyID ) );

				int mpiCnt = PKT_GetEncryptedMessageMPICount( sig->publicKeyAlgID );

				char str[ 80 ];
				sprintf( str, "Public key alg %s, hash alg %s - %d MPIs",
					PKT_GetPublicKeyAlgName( sig->publicKeyAlgID ),
					PKT_GetHashAlgName( sig->hashAlgID ),
					mpiCnt );

				mpiBufOut( recCount, str, sig->mpi, mpiCnt );
			}
			else
			{
				sSignature *sig = (sSignature *)pkt;

				BYTE *spHashed;
				WORD spHashedSize;
				BYTE *spUnhashed;
				WORD spUnhashedSize;
				BYTE *left16Bits;
				BYTE *MPIs;

				res = PKT_Signature_Parse( sig, pktSize, &spHashed, &spHashedSize, &spUnhashed, &spUnhashedSize, &left16Bits, &MPIs );
				if ( res != ppe_ok )
					return res;

				bytesOut( recCount, PKT_Signature_GetTypeName( sig->type ), 0 , 0 );

				printfProbels( recCount + 1 );
				printf( "Hashed subpackets:\n" );

				cSigSubpacketParser shsp( spHashed, spHashedSize );
				while ( shsp.GetSubPacket() )
				{	
					bytesOut( recCount + 2, shsp.GetTypeName( shsp.m_spType ), shsp.m_spBody + 1, shsp.m_spSize - 1 );
				}

				if ( shsp.m_error != ppe_ok )
					return shsp.m_error;

				printfProbels( recCount + 1 );
				printf( "Unhashed subpackets:\n" );

				cSigSubpacketParser susp( spUnhashed, spUnhashedSize );
				while ( susp.GetSubPacket() )
				{	
					bytesOut( recCount + 2, susp.GetTypeName( susp.m_spType ), susp.m_spBody + 1, susp.m_spSize - 1 );
				}

				if ( susp.m_error != ppe_ok )
					return susp.m_error;


				int mpiCnt = PKT_GetEncryptedMessageMPICount( sig->publicKeyAlgID );

				char str[ 80 ];
				sprintf( str, "Public key alg %s, hash alg %s, left 16 %04X - %d MPIs",
					PKT_GetPublicKeyAlgName( sig->publicKeyAlgID ),
					PKT_GetHashAlgName( sig->hashAlgID ),
					(UINT(left16Bits[0]) << 8) | left16Bits[1],
					mpiCnt );

				mpiBufOut( recCount, str, MPIs, mpiCnt );
			}

			break;
		}
		case pkt_PublicSubkey:
		{
			sPublicKey *pk = ( sPublicKey * )pkt;

			res = PKT_PublicKey_Parse( pk, pktSize );

			if ( res != ppe_ok )
				return res;

			bytesOut( recCount, "Time      ", (BYTE *)(&pk->time), 4 );

			const char *algName = PKT_GetPublicKeyAlgName( pk->algID );

			if ( algName == NULL )
				return ppe_invalidAlgorithmID;

			int mpiCnt = PKT_GetPublicKeyMPICount( pk->algID );

			char str[ 80 ];
			sprintf( str, "%s - %d mpis", algName, mpiCnt );

			mpiBufOut( recCount, str, pk->mpi, mpiCnt );

			break;
		}
		case pkt_Image:
		{
			HANDLE hFile = CreateFile(
				"img.jpg",
				GENERIC_WRITE,
				FILE_SHARE_READ,
				NULL,
				OPEN_ALWAYS,
				FILE_ATTRIBUTE_NORMAL,
				NULL );
			
			if ( hFile == INVALID_HANDLE_VALUE )
				break;

			UINT written;

			int hdrSize = 22;

			WriteFile( hFile, pkt + hdrSize, pktSize - hdrSize, &written, NULL );

			CloseHandle( hFile );

			break;

		}

		default:
			break;
	}

	return ppe_ok;
}

int ParsePacket( BYTE *pkt, int totalPktSize, int *parsed )
{
	BYTE tag;
	BYTE *data;
	int dataSize;
	int hdrSize;
	bool partial = 0;

	*parsed = 0;

	int res;
	int recCnt = 1;
	
	do
	{
		res = pgp_PacketHeader_Parse( &tag, &hdrSize, &dataSize, &partial, pkt + *parsed, totalPktSize );

		if ( res != ppe_ok )
			return res;

		data = pkt + *parsed + hdrSize;

		char *tagName;
		res = PKT_GetTagName( tag, &tagName );

		if ( ( tag == pkt_PublicKey ) ||
			 ( tag == pkt_SecretKey )
		   )
			printf("///////////////////////////////////////////////////\n");

		printfProbels( recCnt );


		if (
			( tag == pkt_Signature ) ||
			( tag == pkt_SecretKey )  ||
			( tag == pkt_PublicKey )  ||
			( tag == pkt_PublicSubkey ) ||
			( tag == pkt_SecretSubkey ) 
		   )
			printf( "%s v.%d, size %d\n", tagName, int(pkt[hdrSize]), dataSize );
		else
			printf( "%s(%d), size %d\n", tagName, tag, dataSize );

		if ( res != ppe_ok )
		{
			res = ppe_ok;
//			return res;
		}

		ParsePacketBody( tag, pkt + hdrSize, dataSize, recCnt + 1 );

		*parsed = dataSize + hdrSize;

	}
	while ( partial );

	return res;
}


int ParsePackets( BYTE *ptr, int size )
{
	int res = 0;

	int cnt = 0;

	int totalParsed = 0;
	while ( size > totalParsed )
	{
		int parsed;
		res = ParsePacket( ptr + totalParsed, size - totalParsed, &parsed );
		if ( res != 0 )
			return res;

		cnt++;

		totalParsed += parsed;
	}

	printf( "Total packets %d\n", cnt );

	return res;
}

int ParseKeys( BYTE *ptr, int size )
{
	int res;

	int bfSize = 0;

	bool secret = false;

	res = pgp_EnumKeyIDs( NULL, &bfSize, ptr, size, secret ); 
	if ( res == ppe_bufferTooSmall )
	{
		BYTE *buff = new BYTE[ bfSize ];

		if ( buff == NULL )
			res = ppe_internalError;
		else
		{
			res = pgp_EnumKeyIDs( buff, &bfSize, ptr, size, secret ); 

			if ( res == ppe_ok )
			{
				BYTE *keyPkt;
				int keyPktSize;

				for ( int q = bfSize / pgp_keyIDSize; q--; )
				{
					BYTE *sigPkt;
					int sigPktSize;

					BYTE *userID;
					int userIDSize;

					res = pgp_FindKeyByID( &keyPkt, &keyPktSize, buff + q * pgp_keyIDSize, ptr, size, secret, &sigPkt, &sigPktSize, &userID, &userIDSize );
					if ( res != ppe_ok )
						continue;

					cHashedList hashList;

					res = pgp_MakeKeySignature_0x10_HashList( &hashList, keyPkt, keyPktSize, userID, userIDSize, sigPkt[0] );

					if ( res == ppe_ok )
					{
						if ( sigPkt )
							res = pgp_VerifySignaturePacket( sigPkt, sigPktSize, &hashList, ptr, size );
					}
				}
			}

			delete buff;
		}
	}

	printf( "Total packets %d\n", bfSize / pgp_keyIDSize );

  	return res;
}
/*
class cFileView
{
	HANDLE m_hFile;
	HANDLE m_hMap;

public:
	enum { cfv_ok = 0, cfv_fileDoesNotExist, cfv_errorInOpeningFile };
	int m_err;
	int m_size;
	BYTE *m_ptr;

	cFileView( const char *fileName )
	{
		memset( this, 0, sizeof( *this ) );
		m_err = cfv_fileDoesNotExist;

		if ( _access( fileName, 0 ) != 0 )
			return;

		m_err = cfv_errorInOpeningFile;

		m_hFile =
			CreateFile(
				fileName,
				GENERIC_READ,
				FILE_SHARE_READ | FILE_SHARE_WRITE,
				NULL,
				OPEN_EXISTING,
				FILE_ATTRIBUTE_NORMAL,
				NULL);

		if ( m_hFile == INVALID_HANDLE_VALUE )
			return;

		m_size = GetFileSize( m_hFile, NULL );

		if ( m_size == (UINT)-1 )
			return;

		m_hMap = CreateFileMapping( m_hFile, NULL, PAGE_READONLY, 0, m_size, fileName );

		if ( m_hMap == NULL )
			return;


		m_ptr = (BYTE *)MapViewOfFile( m_hMap, FILE_MAP_READ, 0, 0, m_size );

		if ( m_ptr == NULL )
			return;

		m_err = cfv_ok;
	}

	~cFileView( )
	{
		if ( m_ptr )	UnmapViewOfFile( m_ptr );
		if ( m_hMap )	CloseHandle( m_hMap );
		if ( m_hFile )	CloseHandle( m_hFile );
	}
};

int encryptSessionKey( const char *pkrName, const char *skrName )
{
	int sesAlg = pgpsaTripleDES;
	int hashAlg = pgphaSHA1;

	cPlainTextBlock sesKey;
	cPlainTextBlock decSesKey;

	if ( ( sesKey.Allocate( sa_getKeySize( sesAlg ) ) == NULL ) ||
		 ( decSesKey.Allocate( sa_getKeySize( sesAlg ) ) == NULL ) )
		return 1;

	memset( sesKey.buffer, 0xA5, sesKey.bufferSize );

	cFileView pkRing( pkrName );
	cFileView skRing( skrName );

	if ( pkRing.m_err || skRing.m_err )
		return pkRing.m_err;

	int bfSize = 0;

	bool secret = false;
	cPlainTextBlock idsBuff;

	int res = pgp_EnumKeyIDs( NULL, &bfSize, pkRing.m_ptr, pkRing.m_size, secret ); 
	if ( res == ppe_bufferTooSmall )
	{
		if ( idsBuff.Allocate( bfSize ) == NULL )
			res = ppe_internalError;
		else
		{
			res = pgp_EnumKeyIDs( idsBuff.buffer, &bfSize, pkRing.m_ptr, pkRing.m_size, secret ); 

			if ( res == ppe_ok )
			{
				BYTE *keyPkt;
				int keyPktSize;

				cPlainTextBlock encPkt;
				int encPktSize;
				
				for ( int q = bfSize / keyIDSize; q--; )
				{
					res = pgp_FindKeyByID( &keyPkt, &keyPktSize, idsBuff.buffer + q * keyIDSize, pkRing.m_ptr, pkRing.m_size, secret );

					if ( res == ppe_ok )
					{
						encPktSize = 0;

						res = pgp_FormatPublicKeyEncryptedSessionKeyPacket(
							NULL, &encPktSize, sesAlg, sesKey.buffer, hashAlg, keyPkt, keyPktSize );

						if ( res == ppe_bufferSizeTooSmall )
						{
							if ( encPkt.Allocate( encPktSize ) == NULL )
								return 1;

							res = pgp_FormatPublicKeyEncryptedSessionKeyPacket(
								encPkt.buffer, &encPktSize, sesAlg, sesKey.buffer, hashAlg, keyPkt, keyPktSize );

							int decSesAlg;
							if ( !res )
								res = pgp_DecryptPublicKeyEncryptedSessionKeyFromPacket(
									&decSesAlg, decSesKey.buffer, &decSesKey.bufferSize,
									encPkt.buffer, encPkt.bufferSize, skRing.m_ptr, skRing.m_size, "qqqqqq" );

							if ( !res )
								res = memcmp( sesKey.buffer, decSesKey.buffer, sesKey.bufferSize );

						}
					}
				}
			}
		}
	}

	return res;
}

int parseFile( const char *fileName, bool allPackets = false )
{
	cFileView cfv( fileName );

	if ( cfv.m_err )
		return cfv.m_err;

	BYTE *ptr = cfv.m_ptr;
	int size = cfv.m_size;

	// try to decode base64

	cPlainTextBlock ptb;
	int encSize = 0;
	BYTE *encBlock;

	if ( ppe_ok == 
		findEncodedBlock( 
			&encBlock, &encSize,
			prt, size,
			"-----BEGIN PGP PUBLIC KEY BLOCK-----",
			"-----END PGP PUBLIC KEY BLOCK-----" )
		)
	{
		int decSize = 0;
		base64Decode( NULL, &decSize, encBlock, encSize );
		ptb.Allocate( decSize );
		base64Decode( ptb.buffer, &decSize, encBlock, encSize );
		ptr = ptb.buffer;
		size = decSize;
	}

	return  allPackets ?
				ParsePackets( ptr, size ) :
				ParseKeys( ptr, size );
}

void generateRndBytes( BYTE *bf, int size )
{
	while( size -- )
		bf[size] = (BYTE)rand();
}
*/
/*
int generateTest()
{
	pw_int t,p,rm,r,m,rw;

	BYTE bf[ 4096 / 8 ];


	int res;

	int mgmT = 0;
	int uslT = 0;

	int q = 50;

//	t.setu( 0x1 ); pw_mul_exp2( &t, &t, 64 ); pw_addu( &t, &t, 2 );
//	p.setu( 0x3 ); pw_mul_exp2( &p, &p, 64 ); pw_addu( &p, &p, 4 );
//	pw_mul( &m, &t, &p );

	srand( 0 );

	while( q-- )
	{
		generateRndBytes( bf, sizeof( bf ) );
		t.set( sizeof( bf ), bf );

//		res = pka_setRandomBuffer( bf, sizeof( bf ) );
//		HPKA ska;
//		res = ska_generateKeyAndCreateAlgorithm( &ska, 1, 1024 / 8 );
//		if ( res != pw_OK )	return res;
//
//		ska_free( ska );


		generateRndBytes( bf, sizeof( bf ) );
		bf[ sizeof(bf) - 1 ] |= 1;
		m.set( sizeof( bf ), bf );

		generateRndBytes( bf, sizeof( bf ) );
		p.set( sizeof( bf ), bf );

		res = pw_mod( &t, &t, &m );
		if ( res != pw_OK )	return res;
		res = pw_mod( &p, &p, &m );
		if ( res != pw_OK )	return res;

		/////////////
//t.setu( 0x2 );
//p.setu( 0xb3 );
//m.setu( 0xb );
		/////////////

		clock_t start = clock();

//		res = pw_powm_windows( &rm, &t, &p, &m );
//		res = pw_powm_montgomery( &rm, &t, &p, &m );
		res = pw_powm_montgomery_windows( &rm, &t, &p, &m );
		if ( res != pw_OK )	return res;

		clock_t mgm = clock();

//		res = pw_powm( &r, &t, &p, &m, true );
		res = pw_powm_windows( &r, &t, &p, &m );
		if ( res != pw_OK )
			return res;

		clock_t finish = clock();

		mgmT = mgm - start;
		uslT = finish - mgm;


		res = pw_cmp( &r, &rm );
		if ( res != pw_OK )
			return res;


		printf( "mgm %d, usual %d, ratio %d.%d\n", mgmT, uslT,
				mgmT ? uslT/mgmT : -1,
				mgmT ? (( uslT % mgmT ) * 10 ) / mgmT : -1);
	}

	return res;
}
*/

class cProgressActorPrint : public cProgressActor
{
	UINT sec;

public:
	cProgressActorPrint()
	{
		sec = time(0);
	}

	~cProgressActorPrint()
	{
		printf( "\nTime - " );
		sec = time(0) - sec;
		if ( sec > 60 * 60 )
		{
			printf( "%d:", sec / (60*60) );
			sec = sec % (60*60);
		}

		printf( "%02d:", sec / (60) );
		sec = sec % (60);

		printf( "%02d\n", sec );
	}

	virtual int func( const char *name, int value0, int value1  )
	{
		static int summ, count;

		if ( name )
		{	printf( name );
			summ = 0; 
			count = 0;
		}
		else
		if ( value0 == 0 )
		{
			if ( value1 == -1 )
				printf( "." );
			else
			{
				if ( summ < value1 )
					summ = value1;
				printf( "%d", value1 );
			}
		}
		else
		{
			printf( "%d\n", summ );
		}

		return 0;
	};
} actor;

int WriteSecretKey( cPlainTextBlock *ptb, HPKA hSka, const char *friendlyName, const char *eMailAddress, cPassword *pwd )
{
	WCHAR friendlyNameBuff[ 1024 ];

	int friendlyNameSize = 0;

	if ( strlen( eMailAddress ) != 0 )
	{

#ifdef UNICODE
		wcscpy( friendlyNameBuff, friendlyName );
		wcscat( friendlyNameBuff, _T( " - <" ) );
		wcscat( friendlyNameBuff, eMailAddress );
		wcscat( friendlyNameBuff, _T( ">" ) );

		friendlyNameSize = wcslen( friendlyNameSize );

#else
		strcpy( (char *)friendlyNameBuff, friendlyName );
		strcat( (char *)friendlyNameBuff, " - <" );
		strcat( (char *)friendlyNameBuff, (char *)eMailAddress );
		strcat( (char *)friendlyNameBuff, ">" );

		friendlyNameSize = ConvertToUnicodeString( friendlyNameBuff, sizeof( friendlyNameBuff ) / sizeof( *friendlyNameBuff ), (char *)friendlyNameBuff );

#endif // UNICODE
	}

	return P12_MakeSecretKeyPkt( ptb, hSka, friendlyNameBuff, friendlyNameSize, pwd );
}

int generateKeyPair()
{

	int crtSize = 0;

	BYTE pkAlg = pgppkaRSA;

	const char *oidStr = getSignatureAlgOidStr( pgphaMD5, pkAlg );
	sNameAttributeList rdn;
	rdn.Add( sNameAttribute::id_at_countryName, "Russia", asn1tag_PrintableString );
	rdn.Add( sNameAttribute::id_at_stateOrProvinceName, "Siberia", asn1tag_PrintableString );
	rdn.Add( sNameAttribute::id_at_localityName, "Novosibirsk", asn1tag_PrintableString );
	rdn.Add( sNameAttribute::id_at_organizationName	, "IEA", asn1tag_PrintableString );
	rdn.Add( sNameAttribute::id_at_organizationalUnitName, "Room 17", asn1tag_PrintableString );
	rdn.Add( sNameAttribute::id_at_commonName, "Alexander V. Pichuev", asn1tag_PrintableString );
	rdn.Add( sNameAttribute::pkcs9_emailAddress, "<pav@iae.nsk.su>", asn1tag_IA5String );

	HPKA ska, pka;

	srand(time(0) ^ clock());

#define keySize 1024 * 2

	BYTE pool[ keySize / 8 ]; // = { 0,1,2,3,4,5,6,7,8,9,  0,1,2,3,4,5,6,7,8,9,  0,1,2,3,4,5,6,7,8,9,  0,1,2,3,4,5,6,7,8,9  };
	for ( int i = sizeof( pool ); i--; )
	{
		pool[i] = BYTE(rand());
	}

	cRNG::setPool( pool, sizeof( pool ) );

	ska_RSA_generateSafePrime(  true );

	int res = ska_generateKeyAndCreateAlgorithm( &ska, pkAlg, keySize / 8, &actor ); // in bytes
	if ( res != pkae_ok )
		return res;

	res = ska_derivePublicKeyAlgorithm( &pka, ska );
	if ( res != pkae_ok )
	{
		ska_free( ska );
		return res;
	}

	x509_certificate cer;
	res = cer.createPacket( NULL, &crtSize, pka, ska, rdn.head, rdn.head, 2002, 11, 25, 2003, 11, 25, oidStr, 1 );

	cPlainTextBlock ptb;
	if ( ptb.Allocate( crtSize ) != NULL )
	{
		res = cer.createPacket( ptb.buffer, &crtSize, pka, ska, rdn.head, rdn.head, 2002, 11, 25, 2003, 11, 25, oidStr, 1 );
	}

	pka_free( pka );

	if ( res == x509e_ok )
	{
		res = verifyCertificate( ptb.buffer, crtSize );
		if ( res != x509e_ok )
		{
			ska_free( ska );
			return res;
		}
	}

	WriteToFile( "qqq.cer", ptb.buffer, crtSize );

	//
	//	Writes secret key to p12_pfx_pdu_p7data
	//
	cPassword pwd( "qqqqqq" );

	res = WriteSecretKey( &ptb, ska, "Alexander V. Pichuev", "pav@iae.nsk.su", &pwd );

	if ( res == x509e_ok )
		WriteToFile( "qqq.p12", ptb.buffer, ptb.bufferSize );

	ska_free( ska );

	return res;
}

int PFX_PDU_parse( BYTE *bf, int size, char *pPwd = NULL )
{
	cPassword pwd( pPwd );

	cPlainTextBlock ptbKey;

	int res = P12_PFX_PDU_parse( bf, size, &ptbKey, &pwd );
	if ( res == ppe_ok )
		return res;

	x509_privateKeyInfo pki;

	res = pki.parse( ptbKey.buffer, ptbKey.bufferSize );

	HPKA hSka;

	if ( res == pkae_ok )
		res = ska_createAlgorithm_X509( &hSka, &pki );

	if ( res != pkae_ok )
		return res;

	ska_free( hSka );

	return x509e_ok;
/*
	p12_safeBag bag;
	int res = bag.parse( pf, size );
	if ( strcmp( bag.oid.oidStr, oidP12BtPKCS8SroudedKeyBagOID ) != 0 )
		return x509e_unexpectedOIDType;
*/
}

int findDivider( pw_int *d, pw_int *p );

static int testPrime( int exp, UINTLONG value )
{
	pw_int p;
	p.setu( 0 );
	pw_setBit( &p, exp );
	pw_addu( &p, &p, value );

	pw_int d;

	d.setu( 2 );

	int res = pw_OK;
	
	while( true )
	{
		res = findDivider( &d, &p );

		if ( res == pw_BufferSizeTooSmall )
		{
			printf( "Prime NOT found\n" );
			return res;
		}

		if ( res != pw_OK )
		{
			printf( "Error - %d\n", res );
			return res;
		}

		if ( res != pw_OK )
			return res;

		pw_int q,r;

		res = pw_div( &q, &r, &p, &d );

		if ( pw_cmpu( &r, 0 ) != 0 )
			return pw_InternalError;

		printf( "Prime found %I64X\n", d.data[0] );

		pw_set( &p, &q );
	}

	return res;
}

int parseFile( const char *fileName, bool allPackets = false )
{
	cFileView cfv( fileName );

	if ( cfv.m_err )
		return cfv.m_err;

	BYTE *ptr = cfv.m_ptr;
	int size = cfv.m_size;

	// try to decode base64

	cPlainTextBlock ptb;
	int encSize = 0;
	const BYTE *encBlock;

	if ( ppe_ok == 
		findEncodedBlock( 
			&encBlock, &encSize,
			ptr, size,
			"-----BEGIN PGP MESSAGE-----",
			"-----END PGP MESSAGE-----" )
		)
	{
		int decSize = 0;
		base64Decode( NULL, &decSize, encBlock, encSize );
		ptb.Allocate( decSize );
		base64Decode( ptb.buffer, &decSize, encBlock, encSize );
		ptr = ptb.buffer;
		size = decSize;
	}

	return  allPackets ?
				ParsePackets( ptr, size ) :
				ParseKeys( ptr, size );
}

int main( int argc, char *argv[] )
{
	return parseFile( argv[1], 1 );

//	return testPrime( 64, 0x1B );
	return testPrime( 128, 0x87 );
	return testPrime( 64, 0xd );
//	return testPrime( 3, 0x1 );

	if ( argc == 1 )
		return generateKeyPair();

//	return viewCertificate( argv[1] );
/*
	if ( strcmp( argv[1], "-s" ) == 0 )
		return encryptSessionKey( argv[2], argv[3] );
	else
	if ( strcmp( argv[1], "-p" ) == 0 )
		return parseFile( argv[2], true );
	else
		return parseFile( argv[1] );
*/

//	return generateTest();

	int arg = 1;
	bool skip = false;
	char *pwd = NULL;

//	if ( stricmp( argv[1], "-kr" ) == 0 )  // parse all key ring
//		return parseFile( argv[2], true );
//	else
//	if ( stricmp( argv[1], "-key" ) == 0 ) // keys only
//		return parseFile( argv[2] );
//	else
	if ( strcmp( argv[1], "-s" ) == 0 ) // skip octet parsing
	{
		skip = true;
		arg++;
	}
	else
	if ( strncmp( argv[1], "-p", 2 ) == 0 ) // get key from shrouded key bag
	{
		pwd = argv[1] + 2;
		arg++;
	}

	cFileView cfv( argv[ arg ] );

	if ( cfv.m_err )
		return cfv.m_err;

	BYTE *pkt;
	int pktSize = 0;

	const char *szRSAPrivKeyB64_Begin = "-----BEGIN RSA PRIVATE KEY-----";
	const char *szRSAPrivKeyB64_End   = "-----END RSA PRIVATE KEY-----";

	const BYTE *block;
	int blockSize = 0;
	cPlainTextBlock ptb;

	if ( findEncodedBlock( &block, &blockSize, cfv.m_ptr, cfv.m_size, szRSAPrivKeyB64_Begin, szRSAPrivKeyB64_End ) )
	{
		int decBlockSize = 0;

		int res = base64Decode( NULL, &decBlockSize, block, blockSize );

		if ( res != ppe_bufferTooSmall )
			return res;

		ptb.Allocate( decBlockSize );

		res = base64Decode( ptb.buffer, &decBlockSize, block, blockSize );

		if ( res != ppe_ok )
			return res;

		pkt = ptb.buffer;
		pktSize = decBlockSize;
	}
	else
	{
		pkt = cfv.m_ptr;
		pktSize = cfv.m_size;
	}

	if ( pwd )
	{
		return PFX_PDU_parse( pkt, pktSize, pwd );
	}

//	return addCertificate( cfv.m_ptr, cfv.m_size );

//	return verifyCertificate( cfv.m_ptr, cfv.m_size );

	return x509_test( pkt, pktSize, skip );

}