#include "SBlocks.h"
#include "smemory.h"
#include <assert.h>
#include <stdio.h>

#ifndef _WIN32
//#include "../../misc/misc.h"
#define _myFree SFree
#define _myAlloc SMalloc
#define CheckMem( bf ) SMemCheck( bf )
#else


#ifdef _DEBUG_MEMORY

void CheckMem( BYTE *bf )
{
	bf -= 8;

	SIZE_T sizeB;
	SIZE_T sizeT;

	assert( memcmp( bf, "DBGA", 4 ) == 0 );
	memcpy( &sizeB, bf + 4, 4 );
	memcpy( &sizeT, bf + 8 + sizeB, 4 );
	assert( sizeT == sizeB );

	assert( memcmp( bf + 8 + sizeB + 4, "TAIL", 4 ) == 0 );
}

HGLOBAL DbgGlobalFree( HGLOBAL hMem )
{
	BYTE *bf = (BYTE *)hMem;

	CheckMem( bf );

	bf -= 8;

	return GlobalFree( (HGLOBAL)bf );
 }

HGLOBAL DbgGlobalAlloc( UINT uFlags, SIZE_T dwBytes )
{
	HGLOBAL hGlbl = GlobalAlloc( uFlags, dwBytes + 8 + 8 );
	BYTE *bf = (BYTE *)hGlbl;

	if ( bf == NULL )
		return NULL;

	memcpy( bf, "DBGA", 4 );
	memcpy( bf + 4, &dwBytes, 4 );

	memcpy( bf + 8 + dwBytes, &dwBytes, 4 );
	memcpy( bf + 8 + dwBytes + 4, "TAIL", 4 );

	return (HGLOBAL)(bf + 8);
}

#define _myFree( X ) DbgGlobalFree( HGLOBAL( X ) )
#define _myAlloc (BYTE*)DbgGlobalAlloc

#else
#define _myFree GlobalFree
#define _myAlloc( __size ) GlobalAlloc( GMEM_FIXED, __size )
#define CheckMem( bf )
#endif

int cUniversalString::Set( const wchar_t *wcs, int wcsLen /* = -1 */ )
{
	ClearBuffers();

	size_t num = 0;

	errno_t err = wcstombs_s( &num, NULL, 0, wcs, wcsLen );

	if ( err != 0 )
		return 0;

	bufferSize = (int)num;

	buffer = ( char *)AllocBuffer( bufferSize );
	if ( buffer == 0 )
		return 0;

	wcstombs_s( &num, buffer, bufferSize, wcs, wcsLen );


	if ( wcsLen == -1 )
		wcsLen = (int)wcslen( wcs );

	if ( AllocateUnicodeBuffer( wcsLen + 1 ) == 0 )
		return 0;

	BYTE *leBf = (BYTE*)wcs;

	unicodeBuffer[0] = 0;
	memcpy_s( unicodeBuffer + 1, unicodeBufferSize  - 3, leBf, wcsLen * sizeof( wchar_t ) - 1 );
	unicodeBuffer[ wcsLen * sizeof( wchar_t )  ] = 0;
	unicodeBuffer[ wcsLen * sizeof( wchar_t ) + 1 ] = 0;

	wcBufferSize = unicodeBufferSize;
	wcBuffer = (wchar_t *)AllocBuffer( wcBufferSize );
	if ( wcBuffer == NULL )
		return 0;

	wcsncpy_s( wcBuffer, wcBufferSize / sizeof( wchar_t ), wcs, _TRUNCATE );

	return wcBufferSize;
}

int cUniversalString::MakeFromUnicodeString()
{
	// Converts ASN.1 unicode string to wchar_t 
	if ( wcBuffer )
	{
		memset( wcBuffer, 0, wcBufferSize );
		FreeBuffer( (BYTE *)wcBuffer );
	}
    
	wcBufferSize = unicodeBufferSize;
	wcBuffer = (wchar_t *)AllocBuffer( wcBufferSize );

	BYTE *tmpBuff = (BYTE *)wcBuffer;

	if ( wcBuffer == NULL )
		wcBufferSize = 0;
	else
	for ( int i = 0; i < wcBufferSize; i += 2 )
	{
		tmpBuff[ i ] = unicodeBuffer[ i + 1 ];
		tmpBuff[ i + 1 ] = unicodeBuffer[ i ];
	}

	// Convert wchar_t to char

	if ( buffer )
	{
		memset( buffer, 0, bufferSize );
		FreeBuffer( (BYTE*)buffer );
	}

	bufferSize = unicodeBufferSize / sizeof( wchar_t );
	buffer = (char *)AllocBuffer( bufferSize );

	if ( buffer == NULL )
		bufferSize = 0;
	else
	{
		size_t num;
		wcstombs_s( &num, buffer, bufferSize, wcBuffer, _TRUNCATE );
	}
/*
	for ( int i = 0; i < bufferSize; i++ )
	{
		buffer[ i ] = unicodeBuffer[ i * 2 + 1 ];
		// WideCharToMultiByte does not used for compatibility with Windows 95.
	}
*/
	return bufferSize;
}

#endif // #ifndef _WIN32

//
//	The call can be send as void handler ( see static WriteBytes ).
//	But it has a signature value. So validity of the block can be checked.
//
int cPlainTextBlock::WriteBytesStatic( const void *ptr, BYTE *data, int dataSize )
{
	cPlainTextBlock *ptb = (cPlainTextBlock *)ptr;

	if ( !ptb->checkSig() )
		return 0;

	return ptb->WriteBytes( data, dataSize );
}

void cPlainTextBlock::ClearBuffer()
{
	if ( buffer )
		memset( buffer, 0, bufferSize );
}

void cPlainTextBlock::SeekToBegin()
{
	//
	//	Clear previously filled data...
	//
	ClearBuffer();

	bufferSize = 0;
}

int cPlainTextBlock::WriteBytes( const BYTE *data, int dataSize, bool init )
{
	if ( init )
		SeekToBegin();

	if ( bufferSize + dataSize > allocatedSize )
		if ( ReAllocate( bufferSize + dataSize ) == NULL )
			return 0;

	if ( data != NULL )
		memcpy( buffer + bufferSize, data, dataSize );

	bufferSize += dataSize;

	return dataSize;
}

int cPlainTextBlock::PutByte( BYTE bt )
{
	return WriteBytes( &bt, 1 );
}

void cPlainTextBlock::Free()
{
	if ( buffer != NULL )
	{
	    CheckMem( buffer );
	    _myFree( buffer );
	    buffer = NULL;
	}

	allocatedSize = bufferSize = 0;
}

//
//	Note, ReAllocate does not change bufferSize - current filled data does not changed...
//
BYTE *cPlainTextBlock::ReAllocate( int size )
{
	if ( allocatedSize == 0 )
	{
		BYTE *bf = Allocate( size );
		bufferSize = 0; // See note above
		return bf;
	}

	CheckMem( buffer );

	if ( bufferSize >= size )
	{
		memset( buffer + size, 0, bufferSize - size );
		return buffer;
	}

	if ( allocatedSize >= size )
		return buffer;

	BYTE *newBuffer = (BYTE *)_myAlloc( size );

	if ( newBuffer == NULL )
		return NULL;

	allocatedSize = size;

	memcpy( newBuffer, buffer, bufferSize );

	memset( newBuffer + bufferSize, 0, allocatedSize - bufferSize );

	memset( buffer, 0, bufferSize );

	_myFree( buffer );

	buffer = newBuffer;

	return buffer;
}

BYTE *cPlainTextBlock::Allocate( int size )
{
	if ( buffer )
		Free();

	buffer = (BYTE *)_myAlloc( size  );

	if ( buffer )
	{
		allocatedSize = bufferSize = size;
		ClearBuffer();
	}

	return buffer;
}

BYTE *cPlainTextBlock::Assign( const BYTE *inBuffer, int size )
{
	if ( Allocate( size ) == NULL )
		return NULL;

	memcpy( buffer, inBuffer, size );

	return buffer;
}


cPlainTextBlock::~cPlainTextBlock()
{
	Free();
}

/////////////////////
//
//	class cUniversalString
//
void cUniversalString::ClearBuffers()
{
	if ( buffer )
		memset( buffer, 0, bufferSize );

	if ( unicodeBuffer )
		memset( unicodeBuffer, 0, unicodeBufferSize );

	if ( wcBuffer )
		memset( wcBuffer, 0, wcBufferSize );
}

int cUniversalString::Set( const char *str, int size /* = -1 */ )
{
	ClearBuffers();
	if ( size == -1 )
		size = (int)strlen( str );

	buffer = ( char *)AllocBuffer( size + 1 );

	if ( buffer == NULL )
		return 0;

	memcpy( buffer, str, size );
	buffer[ size ] = 0;

	bufferSize = size + 1;

	return bufferSize;
}

int cUniversalString::AllocateUnicodeBuffer( int sizeInSymbols )
{
	int size = sizeInSymbols * sizeof( wchar_t );

	if ( unicodeBuffer != NULL )
	{
		memset( unicodeBuffer, 0, unicodeBufferSize );

		if ( unicodeBufferSize < size  )
		{
			FreeBuffer( unicodeBuffer );
			
			unicodeBufferSize = 0;
			unicodeBuffer = NULL;
		}
		else
			return unicodeBufferSize;
	}

	if ( unicodeBuffer == NULL )
	{
		unicodeBuffer = AllocBuffer( size );
		if ( unicodeBuffer == NULL )
		{
			unicodeBufferSize = 0;
			return 0;
		}
	}

	unicodeBufferSize = size;

	memset( unicodeBuffer, 0, size );

	return unicodeBufferSize;
}

int cUniversalString::SetUnicode( const wchar_t *wStr, int sizeInWideChars /* = -1*/, bool checkBigEndian /* = false */ )
{
	int size = sizeInWideChars;
	if ( size == -1 )
		size = (int)wcslen( wStr );

//	ClearBuffers();
	if ( AllocateUnicodeBuffer( size + 1 ) == 0 )
		return 0;

	if ( (int)(unicodeBufferSize / sizeof( wchar_t )) < (size + 1) )
		return 0;
 
	bool bigEndian = true;

	if ( checkBigEndian )
	{
		bigEndian = *(BYTE*)wStr == 0;
	}

	if ( bigEndian )
		memcpy( unicodeBuffer, wStr, size * sizeof( wchar_t ) );
	else
	if ( size != 0 )
	{
		memcpy( unicodeBuffer + 1, wStr, size * sizeof( wchar_t ) - 1 );
		unicodeBuffer[ 0 ] = 0;
	}

	wchar_t *wPtr = (wchar_t *)unicodeBuffer;
	wPtr[ size ] = 0;

	return unicodeBufferSize;
}

int cUniversalString::SetUnicode( const BYTE *buff, int sizeInBytes /* = -1 */, bool checkBigEndian /* = false */ )
{
	int sizeInWideChars = ( sizeInBytes == -1 ? -1 : sizeInBytes / 2 );
	return SetUnicode( ( const wchar_t *)buff, sizeInWideChars, checkBigEndian );
}

BYTE *cUniversalString::AllocBuffer( int size )
{
	BYTE *newBuffer = (BYTE *)_myAlloc( size );
	return newBuffer;
}

cUniversalString::cUniversalString(const cUniversalString& rhv)
{
	this->Set(rhv.buffer);
	this->GenerateUnicodeString();
}

const cUniversalString& cUniversalString::operator = (const cUniversalString& rhv)
{
	this->ClearBuffers();
	
	if ( buffer ) {
		FreeBuffer( (BYTE *)buffer );
		buffer = NULL;
	}

	if ( wcBuffer ) {
		FreeBuffer( (BYTE *)wcBuffer );
		wcBuffer = NULL;
	}

	if ( unicodeBuffer ) {
		FreeBuffer( unicodeBuffer );
		unicodeBuffer = NULL;
	}

	this->Set(rhv.buffer);
	this->SetUnicode(rhv.wcBuffer);

	return *this;
}


cUniversalString::~cUniversalString()
{
	if ( buffer ) {
		FreeBuffer( (BYTE *)buffer );
	}

	if ( wcBuffer ) {
		FreeBuffer( (BYTE *)wcBuffer );
	}

	if ( unicodeBuffer ) {
		FreeBuffer( unicodeBuffer );
	}
}

void cUniversalString::FreeBuffer( BYTE *buff )
{
    _myFree( buff );
}

int cUniversalString::GenerateUnicodeString()
{
	if ( unicodeBuffer )
	{
		memset( unicodeBuffer, 0, unicodeBufferSize );
		FreeBuffer( unicodeBuffer );
		unicodeBuffer = NULL;
		unicodeBufferSize = 0;
	}

	if ( ( wcBufferSize != 0 ) && ( wcBuffer[0] != 0 ) )
	{
		wstr2unicodeBuff( wcBuffer );
	}
	else
	if ( bufferSize != 0 )
	{
		int pwdLen = (int)strlen( buffer ) + 1;

		unicodeBufferSize = ( pwdLen ) * 2;
		unicodeBuffer = (BYTE *)AllocBuffer( unicodeBufferSize );

		if ( unicodeBuffer == NULL )
			unicodeBufferSize = 0;
		else
		for ( int i = 0; i < pwdLen; i++ )
		{
			unicodeBuffer[ i * 2 ] = 0;
			unicodeBuffer[ i * 2 + 1 ] = buffer[ i ];
		}
	}

	return unicodeBufferSize;
}


//
//	Wide char string / unicode buffer conversions.
//
//	wstr2unicodeBuff 
//		unicode buffer size has to be equal wcslen( ws ) * 2 bytes
//		returns number of copied wide char symbols
//
int cUniversalString::wstr2unicodeBuff( BYTE *unicodeBuff, wchar_t *ws )
{
	int len = (int)wcslen( ws );
	for( int i = len; i--; )
	{
		unicodeBuff[ i * 2 + 1 ] = BYTE( ws[ i ] );
		unicodeBuff[ i * 2 ] = BYTE( ws[ i ] >> 8 );
	}

	return len;
}

//
//	unicodeBuff2wstr
//		size of wc buffer has to be one symbol more of unicodeBuffByteSize/2
//		for null termination symbol
//
int cUniversalString::unicodeBuff2wstr( wchar_t *ws, int wsBufferSize, BYTE *unicodeBuff, int unicodeBuffByteSize )
{
	int l = ( unicodeBuffByteSize / 2 );

	if ( l > (int)(wsBufferSize / sizeof( wchar_t )) )
		return 0;

	ws[l] = 0;

	for( ; l--; )
	{
		ws[l] = unicodeBuff[ l * 2 + 1 ] | ( wchar_t( unicodeBuff[ l * 2 ] ) << 8 );
	}

	return (int)wcslen( ws );
}
