/////////////////////////////////////////////////////////////////
//
//	Copyright (c) 1994-2005 Jetico, Inc., Finland
//	All rights reserved.
//
//	File: cPassword.h
//	Revision: 
//	Created:	12.05.2005
//	Description: 
//		cPassword
//			Class for storing character string for password.
//			It has buffer with predefined size.
//			Destructor clears allocated memory buffer.
//
//
/////////////////////////////////////////////////////////////////

#ifndef _cPassword_h_
#define _cPassword_h_

//#include <windows.h>
#include "../publicKeyAlgs/Power/Windefs.h"
#include "SBlocks.h"
#include <assert.h>

class cPassword : public cUniversalString
{
  public:
	enum { MaxLenght = 256, MinimumLenght = 8 };

	void InitBuffers();

	cPassword( const char *pwdStr = NULL );
	cPassword( const wchar_t *pwdStr );

	~cPassword();

	bool IsEmpty() { return strlen( buffer ) == 0; }

	bool CheckValidSize();
	bool wcCheckValidSize();


	//
	//	Returns difference of first not matched symbols.
	//
	int Compare( const cPassword *pwd ) const;
	int wcCompare( const cPassword *pwd ) const;

	//
	//	generateKeyFromPassword
	//		Uses P12_producePseudoRandomBytes functions for key generation.
	//
	bool generateKeyFromPassword(
		BYTE *outBuff,	int outBuffSize,
		BYTE hashAlgId, // = bchaSHA256,
		BYTE *salt,		int saltSize, // NULL
		int iteration, // == 1
		bool key // key or IV
	);

	//
	//	The module which uses the function is responsible 
	//	for including the "UnicodeFuncs.h" first
	//
#ifdef _UnicodeFuncs_
	//
	//	ReadFromEditBox
	//		1. Reads wide char text from edit box to wcBuffer,
	//		2. Generates UTF-8 encoded string to the buffer from wcBuffer,
	//		3. Generates UTF-16 unicode string to unicodeBuffer from wcBuffer.
	//
	int ReadFromEditBox( HWND hEB )
	{
		memset( wcBuffer, 0, wcBufferSize );
		GetWindowTextT( hEB, wcBuffer, ( wcBufferSize  - 1 ) / sizeof( wchar_t ) );

		if ( WideCharToMultiByte( CP_UTF8, 0, wcBuffer, -1, buffer, bufferSize, NULL, NULL )  == 0 )
		{
			assert( false );
			// Widows 95 does not support UTF-8
			GetWindowTextA( hEB, buffer, bufferSize );
		}

		return wstr2unicodeBuff( wcBuffer );
	}

	//
	//	ReadFromEditBuffer
	//		1. Fills wcBuffer,
	//		2. Generates UTF-8 encoded string to the buffer from wcBuffer,
	//		3. Generates UTF-16 unicode string to unicodeBuffer from wcBuffer.
	//
	int ReadFromBuffer( wchar_t *buff, UINT cp = CP_UTF8  )
	{
		memset( wcBuffer, 0, wcBufferSize );
		wcsncpy_s( wcBuffer, wcBufferSize / sizeof( wchar_t ), buff, _TRUNCATE );

		if ( WideCharToMultiByte( cp, 0, wcBuffer, -1, buffer, bufferSize, NULL, NULL ) == 0 )
		{
			assert( false );

			// Widows 95 does not support UTF-8 but supports ANSI code page
			if ( WideCharToMultiByte( CP_ACP, 0, wcBuffer, -1, buffer, bufferSize, NULL, NULL ) == 0 )
			{
				assert( false );
				return 0;
			}
		}

		return wstr2unicodeBuff( wcBuffer );
	}
#endif
};

#endif /* _cPassword_h_*/

