#include "certstore.h"

// Uses Library: Crypt32.lib.
// Requires _WIN32_WINNT=0x400 to be defibned.

//#if(_WIN32_WINNT < 0x0400)
//#if defined(_WIN32_WINNT ) 
//#undef _WIN32_WINNT
//#endif // defined(_WIN32_WINNT ) 
#define _WIN32_WINNT 0x400
//#endif // (_WIN32_WINNT < 0x0400)

#include <wincrypt.h>

cCertStore::cCertStore( int storeType )
{
	wchar_t *storeTypeString;

	switch ( storeType )
	{
		case estRoot	: storeTypeString = L"Root";	break;
		case estTrust	: storeTypeString = L"Trust";	break;
		case estMy		: storeTypeString = L"My";		break;
		case estCA		: storeTypeString = L"CA";		break;
		default : 
			hSystemStore = NULL;
			return;
	}

	hSystemStore = CertOpenStore(
		 CERT_STORE_PROV_SYSTEM, // The system store will be a virtual store.
		 0,                      // Encoding type not need with this PROV.
		 NULL,                   // Accept the default HCRYPTPROV. 
		 CERT_SYSTEM_STORE_CURRENT_USER,
		 storeTypeString
		 );
}

cCertStore::~cCertStore()
{
	if ( hSystemStore )
		CertCloseStore( hSystemStore, CERT_CLOSE_STORE_CHECK_FLAG );
}

int cCertStore::findCertificateByDN(
		BYTE **pPtr,	// pointer to found cetificate buffer in store.
		int *pSize,		// pointer to size of found certificate
		BYTE *dn,		// Distinguished Name buffer
		int dnSize		// size od DN buffer.
		)
{
	if ( hSystemStore == NULL )
		return errInvalidStore;

	CERT_NAME_BLOB nb;
	nb.pbData = dn;
	nb.cbData = dnSize;

	PCCERT_CONTEXT  pDesiredCert = NULL;   

	pDesiredCert = CertFindCertificateInStore(
		hSystemStore,
		PKCS_7_ASN_ENCODING | X509_ASN_ENCODING,
		0,                           // No dwFlags needed. 
		CERT_FIND_SUBJECT_NAME, // CERT_FIND_SUBJECT_STR, // CERT_FIND_ANY, //CERT_FIND_SUBJECT_STR,
		&nb,					//L"Thawte",
		pDesiredCert);

	if ( pDesiredCert == NULL )
		return errCertificateNotFound;

	if ( pPtr != NULL )
		*pPtr =  pDesiredCert->pbCertEncoded;
	if ( pSize != NULL )
		*pSize = pDesiredCert->cbCertEncoded;

	return errOK;
}
/*
//
//	Some certificate contains as extension Key ID of athority issued the certificate.
//	
int cCertStore::findCertificateByAthorityKeyID(
		BYTE **pPtr,	// pointer to found cetificate buffer in store.
		int *pSize,		// pointer to size of found certificate
		BYTE *keyID,	// Key ID buffer
		int keyIDSize	// size of key ID buffer.
		)
{
	if ( hSystemStore == NULL )
		return errInvalidStore;
//
//   AuthorityKeyIdentifier ::= SEQUENCE {
//      keyIdentifier             [0] KeyIdentifier           OPTIONAL,
//      authorityCertIssuer       [1] GeneralNames            OPTIONAL,
//      authorityCertSerialNumber [2] CertificateSerialNumber OPTIONAL  }
//

	x509_struct st;

	int res = st.parse( keyID, keyIDSize, 0,0, asn1tag_SEQUENCE );

	if ( res != x509e_ok ) return res;

	x509_struct opt;

	res = opt.parse( st.B, st.L, 0, 0, asn1tag_OPTIONAL );
	if ( res != x509e_ok ) return res;

	int num = opt.getOptionalNum();

	if ( ( num == -1 || ( num > 2 ) ) return x509e_internalError;


	CERT_ID ci;

	switch ( num )
	{
		case 0: ci.dwIdChoice = CERT_ID_KEY_IDENTIFIER; break;
		case 1: return x509e_internalError; // ci.dwIdChoice = CERT_ID_SHA1_HASH; break;
		case 2: ci.dwIdChoice = CERT_ID_ISSUER_SERIAL_NUMBER; break;
	}
	
	ci.KeyId.cbData = opt.L;
	ci.KeyId.pbData = opt.B;

	PCCERT_CONTEXT  pDesiredCert = NULL;   

	pDesiredCert = CertFindCertificateInStore(
		hSystemStore,
		PKCS_7_ASN_ENCODING | X509_ASN_ENCODING,
		0,
		CERT_FIND_CERT_ID,
		&ci,
		pDesiredCert );

	if ( pDesiredCert == NULL )
		return errCertificateNotFound;

	if ( pPtr != NULL )
		*pPtr =  pDesiredCert->pbCertEncoded;
	if ( pSize != NULL )
		*pSize = pDesiredCert->cbCertEncoded;

	return errOK;
}
*/

int cCertStore::addCertificateToStore( BYTE *crtBuff, int size )
{
	BOOL res = CertAddEncodedCertificateToStore(
		hSystemStore,
		PKCS_7_ASN_ENCODING | X509_ASN_ENCODING,
		crtBuff,
		size,
		CERT_STORE_ADD_ALWAYS,
		NULL );

	if ( res )
		return errOK;
	else
		return errCertificateWasNotAdded;
}

#include "..\x509pkts\x509_structs.h"

int verifyX509Certificate( BYTE *ptr, int size )
{
	x509_certificate crt;
	int res = crt.parse( ptr, size );

	if ( res != x509e_ok ) return res;

	cCertStore sto( cCertStore::estRoot );

	BYTE *issuerCrtPtr;
	int issuerCrtSize;

	BYTE *issuerDN = crt.tbsCertificate.issuerName.H;
	int issuerDNSize = crt.tbsCertificate.issuerName.TL;

	res = sto.findCertificateByDN( &issuerCrtPtr, &issuerCrtSize, issuerDN, issuerDNSize );
	if ( res != sto.errOK ) return res;

	x509_certificate issuerCrt;

	res = issuerCrt.parse( issuerCrtPtr, issuerCrtSize );
	if ( res != x509e_ok ) return res;

	res = crt.verify( &(issuerCrt.tbsCertificate.subjectPKInfo) );
	if ( res != x509e_ok ) return res;

	return res;
}
