#include "container.h"
#include "kgghost.h"

#include <alg/alg.h>
#include <alg/manager.h>
#include <common/log.h>

#include <backend/io.h>

#include <string.h>

using namespace bdk;
using namespace kgghost;


static bool GetRandomSeed( unsigned char *seed, int seedLength )
{

    FILE *fd;
    int  i;

    if ( ! (fd=fopen("/dev/urandom","r")) ) return false;

    for ( i=0; i<seedLength; i++ )
        seed[i]=(unsigned char)fgetc(fd);

    fclose (fd);
    return true;
}   


BCContainerViewHandle V8Format::get_view(IBCContainerIO* io)
{
	BCAlgService* serv = bc_core()->get_alg_service(this->algorithm_id());
	unsigned int key_size = (unsigned int)serv->key_size() >> 3;
	unsigned char* key = (unsigned char*) malloc(key_size);

	if( !key )
		throw BCSystemException("raw: no memory to allocate key");

	memset(key, '\0', key_size);

	password = (unsigned char*) getpass("Enter temporary password or leave blank for random generation:");
	if( !password )
		throw BCSystemException("raw: cannot read password\n");

	size_t pwdlen = strlen(password);
	if(pwdlen == 0)
	{		
		if( !GetRandomSeed(pool, sizeof(pool)) ) {
			bc_get_log(BC_ERR).msg("Error: could not get random seed\n");
			return false;
		}

		if( !SHA256_RandomGenerate(key, key_size, pool, sizeof(pool))) {
			bc_get_log(BC_ERR).msg("Error: could not generate random key\n");
			return false;
		}

	} else {

		if( !SHA256_RandomGenerate(key, key_size, password, pwdlen)) {
			bc_get_log(BC_ERR).msg("Error: could not generate random key\n");
			return false;
		}

		memset(password, '\0', passlen);
		free(password);
	}

	memset(pool, '\0', sizeof(pool));		

	// init algorithm
	BCAlgHandle alg = BCAlg::init(algname, AM_CBC, key, pool);
	memset(key, '\0', key_size);
	free(key);

	bc_space_unit_t space;
	space.offset = 0;
	space.length = this->size_bytes();

	// return full file context
	return BCContainerView::init(alg, space, BC_CRYPTO_IV_64BIT);
}




