/*******************************************************************
 *
 *	Copyright (c) 1994-1999 Jetico, Inc., Finland
 *	All rights reserved.
 *
 *	File: pwdlg.cc
 *	Revision:  $Id: pwldlg.cpp,v 1.1 2005-05-12 09:51:39 crypt Exp $
 *	Created:
 *	Description: implementation of procedures that support 
 *		dialog for accepting user's password and 
 *		support generation of seed value for 
 *		random key generation
 *
 *******************************************************************/


#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include "kg_defs.h"
#include "kg_misc.h"
#include "pwldlg.h"

#include <common/types.h>

char pwldlg_cpp[]="$Id: pwldlg.cpp,v 1.1 2005-05-12 09:51:39 crypt Exp $";

/* Global data */

// DWORD   SeedCounter, SeedLength;

/* Exported procedures */

DWORD GetPassword( 
			const char *text,	// Text, containing filename of the container
			const char *caption,	// Caption of the "get password" dialog 
			char   *password 
		)
{
    char *pwd;
    if (text) printf ("%s\n",text);
    if (! (pwd = getpass (caption))) return ERROR_INTERNAL_PROBLEM;
    strncpy(password,pwd,PASSWORD_MAXIMUM_LENGTH);
    ShredData( (BYTE *)pwd, strlen(pwd) );
    //free(pwd);
    return ERROR_NO;
}


DWORD GetPasswordWithVerification(
				const char *text,	// Text, containing filename of the container
				const char *caption,	// Caption of the "get password" dialog window
				char   *password 
				)
{

    char *pwd;

    if (text) printf ("%s\n",text);

    do 
      if (! (pwd  = getpass (caption))) return ERROR_INTERNAL_PROBLEM;
    while (strlen(pwd) < PASSWORD_MINIMUM_LENGTH && printf("Password should be at least %d characters length.\n", PASSWORD_MINIMUM_LENGTH));
    strncpy(password,pwd,PASSWORD_MAXIMUM_LENGTH);
    ShredData( (BYTE *)pwd, strlen(pwd) );
    
    if (! (pwd = getpass ("verify password: "))) return ERROR_INTERNAL_PROBLEM;
    if (strlen(pwd) != strlen(password) || strncmp(pwd,password,strlen(pwd) < PASSWORD_MAXIMUM_LENGTH ? strlen(password) : PASSWORD_MAXIMUM_LENGTH))
      return ERROR_USER_PASSWORDS_MISMATCH;
    ShredData( (BYTE *)pwd, strlen(pwd) );

    return ERROR_NO;

}

#include <common/log.h>

DWORD GetSeedValue( 
			BYTE  *seed, 
			DWORD seedLength )
{

	using namespace bdk;

    FILE *fd;
    
    if (! (fd=fopen("/dev/random","r"))) return ERROR_INTERNAL_PROBLEM;
    
	fprintf(stdout, "generating random seed, randomly move your mouse or press keys to speed the process\n");
	
	DWORD i = 0;
	do {
		fprintf(stdout, "random seed generation: %d\\%d bits (%d%%) 		\r", (i+1)*8, seedLength*8, ((i+1)*100/seedLength));
        	seed[i++]=(char)fgetc(fd);
		
	} while ( i < seedLength );

	fprintf(stdout, "\n");
    fclose (fd);
    return ERROR_NO;
}


