#include "sha_generic.h"

#include <CertificateLib/hashAlgs/hashAlgs.h>
#include <CertificateLib/hashAlgs/SHA1/Sha1.h>
#include <CertificateLib/hashAlgs/SHA256/sha256.h>


static DWORD ConvertKGSHAId( DWORD kgsha_id )
{
	if(kgsha_id == SHA_ID) return haID_SHA1;
	else if(kgsha_id == SHA256_ID) return haID_SHA256;
	else return 0;
}

BOOL CalculateDigest( DWORD hashAlgId, BYTE *Data, DWORD Length, BYTE *Digest )
{
	DWORD id = ConvertKGSHAId(hashAlgId);
	if( 0 == id ) return FALSE;

	return (ha_makeDigestEx(id, Digest, Data, Length) == hae_ok) ? TRUE : FALSE;
}


DWORD DigestSize(DWORD hashAlgId)
{
	DWORD id = ConvertKGSHAId(hashAlgId);
	if( 0 == id ) return FALSE;

	int digest_size;
	return (ha_getDigestSize( &digest_size, id ) == hae_ok) ? digest_size : 0;
}


BOOL RandomGenerate(DWORD hashAlgId, BYTE* byteBuffer, DWORD byteBufferSize, BYTE* seedBuffer, DWORD seedBufferSize )
{
	if( hashAlgId == SHA_ID ) 
	{
		return SHA1RandomGenerate(byteBuffer, byteBufferSize, seedBuffer, seedBufferSize);
	} else if( hashAlgId == SHA256_ID ) {
		return SHA256_RandomGenerate(byteBuffer, byteBufferSize, seedBuffer, seedBufferSize);
	} else {
		return FALSE;
	}
}

