
#ifndef BCMOD_V8_BOOT_RECORD_H
#define BCMOD_V8_BOOT_RECORD_H

#include <common/build.h>

BC_PACKED(
struct BPB
{
	WORD sectSize;
	BYTE sectPerCluster;
	WORD reservedSectors;
	BYTE NumberOfFat;
	WORD maxRootDirEntry;
	WORD totalSectors;
	BYTE mediaDesc;
	WORD sectorsPerFat;
	WORD sectorsPerTrack;
	WORD numberOfHeads;
	DWORD hiddenSectors;
	DWORD totalSectorsLong;
}
);

BC_PACKED(
struct BootRecord 
{
	BYTE jmpCode[3];
	char OEMid[8];

	BPB bpb;
	BYTE driveNo; // 0, 80h
	BYTE reserved;
	BYTE extBootSign;
	DWORD serialNumber;
	char volumeLabel[11];
	char FatType[8];
	   
//------------------- end of boot record (size=62 bytes)
}
);

#define HiddenSectorSize 512
#define KeyDataSize 4
#define SignatureSize 8

BC_PACKED(
struct HiddenSector : public BootRecord
{
  char description[66];
  WORD extent;
  WORD version;
  BYTE keyData[KeyDataSize];

  BYTE reserved[HiddenSectorSize -
               sizeof(BootRecord)-
               66 - // sizeof(description)-
               2 -  // sizeof(extent)-
               2 -  // sizeof(version)-
			   KeyDataSize -  // sizeof(keyData)-
			   4 -	// dwKeySize
			   4 -  // dwDataOffset
			   4 -  // fileSystemId
               4 -  // algorithmId
			   4 -	// keyGenId
               8];

  DWORD dwKeySize;		// Key size. Key is to be placed just after hidden sector in container...
  DWORD dwDataOffset;	// Encrypted Data offset from file begining in bytes...
  DWORD fileSystemId;	// Driver will mark container during formating formating...
  DWORD algorithmId;	// Encryption Algorithm identifier.  
  DWORD keyGenId;		// Key generation identifier...
  char signature[ SignatureSize ];
  
	bool isValid() const
	{
        	return ( strncmp( OEMid, "LOCOS94 ", sizeof(OEMid) ) == 0 );
	}

	int getDataOffsetInSectors() const
	{
        	if ( dwDataOffset == 0 )
                	return 1; // 512; // Old version;
        
        	return dwDataOffset / 512;
	}
}
);


#endif
