#include <windows.h>
#include <windowsx.h>
#include <winuser.h>
#include <stdio.h>
#include <commctrl.h>
#include <time.h>
//#include <tchar.h>
//#include <winnetwk.h>
#include <io.h>

#include "UnicodeFuncs.h"
#include "cDlg.h"
//#include "Translatedll.h"
#include "cWS2Dll.h"

#include "KBlock.h"
#include "..\..\comm_def\kg_defs.h"
#include "client.h"
#include "..\..\comm_def\comm_def.h"
#include "resource.h" 
#include "pwdlg.h"
#include "..\..\comm_def\dllMain.h"

#include "..\..\certificateLib\pk_enc.h"
#include "..\..\..\certificateLib\crtLibErrs.h"
#include "..\..\..\certificateLib\publicKeyAlgs\publicKeyAlgs.h"
#include "..\..\..\certificateLib\SMemory\SBlocks.h"
#include "..\..\..\CertificateLib\SMemory\cPassword.h"
#include "..\..\..\CertificateLib\hashAlgs\SHA1\Sha1.h"

#include "..\..\..\Setups\SetupIncludesT\cUIFile.h"

cWS2Dll ws2dll;

struct sSymKey
{
	sKeyInfo keyInfo;

	enum { SHA1_SIGNATURE_LENGTH = 20 };

	BYTE signature[ SHA1_SIGNATURE_LENGTH ];

	static void createSignature( BYTE *Signature, BYTE *Buffer, int Size )
	{
		memset(Signature, 0, SHA1_SIGNATURE_LENGTH );
		SHA1MakeDigest( Buffer, Size, Signature );
	}

	static BOOL verifySignature( BYTE *Signature, BYTE *Buffer, int Size )
	{
		BYTE calcSig[ SHA1_SIGNATURE_LENGTH ];
		memset(calcSig, 0, SHA1_SIGNATURE_LENGTH );
		SHA1MakeDigest( Buffer, Size, calcSig );

		if ( memcmp(calcSig, Signature, SHA1_SIGNATURE_LENGTH ) != 0 ) return FALSE;
		return TRUE;
	}
};

char BcServerComputerName[64];
BOOL BcServerComputerNameDefined = FALSE;
unsigned short BcServerPort = 0;

char     *ClPassword = NULL;
HWND     HWndParent = NULL;

// Local functions declaration
static BOOL  getBCServerComputerName( char *ComputerName, int NameLength );

static BOOL  openSocket( SOCKET *Socket, char *BCServerComputerName );
static void  closeSocket( SOCKET *Socket );

static DWORD getUserKeys( char *UserName, ASYM_KEY *PrivateKey, ASYM_KEY *PublicKey );
static DWORD putUserKeys( char *UserName, ASYM_KEY *PrivateKey, ASYM_KEY *PublicKey );

static DWORD createNewAsymetricKeyPair( HWND hWndParent, ASYM_KEY *PrivateKey, ASYM_KEY *PublicKey );

static BOOL  putAdministratorPublicKeySignatureToLocalDisk( BYTE *Signature, int sigSize, const char *UserName );
static BOOL  verifyAdministratorPublicKeySignatureFromLocalDisk(ASYM_KEY *AdmPublicKey, 
																ASYM_KEY *UserPublicKey, 
																char *UserName );

static DWORD sendContainerInfoToBCServer(
			SYM_KEY_PKENC *EncryptedUserKey, 
			SYM_KEY_PKENC *EncryptedAdminKey, 
			const char	  *ContainerPath, 
			BYTE          *ContainerId,
			BOOL          Owner,
			char const    *UserAccessed,
			char const    *UserWhoGetsKeyInDatabase
		);

static BOOL getDomainName( char *VersionBuf );
static BOOL parseContainerFile(	const char *ContainerString, char *ContainerName, char *ContainerPath );
static void getTimeString( char *TimeStr );

static BOOL getPasswordAndDecryptPrivateKey( ASYM_KEY *PrivateKey );

static BOOL	getUserRights(  HWND hWndParent, const char *UserName, DWORD *Rights );

static BOOL readPortFromFile( char *ServerName, unsigned short *Port );


// Client_CheckBcServerExistance() checks if BCServer exists and returns 
// Name of Computer where BCServer is running

BOOL Client_CheckBcServerExistance()
{
	ODSFunc;

	SOCKET                    commsock;
	int                       retval;
	COMM_GET_BCSERVER_VERSION commbuf;
	
ODS( L"Point 0" );
	// check if the bcsrv.ini file exists in the directory where BestCrypt installed
	// if it exists, read BCServer computer name from the file
	if (!getBCServerComputerName( BcServerComputerName, sizeof(BcServerComputerName) )) 
		return FALSE;

ODS( L"Point 01" );
	// Read information about TCP/IP port
	if (!readPortFromFile( BcServerComputerName, &BcServerPort ))
		return FALSE;

	// Try to send request to BCServer using Winsock
	// create request package
	memset( &commbuf, 0, sizeof(COMM_GET_BCSERVER_VERSION) );
	memcpy(commbuf.header.eye_catch, EYE_CATCH_STRING_BCSC, 8);
	commbuf.header.command = CMD_GET_BCSERVER_VERSION;
	commbuf.header.result  = RESULT_INVALID;
	
ODS( L"Point 02" );
	CommDef_createSignature( commbuf.header.signature,
	                     (BYTE *)&commbuf.major_version, 
	                      sizeof(COMM_GET_BCSERVER_VERSION) - sizeof(COMM_HEADER));
	
ODS( L"Point 03" );
	// send request package to BCServer
	if (!openSocket( &commsock, BcServerComputerName ))
	{
		closeSocket( &commsock );
		return FALSE;
	}

ODS( L"Point 04" );
	retval = ws2dll.send(commsock, (const char *)&commbuf, sizeof(COMM_GET_BCSERVER_VERSION), 0);

	if (retval == 0) return FALSE;
	
	// Receive answer from Server
	retval = ws2dll.recv( commsock, (char *)&commbuf, sizeof(COMM_GET_BCSERVER_VERSION), 0 );

ODS( L"Point 05" );
	closeSocket( &commsock );

	// verify returned result
	if ((retval == 0) || (commbuf.header.result != RESULT_SUCCESS))
		return FALSE;
	
ODS( L"Point 06" );
	if (!CommDef_verifySignature(commbuf.header.signature, 
		                        (BYTE *)&commbuf.major_version, 
								sizeof(COMM_GET_BCSERVER_VERSION) - sizeof(COMM_HEADER)))
	{
		return FALSE;
	}

	BcServerComputerNameDefined = TRUE;

ODS( L"Point 07" );
	return TRUE;
}



// Registering means that user creates public/private keys, then encrypts
// private key by own password and place the pair to the BCServer's database

#define TEST_BUFFER_SIZE 32

BOOL Client_CheckIfUserRegisteredOnServer( HWND hWndParent, char *NetworkPassword, DWORD *Rights )
{
	char     *userName = NULL;
	wchar_t     userNameW[64];
	BYTE     bTmp[ TEST_BUFFER_SIZE ];
	DWORD    dwRes, dwLength = 64;
	ASYM_KEY privateKey, publicKey, admPrivateKey, admPublicKey;
	HPKA     hSka = 0;
	int      signatureSize = 0;
	BYTE     *signature = NULL;
	BOOL     bResult = FALSE;

	HWndParent = hWndParent;

	// Check if BCServer is available and computer name, where it works, is defined
	if (!BcServerComputerNameDefined)
	{
		if (!Client_CheckBcServerExistance()) return FALSE;
	}

	// Get my User Name
	if (!GetUserNameT( userNameW, &dwLength )) return FALSE;

	// if we can get user's keys, it means the user is registered in BCServer database
	cMBStr mbs( userNameW );

	userName = mbs.m_szBody;

	if ( userName == NULL )
		return false;
	
	dwRes = getUserKeys( userName, &privateKey, &publicKey );

	if (dwRes == RESULT_SUCCESS)
	{
		// check if PrivateKey is initialized, i.e. is not equal to 0
		memset( bTmp, 0, TEST_BUFFER_SIZE );
		if (memcmp( &privateKey, bTmp, TEST_BUFFER_SIZE ) != 0)
		{
			getUserRights( hWndParent, userName, Rights );
			return TRUE;
		}
	}
	else if (dwRes == RESULT_USER_DOES_NOT_EXIST)
	{
		wchar_t msgText[256], captionText[32];

        // TO DO: Public/Private keys can be not only generated, but also 
        // exported from the PGP/SSH/SSH1/SSH2 software formats

		LoadStringT( hMainInst, IDS_USER_NOT_ADDED, msgText, maxLen(msgText) );
		LoadStringT( hMainInst, IDS_WARNING, captionText, maxLen(captionText) );
		MessageBoxT( hWndParent, msgText, captionText, MB_OK | MB_ICONWARNING );
		return FALSE;
	}

	// Request was not successful, let's suggest user to create his private/public key
	// and get registered in BCServer database.

	// Initialize user's private and public keys
	ClPassword = NetworkPassword;

	// before creating new key pair, run gathering of seed
	if (!CommDef_GetRandomSeed( bTmp, 4 )) CommDef_InitRandomSeed();

	if (createNewAsymetricKeyPair( hWndParent, &privateKey, &publicKey ) != ERROR_NO)
	{
		CommDef_StopRandomSeedGathering();
		return FALSE;
	}

	CommDef_StopRandomSeedGathering();

	// Get Administrator's public key and sign it using user's private key.
	dwRes = getUserKeys( ADMINISTRATOR_NAME, &admPrivateKey, &admPublicKey );
	if (dwRes != RESULT_SUCCESS) return FALSE;

	// Calculate size for signature we have to add to calcBufSize
	if (CreateSecretAlgorithm( &hSka, PKALG_ID, publicKey.key, publicKey.size, privateKey.key, privateKey.size ) != sae_ok)
		return FALSE;

	if (SignWithPrivateKey( hSka, NULL, 0, NULL, &signatureSize) != pkae_bufferSizeTooSmall )
		goto exit;
	
	signature = (BYTE *)HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, signatureSize );
	if (!signature) goto exit;

	if (SignWithPrivateKey(	hSka, admPublicKey.signature, sizeof(admPublicKey.signature), 
							signature, &signatureSize) != sae_ok )
		goto exit;

	// Save Administrator's key signature on local machine in BCSERVER_INI_FILE file
	//
	// Later the saved data will be used for verification the data received from BCServer
	// to avoid men-in-the-middle attack.
	// To verify the key it will be enough:
	// a) receive Adm. public key from BCServer; 
	// b) receive user's public key from BCServer;
	// c) verify data in BCSERVER_INI_FILE with user's public key;
	// (see implementation of the Verification procedure in Client_EncryptAndSaveKeyOnBCServer() procedure)

	putAdministratorPublicKeySignatureToLocalDisk( signature, signatureSize, userName );

	// Encrypt private key using password user entered
	CommDef_EncryptDataUsingPassword( privateKey.signature, privateKey.signature, SIGNATURE_LENGTH + privateKey.size, NetworkPassword );

	// Put the keys to BCServer database
	dwRes = putUserKeys( userName, &privateKey, &publicKey );

	if (dwRes != RESULT_SUCCESS)
	{
		return FALSE;
	}
	
	getUserRights( hWndParent, userName, Rights );

	bResult = TRUE;

exit:
	if (hSka)      FreeKeyHandle(hSka);
	if (signature) HeapFree(GetProcessHeap(), 0, signature);
	return bResult;
}



///////////////////////////


BOOL Client_GetUserRights(	const char *UserName, // if user_name is empty ("\0"), default rights are returned
							DWORD      *DefaultRights, 
							DWORD      *UserRights 
						  )
{
	SOCKET           commsock;
	int              retval;
	COMM_USER_RIGHTS bufRights;

	// Check if BCServer is available and computer name, where it works, is defined
	if (!BcServerComputerNameDefined)
	{
		if (!Client_CheckBcServerExistance()) return FALSE;
	}

	// create request package
	memcpy( bufRights.header.eye_catch, EYE_CATCH_STRING_BCSC, 8);
	
	bufRights.header.command = CMD_GET_USER_RIGHTS;
	bufRights.header.result  = RESULT_INVALID;

	strcpy( bufRights.user_name, UserName );

	CommDef_createSignature( bufRights.header.signature,
	                     (BYTE *)bufRights.user_name,
	                      sizeof(COMM_USER_RIGHTS) - sizeof(COMM_HEADER));

	// send request package to BCServer
	if (!openSocket( &commsock, BcServerComputerName ))
	{
		closeSocket( &commsock );
		return FALSE;
	}

	// Send request to Server
	retval = ws2dll.send(commsock, (const char *)&bufRights, sizeof(COMM_USER_RIGHTS), 0);

	if (retval == 0) return FALSE;

	// Receive answer from Server
	retval = ws2dll.recv( commsock, (char *)&bufRights, sizeof(COMM_USER_RIGHTS), 0 );

	closeSocket( &commsock );

	// verify returned result
	if ((retval == 0) || (bufRights.header.result != RESULT_SUCCESS))
		return FALSE;
	
	if (!CommDef_verifySignature(bufRights.header.signature, 
		                        (BYTE *)bufRights.user_name,
								sizeof(COMM_USER_RIGHTS) - sizeof(COMM_HEADER)))
		return FALSE;

	*DefaultRights = bufRights.default_rights;
	*UserRights    = bufRights.user_rights;

	return TRUE;
}


/////////////////////////////////////////////

static BOOL getContPath(
		wchar_t *path,
		const wchar_t *text
	)
{
	// input *Text has a form: "Container:  D:\path\container_name.jbc"
	// we return: path = "D:\path\container_name.jbc", 

	int  i;
	const wchar_t *pathPtr = NULL, *namePtr = NULL;

	if ( text == NULL )
		return FALSE;

	for( i = (int)wcslen( text ); i>0; i--)
	{
		if ((text[i] == ' ') && (text[i-1] == ':'))
		{
			pathPtr = text + i;
			break;
		}
	}

	if ( pathPtr == NULL )
		pathPtr = text;

	if ( pathPtr != NULL )
		wcscpy( path, pathPtr );
	else
		path[0] = L'\0';

	return TRUE;
}


BOOL Client_EncryptAndSaveKeyOnBCServer(
		sKeyInfo *keyInfo, 
		const wchar_t *ContainerPath, 
		BYTE *ContainerId,
		DWORD Rights
	)
{
	wchar_t  userNameW[64];
	DWORD    dwRes, dwLength = 64;
	ASYM_KEY userPublicKey, admPublicKey, tmpAsymKey;
	sSymKey  *pSymKey = NULL;
	SYM_KEY_PKENC encryptedUserKey, encryptedAdminKey;
	int           encBufSize;

	char contPath[ 512 ];
	wchar_t contPathW[ MAX_PATH ];
	getContPath( contPathW, ContainerPath );
	wcstombs( contPath, contPathW, maxLen( contPath ) );

	cPlainTextBlock ptb;
	if ( ptb.Allocate( sizeof( sSymKey ) ) == NULL )
		return FALSE;

	pSymKey = ( sSymKey* )ptb.buffer;

	if (Rights == RIGHT_DO_NOT_BACKUP_KEYS) return TRUE;
	
	// Check if BCServer is available and computer name, where it works, is defined
	if (!BcServerComputerNameDefined)
	{
		if (!Client_CheckBcServerExistance()) return FALSE;
	}

	// Get current User Name
	if (!GetUserNameT( userNameW, &dwLength )) return FALSE;

	cMBStr mbs( userNameW );
	char *userName = mbs.m_szBody;

	if ( userName == NULL )
		return false;

	// Receive public key of the user from BCServer
	dwRes = getUserKeys( userName, &tmpAsymKey, &userPublicKey );
	if (dwRes != RESULT_SUCCESS) return FALSE;

	// Receive public key of Administrator from BCServer
	dwRes = getUserKeys( ADMINISTRATOR_NAME, &tmpAsymKey, &admPublicKey );
	if (dwRes != RESULT_SUCCESS) return FALSE;

	// Verify Administrator's public key with signature stored in the BCSERVER_INI_FILE file
	if (!verifyAdministratorPublicKeySignatureFromLocalDisk( &admPublicKey, &userPublicKey, userName )) 
		return FALSE;

	memcpy( &( pSymKey->keyInfo ) , keyInfo, keyInfo->size );
	sSymKey::createSignature( pSymKey->signature, (BYTE *)&( pSymKey->keyInfo ), SYMM_KEY_LENGTH );

	// encrypt Key using Administrator's key
	HPKA hPka = 0;

	if (CreatePublicAlgorithm( &hPka, PKALG_ID, admPublicKey.key, admPublicKey.size ) != sae_ok )
		return FALSE;

	encBufSize = sizeof(encryptedAdminKey.key);

	if (EncryptSessionKey( hPka, (BYTE *)pSymKey, sizeof( sSymKey ), encryptedAdminKey.key, &encBufSize) != sae_ok )
	{
		FreeKeyHandle(hPka);
		return FALSE;
	}

	encryptedAdminKey.size = encBufSize;

	FreeKeyHandle(hPka);

	// encrypt Key using User's public key
	hPka = 0;

	if (CreatePublicAlgorithm( &hPka, PKALG_ID, userPublicKey.key, userPublicKey.size ) != sae_ok )
		return FALSE;

	encBufSize = sizeof(encryptedAdminKey.key);

	if (EncryptSessionKey( hPka, (BYTE *)pSymKey, sizeof( sSymKey ), encryptedUserKey.key, &encBufSize) != sae_ok )
	{
		FreeKeyHandle(hPka);
		return FALSE;
	}

	encryptedUserKey.size = encBufSize;

	FreeKeyHandle(hPka);

	dwRes = RESULT_SUCCESS;
	
	// Check if we have to save key for our account only.
	// If so, do not send the key for Administrator
	if (Rights == RIGHT_BACKUP_FOR_MY_ACCOUNT)
	{
		memset(&encryptedAdminKey, 0, sizeof(SYM_KEY_PKENC));

		// Send the encrypted *Key with other container's characteristics to BCServer
		dwRes = sendContainerInfoToBCServer( &encryptedUserKey, &encryptedAdminKey, contPath, ContainerId, TRUE, userName, "" );
	}

	else if (Rights == RIGHT_BACKUP_KEY_FOR_ADMIN)
	{
		// Send the encrypted *Key with other container's characteristics to BCServer
		dwRes = sendContainerInfoToBCServer( &encryptedUserKey, &encryptedAdminKey, contPath, ContainerId, TRUE, userName, ADMINISTRATOR_NAME );
	}

	if (dwRes != RESULT_SUCCESS) return FALSE;

	return TRUE;
}



/////////////////////////////////////////////


BOOL Client_GetKeyForContainer( BYTE *ContainerId, sKeyInfo *keyInfo, const char *Password )
{
	COMM_CONTAINER_KEY_FOR_USER_1 bindInfo;
	SOCKET         commsock;
	int            retval, idSize;
	wchar_t        userNameW[64];
	DWORD          dwLength = 64, dwRes;
	ASYM_KEY       userPrivateKey, userPublicKey;
	
	// Check if BCServer is available and computer name, where it works, is defined
	if (!BcServerComputerNameDefined)
	{
		if (!Client_CheckBcServerExistance()) return FALSE;
	}
	
	// Get current User Name
	if (!GetUserNameT( userNameW, &dwLength )) return FALSE;

	cMBStr mbs( userNameW );
	char *userName = mbs.m_szBody;

	if ( userName == NULL )
		return false;

	// create request package
	memcpy( bindInfo.header.eye_catch, EYE_CATCH_STRING_BCSC, 8);
	
	bindInfo.header.command = CMD_GET_CONTAINER_KEY_FOR_USER_1;
	bindInfo.header.result  = RESULT_INVALID;

	strcpy( bindInfo.user_name, userName );
	
	// Fill in containerId field. It's slightly complicated, 
	// because Id in database is 32-bit number, but inside container 
	// we can store more longer sequence of bytes.
	idSize = (CONTAINER_ID_LENGTH >= sizeof(DWORD)) ? sizeof(DWORD) : CONTAINER_ID_LENGTH;
	
	bindInfo.container_id = 0;
	
	for(int i=0; i<idSize; i++) bindInfo.container_id |= ((DWORD)ContainerId[i]) << (i<<3);
	
	CommDef_createSignature( bindInfo.header.signature,
	                     (BYTE *)bindInfo.user_name,
	                      sizeof(COMM_CONTAINER_KEY_FOR_USER_1) - sizeof(COMM_HEADER));

	// send request package to BCServer

	if (!openSocket( &commsock, BcServerComputerName ))
	{
		closeSocket( &commsock );
		return FALSE;
	}

	// Send request to Server
	retval = ws2dll.send(commsock, (const char *)&bindInfo, sizeof(COMM_CONTAINER_KEY_FOR_USER_1), 0);

	if (retval == 0) return FALSE;

	// Receive answer from Server
	retval = ws2dll.recv( commsock, (char *)&bindInfo, sizeof(COMM_CONTAINER_KEY_FOR_USER_1), 0 );

	closeSocket( &commsock );

	// verify returned result
	if ((retval == 0) || (bindInfo.header.result != RESULT_SUCCESS))
		return FALSE;
	
	if (!CommDef_verifySignature(bindInfo.header.signature, 
		                        (BYTE *)bindInfo.user_name,
								sizeof(COMM_CONTAINER_KEY_FOR_USER_1) - sizeof(COMM_HEADER)))
		return FALSE;

	// Receive private key of the user from BCServer
	dwRes = getUserKeys( userName, &userPrivateKey, &userPublicKey );
	if (dwRes != RESULT_SUCCESS) return FALSE;

	if (!CommDef_DecryptDataUsingPassword( userPrivateKey.signature,  userPrivateKey.signature, 
			SIGNATURE_LENGTH + userPrivateKey.size, Password ))
		return FALSE;

	if (!CommDef_verifySignature( userPrivateKey.signature, userPrivateKey.key, userPrivateKey.size ))
		return FALSE;

	// Decrypt the buffer we received with Private Key
	HPKA     hSka = 0;
	sSymKey  *pSymKey = NULL;
	int      symKeyDecSize;

	cPlainTextBlock ptb;
	if ( ptb.Allocate( sizeof( sSymKey ) ) == NULL )
		return FALSE;

	pSymKey = ( sSymKey* )ptb.buffer;

	if (CreateSecretAlgorithm( &hSka, PKALG_ID, userPublicKey.key, userPublicKey.size, userPrivateKey.key, userPrivateKey.size ) != sae_ok)
		return FALSE;

	symKeyDecSize = sizeof( sSymKey );
	
	if (DecryptSessionKey( hSka, bindInfo.key.key, bindInfo.key.size, 
							(BYTE *)pSymKey, &symKeyDecSize) != sae_ok)
	{
		FreeKeyHandle(hSka);
		return FALSE;
	}

	FreeKeyHandle(hSka);

	if (!sSymKey::verifySignature( pSymKey->signature, (BYTE *)&pSymKey->keyInfo, SYMM_KEY_LENGTH ))
		return FALSE;

	if ( !pSymKey->keyInfo.isValid() )
		return false;

	memcpy( keyInfo, &(pSymKey->keyInfo), pSymKey->keyInfo.size );

	return TRUE;
}



/////////////////////////////////////////////


BOOL Client_RemoveContainerFromBCServer( const wchar_t *wContainerPath, BYTE *ContainerId )
{
	SOCKET         commsock;
	int            retval, idSize;
	COMM_CONTAINER commbuf;
	char           fileName[255], filePath[255], 
	               computerName[MAX_COMPUTERNAME_LENGTH + 1], timeStr[64];
	wchar_t        userNameW[64];
	DWORD          dwLength;

	if (!openSocket( &commsock, BcServerComputerName )) return FALSE;

	// Get current User Name
	dwLength = 64;
	if (!GetUserNameT( userNameW, &dwLength )) return FALSE;

	cMBStr mbs( userNameW );
	char *userName = mbs.m_szBody;

	if ( userName == NULL )
		return false;

	char ContainerPath[ 512 ];
	wcstombs( ContainerPath, wContainerPath, maxLen( ContainerPath ) );

	if (!parseContainerFile( ContainerPath, fileName, filePath ))
	{
		return RESULT_INVALID;
	}

	dwLength = MAX_COMPUTERNAME_LENGTH + 1;
	if (!GetComputerName( computerName, &dwLength )) computerName[0] = '\0';
	
	getTimeString( timeStr );

	// Send request to Server

	memcpy(commbuf.header.eye_catch, EYE_CATCH_STRING_BCSC, 8);
	commbuf.header.command = CMD_DELETE_CONTAINER;
	commbuf.header.result  = RESULT_INVALID;

	memcpy( commbuf.container, fileName, PATH_LENGTH );
	memset( &(commbuf.key_user), 0, sizeof(commbuf.key_user) );
	memset( &(commbuf.key_admin), 0, sizeof(commbuf.key_user) );
	memcpy( commbuf.path, filePath, PATH_LENGTH );
	memcpy( commbuf.computer, computerName, NAME_LENGTH );
	memcpy( commbuf.userWhoAccess, userName, strlen(userName) < NAME_LENGTH ? (strlen(userName) + 1) : NAME_LENGTH );
	commbuf.userWhoGetsKeyInDatabase[0] = '\0';
	commbuf.owner = TRUE;
	memcpy( commbuf.time, timeStr, TIME_LENGTH );

	// Fill in containerId field. It's slightly complicated, 
	// because Id in database is 32-bit number, but inside container 
	// we can store more longer sequence of bytes.
	
	idSize = (CONTAINER_ID_LENGTH >= sizeof(DWORD)) ? sizeof(DWORD) : CONTAINER_ID_LENGTH;
	
	commbuf.containerId = 0;
	
	for(int i=0; i<idSize; i++) commbuf.containerId |= ((DWORD)ContainerId[i]) << (i<<3);

	// Get Domain name.
	getDomainName( commbuf.domain );

	commbuf.op_system[0] = '\0';
	commbuf.bc_version[0] = '\0';
	
	CommDef_createSignature( commbuf.header.signature,
	                     (BYTE *)commbuf.container, 
	                      sizeof(COMM_CONTAINER) - sizeof(COMM_HEADER));
	
	retval = ws2dll.send(commsock, (const char *)&commbuf, sizeof(COMM_CONTAINER), 0);

	if (retval == 0)
	{
		closeSocket( &commsock );
		return FALSE;
	}

	// Receive answer from Server
	retval = ws2dll.recv( commsock, (char *)&commbuf, sizeof(COMM_CONTAINER), 0 );

	closeSocket( &commsock );
	
	if (retval == 0)
	{
		closeSocket( &commsock );
		return FALSE;
	}
	
	if (!CommDef_verifySignature(commbuf.header.signature, 
		                        (BYTE *)commbuf.container, 
								sizeof(COMM_CONTAINER) - sizeof(COMM_HEADER)))
		return FALSE;

	return TRUE;
}


//
// Local functions
//

static BOOL getBcsrvIniFilePath( char *Path, char *UninstallPath )
{
	char keyName[ MAX_PATH ];
	char keyNameValue[] = "UninstallString";
	int  i;
	HKEY hKey;

	// Open Registry key name: 
	// HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\BestCrypt

	strcpy( keyName, "Software\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\BestCrypt" );

	if (RegCreateKeyEx(HKEY_LOCAL_MACHINE, keyName, 0, NULL, REG_OPTION_NON_VOLATILE, 
	                   KEY_READ, NULL, &hKey, NULL) != ERROR_SUCCESS)
		return FALSE;

	DWORD dwSize = 0;
	DWORD dwType = REG_SZ;
	LONG  lRes = RegQueryValueEx(hKey, keyNameValue, NULL, &dwType, NULL, &dwSize);

	if ((lRes == ERROR_SUCCESS) && (dwType == REG_SZ))
	{
		// we use keyName here to store string read from Registry
		lRes = RegQueryValueEx(hKey, keyNameValue, NULL, &dwType, (LPBYTE)keyName, &dwSize);
	}
	else
	{
		RegCloseKey(hKey);
		return FALSE;
	}

	RegCloseKey(hKey);

	// We have got string from Registry, now we should extract path to BestCrypt directory
	// example of the string we have read: "C:\WINNT\BCUnInstall.exe" C:\Program Files\Jetico\BestCrypt\UnInstall.log
	char *path;

	for (i=1; i < (int)strlen(keyName); i++)
	{
		path = &(keyName[i]);

		if (keyName[i] == '"')
		{
			path = &(keyName[i+2]);
			break;
		}
	}

	strcpy( Path, path );

	// remove Uninstal.log name from the Path
	for ( i = (int)strlen(Path); i>0; i-- )
	{
		if (Path[i] == '\\') break;
		else Path[i] = '\0';
	}

	// create path to bcsrv.ini file 
	strcat( Path, BCSERVER_INI_FILE );

	// Now we have pointer to path string of Uninstall.log file in *path variable,
	// and path to BCSERVER_INI_FILE in Path parameter, 
	// copy path to Uninstall.log to input parameter

	strcpy( UninstallPath, path );

	return TRUE;
}



////////////////////////////////

static bool readComputerNameFromUninstallFile( char *UninstallPath, char *ComputerName, int maxlen )
{
	UIFile_V2 f( getWCStr( UninstallPath ), false );

	if (f.error) return false;

	while(f.readNext())
	{
		if ( f.command == LF_ReportToBCServerAboutUninstallation )
		{
			getMBStr( f.buffChar, ComputerName, maxlen );
			return true;
		}
	}

	return false;
}

//#define BCSERVER_NAME_ID_OPERATION 0xD
//
//static BOOL readComputerNameFromUninstallFile( char *UninstallPath, char *ComputerName, int maxlen )
//{
//	ODSFuncS( getWCStr( UninstallPath ) );
//
//	int  len, idOperation, objInt;
//	FILE *file;
//	BOOL bRes = FALSE;
//
//	file = fopen(UninstallPath,"rb");
//
//ODS( L"Point 0" );
//	if (!file) return FALSE;
//	
//ODS( L"Point 01" );
//	for(;;)
//	{
//ODS( L"Point 01.1" );
//		if (( fread(&idOperation,sizeof(int),1,file) == 1 )
//			&& ( fread(&objInt,sizeof(int),1,file) == 1 )
//			&& ( fread(&len,sizeof(int),1,file) == 1 )
//			&& ( len < maxlen )
//			&& ( idOperation == BCSERVER_NAME_ID_OPERATION )
//			)
//		{
//			bRes = ( fread( ComputerName, len, 1, file ) == 1 );
//
//ODS( getWCStr( ComputerName ) );
//
//			bRes = TRUE;
//			break;
//		}
//
//		if (feof(file))
//			break;
//	}
//
//	fclose( file );
//
//ODS( L"Point 03" );
//	return bRes;
//}
//


////////////////////////////////


static BOOL getBCServerComputerName( char *ComputerName, int NameLength )
{
	ODSFunc;

	FILE *file;
	char path[ MAX_PATH ], uninstallPath[ MAX_PATH ], *defStr = "\0";
	int  nSize;

ODS( L"Point 0" );
	if (!getBcsrvIniFilePath( path, uninstallPath )) return FALSE;

ODS( L"Point 01" );
	file = fopen( path, "r");

ODS( L"Point 02" );
	if (!file)
	{
		// use uninstallPath to read BCServer computer name 
		// from Uninstall.Log file
		if (!readComputerNameFromUninstallFile( uninstallPath, ComputerName, NameLength ))
			return FALSE;

		nSize = WritePrivateProfileString( BCSMGR_SERVER_SECTION, BCSMGR_SERVER_KEY, ComputerName, path );

		return TRUE;
	}

	fclose( file );

ODS( L"Point 03" );
	nSize = GetPrivateProfileString( BCSMGR_SERVER_SECTION, BCSMGR_SERVER_KEY, defStr, ComputerName, NameLength, path );

	if (strlen(ComputerName) == 0) return FALSE;

ODS( L"Point 04" );
	return TRUE;
}


/*********** Local communication functions ***********/

static BOOL openSocket( SOCKET *Socket, char *BCServerComputerName )
{
	WSADATA            wsaData;
	struct hostent     *hp;
	struct sockaddr_in server;
	int                socket_type = DEFAULT_PROTOCOL;
	unsigned int       addr;

	ws2dll.Load( false );

	if ( ws2dll.WSAStartup(0x202,&wsaData) == SOCKET_ERROR) 
	{	ws2dll.WSACleanup();
		return FALSE;
	}

	if (isalpha(BCServerComputerName[0]))  /* server address is a name */
	{  	hp = ws2dll.gethostbyname( BCServerComputerName );
	}
	else                          /* Convert nnn.nnn address to a usable one */
	{ 	addr = ws2dll.inet_addr(BCServerComputerName);
		hp = ws2dll.gethostbyaddr((char *)&addr,4,AF_INET);
	}

	if (hp == NULL )
	{	ws2dll.WSACleanup();
		return FALSE;
	}

	memset( &server, 0, sizeof(server));
	memcpy( &(server.sin_addr), hp->h_addr, hp->h_length);

	server.sin_family = hp->h_addrtype;
	server.sin_port = ws2dll.htons(BcServerPort);

	*Socket = ws2dll.socket( AF_INET, socket_type, 0);
	if ( (*Socket) == INVALID_SOCKET) 
	{	ws2dll.WSACleanup();
		return FALSE;
	}

	if (ws2dll.connect(*Socket, (struct sockaddr*)&server, sizeof(server)) == SOCKET_ERROR) 
	{
		ws2dll.WSACleanup();
		return FALSE;
	}

	return TRUE;
}

////////////////////////

static void closeSocket( SOCKET *Socket )
{
	if ( (*Socket) != INVALID_SOCKET)
	{
		ws2dll.closesocket(*Socket);
	}

	ws2dll.WSACleanup();
}


/*
 * The getUserKeys is called to get user's public and private keys,
 *  at this time sender cannot sign the request by his private key. 
 *  Hence, the request is sent not signed.
 *
 *  There is no security leak here, because private key is returned encrypted
 *  by user's password, and will be verified by client after decryption. 
 *
 */

static DWORD getUserKeys( char *UserName, ASYM_KEY *PrivateKey, ASYM_KEY *PublicKey )
{
	SOCKET         commsock;
	int            retval;
	COMM_USER_KEYS commbuf;

	if (!openSocket( &commsock, BcServerComputerName )) return RESULT_COMMUNICATION_ERROR;

	// Send request to Server
	memcpy(commbuf.header.eye_catch, EYE_CATCH_STRING_BCSC, 8);
	commbuf.header.command = CMD_GET_USER_KEYS;
	commbuf.header.result  = RESULT_INVALID;

	memcpy( commbuf.user, UserName, strlen(UserName) < NAME_LENGTH ? (strlen(UserName) + 1) : NAME_LENGTH );
	memset( &commbuf.public_key, 0, sizeof(ASYM_KEY) );
	memset( &commbuf.private_key, 0, sizeof(ASYM_KEY) );
	
	CommDef_createSignature( commbuf.header.signature,
	                     (BYTE *)commbuf.user, 
	                      sizeof(COMM_USER_KEYS) - sizeof(COMM_HEADER));

	retval = ws2dll.send(commsock, (const char *)&commbuf, sizeof(COMM_USER_KEYS), 0);

	if (retval == 0)
	{
		closeSocket( &commsock );
		return RESULT_COMMUNICATION_ERROR;
	}

	// Receive answer from Server
	retval = ws2dll.recv( commsock, (char *)&commbuf, sizeof(COMM_USER_KEYS), 0 );

	closeSocket( &commsock );
	
	if (retval == 0)
	{
		closeSocket( &commsock );
		return RESULT_COMMUNICATION_ERROR;
	}
	
	
	if (!CommDef_verifySignature(commbuf.header.signature, 
		                        (BYTE *)commbuf.user, 
								sizeof(COMM_USER_KEYS) - sizeof(COMM_HEADER)))
		return RESULT_INVALID_SIGNATURE;

	PrivateKey->size = commbuf.private_key.size;
	memcpy( PrivateKey->signature, commbuf.private_key.signature, SIGNATURE_LENGTH );
	memcpy( PrivateKey->key, commbuf.private_key.key, commbuf.private_key.size );
		
	PublicKey->size = commbuf.public_key.size;
	memcpy( PublicKey->signature, commbuf.public_key.signature, SIGNATURE_LENGTH );
	memcpy( PublicKey->key, commbuf.public_key.key, commbuf.public_key.size );
		
/*	
	memcpy( PrivateKey, &commbuf.private_key, sizeof(ASYM_KEY) );
	memcpy( PublicKey,  &commbuf.public_key,  sizeof(ASYM_KEY) );
*/
	return commbuf.header.result;
}


/////////////////////////////////////////

static void resetMoreDetailWindow( HWND hWnd, int *oldCY )
{
	HWND lineHwnd;
	int  cx, cy;
	RECT rDlg, rLine;

	lineHwnd = GetDlgItem( hWnd, IDC_STATIC_LINE );

	if (!lineHwnd) return;

	if (!GetWindowRect( hWnd, &rDlg ) || !GetWindowRect( lineHwnd, &rLine )) return;

	*oldCY = rDlg.bottom - rDlg.top;
	cx = rDlg.right - rDlg.left;
	cy = rLine.top - rDlg.top;

	SetWindowPos( hWnd, NULL, 0, 0, cx, cy, SWP_NOMOVE | SWP_NOOWNERZORDER | SWP_NOZORDER );
}


static void setMoreDetailWindow( HWND hWnd, int oldCY )
{
	int  cx;
	RECT rDlg;

	if (!GetWindowRect( hWnd, &rDlg )) return;

	cx = rDlg.right - rDlg.left;

	SetWindowPos( hWnd, NULL, 0, 0, cx, oldCY, SWP_NOMOVE | SWP_NOOWNERZORDER | SWP_NOZORDER );
}


LRESULT CALLBACK createAsymKeysDlgProc( HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
    WORD       pwdLength;
	char       tmpStr[ PASSWORD_MAXIMUM_LENGTH + 4 ];
	wchar_t    msgText[128], captionText[32];
	WORD       tmpStrLength;
	HWND       editHwndA, editHwndB;
	static int oldCY;

	switch (uMsg) 
	{ 
        case WM_INITDIALOG: 

			//if ( pTranslateDll != NULL )
			//	pTranslateDll->TranslateDialog( hwnd );

			oldCY = 0;

			editHwndA = GetDlgItem(hwnd, IDC_PASSA); 
			editHwndB = GetDlgItem(hwnd, IDC_PASSB);

			resetMoreDetailWindow( hwnd, &oldCY );

			return TRUE; 

        case WM_COMMAND: 
 
			switch(wParam) 
			{ 
				case IDC_INFO_BUTTON:
					if (oldCY) setMoreDetailWindow( hwnd, oldCY );
					EnableWindow( GetDlgItem(hwnd, IDC_INFO_BUTTON), FALSE); 
					break;

				case IDOK: 
					/* Get number of characters. */ 
					pwdLength = (WORD) SendDlgItemMessage( hwnd, IDC_PASSA,
									EM_LINELENGTH, (WPARAM) 0, (LPARAM) 0); 

                    if (pwdLength < PASSWORD_MINIMUM_LENGTH) 
					{
						LoadStringT( hMainInst, IDS_PWD_TOO_SHORT, msgText, maxLen(msgText) );
						LoadStringT( hMainInst, IDS_INVALID_PWD, captionText, maxLen(captionText) );
						MessageBoxT(NULL, msgText, captionText, MB_OK | MB_ICONWARNING );
						SetFocus( GetDlgItem( hwnd, IDC_PASSA ));
						return 0; /* Let user to continue to edit password */ 
                    } 
                    
					pwdLength = pwdLength < PASSWORD_MAXIMUM_LENGTH ? pwdLength : PASSWORD_MAXIMUM_LENGTH;

					/* Put the number of characters into first word of buffer */
					*((LPWORD)ClPassword) = pwdLength; 
 
					/* Get the characters. */ 
					SendDlgItemMessage(hwnd, IDC_PASSA, EM_GETLINE, (WPARAM) 0, (LPARAM)ClPassword); 
 
					ClPassword[pwdLength] = 0; /* Null-terminate the string. */ 

					/*Get the password verification string */
					tmpStrLength = (WORD) SendDlgItemMessage( hwnd, IDC_PASSB, 
										EM_LINELENGTH, (WPARAM) 0, (LPARAM) 0);

					if (tmpStrLength == 0)
					{	SetFocus( GetDlgItem( hwnd, IDC_PASSB ));
						return 0; /* let user to type the second copy of password */
					}
					else if (tmpStrLength != pwdLength)
					{
						LoadStringT( hMainInst, IDS_PASSWORD_NOT_SAME, msgText, maxLen(msgText) );
						LoadStringT( hMainInst, IDS_PASSWORD_MISMATCH, captionText, maxLen(captionText) );
						MessageBoxT(NULL, msgText, captionText, MB_OK | MB_ICONWARNING );
						return 0; /* let user to type the passwords again */
					}

					// Get verificatiopn password strings
					*((LPWORD)tmpStr) = pwdLength; 
					SendDlgItemMessage(hwnd, IDC_PASSB, EM_GETLINE, (WPARAM) 0, (LPARAM)tmpStr); 
					tmpStr[pwdLength] = 0; /* Null-terminate the string. */ 

					if (strcmp( tmpStr, ClPassword ) != 0)
					{
						LoadStringT( hMainInst, IDS_PASSWORD_NOT_SAME, msgText, maxLen(msgText) );
						LoadStringT( hMainInst, IDS_PASSWORD_MISMATCH, captionText, maxLen(captionText) );
						MessageBoxT(NULL, msgText, captionText, MB_OK | MB_ICONWARNING );
						return 0; /* let user to type the passwords again */
					}

					Static_SetText(GetDlgItem(hwnd, IDC_PASSA), "qwerty");
					Static_SetText(GetDlgItem(hwnd, IDC_PASSB), "qwerty");
					EndDialog(hwnd, ERROR_NO); 
					return TRUE; 
 
				case IDCANCEL: 
					Static_SetText(GetDlgItem(hwnd, IDC_PASSA), "qwerty");
					Static_SetText(GetDlgItem(hwnd, IDC_PASSB), "qwerty");
					EndDialog(hwnd, (int)ERROR_CANCELLED_BY_USER); 
					return TRUE; 
            } 
            return 0; 
    } 
    return FALSE;

	UNREFERENCED_PARAMETER(lParam); 
}



/////////////////////////////////////////

#define PROGRESS_LENGTH 300
BOOL KeyGenerateSuccess;

DWORD WINAPI keyGenerate( LPVOID lpParam )
{
	HWND hWnd, progressHwnd;
	int i;
	DWORD res = FALSE, sleepTime;

	if (!lpParam) return FALSE;

	hWnd = *((HWND *)lpParam);
	
	if (!hWnd)
	{
		free( lpParam );
		return FALSE;
	}

	progressHwnd = GetDlgItem(hWnd, IDC_PROGRESS1); 

	sleepTime = 100; // 0.1 sec
	
	do
	{
		for(i=0; i<PROGRESS_LENGTH; i++)
		{
			Sleep( sleepTime );
			SendMessage( progressHwnd, PBM_SETPOS, (WPARAM)i, 0);
			if (KeyGenerateSuccess) sleepTime = 10;
		}
	} while (!KeyGenerateSuccess);

	KeyGenerateSuccess = TRUE;
	res = TRUE;
	EndDialog(hWnd, 0);
	free( lpParam );
	return res;
}


//////////////////////////////////////


LRESULT CALLBACK waitKeysGenerationDlgProc( HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	HWND   progressHwnd, *mHwnd;
	HANDLE threadHandle;
	DWORD  threadId;
	BOOL   bTRes;

	switch (uMsg) 
	{ 
        case WM_INITDIALOG:

			//if ( pTranslateDll != NULL )
			//	pTranslateDll->TranslateDialog( hwnd );

			// Retrieve the handle of the progress indicator control
			progressHwnd = GetDlgItem(hwnd, IDC_PROGRESS1); 
			SendMessage( progressHwnd, PBM_SETRANGE, 0, MAKELPARAM(0, PROGRESS_LENGTH));

			//Create thread to generate keys
			mHwnd = (HWND *)malloc( sizeof(HWND) );
			*mHwnd = hwnd;

			threadHandle = CreateThread( NULL, 0, keyGenerate, mHwnd, 0, &threadId );
			bTRes = SetThreadPriority( threadHandle, THREAD_PRIORITY_HIGHEST );

			return TRUE;

		default:
			return 0;
	}
	return 0;
}


typedef struct GENKEY_STRUCT_
{
	cPlainTextBlock *key;
	HPKA            *hpka;
	int             res;
} GENKEY_STRUCT;


DWORD WINAPI generateProcess( LPVOID lpParam )
{
	GENKEY_STRUCT *gs;

	gs = (GENKEY_STRUCT *)lpParam;

	gs->res = GenerateKeyAndCreateAlgorithm( gs->hpka, PKALG_ID, PKEY_LENGTH_BITS, 
		gs->key->buffer, gs->key->bufferSize );
	
	KeyGenerateSuccess = TRUE;

	return 0;
}

static DWORD createNewAsymetricKeyPair(
				HWND      hWndParent,
				ASYM_KEY  *privateKey,
				ASYM_KEY  *publicKey )
{
	int             iResult;
	HANDLE          threadHandle;
	DWORD           threadId, dwResult;
	HPKA            ska = 0;
//	int             sres;
	cPlainTextBlock key, seed;
	GENKEY_STRUCT   gs;

	KeyGenerateSuccess = FALSE;

	//iResult = (int)DialogBox( hMainInst, MAKEINTRESOURCE(IDD_CreateAsymKeysDlg), 
	//					hWndParent, (DLGPROC)createAsymKeysDlgProc );

	cDlg dlg;
	iResult = (int)dlg.DoModal( hMainInst, IDD_CreateAsymKeysDlg, hWndParent, (DLGPROC)createAsymKeysDlgProc );

	if (iResult == -1) /* system error */
	{
		assert( false );
		return ERROR_INTERNAL_PROBLEM;
	}

	if (iResult != ERROR_NO) return (DWORD)iResult;

	// A code to generate User's public and private keys 

	if (( key.Allocate( PKEY_LENGTH_BITS / 8 ) == NULL ) || 
	    ( seed.Allocate( PKEY_LENGTH_BITS / 8 ) == NULL ))
	{
		assert( false );
		return ERROR_INTERNAL_PROBLEM;
	}

	if (!CommDef_GetRandomSeed( seed.buffer, seed.bufferSize ))
	{
		assert( false );
		return ERROR_INTERNAL_PROBLEM;
	}

	CommDef_GenerateRandomKey( key.buffer,  key.bufferSize, seed.buffer, seed.bufferSize );

	gs.key  = &key;
	gs.hpka = &ska;
	
	threadHandle = CreateThread( NULL, 0, generateProcess, &gs, 0, &threadId );
	
	//iResult = (int)DialogBox( hMainInst, MAKEINTRESOURCE(IDD_WaitKeysGenerationDlg), 
	//					hWndParent, (DLGPROC)waitKeysGenerationDlgProc );

	iResult = (int)dlg.DoModal( hMainInst, IDD_WaitKeysGenerationDlg, hWndParent, (DLGPROC)waitKeysGenerationDlgProc );

	dwResult = ERROR_IN_GENERATING_ASYM_KEYS;

	if( gs.res != sae_ok )
		goto exit;

	privateKey->size = ASYMM_KEY_LENGTH;
	publicKey->size = ASYMM_KEY_LENGTH;

	if (sae_ok != SaveKey( ska, publicKey->key, &publicKey->size, 
						privateKey->key, &privateKey->size ))
		goto exit;

	// Create Signature for the keys
	CommDef_createSignature( privateKey->signature, privateKey->key, privateKey->size );
	CommDef_createSignature( publicKey->signature,  publicKey->key,  publicKey->size );

	dwResult = ERROR_NO;
	
exit:
	if (ska) FreeKeyHandle( ska );

	if (dwResult != ERROR_NO)
	{
		memset( publicKey,  0, sizeof(ASYM_KEY) );
		memset( privateKey, 0, sizeof(ASYM_KEY) );
	}

	return dwResult;
}


/////////////////////////

static DWORD putUserKeys( char *UserName, ASYM_KEY *PrivateKey, ASYM_KEY *PublicKey )
{
	SOCKET         commsock;
	int            retval;
	COMM_USER_KEYS commbuf;

	if (!openSocket( &commsock, BcServerComputerName )) return RESULT_COMMUNICATION_ERROR;

	// Send request to Server
	memcpy(commbuf.header.eye_catch, EYE_CATCH_STRING_BCSC, 8);
	commbuf.header.command = CMD_PUT_USER_KEYS;
	commbuf.header.result  = RESULT_INVALID;

	memcpy( commbuf.user, UserName, strlen(UserName) < NAME_LENGTH ? (strlen(UserName) + 1) : NAME_LENGTH );
	memcpy( &commbuf.public_key,  PublicKey,  sizeof(ASYM_KEY) );
	memcpy( &commbuf.private_key, PrivateKey, sizeof(ASYM_KEY) );
	
	CommDef_createSignature( commbuf.header.signature,
	                     (BYTE *)commbuf.user, 
	                      sizeof(COMM_USER_KEYS) - sizeof(COMM_HEADER));
	
	retval = ws2dll.send(commsock, (const char *)&commbuf, sizeof(COMM_USER_KEYS), 0);

	if (retval == 0)
	{
		closeSocket( &commsock );
		return RESULT_COMMUNICATION_ERROR;
	}

	// Receive answer from Server
	retval = ws2dll.recv( commsock, (char *)&commbuf, sizeof(COMM_USER_KEYS), 0 );

	closeSocket( &commsock );
	
	if (retval == 0)
	{
		closeSocket( &commsock );
		return RESULT_COMMUNICATION_ERROR;
	}
	
	if (!CommDef_verifySignature(commbuf.header.signature, 
		                        (BYTE *)commbuf.user, 
								sizeof(COMM_USER_KEYS) - sizeof(COMM_HEADER)))
		return RESULT_INVALID_SIGNATURE;
	
	return commbuf.header.result;
}


////////////////

char KeyString[ PKENC_BUF_LENGTH * 3 + 24];

static BOOL  putAdministratorPublicKeySignatureToLocalDisk( BYTE *Signature, int sigSize, const char *UserName )
{
	FILE *file;
	char chStr[MAX_PATH], uninstallPath[ MAX_PATH ];
	char *chPtr = NULL;
	int  i, nSize;

	if ( sigSize > PKENC_BUF_LENGTH) return FALSE;

	if (!getBcsrvIniFilePath( chStr, uninstallPath )) return FALSE;

	file = fopen( chStr, "r+");

	if (!file) return FALSE;

	fclose( file );

	// Create size string
	sprintf( KeyString, "%d", sigSize );
	nSize = WritePrivateProfileString( UserName, BCSERVER_INI_FILE_KEY_SIZE, KeyString, chStr );

	// Create signature string
	memset( KeyString, 0, sizeof(KeyString) );

	for (i=0; i< sigSize; i++) 
		sprintf( &(KeyString[strlen(KeyString)]), "%02x ", Signature[i]);

	nSize = WritePrivateProfileString( UserName, BCSERVER_INI_FILE_KEY_SIGNATURE, KeyString, chStr );

	memset( KeyString, 0, sizeof(KeyString) );

	return TRUE;
}



/////////////////

static BOOL getPasswordAndDecryptPrivateKey( ASYM_KEY *PrivateKey )
{
	DWORD    errorCode;
	int      i;
	BOOL     bPwdCorrect = FALSE;
	ASYM_KEY tmpKey;

	cPassword pwd;

	for ( i=0; i<3; i++ )  // 3 attempts to enter a valid password
	{
		errorCode = GetPassword( HWndParent,
						LoadStringOnceT( hMainInst, IDS_PRIVATE_KEY_PWD_TEXT ),
						LoadStringTwiceT( hMainInst, IDS_ENTER_PASSWORD ),
						&pwd );

		if (errorCode == ERROR_NO)
		{	
			if (!CommDef_DecryptDataUsingPassword( PrivateKey->signature, tmpKey.signature, 
						SIGNATURE_LENGTH + PrivateKey->size, pwd.buffer ))
				break;

			tmpKey.size = PrivateKey->size;

			if (!CommDef_verifySignature( tmpKey.signature, tmpKey.key, tmpKey.size  ))
			{
				MessageBoxT( HWndParent,
					LoadStringOnceT( hMainInst, IDS_INCORRECT_PASSWORD ),
					LoadStringTwiceT( hMainInst, IDS_INVALID_PWD ),
					MB_OK | MB_ICONWARNING );

				continue;
			}
			
			memset( PrivateKey, 0, sizeof(ASYM_KEY) );
			memcpy( PrivateKey, &tmpKey,  ASYMM_KEY_LENGTH );
			bPwdCorrect = TRUE;
			break;
		}
		else if (errorCode == ERROR_CANCELLED_BY_USER) 
		{
			break;
		}
	}

	if (!bPwdCorrect)
	{
		MessageBoxT( HWndParent,
			LoadStringOnceT( hMainInst, IDS_PWD_CANCELLED ),
			LoadStringTwiceT( hMainInst, IDS_WARNING ),
			MB_OK | MB_ICONWARNING );

		return FALSE;
	}

	return TRUE;
}




/////////////////


static BOOL verifyAdministratorPublicKeySignatureFromLocalDisk(ASYM_KEY *AdmPublicKey, 
															   ASYM_KEY *UserPublicKey, 
															   char *UserName )
{
	FILE  *file;
	char  chStr[ MAX_PATH ], uninstallPath[ MAX_PATH ], *defString = "\0";
	char  *chPtr = NULL, *ch;
	int   i, nSize, iTmp, signSize;
	BYTE  signature[ PKENC_BUF_LENGTH ];
	DWORD dwTmp;
	HPKA  hPka = 0;

	if (!getBcsrvIniFilePath( chStr, uninstallPath )) return FALSE;

	file = fopen( chStr, "r+");
	if (!file) goto write_key;

	fclose( file );

	// Read signature size
	nSize = GetPrivateProfileString( UserName, BCSERVER_INI_FILE_KEY_SIZE, defString, KeyString, sizeof(KeyString), chStr);
	if (strlen(KeyString) == 0) goto write_key;

	sscanf( KeyString, "%d", &signSize );

	memset( KeyString, 0, sizeof(KeyString) );

	if (signSize > PKENC_BUF_LENGTH) goto write_key;

	// Read key signature
	nSize = GetPrivateProfileString( UserName, BCSERVER_INI_FILE_KEY_SIGNATURE, defString, KeyString, sizeof(KeyString), chStr);
	if (strlen(KeyString) == 0) goto write_key;

	for (i=0, ch = KeyString; i<signSize; i++)
	{
		if ( ch[0] == '\0' ) goto write_key;
		sscanf( ch, "%x", &iTmp );
		signature[i] = (BYTE)iTmp;
		for(; (ch[0] != ' ') && (ch[0] != '\0'); ch++) ;
		ch++;
	}

	memset( KeyString, 0, sizeof(KeyString) );

	// Verify signature of the buffer we got with Administrators' public key

	if (CreatePublicAlgorithm( &hPka, PKALG_ID, UserPublicKey->key, UserPublicKey->size ) != sae_ok )
		return FALSE;

	if (VerifyWithPublicKey( hPka, signature, signSize,
							AdmPublicKey->signature, sizeof(AdmPublicKey->signature))
							!= sae_ok)
	{
		int iRes = MessageBoxT( HWndParent,
					LoadStringOnceT( hMainInst, IDS_ADM_KEY_WARNING ),
					LoadStringTwiceT( hMainInst, IDS_WARNING ),
					MB_OKCANCEL | MB_ICONWARNING );

		FreeKeyHandle( hPka );

		if (iRes == IDOK) goto write_key;

		return FALSE;
	}

	FreeKeyHandle( hPka );

	return TRUE;

write_key:

	// User is initialized in BCServer database, but Ini-file does not contain 
	// public key of Administrator for verification.
	// Let's create such a section

	ASYM_KEY privateKey, publicKey;

	dwTmp = getUserKeys( UserName, &privateKey, &publicKey );
	if (dwTmp != RESULT_SUCCESS) return FALSE;

	// We received User's private key encrypted by password.
	// Let's ask for the password, decrypt user's private key and encrypt 
	// Administrator's public key by user's private key 
	// (see also the Client_CheckIfUserRegisteredOnServer() procedure )

	if (!getPasswordAndDecryptPrivateKey( &privateKey )) return FALSE;

	// Calculate size for signature we have to add to calcBufSize
	hPka = 0;

	if (CreateSecretAlgorithm( &hPka, PKALG_ID, publicKey.key, publicKey.size, privateKey.key, privateKey.size ) != sae_ok)
		return FALSE;

	// Calculate size for signature we have to add to calcBufSize
	if (SignWithPrivateKey( hPka, NULL, 0, NULL, &signSize) != pkae_bufferSizeTooSmall )
	{
		FreeKeyHandle(hPka);
		return FALSE;
	}

	if (signSize > PKENC_BUF_LENGTH)
	{
		FreeKeyHandle(hPka);
		return FALSE;
	}

	if (SignWithPrivateKey(	hPka, AdmPublicKey->signature, sizeof(AdmPublicKey->signature), 
							signature, &signSize) != sae_ok )
	{
		FreeKeyHandle(hPka);
		return FALSE;
	}

	FreeKeyHandle(hPka);

	return putAdministratorPublicKeySignatureToLocalDisk( signature, signSize, UserName );
}


////////////////////

static void getTimeString( char *TimeStr )
{
	char timebuf[32], datebuf[32];
	_strtime( timebuf );
	_strdate( datebuf );
	sprintf( TimeStr, "%s %s",timebuf, datebuf);
}


////////////////////

static BOOL parseContainerFile(
		const char *ContainerString,
		char *ContainerName,
		char *ContainerPath
	)
{
	// input *ContainerString has a form: "Container:  D:\path\container_name.jbc"
	// we return: ContainerName = "container_name.jbc", 
	//            ContainerPath = "D:\path\"

	int  i,j;
	const char *pathPtr = NULL, *namePtr = NULL;

	if ( ContainerString == NULL )
		return FALSE;

	for( i = (int)strlen(ContainerString); i>0; i--)
	{
		if ((ContainerString[i] == ' ') && (ContainerString[i-1] == ':'))
		{
			pathPtr = &(ContainerString[i]);
			j = i;
			while (ContainerString[j++] == ' ') pathPtr = &(ContainerString[j]);
			break;
		}
	}

	if (!pathPtr) pathPtr = ContainerString;

	for( i = (int)strlen(ContainerString); i>0; i--)
	{
		if (ContainerString[i] == '\\')
		{
			namePtr = &(ContainerString[i+1]);
			break;
		}
	}

	if (!namePtr) namePtr = ContainerString;

	strcpy( ContainerName, namePtr );

	strcpy( ContainerPath, pathPtr );

	for( i = (int)strlen(ContainerPath); i>0; i--)
	{
		if (ContainerPath[i] == '\\')
		{
			ContainerPath[i+1] = '\0';
			break;
		}
	}

	return TRUE;
}


////////////////////

static BOOL getBestCryptVersion( char *VersionBuf )
{
	char  path[ MAX_PATH ], uninstallPath[ MAX_PATH ];
	int   i;
	DWORD zero, size; 
	unsigned int dwBytes;
	BYTE *versionInfo;
	void *fileVersion;
	
	VersionBuf[0] = '\0';

	if (!getBcsrvIniFilePath( path, uninstallPath )) return FALSE;

	// remove BCSERVER_INI_FILE name from the path
	for ( i = (int)strlen(path); i>0; i-- )
	{
		if (path[i] == '\\') break;
		else path[i] = '\0';
	}

	// create path to bcsrv.ini file 
	strcat( path, BESTCRYPT_EXE_NAME );

	size = GetFileVersionInfoSize( path,&zero );
	versionInfo = new BYTE[size];

	GetFileVersionInfo( path, zero, size, versionInfo );

	dwBytes = 0;
	fileVersion = 0;

	VerQueryValue(versionInfo, TEXT("\\StringFileInfo\\040904b0\\FileVersion"), 
              &fileVersion, &dwBytes);

	if ( dwBytes  == 0 ) // 041904b0
		VerQueryValue(versionInfo, TEXT("\\StringFileInfo\\041904b0\\FileVersion"), 
				  &fileVersion, &dwBytes);
	
	size = ((VERSION_LENGTH - 1) < dwBytes) ? (VERSION_LENGTH - 1) : dwBytes;
	memcpy( VersionBuf, fileVersion, size);
	VersionBuf[size] = '\0';

	delete versionInfo;

	return TRUE;
}


////////////////////

static BOOL getOperatingSystemVersion( char *VersionBuf )
{
	OSVERSIONINFO vi;

	VersionBuf[0] = '\0';
	
	vi.dwOSVersionInfoSize = sizeof( vi );
	
	GetVersionEx(&vi);

	if (vi.dwPlatformId == VER_PLATFORM_WIN32_NT)
	{
		strcpy( VersionBuf, "Windows NT/2000" );
	}
	else if (vi.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS)
	{
		strcpy( VersionBuf, "Windows 95/98" );
	}
	else
	{
		strcpy( VersionBuf, "Windows" );
	}

	return TRUE;
}


/////////////////////

// In the current release we don't store information about domain name.
// So to let the possibility for future, we will change the domain information 
// in the input buffer to default domain name.
//
// See also Database_UpdateContainerInfo() procedure in 'bcserver' project, database.cpp file

#define DEFAULT_DOMAIN_NAME "Default Domain"

static BOOL getDomainName( char *VersionBuf )
{

	VersionBuf[0] = '\0';
	strcpy( VersionBuf, DEFAULT_DOMAIN_NAME );

/* First variant to get domain name in Windows NT
{
		HANDLE hProcess = NULL;
        CString s = "";
        HANDLE hToken   = NULL;  
        PSECURITY_DESCRIPTOR    pSecDesc = NULL; // address of security descriptor
        CHAR                    nsz[1024];
        CHAR                    dsz[1024];
        DWORD                   lnsz = sizeof(nsz);
        DWORD                   ldsz = sizeof(dsz);
        SID_NAME_USE    eUse;
        DWORD                   ReturnLength;
        BYTE                    work[1024];
        PTOKEN_USER             ptu = (PTOKEN_USER)work;
 
        // Get a token for this process.
 
//      hProcess = OpenProcess(  PROCESS_QUERY_INFORMATION | READ_CONTROL , FALSE, PID );
        hProcess = OpenProcess(  PROCESS_QUERY_INFORMATION, FALSE, PID );
        if (hProcess == 0)
        {
                goto errexit;
        }

        if (!OpenProcessToken(hProcess, TOKEN_QUERY, &hToken)) 
        {       
                goto errexit;
        }

        if (!GetTokenInformation(hToken, TokenUser, ptu, sizeof(work), &ReturnLength))
        {       
                goto errexit;
        }

        if (!LookupAccountSid(NULL, ptu->User.Sid, nsz, &lnsz, dsz, &ldsz, &eUse))
        {       
                goto errexit;
        }

        s = CString(dsz) + " \\ " + CString(nsz);

errexit:
        if (hToken) 
        {
                CloseHandle(hToken);
        }
    if (hProcess) 
        {
                CloseHandle( hProcess );
        }
        return s;
}
*/

/* Second variant to get domain name in Windows NT

	if (!szDomainName)
	{
		//Call DsGetDcName to get the name of this computer's domain.
		PDOMAIN_CONTROLLER_INFO DomainControllerInfo = NULL;
		DWORD dReturn = 0L;

		dReturn = DsGetDcName( NULL, NULL, NULL, NULL,
			DS_DIRECTORY_SERVICE_REQUIRED, &DomainControllerInfo );
    
		if (dReturn==NO_ERROR)
		{
			wcscpy(szInitDomain, DomainControllerInfo->DomainName);
			hr = S_OK;    
		}     
		//Free the buffer.    if (DomainControllerInfo)
		NetApiBufferFree(DomainControllerInfo);
	}
	else
	{
		wcscpy(szInitDomain, szDomainName);    
		hr = S_OK;
	}
*/

	return TRUE;
}


//////////////////////

static void	correctPathIfNetwork( char *Path )
{
	char  drvName[4];
	UINT  type;
	CHAR  szDeviceName[80];
	char  infoStr[128];
	DWORD dwResult, cchBuff = maxLen(szDeviceName); 

	if ( ((Path[0] >= 'a') && (Path[0] <= 'z') || (Path[0] >= 'A') && (Path[0] <= 'Z')) &&
		 (Path[1] == ':')
	   )
	{
		memset( drvName, 0, 4 );
		memcpy( drvName, Path, 2 );
		type = GetDriveType( drvName );
		if (type != DRIVE_REMOTE) return;

		dwResult = WNetGetConnection( drvName, (LPSTR)szDeviceName, &cchBuff); 

		if (dwResult != NO_ERROR) return;

		sprintf( infoStr, " (%s mapped on '%s') ", drvName, szDeviceName );

		if (strlen(Path) + strlen(infoStr) > 255)
		{
			infoStr[255 - strlen(Path)] = '\0';
		}

		strcat( Path, infoStr );
	}
}

static DWORD sendContainerInfoToBCServer(
			SYM_KEY_PKENC *EncryptedUserKey, 
			SYM_KEY_PKENC *EncryptedAdminKey, 
			const char	  *ContainerPath, 
			BYTE          *ContainerId,
			BOOL          Owner,
			char const    *UserAccessed,
			char const    *UserWhoGetsKeyInDatabase
		)
{
	SOCKET         commsock;
	int            retval, idSize;
	COMM_CONTAINER commbuf;
	char           fileName[256], filePath[256], 
	               computerName[MAX_COMPUTERNAME_LENGTH + 1], timeStr[64];
	DWORD          dwSize;

	if (!openSocket( &commsock, BcServerComputerName )) return RESULT_COMMUNICATION_ERROR;

	if (!parseContainerFile( ContainerPath, fileName, filePath ))
	{
		strcpy(fileName, ContainerPath );
		filePath[0] = '\0';
	}

	correctPathIfNetwork( filePath );

	dwSize = MAX_COMPUTERNAME_LENGTH + 1;
	if (!GetComputerName( computerName, &dwSize )) computerName[0] = '\0';
	
	getTimeString( timeStr );

	// Send request to Server

	memcpy(commbuf.header.eye_catch, EYE_CATCH_STRING_BCSC, 8);
	commbuf.header.command = CMD_UPDATE_CONTAINER;
	commbuf.header.result  = RESULT_INVALID;

	memcpy( commbuf.container, fileName, PATH_LENGTH );
	memcpy( &(commbuf.key_user), EncryptedUserKey, sizeof(SYM_KEY_PKENC) );
	memcpy( &(commbuf.key_admin), EncryptedAdminKey, sizeof(SYM_KEY_PKENC) );
	memcpy( commbuf.path, filePath, PATH_LENGTH );
	memcpy( commbuf.computer, computerName, NAME_LENGTH );
	memcpy( commbuf.userWhoAccess, UserAccessed, strlen(UserAccessed) < NAME_LENGTH ? (strlen(UserAccessed) + 1) : NAME_LENGTH );
	memcpy( commbuf.userWhoGetsKeyInDatabase, UserWhoGetsKeyInDatabase, strlen(UserWhoGetsKeyInDatabase) < NAME_LENGTH ? (strlen(UserWhoGetsKeyInDatabase) + 1) : NAME_LENGTH );
	commbuf.owner = Owner;
	memcpy( commbuf.time, timeStr, TIME_LENGTH );

	// Fill in containerId field. It's slightly complicated, 
	// because Id in database is 32-bit number, but inside container 
	// we can store more longer sequence of bytes.
	
	idSize = (CONTAINER_ID_LENGTH >= sizeof(DWORD)) ? sizeof(DWORD) : CONTAINER_ID_LENGTH;
	
	commbuf.containerId = 0;
	
	for(int i=0; i<idSize; i++) commbuf.containerId |= ((DWORD)ContainerId[i]) << (i<<3);

	// Get Domain name.
	getDomainName( commbuf.domain );

	// Get operating system name
	getOperatingSystemVersion( commbuf.op_system );

	// Get BestCrypt version
	getBestCryptVersion( commbuf.bc_version );
	
	CommDef_createSignature( commbuf.header.signature,
	                     (BYTE *)commbuf.container, 
	                      sizeof(COMM_CONTAINER) - sizeof(COMM_HEADER));
	
	retval = ws2dll.send(commsock, (const char *)&commbuf, sizeof(COMM_CONTAINER), 0);

	if (retval == 0)
	{
		closeSocket( &commsock );
		return RESULT_COMMUNICATION_ERROR;
	}

	// Receive answer from Server
	retval = ws2dll.recv( commsock, (char *)&commbuf, sizeof(COMM_CONTAINER), 0 );

	closeSocket( &commsock );
	
	if (retval == 0)
	{
		closeSocket( &commsock );
		return RESULT_COMMUNICATION_ERROR;
	}
	
	if (!CommDef_verifySignature(commbuf.header.signature, 
		                        (BYTE *)commbuf.container, 
								sizeof(COMM_CONTAINER) - sizeof(COMM_HEADER)))
		return RESULT_INVALID_SIGNATURE;

	return commbuf.header.result;
}

//////////////////////


BOOL readStringFromIniFile( char *String, const char *Section, const char *Key )
{
	char fileName[ MAX_PATH ], uninstallPath[ MAX_PATH ], returnedStr[128];
	int   nSize;

	if (!getBcsrvIniFilePath( fileName, uninstallPath )) return FALSE;

	String[0] = returnedStr[0] = '\0';
	
	// Check if the file exists
	if ( _access( fileName, 0 ) != 0 )
		return FALSE;

	// Read Key string from the specified section
	nSize = GetPrivateProfileString( Section, Key, String, returnedStr, 128, fileName );

	strcpy( String, returnedStr );

	return TRUE;
}


////////////////////////////////////


BOOL writeStringToIniFile( char *String, const char *Section, const char *Key )
{
	char fileName[ MAX_PATH ], uninstallPath[ MAX_PATH ];
	int   nSize;

	if (!getBcsrvIniFilePath( fileName, uninstallPath )) return FALSE;

	// Write Key string from the specified section
	nSize = WritePrivateProfileString( Section, Key, String, fileName );

	return TRUE;
}

////////////////////////////////////

BOOL showRightsWarning(  HWND hWndParent, DWORD ServerRights, DWORD LocalRights ) // returns FALSE if user refuses to continue
{
	wchar_t  msgText[768], tmp[256], captionText[32];
	UINT     ids1, ids2, ids3;
		
	if (ServerRights == LocalRights) return TRUE;

	LoadStringT( hMainInst, IDS_WARNING, captionText, sizeof(captionText) );

	switch (ServerRights)
	{
		case RIGHT_DO_NOT_BACKUP_KEYS:
			ids1 = IDS_Rights1; // encryption Key for the container will not be saved on Server
			ids2 = IDS_Rights2; // the Key will be saved on Server so that only you can decrypt it
			ids3 = IDS_Rights3; // the Key will be saved on Server so that Administrator can decrypt it.
			break;

		case RIGHT_BACKUP_FOR_MY_ACCOUNT:
			ids1 = IDS_Rights2; // the Key will be saved on Server so that only you can decrypt it
			ids2 = IDS_Rights1; // encryption Key for the container will not be saved on Server
			ids3 = IDS_Rights3; // the Key will be saved on Server so that Administrator can decrypt it.
			break;

		case RIGHT_BACKUP_KEY_FOR_ADMIN:
			ids1 = IDS_Rights3; // encryption Key for the container will not be saved on Server
			ids2 = IDS_Rights1; // the Key will be saved on Server so that only you can decrypt it
			ids3 = IDS_Rights2; // the Key will be saved on Server so that Administrator can decrypt it.
			break;

		default:
			LoadStringT( hMainInst, IDS_RightsError, msgText, maxLen(msgText) );
			MessageBoxT( hWndParent, msgText, captionText, MB_YESNO | MB_ICONWARNING );
			return FALSE;
	}

	LoadStringT( hMainInst, IDS_RightsStart, msgText, maxLen(msgText) );
	LoadStringT( hMainInst, ids1, tmp, maxLen(tmp) );
	wcscat( msgText, tmp );

	LoadStringT( hMainInst, IDS_RightsAdvice, tmp, maxLen(tmp) );
	wcscat( msgText, tmp );
	
	LoadStringT( hMainInst, ids2, tmp, maxLen(tmp) );
	wcscat( msgText, tmp );

	LoadStringT( hMainInst, ids3, tmp, maxLen(tmp) );
	wcscat( msgText, tmp );

	LoadStringT( hMainInst, IDS_RightsQuestion, tmp, maxLen(tmp) );
	wcscat( msgText, tmp );

	int iRes = MessageBoxT( hWndParent, msgText, captionText, MB_YESNO | MB_ICONWARNING );
	
	if (iRes == IDYES) return TRUE;
	
	return FALSE;

	/*
		"Administrator has configured backuping of encryption Keys for your 
		"containers so that - "
		" - encryption Key for the container will not be saved on Server."

		Please contact Administrator if you believe that the current settings 
		should be changed to one of the following:

		"- encryption Key for the container will not be saved on Server."
		"- the Key will be saved on Server so that only you can decrypt it."
		"- the Key will be saved on Server so that Administrator can decrypt it."

		 You may also stop the process of accessing the container and wait 
		 while Administrator changes the settings. 

		Do you want to continue with the current settings?
		*/

}


////////////////////////////////


static BOOL getUserRights(  HWND hWndParent, const char *UserName, DWORD *Rights )
{
	DWORD defaultRights, serverRights, localRights;
	char  strRights[16];
	BOOL  bContinue = FALSE, saveNewRights = FALSE;

	// Send request to Server to check what rights the User has
	if (!Client_GetUserRights( UserName, &defaultRights, &serverRights ))
	{
		*Rights = RIGHT_DO_NOT_BACKUP_KEYS;
		return FALSE;
	}

	if (serverRights == DEFAULT_USER_RIGHTS) serverRights = defaultRights;

	// Read our rights from local disk
	if (!readStringFromIniFile( strRights, KGSHANW_SAVEKEY_SECTION, KGSHANW_SAVEKEY_KEY ) 
		|| (strlen(strRights) == 0))
	{
		bContinue = showRightsWarning( hWndParent, serverRights, RIGHT_INVALID );
		if (!bContinue)
		{	*Rights = RIGHT_INVALID;
			return FALSE;
		}

		// save the Rights on local computer
		sprintf( strRights, "%d", serverRights );
		writeStringToIniFile( strRights, KGSHANW_SAVEKEY_SECTION, KGSHANW_SAVEKEY_KEY );
		*Rights = serverRights;
		return TRUE;
	}

	sscanf( strRights, "%d", &localRights );
	
	if ( localRights != serverRights )
	{
		bContinue = showRightsWarning( hWndParent, serverRights, localRights );
		if (!bContinue)
		{
			*Rights = RIGHT_INVALID;
			return FALSE;
		}

		// save the Rights on local computer
		sprintf( strRights, "%d", serverRights );
		writeStringToIniFile( strRights, KGSHANW_SAVEKEY_SECTION, KGSHANW_SAVEKEY_KEY );
		*Rights = serverRights;
		return TRUE;
	}

	*Rights = serverRights;

	// check that the rights are valid
	if (((*Rights) != RIGHT_DO_NOT_BACKUP_KEYS) &&
		((*Rights) != RIGHT_BACKUP_FOR_MY_ACCOUNT) &&
		((*Rights) != RIGHT_BACKUP_KEY_FOR_ADMIN))
	{
		*Rights = RIGHT_DO_NOT_BACKUP_KEYS;
		return FALSE;
	}

	return TRUE;
}


////////////////////////////////


static BOOL readPortFromFile( char *ServerName, unsigned short *Port )
{
	char fileName[64];
	char portStr[32], iniStr[32];
	FILE *file;
	int nSize;

	*Port = 0;

	strcpy( fileName, "\\\\" );
	strcat( fileName, ServerName );
	strcat( fileName, "\\" );
	strcat( fileName, BCSERVER_SHARED_DIRECTORY_ANSI );
	strcat( fileName, "\\" );
	strcat( fileName, CLIENT_INI_FILE );

	file = fopen( fileName, "r" );
	if (!file) return FALSE;
	fclose( file );

	// Read Key string from the specified section

	iniStr[0] = portStr[0] = '\0';

	nSize = GetPrivateProfileString( CLIENT_INI_FILE_PORT_SECTION,
		CLIENT_INI_FILE_PORT_KEY, iniStr, portStr, 32, fileName );

	if (strlen(portStr) == 0) return FALSE;

	if (sscanf( portStr, "%hd", Port ) != 1) return FALSE;

	return TRUE;
}

