/*******************************************************************
 *
 *	Copyright (c) 1994-2000 Jetico, Inc., Finland
 *	All rights reserved.
 *
 *	File: commands.cpp
 *	Revision: 
 *	Created:
 *	Description: the file implements class to add and call 
 *  additional commands for Key Generator
 *
 *******************************************************************/

#include <memory.h>
#include <string.h>
#include "commands.h"
#include "kblock.h"
#include "..\..\comm_def\kg_defs.h"
#include "..\..\comm_def\dllMain.h"
#include <stdio.h>
#include "resource.h"
#include "UnicodeFuncs.h"
#include "cDlg.h"
#include "Alg.h"
#include "kgGhost.h"

CCommands::CCommands()
{
	int i;
	
	m_nextCmdOffset = 0;
	m_hModule = hMainInst;
	m_commandsNumber = 0;

	for( i=0; i<MAX_NUMBER_OF_COMMANDS; i++)
	{
		m_idsTable[i] = 0;
		m_commands[i] = NULL;
		m_refTable[i] = NULL;
	}

	memset( m_buffer, 0, sizeof( m_buffer )  );
}

BOOL CCommands::AddCommand( UINT uID, CommandFunction *Function )
{
	wchar_t cmdName[ MAX_COMMAND_NAME_LENGTH ];
	int  cmdLength;

	cmdLength = LoadStringT( m_hModule, uID, cmdName, MAX_COMMAND_NAME_LENGTH );

	if (!cmdLength) return FALSE; // invalid uID

	cmdLength = (int)wcslen( cmdName );

	if ( (DWORD)cmdLength >= ( maxLen( m_buffer ) - m_nextCmdOffset - 1))
	{
		return FALSE;
	}

	wcscpy( &(m_buffer[ m_nextCmdOffset ]), cmdName );

	m_refTable[ m_commandsNumber ] = &(m_buffer[ m_nextCmdOffset ]);

	m_nextCmdOffset += cmdLength + 1;

	m_idsTable[ m_commandsNumber ] = uID;

	m_commands[ m_commandsNumber ] = Function;

	m_commandsNumber++;

	return TRUE;
}


BOOL CCommands::RunCommand( 
		wchar_t *CommandName,   
		wchar_t *contPath,
		BYTE    *DataBlock,
		int     dbSize,
		void    *params           // see the kg_defs.h file
		)
{
	int  i;
	CommandFunction *functionToRun = NULL;
	BOOL result;

	for ( i=0; i < m_commandsNumber; i++ )
	{
		if ( wcscmp( m_refTable[i], CommandName ) == 0)
		{
			functionToRun = m_commands[i];
			break;
		}
	}

	if (!functionToRun) return FALSE;

	result = functionToRun( contPath, DataBlock, dbSize, params );

	return result;
}

DWORD CCommands::GetCommandsBuffer( wchar_t *Buffer, int *BufferSize )
{
	if (*BufferSize <= m_nextCmdOffset)
	{
		*BufferSize = m_nextCmdOffset + 1;
		return ERROR_BUFFER_TOO_SMALL;
	}

	memcpy( Buffer, m_buffer, (m_nextCmdOffset + 1) * sizeof( *Buffer ) );

	return ERROR_NO;
}


//
//	GetAdditionalCommandNames() is called by BestCrypt to understand if the Key Generator
//  module supports additional commands that user can call from the BC Control Panel "File" menu
//

// just an example - we set a global flag, defining if we run evaluation version,
// and if the evaluation period has expired

BOOL EvaluationVersion = FALSE;
BOOL EvaluationVersionHasExpired = FALSE;

class CCommandsEx : public CCommands
{
public:

	bool m_err;

	CCommandsEx()
	{
		m_err = 0 
//			||(!AddCommand( IDS_cmdShowKeyBlockInfo, cmdShowKeyBlockInfo ))
//			||(!AddCommand( IDS_cmdEncryptHeader, cmdEncryptHeader ))
//			||(!AddCommand( IDS_cmdBackupDataBlock, cmdBackupDataBlock ))
//			||(!AddCommand( IDS_cmdRestoreKeyBlockFromBackup, cmdRestoreKeyBlockFromBackup ))
//			||(!AddCommand( IDS_ADDITIONAL_COMMAND_3, MyFunction0 ))
			;
	}
};

extern "C" BOOL GetAdditionalCommandNames(
		wchar_t	*CommandNamesBuffer,	// buffer where the function returns strings of commands, like "comm1\0comm2\0com3\0\0"
		int		*BufferSize,			// max length in wchars of the buffer for the command names
		DWORD	*pErrCode,				// if the function returns FALSE, ErrorCode is valid, see kg_defs.h
		DWORD	*Flags					// some flags BestCrypt tells to the Key Generator, see kg_defs.h
		)
{
//	assert( false );

	CCommandsEx cmdClass;

	if ( cmdClass.m_err )
	{
		assert( false );
		return ERROR_INTERNAL_PROBLEM;
	}

    *pErrCode = cmdClass.GetCommandsBuffer( CommandNamesBuffer, BufferSize );

	// example of using flags in this procedure
	EvaluationVersion = ( *Flags & FLAG_EVALUATION_VERSION ) == 0 ? FALSE : TRUE;
	EvaluationVersionHasExpired = ( *Flags & FLAG_EVALUATION_VERSION_EXPIRED) == 0 ? FALSE : TRUE;
	
	if ( *pErrCode != ERROR_NO) return FALSE;

	return TRUE;
}

//
//
//	RunAdditionalCommand() is called by BestCrypt to run some of the command returned in 
//  preceeding call of the GetAdditionalCommandNames()
//
//

extern "C" BOOL RunAdditionalCommand(
		HWND    hWnd,
		wchar_t	*CommandName,   
		wchar_t	*contPath,
		BYTE    *DataBlock,
		int     dbSize,
		void	*params
		)
{
	sKGCmdParams   *sParams = (sKGCmdParams *)params;
	CCommandsEx cmdClass;
	BOOL      result;

	if ( cmdClass.m_err )
	{
		return FALSE; // or TRUE if we don't want BC Control Panel says something
	}

	result = cmdClass.RunCommand( CommandName, contPath, DataBlock, dbSize, sParams );
	
	return result;
}
 
#include "..\..\COMM_DEF\kbd.h"
//#include "bc_ioctl.h"
#include "..\..\COMM_DEF\mhk.h"

bool GetAlarmCrashHotkey( DWORD *pDWValue )
{
	AC_HOTKEY_BUFFER acBuf;

	memset( &acBuf, 0, sizeof(AC_HOTKEY_BUFFER) );

	if (!KBD_sendCommandToMhkDriver( (LPVOID)&acBuf, sizeof(AC_HOTKEY_BUFFER), IOCTL_PRIVATE_MHK_GET_ALARM_CRASH_HOTKEY ) )
	{
		return FALSE;
	}

	DWORD hotKey = ( acBuf.ScanCode << 16 );

	if ( acBuf.ControlUsed  )	hotKey |= MOD_CONTROL;
	if ( acBuf.AltUsed )		hotKey |= MOD_ALT;

	if ( pDWValue != NULL )
		*pDWValue = hotKey;

	return TRUE;
}

bool SetAlarmCrashHotkey( DWORD dwValue )
{
	AC_HOTKEY_BUFFER acBuf;

	memset( &acBuf, 0, sizeof(AC_HOTKEY_BUFFER) );

	acBuf.ScanCode    = dwValue >> 16;
	acBuf.AltUsed     = (( dwValue & MOD_ALT ) != 0 );
	acBuf.ControlUsed = (( dwValue & MOD_CONTROL ) != 0 );

	if (!KBD_sendCommandToMhkDriver( (LPVOID)&acBuf, sizeof(AC_HOTKEY_BUFFER), IOCTL_PRIVATE_MHK_SET_ALARM_CRASH_HOTKEY ) )
	{
		return FALSE;
	}

	return TRUE;
}
