/******************************************************************\
*
*	Copyright (c) 2005 Jetico, Inc., Finland
*	All rights reserved.
*
*	File: GeberateSeed.cpp
*	Revision: 
*	Created:
*	Description: generation of seed value for random key generation
*
\******************************************************************/

#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>
#include <string.h>
#include <stdio.h>
#include "GenerateSeed.h"
#include "resource.h" 
#include "..\..\Comm_def\kg_errors.h"
//#include "kg_defs.h"
#include <time.h>
#include <assert.h>

//
//	For GetPasswordFromCommandLine only
//
#include "pwdlg.h"
#include "..\..\..\CertificateLib\SMemory\cPassword.h"


#include "UnicodeFuncs.h"
//#include "Translatedll.h"
#include "cDlg.h"

extern HINSTANCE hMainInst;

#define MSG_SEED_READY		(WM_USER + 0x529)

typedef BOOL CALLBACK DlgProcType( HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam );

BOOL CALLBACK getRandomDlgProc( HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam );
BOOL UpdateSeed( BYTE *Seed, DWORD dwData );

/* Global data */

#define BUFFER_SIZE 1024

BYTE    *Seed;
DWORD   SeedCounter, SeedLength;

WNDPROC wpOrigWndProc; // Progerss original procedure
//WNDPROC wpEditOrigWndProc;
//WNDPROC wpEditOrigWndProcA;
//WNDPROC wpEditOrigWndProcB;


DWORD
	GetSeedValue( 
		HWND  hWndParent,
		BYTE  *pSeed, 
		DWORD pSeedLength
	)
{
//#ifdef _DEBUG
//	assert( false );
//	memset( pSeed, 'S', pSeedLength );
//	return ERROR_NO;
//#endif

	int result;
	
	if ((!pSeed) || (pSeedLength <= 0)) 
	{
		assert( false );
		return ERROR_INTERNAL_PROBLEM;
	}

	Seed = pSeed;
	SeedCounter = 0;
	SeedLength = pSeedLength;

	//
	//	If password is specified in command line, so seed value is generated from the password.
	//	Note, it is not secure.
	//
	cPassword tmpPwd;
	if ( GetPasswordFromCommandLine( tmpPwd.wcBuffer ) )
	{
		int len = (int)wcslen( tmpPwd.wcBuffer );

		for( ; SeedCounter < SeedLength; SeedCounter++ )
		{
			BYTE value = (BYTE)SeedCounter;
			wchar_t wData = tmpPwd.wcBuffer[ SeedCounter % len ];
			for( int i = sizeof( wData ); i--; )
				value ^= BYTE( wData >> ( i * 8 ) );

			Seed[ SeedCounter ++ ] = value;
		}

		return ERROR_NO;
	}

	//result = (int)DialogBox( hMainInst, MAKEINTRESOURCE( IDD_RANDOMDLG ), hWndParent, (DLGPROC)getRandomDlgProc );
	cDlg dlg;
	result = (int)dlg.DoModal( hMainInst, IDD_RANDOMDLG, hWndParent, (DLGPROC)getRandomDlgProc );

	if (result == -1) /* system error */
	{
		assert( false );
		return ERROR_INTERNAL_PROBLEM;
	}

	if ( result == 0 )
	{
		assert( false );
	}

	return ERROR_NO;
}


// Subclass procedure
LRESULT APIENTRY progressSubclassProc( HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	HWND parentHwnd;

	switch (uMsg)
	{
		case WM_GETDLGCODE:
			return DLGC_WANTALLKEYS;

		case WM_KEYDOWN:
		case WM_SYSKEYDOWN:
			if (!UpdateSeed( Seed, (DWORD)wParam ))
			{	parentHwnd = GetParent( hwnd );
				SendMessage( parentHwnd, MSG_SEED_READY, 0, 0);
			}
			else
			{	SendMessage( hwnd, PBM_SETPOS, (WPARAM)(SeedCounter), 0);
			}

			break;
		case WM_MOUSEMOVE:
			if (!UpdateSeed( Seed, (DWORD)lParam ))
			{	SendMessage( GetParent( hwnd ), MSG_SEED_READY, 0, 0);
			}
			else
			{	SendMessage( hwnd, PBM_SETPOS, (WPARAM)(SeedCounter), 0);
			}

			break;
		default:
			break;
	}
	return CallWindowProc(wpOrigWndProc, hwnd, uMsg, wParam, lParam);
}

BOOL CALLBACK getRandomDlgProc( HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	HWND   prHwnd;
	
	switch (uMsg) 
	{ 
		case WM_MOUSEMOVE:
			if (!UpdateSeed( Seed, (DWORD)lParam ))
			{	SendMessage( hwnd, MSG_SEED_READY, 0, 0);
			}
			else
			{	SendMessage( GetDlgItem( hwnd, IDC_PROGRESS  ), PBM_SETPOS, (WPARAM)(SeedCounter), 0);
			}
			break;

        case WM_INITDIALOG:

			//if ( pTranslateDll != NULL )
			//	pTranslateDll->TranslateDialog( hwnd );

			// Retrieve the handle of the progress indicator control
			prHwnd = GetDlgItem(hwnd, IDC_PROGRESS); 
			SendMessage( prHwnd, PBM_SETRANGE, 0, MAKELPARAM(0, SeedLength));
 
			EnableWindow( GetDlgItem(hwnd, IDOK), FALSE); 

			// Subclass the  progress indicator control
			wpOrigWndProc = (WNDPROC) SetWindowLongPtrT(prHwnd, GWLP_WNDPROC, (LONG_PTR) progressSubclassProc);
			return TRUE;

		case WM_DESTROY:
			// Retrieve the handle of the  progress indicator control
			prHwnd = GetDlgItem(hwnd, IDC_PROGRESS);

			// Remove the subclass from the progress indicator control.
			SetWindowLongPtrT(prHwnd, GWLP_WNDPROC, (LONG_PTR) wpOrigWndProc); 

			break;
 
		case MSG_SEED_READY:
			EnableWindow( GetDlgItem(hwnd, IDOK), TRUE); 

			SetFocus( GetDlgItem(hwnd, IDOK) ); 
			break;

		case WM_COMMAND:
			switch (wParam)
			{
				case IDOK: 
					EndDialog(hwnd, (int)ERROR_CANCELLED_BY_USER); 
					return TRUE; 
			}
			break;

		default:
			return 0;
	}
	return 0;
}

BOOL UpdateSeed( BYTE *Seed, DWORD dwData )
{
#define _CheckPrevData
#ifdef _CheckPrevData

	// Skips series of same values

	static int failCount = 0;
	static DWORD prevData = -1;

	if ( SeedCounter == 0 )
		failCount = 0;

	dwData ^= (DWORD)clock();

	if ( dwData == prevData )
	{
		failCount ++;
		return TRUE;
	}

	prevData = dwData;

#endif _CheckPrevData

	if (SeedCounter >= SeedLength)
	{

#ifdef _DEBUG
#ifdef _CheckPrevData

		static bool shown = false;
		if ( ( failCount != 0 ) && !shown )
		{
			shown = true;
			MessageBoxT( 0, FormatOnceT_S( L"Fail count %d data %X", (wchar_t*)failCount, (wchar_t*)dwData ), L"UpdateSeed DBG", MB_OK );
			failCount = 0;
		}
#endif // _CheckPrevData
#endif // _DEBUG

		return FALSE;
	}

	BYTE value = 0;
	for( int i = sizeof( dwData ); i--; )
		value ^= BYTE( dwData >> ( i * 8 ) );

	Seed[ SeedCounter ++ ] = value;

	return TRUE;
}