/*******************************************************************
 *
 *	Copyright (c) 1994-2007 Jetico, Inc., Finland
 *	All rights reserved.
 *
 *	File: hdlg.cpp
 *	Revision: 
 *	Created:
 *	Description: implementation of procedures that support 
 *		dialog boxes for creating hidden part of containers 
 *		and showing an information message.
 *
 *******************************************************************/

#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>
#include <string.h>
#include <stdio.h>
#include "kblock.h"
#include "..\..\comm_def\kg_defs.h"
#include "resource.h"
#include "hdlg.h"
#include "..\..\comm_def\dllMain.h"
#include "UnicodeFuncs.h"
#include "cDlg.h"
#include "KgGhost.h"
#include "cAppFlags.h"
#include "FileView.h"

const wchar_t *szHiddenInfoFileName = L"hinfoEx.txt";

#define MSG_PERCENT_CHANGED		(WM_USER + 0x583)

DWORDLONG
//	HDLG_offset, 
//	HDLG_WholeSize,
	HDLG_HP_MinSize,
	HDLG_HP_MaxSize;

#define HDLG_WholeSize HDLG_HP_MaxSize

DWORD	VirtualDriveNumber;

wchar_t *HDLG_FileName;

HWND    ScrollHwnd, EditPercentHwnd;

WNDPROC wpScrollOrigWndProc;

// #define GetNewSize( percents ) (((((HDLG_WholeSize) / 100) * (percents))) / _4K) * _4K
DWORDLONG GetNewSize( DWORDLONG size, int percents )
{
	return ((( size * percents ) / 100 )  / _4K ) * _4K;
}

#define GetNewSizeAndCheckBounds( result, percents ) \
	{ result = HDLG_HP_MinSize + GetNewSize( HDLG_WholeSize - HDLG_HP_MinSize, percents ); \
      if (result > HDLG_HP_MaxSize) { result = HDLG_HP_MaxSize; } \
	}

//      if (result < HDLG_HP_MinSize) { result = HDLG_HP_MinSize; } \


/**** Local procedures definition ******/

//BOOL CALLBACK getOffsetDlgProc( HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam );
BOOL CALLBACK hiddenInfoDlgProc( HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam );
BOOL CALLBACK showInfoMsgDlgProc( HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam );

static cDlgRect dlgRect;

class cHiddenInfoDlg : public cColoredDlg
{
public:
	virtual LRESULT RealDlgProc( UINT uMsg, WPARAM wParam, LPARAM lParam )
	{
		switch( uMsg )
		{
			case WM_INITDIALOG:
//				SetRCValues( IDC_LINE, 0 , 0, IDI_Hidden );
				m_dlgRect = &dlgRect;
				break;
			default:
				break;
		}

		return hiddenInfoDlgProc( m_hDlg, uMsg, wParam, lParam );
	}
};

//
//	ShowMessage
//		Show dialog message according parameters
//
int ShowMessage( HWND hWnd, sParams *params )
{
	INT_PTR res = 0;
	
	if ( ( params == NULL ) || ( params->size <= sizeof( sParams ) ) )
	{
		cHiddenInfoDlg infoDlg; 
		// Draw information dialog
		res = infoDlg.DoModal( hMainInst, IDD_HiddenInfoDlg, hWnd );
		if ( res == 0 )
			res = IDOK;
		else
		if ( res == ERROR_CANCELLED_BY_USER )
			return IDCANCEL;
	}

	return (int)res;
}

class cGetOffsetDlg : public cColoredDlg
{
public:
	int   posMax;
	HWND  spinHwnd;
	char  strSize[64];
	HWND  cmbHwnd;
	int   i, selDrive, curSel;
	DWORD curLogicalDisks;
	char  driveName[4];

	int m_pos;
	bool m_disableItems;

	cGetOffsetDlg()
	{
		m_disableItems = true;

		strcpy( driveName, "A:" );
		selDrive = 0;
	}

	virtual LRESULT RealDlgProc( UINT uMsg, WPARAM wParam, LPARAM lParam )
	{
		switch( uMsg )
		{
			case WM_INITDIALOG:
				SetRCValues( IDC_LINE, 0 , 0, IDI_Hidden );
				m_dlgRect = &dlgRect;

				break;
			default:
				break;
		}

		return getOffsetDlgProc( uMsg, wParam, lParam );
	}

	BOOL getOffsetDlgProc( UINT uMsg, WPARAM wParam, LPARAM lParam );
};


static BOOL estimateHiddenPartSize(
		DWORDLONG contBodySize,       // in 1 KBytes blocks
		DWORDLONG *HP_MinSize,         // in 1 KBytes blocks
		DWORDLONG *HP_MaxSize,
		BOOL  isNTFS = true
	);


/**** Global procedures ******/
//		DWORD  ContainerFilesystem, // FAT or NTFS or other, see kg_defs.h

BOOL HDLG_GetHiddenOffset(
			HWND   hWnd,
			wchar_t   *contFileName, 
			DWORDLONG  mainBodySize,

			// [OUT] Positions and size of hidden part into main body.
			DWORDLONG  *HPartOffset,        // returns offset in bytes /// !!1 KBytes sectors
			DWORDLONG  *HPartSize,

			wchar_t  *mountPoint,
//			DWORD  *VDriveNumber,
			
			bool allSpaceIsFree
			)
{
	*HPartOffset = 0;
	*HPartSize = mainBodySize;

	HDLG_FileName = contFileName;
	INT_PTR result;

	wchar_t msgText[128], captionText[32];

	cHiddenInfoDlg infoDlg;
	cGetOffsetDlg offsetDlg;
	
	if ( allSpaceIsFree )
	{
		HDLG_WholeSize = mainBodySize;
		HDLG_HP_MinSize = _4K;
		HDLG_HP_MaxSize = mainBodySize;
	}
	else
	{
        if (!estimateHiddenPartSize(
						mainBodySize,
						&HDLG_HP_MinSize,
						&HDLG_HP_MaxSize,
						false ))
		{
			LoadStringT( hMainInst, IDS_CONTAINER_TOO_SMALL, msgText, maxLen(msgText) );
			LoadStringT( hMainInst, IDS_CONTAINER_TOO_SMALL_C, captionText, maxLen(captionText) );
			MessageBoxT(hWnd, msgText, captionText, MB_OK | MB_ICONSTOP );
			return FALSE;
		}
	}
	
	HDLG_WholeSize = HDLG_HP_MaxSize;

	for(;;)
	{
		// 
		//	if allSpaceIsFree so the function is called by BestCrypt->SpecHiddenPartCreate function
		//	But the function calls the infoDlg before, so the dialog is skipped now
		//
		if ( !allSpaceIsFree )
		{
			// Draw information dialog
			result = infoDlg.DoModal( hMainInst, IDD_HiddenInfoDlg, hWnd );
			
			if (result != ERROR_NO)
				return FALSE;

			offsetDlg.m_disableItems = false;
		}

		// Draw dialog to get offset of the hidden part
		result = offsetDlg.DoModal( hMainInst, IDD_GetOffsetDlg, hWnd );
	
		if ( result == ERROR_NO_JUST_GO_BACK ) continue;
		
		if ( result == ERROR_NO ) break;

		return FALSE;
	}

	GetNewSizeAndCheckBounds((*HPartSize), offsetDlg.m_pos );

	if ((*HPartSize) > HDLG_WholeSize)
		return FALSE;

    *HPartOffset = (HDLG_WholeSize - (*HPartSize));

	wcscpy( mountPoint, L"A:\\" );
	mountPoint[0] = wchar_t( L'A' + VirtualDriveNumber );
//	*VDriveNumber = VirtualDriveNumber;

	return TRUE;
}


void HDLG_ShowInformationMessage( HWND hWnd )
{
	cDlg dlg;
	dlg.DoModal( hMainInst, IDD_HP_INFO_DIALOG, hWnd, (DLGPROC)showInfoMsgDlgProc );

//    DialogBox( hMainInst, MAKEINTRESOURCE(IDD_HP_INFO_DIALOG), 
//			hWnd, (DLGPROC)showInfoMsgDlgProc );
}

/**** Local procedures ******/

BOOL lButtonPressed;

// Scroll subclass procedure
LRESULT APIENTRY scrollSubclassProc( HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	int  position, len;
	DWORDLONG hp_size;
	char str[64];

	switch (uMsg)
	{
		case MSG_PERCENT_CHANGED:
			Edit_GetText(EditPercentHwnd, str, 16);
			len = (int)strlen(str);

			if (len > 0)	sscanf( str, "%d", &position );
			else			position = 0;
			
			if		(position > 100) position = 100;
			else if (position < 0)   position = 0;

			SendMessage( ScrollHwnd, TBM_SETPOS, 1L, position );
			SendMessage( ScrollHwnd, TBM_SETSELEND, 1L, position);

			GetNewSizeAndCheckBounds( hp_size, (DWORD)position );
			GetSizeText( str, sizeof( str ), hp_size ); // sprintf( str, "%d KBytes", hp_size );
			SetWindowText( GetDlgItem(GetParent(hwnd), IDC_HP_SIZE), str );

			break;

		case WM_KEYDOWN:
				position = (int)SendMessage( ScrollHwnd, TBM_GETPOS, 0, 0 );
				sprintf( str, "%d", position );
				Edit_SetText( EditPercentHwnd, str);
				SendMessage( ScrollHwnd, TBM_SETSELEND, 1L, position );
				break; 

		case WM_MOUSEMOVE:
			if (lButtonPressed)
			{
				position = (int)SendMessage( ScrollHwnd, TBM_GETPOS, 0, 0 );
				sprintf( str, "%d", position );
				Edit_SetText( EditPercentHwnd, str);
				SendMessage( ScrollHwnd, TBM_SETSELEND, 1L, position);
			}
			break;
		case WM_LBUTTONDOWN:
			lButtonPressed = TRUE;
				break;
		case WM_LBUTTONUP:
			position = (int)SendMessage( ScrollHwnd, TBM_GETPOS, 0, 0 );
			sprintf( str, "%d", position );
			Edit_SetText( EditPercentHwnd, str);
			lButtonPressed = FALSE;
			SendMessage( ScrollHwnd, TBM_SETSELEND, 1L, position);
	}
	return CallWindowProc(wpScrollOrigWndProc, hwnd, uMsg, wParam, lParam);
}

enum eFlags { efShowHPInfo = 0 };
cAppFlags
	appFlags(
		HKEY_CURRENT_USER,
		L"SOFTWARE\\Jetico\\BestCrypt",
		L"KG-GhostFlags" );

BOOL CALLBACK showInfoMsgDlgProc( HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	if (uMsg == WM_INITDIALOG)
	{
		if ( !appFlags.GetFlag( efShowHPInfo ) )
		{
			EndDialog(hwnd, ERROR_NO);
			return false;
		}
		Button_SetCheck( GetDlgItem( hwnd, IDC_chkShow ), BST_CHECKED );
		return TRUE; 
	}
	else if (uMsg == WM_COMMAND )
	{
		WORD cmd = LOWORD( wParam );
		if ( cmd == IDOK )
		{
			bool show = ( Button_GetCheck( GetDlgItem( hwnd, IDC_chkShow ) ) == BST_CHECKED );
			appFlags.SetFlag( efShowHPInfo, show );

			EndDialog(hwnd, ERROR_NO);
			return FALSE;
		}
	}

	return FALSE;
}

BOOL cGetOffsetDlg::getOffsetDlgProc( UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	wchar_t bf[ 100 ];

	switch (uMsg) 
	{ 
        case WM_INITDIALOG:
		{
			lButtonPressed = FALSE;

			if (HDLG_FileName) SetWindowTextT( GetDlgItem( m_hDlg, IDC_HP_FRAME), HDLG_FileName );

			DWORDLONG dwl = GetFileSizeT( HDLG_FileName );
			GetSizeTextT( bf, maxLen( bf ), dwl ); // HDLG_WholeSize );
			SetWindowTextT( GetDlgItem(m_hDlg, IDC_WHOLE_SIZE), bf );

			GetSizeTextT( bf, maxLen( bf ), HDLG_HP_MinSize );
			SetWindowTextT( GetDlgItem(m_hDlg, IDC_HP_SIZE), bf );

			GetSizeTextT( bf, maxLen( bf ), HDLG_HP_MaxSize );
			SetWindowTextT( GetDlgItem(m_hDlg, IDC_MAX_SIZE), bf );

			GetSizeTextT( bf, maxLen( bf ), HDLG_HP_MinSize );
			SetWindowTextT( GetDlgItem(m_hDlg, IDC_MIN_SIZE), bf );

			ScrollHwnd		= GetDlgItem(m_hDlg, IDC_SliderOffset);
			EditPercentHwnd	= GetDlgItem(m_hDlg, IDC_EditPercent);
			spinHwnd		= GetDlgItem(m_hDlg, IDC_SpinPercent);

			//GetScrollRange(ScrollHwnd, SB_HORZ, &posMin, &posMax);
			SendMessage( spinHwnd, UDM_SETRANGE, 0L, MAKELONG(100,0));
			wpScrollOrigWndProc = (WNDPROC)SetWindowLongPtrT(ScrollHwnd, GWLP_WNDPROC, (LONG_PTR) scrollSubclassProc);

			m_pos = 0;

			if ( m_disableItems )
			{
				m_pos = 100;
				ShowWindow( GetDlgItem( m_hDlg, IDC_BACK ), SW_HIDE );
				ShowWindow( GetDlgItem( m_hDlg, IDC_stContSize ), SW_HIDE );
				ShowWindow( GetDlgItem( m_hDlg, IDC_WHOLE_SIZE ), SW_HIDE );
			}

			char strPercent[ 20 ];
			sprintf( strPercent, "%d", m_pos );
			SetWindowText( EditPercentHwnd, strPercent );

			SendMessage( ScrollHwnd , MSG_PERCENT_CHANGED, 0, 0);

			curLogicalDisks = GetLogicalDrives();

			cmbHwnd = GetDlgItem(m_hDlg, IDC_DriveList);

			for (i=2, curSel=0; i<26; i++ )
			{
				if ((curLogicalDisks & (1 << i)) == 0)
				{	driveName[0] = 'A' + i;
					ComboBox_InsertString( cmbHwnd, curSel, driveName);
					curSel++;
				}
			}
			
			ComboBox_SetCurSel( cmbHwnd, 0);

//			DWORD dwStyle = GetWindowLong( ScrollHwnd, GWL_STYLE );
//			SetWindowLong( ScrollHwnd, GWL_STYLE, (LONG)( dwStyle | TBS_FIXEDLENGTH ) );
//			SendMessage( ScrollHwnd, TBM_SETTHUMBLENGTH, 100, 0 );

			return TRUE;
		}
		case WM_DESTROY:
			// Remove the subclass from the edit control.
			break;

        case WM_COMMAND:

			if ((HIWORD(wParam) == EN_CHANGE) && ((HWND)lParam == EditPercentHwnd))
			{
				SendMessage( ScrollHwnd , MSG_PERCENT_CHANGED, 0, 0);
			}

			switch( LOWORD( wParam ) ) 
			{ 
				case IDC_BACK:

					EndDialog(m_hDlg, ERROR_NO_JUST_GO_BACK);
					return TRUE; 

				case IDOK:

					m_pos = (int)SendMessage( ScrollHwnd, TBM_GETPOS, 0, 0 );

					curSel = ComboBox_GetCurSel( GetDlgItem(m_hDlg, IDC_DriveList) );
					ComboBox_GetText( GetDlgItem(m_hDlg, IDC_DriveList), driveName, sizeof(driveName));
					VirtualDriveNumber = (DWORD)(driveName[0] - 'A');
 
					EndDialog(m_hDlg, ERROR_NO);
					return TRUE; 
 
				case IDCANCEL: 
					EndDialog(m_hDlg, (int)ERROR_CANCELLED_BY_USER); 
					return TRUE; 

			} 
            return 0; 
    } 
    return FALSE; 

	UNREFERENCED_PARAMETER(lParam);
}

/*********************************************************************
 ******* Local functions required to draw Information dialog *********
 *********************************************************************/

////////////
//
//	FillLBText functions.
//

HFONT save,current;
static int windowSize = 0;

static int getLastVisible( HWND lBox, const wchar_t *string )
{
//	assert( windowSize );

	HDC hDC = GetDC( lBox );

	LOGFONT lf;
	memset( &lf, 0, sizeof(lf) );
	current = GetWindowFont( lBox );

	save = ( HFONT )SelectObject( hDC, current );

	SIZE size;

	int len = (int)wcslen( string );

	for ( ; len; len-- )
	{
		GetTextExtentPoint32T( hDC, string, len, &size );
		if ( size.cx < windowSize )
			break;
	}

	SelectObject( hDC, save );
//	DeleteObject( current );

	ReleaseDC( lBox, hDC );

	return len;
}

static void AddString( HWND lBox, const wchar_t *string )
{
	int len = (int)wcslen( string );

	int last = getLastVisible( lBox, string );

	if ( last >= len )
		ListBox_AddStringT( lBox, string );
	else
	{
		int lastProbel = last;
		for( ;; lastProbel-- )
		{	if ( lastProbel == 0 )
			{
				lastProbel = last;
				break;
			}
			else
			if ( string[ lastProbel ] == L' ')
				break;
		}

		wchar_t *ptr = ( wchar_t * )string + lastProbel + 1;

		// temporary changes the constant string.
		wchar_t save = *ptr;
		*ptr = L'\0';

		ListBox_AddStringT( lBox, string );

		*ptr = save;

		AddString( lBox, ptr );
	}
}

void FillLBText( HWND lBox, const wchar_t *fileName )
{
	wchar_t path[ MAX_PATH ];

	GetModuleFileNameT( NULL, path, maxLen( path ));

	int pos = splitPath( path );
	if ( pos == 0 )
		return;

	wcscpy( path + pos, fileName );

	cStringFile sf( path );

	if ( sf.m_err )
	{
		MessageBoxT( lBox, fileName, L"Error in opening file", MB_ICONSTOP | MB_TOPMOST );
		return;
	}

	ListBox_ResetContent( lBox );

	RECT r;
	GetWindowRect( lBox, &r );

	int scrollBarSize = 30;

	windowSize = r.right - r.left - scrollBarSize;

	wchar_t string[0x800];

	sf.m_skipEmptyString = false;

	if ( sf.m_isUnicode )
	{
		while( sf.ReadStringW( string, maxLen( string ) ) )
			AddString( lBox, string );
	}
	else
	{
		char stringA[0x800];
		while( sf.ReadString( stringA, maxLen( stringA ) ) )
		{
			mbstowcs( string, stringA, maxLen( string ) );
			AddString( lBox, string );
		}
	}
}
//static windowSize = 0;
//HFONT  save,current;
//
//static int getLastVisible( HWND lBox, char *string )
//{
////	assert( windowSize );
//
//	HDC hDC = GetDC( lBox );
//
//	LOGFONT lf;
//	memset( &lf, 0, sizeof(lf) );
//	current = GetWindowFont( lBox );
//
//	save = (HFONT)SelectObject( hDC, current );
//
//	SIZE size;
//
//	int len = strlen( string );
//
//	for ( ; len; len-- )
//	{
//		GetTextExtentPoint32( hDC, string, len, &size );
//		if ( size.cx < windowSize )
//			break;
//	}
//
//	SelectObject( hDC, save );
////	DeleteObject( current );
//
//	ReleaseDC( lBox, hDC );
//
//	return len;
//}
//
//
//static void AddString( HWND lBox, char *string )
//{
//	int len = strlen( string );
//
//	int last = getLastVisible( lBox, string );
//
//	if ( last >= len )
//		ListBox_AddString( lBox, string );
//	else
//	{
//		int lastProbel = last;
//		for( ;; lastProbel-- )
//		{	if ( lastProbel == 0 )
//			{
//				lastProbel = last;
//				break;
//			}
//			else
//			if ( string[ lastProbel ] == ' ')
//				break;
//		}
//
//		char save = string[ lastProbel + 1];
//		string[ lastProbel + 1] = 0;
//
//		ListBox_AddString( lBox, string );
//
//		string[ lastProbel + 1 ] = save;
//
//		AddString( lBox, string + lastProbel + 1);
//	}
//}
//
//
//void FillLBText( HWND lBox, const char *fileName )
//{
//	char string[0x800];
//
//	FILE *f;
//
//	char path[ MAX_PATH ];
//
//	GetModuleFileName( NULL, path, sizeof( path ));
//
//	char drive[_MAX_DRIVE];
//	char dir[_MAX_DIR];
//	char fname[_MAX_FNAME];
//	char ext[_MAX_EXT];
//
//	_splitpath( path, drive, dir, fname, ext );
//
//	_makepath( path, drive, dir, fileName, NULL );
//
//	f = fopen( path, "r" );
//
//	if ( !f )
//		return;
//
//	ListBox_ResetContent( lBox );
//
//	RECT r;
//	GetWindowRect( lBox, &r );
//
//	//	GetScrollInfo
//
//	int scrollBarSize = 30;
//
//	windowSize = r.right - r.left - scrollBarSize;
//
//	while( fgets( string, sizeof( string ) , f ) )
//	{
//		int len = strlen( string );
//		if ( string[ len - 1 ] == '\n' )
//		{	string[ --len ] = 0;
//		}
//
//		AddString( lBox, string );
//	}
//}


BOOL CALLBACK hiddenInfoDlgProc( HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
    switch (uMsg)
	{ 
 		case WM_COMMAND:
		{
			if ( LOWORD(wParam) == IDOK )
			{
				EndDialog(hwnd, ERROR_NO);
				return TRUE;
			}
			else if ( LOWORD(wParam) == IDCANCEL )
			{
				EndDialog(hwnd, ERROR_CANCELLED_BY_USER);
				return TRUE;
			}
			else
			if ( LOWORD(wParam) == IDC_CheckHidden )
			{
				BOOL checked = Button_GetCheck( GetDlgItem( hwnd, IDC_CheckHidden ));
				EnableWindow( GetDlgItem( hwnd, IDOK ), checked );
			}

			return FALSE;
		}
		case WM_INITDIALOG:
		{
			SetFocus( hwnd );
			FillLBText( GetDlgItem( hwnd, IDC_HP_LBOX ), szHiddenInfoFileName );
			return FALSE;
		}
/*
		case WM_PAINT:
		{
			OnPaint(hwnd,wParam);
			return FALSE;
		}
*/
		default: return FALSE;
	}

	return FALSE;	
}


//TBM_CLEARSEL TBM_CLEARTICS TBM_GETBUDDY TBM_GETCHANNELRECT TBM_GETLINESIZE TBM_GETNUMTICS TBM_GETPAGESIZE TBM_SETPOS TBM_GETPTICS TBM_GETRANGEMAX TBM_GETRANGEMIN TBM_GETSELEND TBM_GETSELSTART TBM_GETTHUMBLENGTH TBM_GETTHUMBRECT TBM_GETTIC TBM_GETTICPOS TBM_GETTOOLTIPS TBM_GETUNICODEFORMAT TBM_SETBUDDY TBM_SETLINESIZE TBM_SETPAGESIZE TBM_SETPOS TBM_SETRANGE TBM_SETRANGEMAX TBM_SETRANGEMIN TBM_SETSEL TBM_SETSELEND TBM_SETSELSTART TBM_SETTHUMBLENGTH TBM_SETTIC TBM_SETTICFREQ TBM_SETTIPSIDE TBM_SETTOOLTIPS TBM_SETUNICODEFORMAT

/* available Partition types - from <ntdddisk.h> */
#define PARTITION_ENTRY_UNUSED          0x00      // Entry unused
#define PARTITION_FAT_12                0x01      // 12-bit FAT entries
#define PARTITION_XENIX_1               0x02      // Xenix
#define PARTITION_XENIX_2               0x03      // Xenix
#define PARTITION_FAT_16                0x04      // 16-bit FAT entries
#define PARTITION_EXTENDED              0x05      // Extended partition entry
#define PARTITION_HUGE                  0x06      // Huge partition MS-DOS V4
#define PARTITION_IFS                   0x07      // IFS Partition
#define PARTITION_FAT32                 0x0B      // FAT32
#define PARTITION_FAT32_XINT13          0x0C      // FAT32 using extended int13 services
#define PARTITION_XINT13                0x0E      // Win95 partition using extended int13 services
#define PARTITION_XINT13_EXTENDED       0x0F      // Same as type 5 but uses extended int13 services
#define PARTITION_PREP                  0x41      // PowerPC Reference Platform (PReP) Boot Partition
#define PARTITION_UNIX                  0x63      // Unix
/**/

//
//	Assumes all calkulations in bytes
//
static BOOL estimateHiddenPartSize(
		DWORDLONG contBodySize,       // in bytes
		DWORDLONG *HP_MinSize,         // in bytes
		DWORDLONG *HP_MaxSize,
		BOOL  isNTFS // = true
	)
{
	DWORDLONG max99percent = GetNewSize( HDLG_WholeSize, 99 );

	*HP_MinSize = _4K;
	
	if ( *HP_MinSize * 2 > contBodySize )
		return FALSE;

	if (contBodySize < _4K * 4 )
	{
		if ( isNTFS )
			return false;

		*HP_MaxSize = contBodySize - *HP_MinSize;
	}
	else if ( contBodySize < _4K * 25 )
	{ 
		*HP_MaxSize = contBodySize - _4K * 2;
	}
	else if ( contBodySize < _4K * 128 )
	{ 
		*HP_MaxSize = contBodySize - _4K * 6;
	}
	else if ( contBodySize < _4K * 256 )
	{ 
		*HP_MaxSize = contBodySize - _4K * 8;
	}
	else if (contBodySize < ((1536/4)*_4K) )
	{ 
		*HP_MaxSize = contBodySize - ((128/4)*_4K);
	}
	else if ((contBodySize < 5*((1024/4)*_4K) ) && isNTFS)
	{ 
		*HP_MaxSize = contBodySize - ((2256/4)*_4K);
	}
	else if (contBodySize < 5*((1024/4)*_4K) )
	{ 
		*HP_MaxSize = contBodySize - ((128 /4)*_4K);
	}
	else if ((contBodySize < 100*((1024/4)*_4K) ) && isNTFS)
	{ 
		*HP_MaxSize = contBodySize - ((2400 /4)*_4K);
	}
	else if (contBodySize < 10*((1024/4)*_4K) )
	{ 
		*HP_MaxSize = contBodySize - ((2400 /4)*_4K);
	}
	else if ((contBodySize < 1024*((1024/4)*_4K) ) && isNTFS)
	{ 
		*HP_MaxSize = contBodySize - ((4608/4)*_4K);
	}
	else if (isNTFS)
	{ 
		*HP_MaxSize = max99percent;
	}
	else if (contBodySize < 256*((1024/4)*_4K) )
	{ 
		*HP_MaxSize = contBodySize - ((256 /4)*_4K);
	}
	else
	{
		*HP_MaxSize = max99percent;
	}

	// Size of Hidden Part cannot be more than 99 percents of whole container size
	if ((*HP_MaxSize) > max99percent)
	{
		*HP_MaxSize = max99percent;
	}

	return TRUE;
}
