/*******************************************************************
 *	Copyright (c) 1994-2004 Jetico, Inc., Finland
 *	All rights reserved.
 *
 *	File: kbd.cpp
 *	Revision: 
 *	Created:
 *	Description: interface with the BestCrypt keyboard driver
 *
 *******************************************************************/

#include <windows.h>
#include <winuser.h>
#include <memory.h>
#include <ctype.h>
#include "kbd.h"
#include "bc_ioctl.h"
//#include "kg_misc.h"
#include "mhk.h"

const char *kbdParams::szSemaphoreName = "BCKeyGenSemaphore";
extern void ShredData( BYTE *data, int dataLength );

BYTE *kbdTmp;

UINT CharacterOrder;

void freeTables(UINT   **VkOriginal,
				UINT   **VkMessed,
				USHORT **ScodOriginal,
				USHORT **ScodMessed,
				BYTE   **ShiftTable,
				UINT   ShiftTableSize )
{
	if (*VkOriginal)	
	{	ShredData( (BYTE *)(*VkOriginal), ENCODING_TABLE_SIZE*2*sizeof(UINT) );
		delete *VkOriginal;
		*VkOriginal = NULL;
	}
	if (*VkMessed)
	{	ShredData( (BYTE *)(*VkMessed), ENCODING_TABLE_SIZE*2*sizeof(UINT) );
		delete *VkMessed;
		*VkMessed = NULL;
	}
	if (*ScodOriginal)
	{	ShredData( (BYTE *)(*ScodOriginal), ENCODING_TABLE_SIZE*2*sizeof(USHORT) );
		delete *ScodOriginal;
		*ScodOriginal = NULL;
	}
	if (*ScodMessed)
	{	ShredData( (BYTE *)(*ScodMessed), ENCODING_TABLE_SIZE*2*sizeof(USHORT) );
		delete *ScodMessed;
		*ScodMessed = NULL;
	}
	if (*ShiftTable)
	{	ShredData( (BYTE *)(*ShiftTable), ShiftTableSize*sizeof(BYTE) );
		delete *ShiftTable;
		*ShiftTable = NULL;
	}
}


static BOOL virtualKeyIsValid( ULONG Vk )
{
	BYTE  keyState[256];
	HKL   hkl;
  	UINT  scanCode;
	WORD  chars;
	int   charsGot;

	memset( keyState,  0x00, 256 ); // nothing is pressed or toggled

	hkl = GetKeyboardLayout( 0 );
	
	/* call ToAsciiEx just to clear keyboard from dead key, if it was pressed */
	scanCode = MapVirtualKey( Vk, 0 ); 
	charsGot = ToAsciiEx( Vk, scanCode, keyState, &chars, 0, hkl );

	/* Get scan code for the virtual key */
	scanCode = MapVirtualKey( Vk, 0 ); 

	/* Get character for the given virtual key */
	charsGot = ToAsciiEx(  Vk, scanCode, keyState, &chars, 0, hkl );
		
	if (charsGot != 1)
	{
		return FALSE;
	}

	return TRUE;
}


#define ADD_VIRTUAL_KEY( Vk, i ) \
	if (virtualKeyIsValid( Vk )) \
	{ tableNumL[i] = tableSmal[i] = Vk;	i++; } \
	else \
	{ (*SizeNumL)--; (*SizeSmal)--; }


//#define ENCODING_TABLE_SIZE 64 - max numbers
#define NUMBER_OF_LETTERS       26
#define NUMBER_OF_USUAL_DIGITS  10
#define NUMBER_OF_SPACE_LETTERS 1
#define NUMBER_OF_OTHER_VK      11
#define NUMBER_OF_NUMPAD_DIGITS 10


BOOL createOriginalTable_VK( UINT **Table, UINT *SizeNumL, UINT *SizeSmal )
{
	UINT i,j;
	UINT start, *tableNumL, *tableSmal;

	// set the SizeNumL and SizeSmal to maximum numbers
	*SizeNumL = NUMBER_OF_USUAL_DIGITS + NUMBER_OF_LETTERS + NUMBER_OF_SPACE_LETTERS + NUMBER_OF_OTHER_VK + NUMBER_OF_NUMPAD_DIGITS;
	*SizeSmal = NUMBER_OF_USUAL_DIGITS + NUMBER_OF_LETTERS + NUMBER_OF_SPACE_LETTERS + NUMBER_OF_OTHER_VK;

	if ( ((*SizeNumL) > ENCODING_TABLE_SIZE) || ((*SizeSmal) > ENCODING_TABLE_SIZE) )
		return FALSE;
	
	*Table = new UINT[ ENCODING_TABLE_SIZE * 2 ];
	if (!(*Table)) return FALSE;

	memset( *Table, 0, ENCODING_TABLE_SIZE * 2 * sizeof(UINT) );

	tableNumL = *Table;
	tableSmal = &((*Table)[ENCODING_TABLE_SIZE]);

	i = 0;

	// Fill in two halves of the table - one of them for the case when NumLock is pressed

	start = 0x30; /* VK_0 ... VK_9 */
	for( j=0; j<10; i++, j++ ) tableNumL[i] = tableSmal[i] = start + j;

	start = 0x41; /* VK_A ... VK_Z */
	for( j=0; j<26; i++, j++ ) tableNumL[i] = tableSmal[i] = start + j;

	/* Plus additional VK_SPACE virtual key */

	ADD_VIRTUAL_KEY( VK_SPACE, i )

	/* Plus OTHER_VK */

	// main keyboard
/*
#define VK_OEM_MINUS      0xBD   // '-' any country
#define VK_OEM_PLUS       0xBB   // '+' any country
#define VK_OEM_2          0xBF   // '/?' for US
#define VK_OEM_PERIOD     0xBE   // '.' any country
#define VK_OEM_5          0xDC  //  '\|' for US
#define VK_OEM_COMMA      0xBC   // ',' any country
#define VK_OEM_3          0xC0   // '`~' for US
#define VK_OEM_4          0xDB  //  '[{' for US
#define VK_OEM_6          0xDD  //  ']}' for US
#define VK_OEM_1          0xBA   // ';:' for US
#define VK_OEM_7          0xDE  //  ''"' for US
*/

	ADD_VIRTUAL_KEY( 0xBD, i )
	ADD_VIRTUAL_KEY( 0xBB, i )
	ADD_VIRTUAL_KEY( 0xBF, i )
	ADD_VIRTUAL_KEY( 0xBE, i )
	ADD_VIRTUAL_KEY( 0xDC, i )
	ADD_VIRTUAL_KEY( 0xBC, i )
	ADD_VIRTUAL_KEY( 0xC0, i )
	ADD_VIRTUAL_KEY( 0xDB, i )
	ADD_VIRTUAL_KEY( 0xDD, i )
	ADD_VIRTUAL_KEY( 0xBA, i )
	ADD_VIRTUAL_KEY( 0xDE, i )

	/* Plus additional VK_NUMPAD* virtual keys */

	tableNumL[i++] = VK_NUMPAD0;
	tableNumL[i++] = VK_NUMPAD1;
	tableNumL[i++] = VK_NUMPAD2;
	tableNumL[i++] = VK_NUMPAD3;
	tableNumL[i++] = VK_NUMPAD4;
	tableNumL[i++] = VK_NUMPAD5;
	tableNumL[i++] = VK_NUMPAD6;
	tableNumL[i++] = VK_NUMPAD7;
	tableNumL[i++] = VK_NUMPAD8;
	tableNumL[i++] = VK_NUMPAD9;

	return TRUE;
}




int getRandomToMess( BYTE *RandomArray, int Bound )
{	
	int result; /* result always < 256 because maximum number of virtual keys is 256 */

	result = (int)RandomArray[ Bound ];

	if (Bound > 0) result = result % Bound;
	else           result = 0;

	return result;
}


BOOL messTable( UINT **Table, UINT Size )
{
	int  i, index;
	UINT tmp;
	BYTE randomArray[ ENCODING_TABLE_SIZE ];

	/* Initialize random numbers array. ShredData uses SHA-1 PRNG */
    ShredData( randomArray, ENCODING_TABLE_SIZE );
	
	/*	Use the Moses-Oakford shuffling algorithm 
		(Tables of Random Permutations, Stanford University Press, 1963)
	 */
	for( i=Size-1; i>0; i-- )
	{
		index = getRandomToMess(randomArray, i-1);
		tmp = (*Table)[i];
		(*Table)[i] = (*Table)[index];
		(*Table)[index] = tmp;
	}
	return TRUE;
}



BOOL createMessedTable_VK(UINT *TableOriginal, 
						  UINT **TableMessed, 
						  UINT SizeNumL,
						  UINT SizeSmal )
{
	UINT *tableMessSmal;
	
	*TableMessed = new UINT[ ENCODING_TABLE_SIZE * 2 ];
	if (!(*TableMessed)) return FALSE;

	memcpy( *TableMessed, TableOriginal, ENCODING_TABLE_SIZE * 2 * sizeof(UINT));

	if (!messTable( TableMessed, SizeNumL )) return FALSE;

	tableMessSmal = &((*TableMessed)[ENCODING_TABLE_SIZE]);

	if (!messTable( &tableMessSmal, SizeSmal )) return FALSE;

	return TRUE;
}


BOOL createTable_scode( UINT *VkTable, USHORT **ScodTable, UINT SizeNumL, UINT SizeSmal )
{
	UINT i;
	USHORT *table;

	table = *ScodTable = new USHORT[ ENCODING_TABLE_SIZE * 2 ];
	if (!table) return FALSE;

	memset( table, 0, ENCODING_TABLE_SIZE * 2 * sizeof(USHORT));
	for( i=0; i<SizeNumL; i++)
	{
		/* Get scan code for the virtual key */
		table[i] = MapVirtualKey( VkTable[i], 0 ); 
	}

	table = &((*ScodTable)[ENCODING_TABLE_SIZE]);

	for( i=0; i<SizeSmal; i++)
	{
		/* Get scan code for the virtual key */
		table[i] = MapVirtualKey( VkTable[i+ENCODING_TABLE_SIZE], 0 ); 
	}
	
	return TRUE;
}


BOOL createShiftTable( BYTE **Table, UINT *Size )
{
	BYTE *table;

	*Size = SHIFT_TABLE_SIZE;

	table = *Table = new BYTE[ *Size ];
	if (!table) return FALSE;

	ShredData( (BYTE *)(*Table), SHIFT_TABLE_SIZE ); // Fill in the table by random data

	return TRUE;
}

BOOL KBD_GenerateEncodingTable(
		UINT   **VkOriginal,
		UINT   **VkMessed,
		USHORT **ScodOriginal,
		USHORT **ScodMessed,
		UINT   *TableSizeNL,
		UINT   *TableSizeSm,
   		BYTE   **ShiftTable,
		UINT   *ShiftTableSize,
		BOOL   UseSecureTyping
	)
{
	UINT  tSizeNL = 0, tSizeSm = 0, tsSize = 0;
	BOOL result;

	result = FALSE;

	if (!UseSecureTyping) return TRUE;

	CharacterOrder = 0;

	*TableSizeNL = tSizeNL = *TableSizeSm = tSizeSm = 0;
	*ShiftTableSize = tsSize = 0;

	if (!createOriginalTable_VK( VkOriginal, &tSizeNL, &tSizeSm ))
		goto error_exit;
	
	if (!createMessedTable_VK( *VkOriginal, VkMessed, tSizeNL, tSizeSm )) 
		goto error_exit;
	
	if (!createTable_scode( *VkOriginal, ScodOriginal, tSizeNL, tSizeSm )) 
		goto error_exit;

	if (!createTable_scode( *VkMessed, ScodMessed, tSizeNL, tSizeSm )) 
		goto error_exit;

	if (!createShiftTable( ShiftTable, &tsSize )) 
		goto error_exit;

	result = TRUE;
	goto success_exit;

error_exit:
	freeTables( VkOriginal, VkMessed, ScodOriginal, ScodMessed, 
		        ShiftTable, *ShiftTableSize );

success_exit:
	*TableSizeNL = tSizeNL;
	*TableSizeSm = tSizeSm;
	*ShiftTableSize = tsSize;
	return result;
}


#define MHK_HOOK_DEVICE_NT    TEXT("\\\\.\\MHK")
#define MHK_HOOK_DEVICE_95    TEXT("\\\\.\\bcrypt.vxd")

void getServiceName( char *ServiceName )
{
	OSVERSIONINFO vi;
	vi.dwOSVersionInfoSize = sizeof( vi );

	GetVersionEx( &vi );

	BOOL isNT = (vi.dwPlatformId  == VER_PLATFORM_WIN32_NT);

	if ( isNT )  strcpy(ServiceName, MHK_HOOK_DEVICE_NT);
	else         strcpy(ServiceName, MHK_HOOK_DEVICE_95);
}



int KBD_SetEncodingTable( 	USHORT *ScodOriginal,
							USHORT *ScodMessed,
							UINT   TableSizeNL,
							UINT   TableSizeSm,
   							BYTE   *ShiftTable,
                            UINT   ShiftTableSize,
							BOOL   UseSecureTyping )
{
   char            kbdService[32];
   ENCODING_BUFFER eBuf;
   BOOL            res = TRUE;
   DWORD           lBytesReturned;
   HANDLE          hDevice;
   UINT            i;
   SHORT           state;

   if (!UseSecureTyping) return TRUE;

   //CharacterOrder = 0;
      
   if ( (TableSizeNL > ENCODING_TABLE_SIZE) || (TableSizeSm > ENCODING_TABLE_SIZE) || 
	    (!ScodOriginal) || (!ScodMessed))
      return FALSE;

   eBuf.FlagSetEncoding = 1;
   eBuf.TableSizeNL = (DWORD)TableSizeNL;
   eBuf.TableSizeSm = (DWORD)TableSizeSm;
   eBuf.ShiftTableSize = (DWORD)ShiftTableSize;
   eBuf.CharacterOrder = CharacterOrder;

   memset( eBuf.OriginalTable, 0, sizeof(USHORT)*ENCODING_TABLE_SIZE*2 );
   memset( eBuf.MessedTable,   0, sizeof(USHORT)*ENCODING_TABLE_SIZE*2 );
   memset( eBuf.ShiftTable,    0, sizeof(BYTE) * SHIFT_TABLE_SIZE );
   memset( eBuf.Info,          0, MHK_INFO_SIZE * sizeof(USHORT));

   eBuf.Info[INFO_NUMLOCK_CODE_BYTE] = MapVirtualKey( VK_NUMLOCK, 0 );
   eBuf.Info[INFO_SHIFT_CODE_BYTE]   = MapVirtualKey( VK_SHIFT, 0 );
   eBuf.Info[INFO_LSHIFT_CODE_BYTE]  = MapVirtualKey( VK_LSHIFT, 0 );
   eBuf.Info[INFO_RSHIFT_CODE_BYTE]  = MapVirtualKey( VK_RSHIFT, 0 );
   eBuf.Info[INFO_CTRL_CODE_BYTE]    = MapVirtualKey( VK_CONTROL, 0 );
   eBuf.Info[INFO_LCTRL_CODE_BYTE]   = MapVirtualKey( VK_LCONTROL, 0 );
   eBuf.Info[INFO_RCTRL_CODE_BYTE]   = MapVirtualKey( VK_RCONTROL, 0 );

   state = GetKeyState( VK_NUMLOCK );
   if ((state & 1) > 0 )
   {
          SET_BIT( eBuf.Info[INFO_STATUS_BYTE], STATUS_NUMLOCK_BIT );
   }

   state = GetKeyState( VK_SHIFT );
   if (((state >> 8) & 0x80) > 0 )
   {
          SET_BIT( eBuf.Info[INFO_STATUS_BYTE], STATUS_SHIFT_BIT );
   }

   state = GetKeyState( VK_CONTROL );
   if (((state >> 8) & 0x80) > 0 )
   {
          SET_BIT( eBuf.Info[INFO_STATUS_BYTE], STATUS_CTRL_BIT );
   }
   
   for( i=0; i<ENCODING_TABLE_SIZE*2; i++)
   { eBuf.OriginalTable[i] = ScodOriginal[i];
     eBuf.MessedTable[i]   = ScodMessed[i];
   }
   
   for( i=0; i<ShiftTableSize; i++)
   { eBuf.ShiftTable[i] = ShiftTable[i];
   }
   
   getServiceName(kbdService);
   
   strcpy( eBuf.Signature, "LOCOS97 ");
   
   hDevice = CreateFile( kbdService,
                         GENERIC_READ | GENERIC_WRITE, 0, NULL,
                         OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL );
   if (hDevice == INVALID_HANDLE_VALUE)
   {
	   assert( false );
	   return FALSE;
   }
  
   res = DeviceIoControl( hDevice, IOCTL_PRIVATE_SET_RESET_ENCODING,
						 &eBuf, sizeof(ENCODING_BUFFER),
						 &eBuf, sizeof(ENCODING_BUFFER),
						 &lBytesReturned, NULL);
   assert( res );

   CloseHandle(hDevice);
   
   if (strncmp(eBuf.Signature, "LS06CXX ", 8) != 0)
   {
	   assert( false );
	   return FALSE;
   }

   if ((eBuf.Status) == STATUS_ERROR_ENCODING_ALREADY_SET )
	   return 2;

   if ((eBuf.Status) != STATUS_KBD_SUCCESS)
   {
	   assert( false );
	   return FALSE;
   }

   return res;
}


BOOL KBD_ResetEncodingTable( BOOL   UseSecureTyping )
{
   char            kbdService[32];
   ENCODING_BUFFER eBuf;
   BOOL            res = TRUE;
   DWORD           lBytesReturned;
   HANDLE          hDevice;

   if (!UseSecureTyping) return TRUE;
   
   memset( &eBuf, 0, sizeof(ENCODING_BUFFER) );
   
   eBuf.FlagSetEncoding = 0;
     
   getServiceName(kbdService);
   
   strcpy( eBuf.Signature, "LOCOS97 ");
   
   hDevice = CreateFile( kbdService,
                         GENERIC_READ | GENERIC_WRITE, 0, NULL,
                         OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL );
   if (hDevice == INVALID_HANDLE_VALUE) return FALSE;
  
   res = DeviceIoControl( hDevice, IOCTL_PRIVATE_SET_RESET_ENCODING,
						 &eBuf, sizeof(ENCODING_BUFFER),
						 &eBuf, sizeof(ENCODING_BUFFER),
						 &lBytesReturned, NULL);
   CloseHandle(hDevice);
   
   if (strncmp(eBuf.Signature, "LS06CXX ", 8) != 0)
	   return FALSE;

   if ((eBuf.Status) != STATUS_KBD_SUCCESS) return FALSE;

   return res;
}


BOOL KBD_FreeTables(UINT   **VkOriginal,
					UINT   **VkMessed,
					USHORT **ScodOriginal,
					USHORT **ScodMessed,
					BYTE   **ShiftTable,
                    UINT   ShiftTableSize,
					BOOL   UseSecureTyping )
{
	if (!UseSecureTyping) return TRUE;

	freeTables( VkOriginal, VkMessed, ScodOriginal, ScodMessed,
		        ShiftTable, ShiftTableSize );

	return TRUE;
}




/***************************************************************

 Translate single character depending on whether the following 
 keys are pressed:
	VK_LSHIFT
	VK_RSHIFT
	VK_LCONTROL
	VK_RCONTROL
	VK_CAPITAL
	VK_NUMLOCK

 ***************************************************************/



BOOL createTable_charEx( UINT *VkTable, char **CharTable, UINT Size )
{
	UINT i;
	char  *table;
  	UINT  scanCode;
	BYTE  keyState[256];
	SHORT state;
	WORD  chars;
	int   charsGot;

	table = *CharTable = new char[ Size ];
	if (!table) return FALSE;

	memset( keyState,  0x00, 256 ); // nothing is pressed or toggled
	
	/* set correct state of Shift, Control and Caps Lock */
	state = GetKeyState( VK_SHIFT );
	keyState[VK_SHIFT] = (((BYTE)state) & 1) | (((BYTE)(state >> 8)) & 0x80);

	state = GetKeyState( VK_CONTROL );
	keyState[VK_CONTROL] = (((BYTE)state) & 1) | (((BYTE)(state >> 8)) & 0x80);

	state = GetKeyState( VK_LSHIFT );
	keyState[VK_LSHIFT] = (((BYTE)state) & 1) | (((BYTE)(state >> 8)) & 0x80);

	state = GetKeyState( VK_RSHIFT );
	keyState[VK_RSHIFT] = (((BYTE)state) & 1) | (((BYTE)(state >> 8)) & 0x80);

	state = GetKeyState( VK_LCONTROL );
	keyState[VK_LCONTROL] = (((BYTE)state) & 1) | (((BYTE)(state >> 8)) & 0x80);

	state = GetKeyState( VK_RCONTROL );
	keyState[VK_RCONTROL] = (((BYTE)state) & 1) | (((BYTE)(state >> 8)) & 0x80);

	state = GetKeyState( VK_CAPITAL );
	keyState[VK_CAPITAL] = (((BYTE)state) & 1) | (((BYTE)(state >> 8)) & 0x80);

	state = GetKeyState( VK_NUMLOCK );
	keyState[VK_NUMLOCK] = (((BYTE)state) & 1) | (((BYTE)(state >> 8)) & 0x80);


	HKL hkl = GetKeyboardLayout( 0 );
	
	/* call ToAsciiEx just to clear keyboard from dead key, if it was pressed */
	scanCode = MapVirtualKey( VkTable[0], 0 ); 
	charsGot = ToAsciiEx( VkTable[0], scanCode, keyState, &chars, 0, hkl );

	for( i=0; i<Size; i++)
	{
		/* Get scan code for the virtual key */
		scanCode = MapVirtualKey( VkTable[i], 0 ); 

		/* Get character for the given virtual key */
		charsGot = ToAsciiEx(  VkTable[i], scanCode, keyState, &chars, 0, hkl );
		if (charsGot != 1) return FALSE;
		table[i] = (char)chars;
	}
	
	return TRUE;
}


/*************************************************************

  ShiftTable is used as 
   a) direct  transformation: N = (X + R) % L 
   b) reverse transformation: X = (N + (L - (R % L))) % L

  where X - initial index, 
        R - random from ShiftTable, 
		L - TableSize - number of messed characters,
		N - new index

  In KBD_TranslateChar() we use reverse transformation

**************************************************************/

BOOL KBD_TranslateChar(		char   *letter,
							UINT   **VkOriginal,
							UINT   **VkMessed,
							USHORT **ScodOriginal,
							USHORT **ScodMessed,
							UINT   TableSizeNL,
							UINT   TableSizeSm,
							BYTE   **ShiftTable,
							UINT   ShiftTableSize,
							BOOL   UseSecureTyping,
							UINT   ScanCode )
{
	UINT   i, index, tabSize;
	char   *charOriginal, *charMessed;
	UINT   *vkOriginal, *vkMessed;
	BOOL   bNumLock;

//	UINT virtKey = MapVirtualKey( ScanCode, 1 ); // 3 - for Windows NT
	
	if (!UseSecureTyping) return TRUE;

	// check if NumLock is pressed
	tabSize = TableSizeSm;
	vkOriginal = &((*VkOriginal)[ENCODING_TABLE_SIZE]);
	vkMessed   = &((*VkMessed)[ENCODING_TABLE_SIZE]);
	bNumLock   = FALSE;

	if ((GetKeyState( VK_NUMLOCK ) & 1) &&
	   (!((GetKeyState( VK_SHIFT ) >> 8) & 0x80)) &&
	   (!((GetKeyState( VK_CONTROL ) >> 8) & 0x80)) )
	{
		tabSize = TableSizeNL;
		vkOriginal = *VkOriginal;
		vkMessed   = *VkMessed;
		bNumLock   = TRUE;
	}

	charOriginal = charMessed = NULL;
	
	if ((!letter) || (!(*VkOriginal)) || 
		(!(*VkMessed)) || (!(*ScodOriginal)) || (!(*ScodMessed)) )
		goto ts_exit;
	
	if (!createTable_charEx( vkOriginal, &charOriginal, tabSize )) 
		goto ts_exit;

	if (!createTable_charEx( vkMessed, &charMessed, tabSize )) 
		goto ts_exit;

	for(i=0; i<tabSize; i++)
	{ 
		if ((*letter) == charMessed[i])
		{
			UINT vkl = vkMessed[i];
			UINT scanCode = MapVirtualKey( vkl, 0 );
			if ((scanCode & 0x1FF) != (ScanCode & 0x1FF)) 
				continue; // the numbers was pressed on num pad

			index = (i + (tabSize - ( ((*ShiftTable)[CharacterOrder%ShiftTableSize]) % tabSize))) % tabSize;
			*letter = charOriginal[index];

			// Commented to fix the problem with Windows Commander.
			// See also keyboard hook in mhk.sys (Windows NT) and bcrypt.vxd (Windows 95/98)
			// CharacterOrder++;
			break;
		}
	}

ts_exit:
	if (charOriginal)
	{	ShredData( (BYTE *)(charOriginal), tabSize*sizeof(char) );
		delete charOriginal;
	}
	if (charMessed)
	{	ShredData( (BYTE *)(charMessed), tabSize*sizeof(char) );
		delete charMessed;
	}
	return TRUE;
}



void KBD_IncrementCharacterOrder(int	VirtualKey, 
								 UINT	*VkMessed,
								 UINT	TableSizeNL,
								 UINT	TableSizeSm,
								 BOOL	UseSecureTyping )
{
	UINT i;
	UINT *vkMessed, tabSize;
	
	if (!UseSecureTyping) return;

	// check if NumLock is pressed
	tabSize = TableSizeSm;
	vkMessed   = &(VkMessed[ENCODING_TABLE_SIZE]);

	if ((GetKeyState( VK_NUMLOCK ) & 1) &&
	   (!((GetKeyState( VK_SHIFT ) >> 8) & 0x80)) &&
	   (!((GetKeyState( VK_CONTROL ) >> 8) & 0x80)) )
	{
		tabSize  = TableSizeNL;
		vkMessed = VkMessed;
	}

	for(i=0; i<tabSize; i++)
	{ 
		if (VirtualKey == (int)(vkMessed[i]))
		{
			CharacterOrder++;
			break;
		}
	}
}

void KBD_IncrementCharacterOrderCNTRL(	int		VirtualKey, 
										UINT	*VkMessed,
										UINT	TableSizeNL,
										UINT	TableSizeSm,
										BOOL	UseSecureTyping )
{
    // Commented to fix the problem with Windows Commander.
	// See also keyboard hook in mhk.sys (Windows NT) and bcrypt.vxd (Windows 95/98)

	return;

/*
	UINT i;

	if (!UseSecureTyping) return;


	SHORT state;
	BYTE  pressed;

	/ * set correct state of Shift, Control and Caps Lock * /
	state   = GetKeyState( VK_CONTROL );
	pressed = ((BYTE)(state >> 8)) & 0x80;

	if (pressed == 0) return;

	for(i=0; i<TableSize; i++)
	{ 
		if (VirtualKey == (int)(VkMessed[i]))
		{
			CharacterOrder++;
			break;
		}
	}
*/
}
