/*******************************************************************
 *	Copyright (c) 1994-2004 Jetico, Inc., Finland
 *	All rights reserved.
 *
 *	File: kbd.h
 *	Revision: 
 *	Created:
 *	Description: interface with the BestCrypt keyboard driver
 *		09.12.2004 - Semaphores added for Set/Reset 
 *		encoding table procedure.( See description at
 *		kbdParams::hSemaphore
 *
 *******************************************************************/


#ifndef __KBD
#define __KBD

#include <assert.h>
#include <stdio.h>
#include <time.h>

extern BOOL KBD_GenerateEncodingTable(
										UINT   **VkOriginal,
										UINT   **VkMessed,
										USHORT **ScodOriginal,
										USHORT **ScodMessed,
										UINT   *TableSizeNL,
										UINT   *TableSizeSm,
										BYTE   **ShiftTable,
										UINT   *ShiftTableSize,
										BOOL   UseSecureTyping);


//
//	KBD_SetEncodingTable
//		returns:
//			1 if success
//			0 if error
//			2 if Prev table was not reset yet.
//
extern int KBD_SetEncodingTable( 	USHORT *ScodOriginal,
									USHORT *ScodMessed,
								    UINT   TableSizeNL,
								    UINT   TableSizeSm,
   									BYTE   *ShiftTable,
									UINT    ShiftTableSize,
									BOOL   UseSecureTyping);

extern BOOL KBD_ResetEncodingTable( BOOL   UseSecureTyping );


extern BOOL KBD_FreeTables(	UINT   **VkOriginal,
							UINT   **VkMessed,
							USHORT **ScodOriginal,
							USHORT **ScodMessed,
							BYTE   **ShiftTable,
							UINT   ShiftTableSize,
							BOOL   UseSecureTyping );


extern BOOL KBD_TranslateChar(
							char   *letter,
							UINT   **VkOriginal,
							UINT   **VkMessed,
							USHORT **ScodOriginal,
							USHORT **ScodMessed,
						    UINT   TableSizeNL,
						    UINT   TableSizeSm,
							BYTE   **ShiftTable,
							UINT   ShiftTableSize,
							BOOL   UseSecureTyping,
							UINT   ScanCode );

extern void KBD_IncrementCharacterOrder( int	VirtualKey, 
										 UINT	*VkMessed,
										 UINT   TableSizeNL,
										 UINT   TableSizeSm,
										 BOOL	UseSecureTyping );

extern void KBD_IncrementCharacterOrderCNTRL(	int		VirtualKey, 
												UINT	*VkMessed,
												UINT	TableSizeNL,
												UINT	TableSizeSm,
												BOOL	UseSecureTyping );

// #define _DEBUG_PRINT

#ifndef _DEBUG_PRINT
#define DbgPrint( a, b ) {};
#endif

struct kbdParams
{
	UINT   *VkOriginal,   *VkMessed;
	USHORT *ScodOriginal, *ScodMessed;
	BYTE   *ShiftTable;
	UINT   TableSizeNL, TableSizeSm, ShiftTableSize;

	//
	//	The semaphore object has been added by one reason:
	//	On any machines the WM_KILLOCUS message sends later then WM_SETFOCUS.
	//	As result in UseSecureTyping mode the encoding tables in kbd driver 
	//	may be changed not correctly. 
	//	One edit box may try to set tables when other edit box does not reset 
	//	it's own tables.
	//
	HANDLE hSemaphore;
	bool exist;
	
	static const char *kbdParams::szSemaphoreName;

	kbdParams()
	{
		Init();
//		DbgPrint( "kbdParams::New" );
	}

	static kbdParams *Allocate()
	{
		HGLOBAL hgl = GlobalAlloc( GMEM_FIXED, sizeof( kbdParams ) );

		if ( hgl == NULL )
		{
			assert( false );
			return false;
		}

		kbdParams *kbd = (kbdParams *)hgl;

		kbd->Init();		

		return kbd;
	}

	void Init()
	{
		memset( this, 0, sizeof( *this ) );
	}

	static void Free( kbdParams *hgl )
	{
		GlobalFree( hgl );
	}

	static kbdParams *GetParams( HWND hWnd )
	{
		kbdParams *kbd = (kbdParams *)GetWindowLong( hWnd, DWL_USER );
		if ( kbd == NULL )	{	assert( false ); return FALSE; }

//		char msg[ 1024 ];
//		sprintf( msg, "GetParams( %X ) - %X", hWnd, kbd );
//		kbd->DbgPrint( msg );

		return kbd;
	}

	BOOL GenerateEncodingTable ( BOOL UseSecureTyping )
	{
		if ( VkMessed != 0 )
		{
			DbgPrint( "GenerateEncodingTable VkMessed is not NULL", (int)VkMessed );

			assert( false );
		}

		BOOL res = KBD_GenerateEncodingTable(
				&VkOriginal, &VkMessed,
				&ScodOriginal, &ScodMessed,	
				&TableSizeNL, &TableSizeSm,
				&ShiftTable, &ShiftTableSize, 
				UseSecureTyping );

//		DbgPrint( "GenerateEncodingTable", res );

		hSemaphore = CreateSemaphore( NULL, 0, 100, szSemaphoreName );

		assert( hSemaphore != NULL );
		res = GetLastError();

		exist = ( res == ERROR_ALREADY_EXISTS );

		LONG prev = 0;
		if ( !exist )
		{	ReleaseSemaphore( hSemaphore, 1, &prev );
			DbgPrint( "CreateSemaphore - ReleaseSemaphore", prev );
		}

		DbgPrint( "CreateSemaphore", exist );
		return res;
	}

	BOOL ResetEncodingTable( BOOL UseSecureTyping )
	{
		BOOL res = KBD_ResetEncodingTable( UseSecureTyping );

		LONG prev = 0;
		DbgPrint( "ResetEncodingTable ReleaseSemaphore start", res );
		ReleaseSemaphore( hSemaphore, 1, &prev );
		DbgPrint( "ResetEncodingTable ReleaseSemaphore", (int)prev );
		return res;
	}

	BOOL SetEncodingTable( BOOL UseSecureTyping )
	{
		DWORD dwRes;

		// Tries to get semaphore state
/*
		LONG prev = 0;
		ReleaseSemaphore( hSemaphore, 1, &prev );
		dwRes = WaitForSingleObject( hSemaphore, 5000 );
		if ( dwRes != WAIT_OBJECT_0 )
		{
			DbgPrint( "First SetEncodingTable WaitForSingleObject Error", dwRes );
			assert( false );
			return false;
		}
*/
		DbgPrint( "SetEncodingTable WaitForSingleObject Start", (int)prev );
		dwRes = WaitForSingleObject( hSemaphore, 5000 );

		if ( dwRes != WAIT_OBJECT_0 )
		{
			DbgPrint( "SetEncodingTable WaitForSingleObject Error", dwRes );
			assert( false );
			return false;
		}

		int res;
		res = KBD_SetEncodingTable( ScodOriginal, ScodMessed, TableSizeNL, TableSizeSm, ShiftTable, ShiftTableSize, UseSecureTyping );
		DbgPrint( "SetEncodingTable", res );

		assert( res == 1 );

		//
		//	The res == 2 means that STATUS_ERROR_ENCODING_ALREADY_SET table is currently set
		//
		//	Note, the code does nopt work for usual situation - semaphore ( see description at hSemaphore )
		//	properly waits permition for new table setting. But if a password dialog has been acsidentally
		//	terminated the following code allows to reset old invalid table and assing the new one.
		//
		if ( res == 2 )
		{
			KBD_ResetEncodingTable( UseSecureTyping );
			res = KBD_SetEncodingTable( ScodOriginal, ScodMessed, TableSizeNL, TableSizeSm, ShiftTable, ShiftTableSize, UseSecureTyping );
			DbgPrint( "SetEncodingTable - second try", res );
		}

		return res == 1;
	}

	BOOL FreeTables	( BOOL UseSecureTyping )
	{
		if ( hSemaphore != NULL )
		{
			CloseHandle( hSemaphore );
			hSemaphore = NULL;
		}

		DbgPrint( "FreeTables", 0 );
		BOOL res = KBD_FreeTables( &VkOriginal, &VkMessed, &ScodOriginal, &ScodMessed, &ShiftTable, ShiftTableSize, UseSecureTyping );
		return res;
	}

	BOOL TranslateChar(
				char		*pLetter,
				BOOL		UseSecureTyping,
				UINT		ScanCode
		)
	{
		if ( UseSecureTyping && ( VkMessed == NULL ) )
		{
			assert( false );
			return FALSE;
		}

		BOOL res = KBD_TranslateChar(	pLetter,
									&VkOriginal, &VkMessed,
									&ScodOriginal, &ScodMessed, TableSizeNL, TableSizeSm,
									&ShiftTable, ShiftTableSize, 
									UseSecureTyping, ScanCode );

		DbgPrint( "TranslateChar", (int)*pLetter );

		return res;
	}

	void IncrementCharacterOrder(
				int			VirtualKey, 
				BOOL		UseSecureTyping
		)
	{
		if ( UseSecureTyping && ( VkMessed == NULL ) )
		{
			assert( false );
			return;
		}

		KBD_IncrementCharacterOrder( VirtualKey, VkMessed, TableSizeNL, TableSizeSm, UseSecureTyping );
	}

	void IncrementCharacterOrderCNTRL(
				int			VirtualKey,
				BOOL		UseSecureTyping
		)
	{
		if ( UseSecureTyping && ( VkMessed == NULL ) )
		{
			assert( false );
			return;
		}
		KBD_IncrementCharacterOrderCNTRL( VirtualKey, VkMessed, TableSizeNL, TableSizeSm, UseSecureTyping );
	}

#ifdef _DEBUG_PRINT
	void DbgPrint( const char *msg, int res = 0 )
	{
		char str[ 1024 ];
		sprintf( str, "\n%08d : %08X - %s, res %d", GetTickCount(), DWORD(VkMessed ? VkMessed[0]: 0 ), msg, res ); 

		HANDLE hFile = CreateFile( "C:\\dbg.txt", GENERIC_READ | GENERIC_WRITE, FILE_SHARE_READ, NULL, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL );
		if ( hFile == INVALID_HANDLE_VALUE )
		{
			MessageBox( 0, "Error in opening file", "DbgPrint", MB_ICONSTOP );
			HANDLE hFile = CreateFile( "C:\\dbg.txt", GENERIC_READ | GENERIC_WRITE, FILE_SHARE_READ, NULL, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL );
			if ( hFile == INVALID_HANDLE_VALUE )
				return;
		}

		DWORD pos = SetFilePointer( hFile, 0, 0, FILE_END );

		DWORD dwWritten = 0;
		WriteFile( hFile, (BYTE *)str, strlen( str ), &dwWritten, NULL );

		CloseHandle( hFile );
	}
#endif
};


#endif /* __KBD */
