//======================================================================
// 
//  mhk.h
//
//  This module contains the PRIVATE (driver-only) definitions for the
//  code that implements the mono device driver.
//
//======================================================================

#include <windef.h> 

#define STATUS_KBD_SUCCESS 0
#define STATUS_ERROR_ENCODING_ALREADY_SET 1
#define STATUS_ERROR_TABLE_TOO_BIG 2

#define ENCODING_TABLE_SIZE 64
#define SHIFT_TABLE_SIZE    244

// Define the various device type values.  Note that values used by Microsoft
// Corporation are in the range 0-32767, and 32768-65535 are reserved for use
// by customers.

#define FILE_DEVICE_DISK   0x00000007
#define IOCTL_DISK_BASE    FILE_DEVICE_DISK
#define METHOD_BUFFERED    0
#define METHOD_IN_DIRECT   1
#define FILE_ANY_ACCESS    0

#define CTL_CODE( DeviceType, Function, Method, Access ) (                 \
    ((DeviceType) << 16) | ((Access) << 14) | ((Function) << 2) | (Method) \
)

#define FILE_DEVICE_MHK  0x00008340


#define IOCTL_PRIVATE_GET_AND_RESET_FLAG \
              CTL_CODE(IOCTL_DISK_BASE, 0x855, METHOD_IN_DIRECT, FILE_ANY_ACCESS)


typedef struct  _FLAG_BUFFER { char  Signature[8];	// input: "LOCOS97",00 ; output: "LS06CXX",00
                               UCHAR Status;
                               DWORD Flag;
                             } FLAG_BUFFER, *PFLAG_BUFFER;

typedef struct _BUFFER_HEADER { char   Signature[8];	// input: "LOCOS97",00 ; output: "LS06CXX",00
                                UCHAR   Status;
                              } BUFFER_HEADER, *PBUFFER_HEADER;

#define IOCTL_PRIVATE_SET_RESET_ENCODING \
              CTL_CODE(IOCTL_DISK_BASE, 0x856, METHOD_IN_DIRECT, FILE_ANY_ACCESS)

#define IOCTL_PRIVATE_MHK_SET_ALARM_CRASH_HOTKEY \
              CTL_CODE(IOCTL_DISK_BASE, 0x857, METHOD_IN_DIRECT, FILE_ANY_ACCESS)

#define IOCTL_PRIVATE_MHK_GET_ALARM_CRASH_HOTKEY \
              CTL_CODE(IOCTL_DISK_BASE, 0x858, METHOD_IN_DIRECT, FILE_ANY_ACCESS)

typedef struct  _AC_HOTKEY_BUFFER { char  Signature[8]; // input: "LOCOS97",00 ; output: "LS06CXX",00
                                    UCHAR Status;
                                    DWORD ScanCode;
                                    BOOL  AltUsed;
                                    BOOL  ControlUsed;
                                    DWORD Reserved_0;
                                    DWORD Reserved_1;
                                    DWORD Reserved_2;
                                    DWORD Reserved_3;
                             } AC_HOTKEY_BUFFER, *PAC_HOTKEY_BUFFER;

#define MHK_INFO_SIZE 16

#define INFO_STATUS_BYTE       0
#define INFO_SHIFT_CODE_BYTE   1 
#define INFO_LSHIFT_CODE_BYTE  2
#define INFO_RSHIFT_CODE_BYTE  3
#define INFO_CTRL_CODE_BYTE    4
#define INFO_LCTRL_CODE_BYTE   5
#define INFO_RCTRL_CODE_BYTE   6
#define INFO_NUMLOCK_CODE_BYTE 7

#define STATUS_SHIFT_BIT   0
#define STATUS_CTRL_BIT    1
#define STATUS_NUMLOCK_BIT 2

#define BIT_IS_SET(ushort, position) ((ushort >> position) & 1)
#define SET_BIT(ushort, position)    (ushort |= (1 << position) )
#define RESET_BIT(ushort, position)  (ushort &= 0xFFFF ^ (1 << position))

typedef struct  _ENCODING_BUFFER { char   Signature[8];	// input: "LOCOS97",00 ; output: "LS06CXX",00
                                   UCHAR  Status;
                                   UCHAR  Dummy[3];
                                   DWORD  FlagSetEncoding;
                                   DWORD  TableSizeNL;
                                   DWORD  TableSizeSm;
                                   USHORT OriginalTable[ ENCODING_TABLE_SIZE*2 ];
                                   USHORT MessedTable[ ENCODING_TABLE_SIZE*2 ];
                                   DWORD  ShiftTableSize;
                                   BYTE   ShiftTable[ SHIFT_TABLE_SIZE ];
								   DWORD  CharacterOrder;
								   USHORT Info[ MHK_INFO_SIZE ];
                                 } ENCODING_BUFFER, *PENCODING_BUFFER;

