/*******************************************************************\
*
*	Copyright (c) 2004-2009 Jetico, Inc., Finland
*	All rights reserved.
*
*	File: 		cParamsPBE.cpp, PBE_proc.cpp
*	Revision: 	28.05.2009, renamed, refactored into key block type
*	Created:	21.10.2004
*	Description:	Password based encryption parameters for key block parsing
*
\*******************************************************************/

#include "PBE_block.h"
#include "alg_utils.h"
#include "enc_block.h"
#include "datablock.h"

#define PWD_HASH_ITERATIONS	0x100

using namespace bdk;
using namespace kgghost;


bool CPBEProcessor::decode(const bc_byte_t* kbEncoded, unsigned int & kbSize, key_info_t& key, cPassword const & init_pwd)
{
	BC_DEBUG_PRINT(("Debug: v8: Decoding PBE block size %d\n", kbSize));
	// check src sizes
	if (kbSize < sizeof(PBE_EncodedKey))
		return false;

	// decrypt PBE keyblock into key
	if( !this->decrypt_key(key, (PBE_EncodedKey *)kbEncoded, init_pwd ) )
		return false;

	

	// update descriptor
	kbSize = sizeof(PBE_EncodedKey);
	return true;
}


bool CPBEProcessor::decrypt_key( key_info_t & key_out, PBE_EncodedKey* kb, cPassword const & pwd )
{
	// configure algorithm
	int rc;
	BCAlgHandle alg;

	rc = alg_init_from_password(alg, key_out.alg_id, key_out.mode_id, NULL /* zero pool */, &pwd, kb->salt, sizeof(kb->salt), key_out.hash_id, PWD_HASH_ITERATIONS);
	if( rc != ERROR_NO ) return false;

	return kb->key_info.decrypt_to(&key_out, alg, key_out.hash_id); // decrypt + verify digest
}


bool CPBEProcessor::encode(bc_byte_t* kbPlain, unsigned int & kbSize, key_info_t const & key, cPassword const & init_pwd)
{
	BC_DEBUG_PRINT(("Debug: v8: Encoding PBE block size %d\n", kbSize));

	// check src sizes
	if(kbSize < sizeof(PBE_EncodedKey))
		return false;

	// encrypt key into PBE keyblock
	this->encrypt_key(key, (PBE_EncodedKey*)kbPlain, init_pwd);
	
	// update descriptor
	kbSize = sizeof(PBE_EncodedKey);
	return true;
}


bool CPBEProcessor::encrypt_key( key_info_t const & key_in, PBE_EncodedKey* kb_out, cPassword const & pwd )
{
	// save password generation salt
	ShredData((bc_byte_t*)kb_out, sizeof(PBE_EncodedKey));

	// configure algorithm
	int rc;
	BCAlgHandle alg;
	
	rc = alg_init_from_password(alg, key_in.alg_id, key_in.mode_id, NULL /* zero pool */, &pwd, kb_out->salt, sizeof(kb_out->salt), key_in.hash_id, PWD_HASH_ITERATIONS);
	if( rc != ERROR_NO ) 
		return false;

	// encrypt keyblock's encrypted block < key_t > with it's own random iv
	kb_out->key_info.encrypt_from(&key_in, alg, key_in.hash_id, NULL);
	return true;
}



