#ifndef V8_PBE_PROCESSOR_H
#define V8_PBE_PROCESSOR_H

#include "keyblock.h"
#include "enc_block.h"

#include "CertificateLib/SMemory/cPassword.h"

namespace kgghost
{

#if 0
	/**
	 * @brief	Processor for Password Based Encryption key blocks
	 */
	class CKeyBlock_PBE : public IKeyBlock
	{
	public:


		/**
		 * @brief	Construct PBE keyblock
		 *
		 */
		CKeyBlock_PBE();


		/**
		 * @brief	Verify password against this keyblock
		 */
		BC_INHERITED	bool verify(const cPassword* pwd);


		/**
		 * @brief	Decode PBE key block into encryption key
		 */
		BC_INHERITED 	bool decode(key_info_t & key, const cPassword * init_pwd, keyblock_descriptor_t& kbdesc);


		/**
		 * @brief	Encode encryption key into PBE key block
		 */
		BC_INHERITED 	bool encode(key_info_t const & key, const cPassword * init_pwd, keyblock_descriptor_t& kbdesc);


	protected:


		/**
		 * @brief 	Password Based Encryption encoded block
		 */
		struct PBE_EncodedKey
		{
			/* salt buffer, randomly initialized when encoding */
			char salt[ SALT_SIZE ];

			/* encrypted block of key info structure */
			CEncBlock< key_info_t > key_info;
		};


		/**
		 * @brief	Encrypts key with the password
		 *
	 	 * @p kb_out	PBE key block buffer result
		 * @p key_in	Key buffer input to encrypt with password
		 */
		bool encrypt_key( key_info_t const & key_in, PBE_EncodedKey* kb_out, const cPassword* pwd );


		/**
		 * @brief	Decrypts key block with the password
		 *
 		 * @p kb_in	PBE key block input
		 * @p key_out	Key buffer result decrypted from PBE key block
		 */
		bool decrypt_key( key_info_t & key_out, PBE_EncodedKey* kb_in, const cPassword* pwd );

	};
#endif

	class CPBEProcessor : public IKeyBlockProcessor
	{
	public:


		/**
		 * @brief	Decode PBE key block into encryption key
		 */
		BC_INHERITED	bool decode(const bc_byte_t* kbEncrypted, unsigned int & kbSize, key_info_t& key, cPassword const & init_pwd);


		/**
		 * @brief	Encode encryption key into PBE key block
		 */
		BC_INHERITED 	bool encode(bc_byte_t* kbEncrypted, unsigned int & kbSize, key_info_t const & key, cPassword const & init_pwd);


	protected:


		/**
		 * @brief 	Password Based Encryption encoded block
		 */
		struct PBE_EncodedKey
		{
			/* salt buffer, randomly initialized when encoding */
			char salt[ SALT_SIZE ];

			/* encrypted block of key info structure */
			CEncBlock< key_info_t > key_info;
		};


		/**
		 * @brief	Encrypts key with the password
		 *
	 	 * @p kb_out	PBE key block buffer result
		 * @p key_in	Key buffer input to encrypt with password
		 */
		bool encrypt_key( key_info_t const & key_in, PBE_EncodedKey* kb_out, cPassword const & pwd );


		/**
		 * @brief	Decrypts key block with the password
		 *
 		 * @p kb_in	PBE key block input
		 * @p key_out	Key buffer result decrypted from PBE key block
		 */
		bool decrypt_key( key_info_t & key_out, PBE_EncodedKey* kb_in, cPassword const & pwd );

	};


} // namespace kgghost

#endif
