/*******************************************************************\
*
*	Copyright (c) 2005-2008 Jetico, Inc., Finland
*	All rights reserved.
*
*	File: pwdlg.cpp
*	Revision: 
*	Created:
*	Description:
*		implementation of procedures that support 
*		dialog boxes for accepting user's advanced 
*		key block parameters for
*			1. Password based encryption	- PBE,
*			2. Public key encryption		- PKE,
*			3. Shamir secret shared scheme	- SSS 
*
\*******************************************************************/


#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>
#include <string.h>

#include <stdio.h>

#include "UnicodeFuncs.h"
//#include "kg_defs.h"
#include "..\..\comm_def\dllMain.h"
#include "pwdlg.h"

#include "cDlgKBParams.h"
#include "resource.h"
#include "diskDriver.h"

static bool convertTextToFilenamePath( wchar_t *fileName, wchar_t *pathName, const wchar_t *text )
{
	ODSFuncS( text );

	fileName[0] = L'\0';
	pathName[0] = L'\0';

	if ( FileExistsT( text ) )
	{
		if ( splitPath( text, pathName, fileName ) != 0 )
			return true;
	}

	// input Text has a form: "Container:  D:\path\container_name.jbc"
	// we return: ContainerName = "container_name.jbc", 
	//            ContainerPath = "D:\path\"

	int  i,j;
	const wchar_t *pathPtr = NULL, *namePtr = NULL;

	if (!text) return FALSE;

	for( i = (int)wcslen( text );  ; i-- )
	{
		if ( i <= 0 )
			return false;

		if ((text[i] == L' ') && ( text[i-1] == L':'))
		{
			pathPtr = &(text[i]);
			j = i;
			while (text[j++] == L' ') pathPtr = &(text[j]);
			break;
		}
	}

	if ( !pathPtr )
		pathPtr = text;

	return ( splitPath( pathPtr, pathName, fileName ) != 0 );
}

BOOL GetPasswordFromCommandLine(
			wchar_t *Password,	// buffer size is 256 symbols
			DWORD   *pFlags,	// = NULL
			wchar_t  *mountPoint// == NULL
		)
{
	///// Try to get password from a command-line: BestCrypt.exe Open G:\30.jbc X: -p:=password=-p
	wchar_t pwd[ 256 ];
	int i, j, k, cmdLineLength;
	BOOL bPwdFound = FALSE;

	memset( pwd, 0, sizeof( pwd ) );

	wchar_t *cmdLine;
	cPlainTextBlock ptbCmdLine;
	
	DWORD dwVersion = GetVersion();
	bool isNT = dwVersion < 0x80000000;

	if ( isNT )
		cmdLine = GetCommandLineW();
	else
	{
		char *cmdLineA = GetCommandLine();
		cmdLineLength = (int)strlen( cmdLineA );

		cmdLine = (wchar_t *)ptbCmdLine.Allocate( ( cmdLineLength + 1 ) * sizeof( wchar_t ) );
		if ( cmdLine == NULL )
			return false;

		mbstowcs( cmdLine, cmdLineA, ( cmdLineLength + 1 ) );
	}
	
	cmdLineLength = (int)wcslen(cmdLine);

	for ( i=0; i < cmdLineLength; i++ )
	{
//		if ((cmdLine[i] == L'-') && (cmdLine[i+1] == L'p') && (cmdLine[i+2] == L':') && (cmdLine[i+3] == L'='))
		if ( wcsncmp( cmdLine + i, L"-p:=", 4 ) == 0 )
		{
			for ( j=i+4, k=0; j<cmdLineLength; j++, k++)
			{
//				if ((cmdLine[j] == '=') && (cmdLine[j+1] == '-') && (cmdLine[j+2] == 'p')) 
				if ( wcsncmp( cmdLine + j, L"=-p", 3 ) == 0 )
				{
					bPwdFound = TRUE;
					break;
				}

				pwd[k] = cmdLine[j];

				if (bPwdFound) break;
			}
		}
	}

	if ( bPwdFound )
		wcsncpy( Password, pwd, maxLen( pwd ) );
	else
		return FALSE;

	///// End of "Try to get password from a command-line"

	if ( mountPoint == NULL )
		return true;

	//// Get Logical drive letter

	DWORD curLogicalDisks = 0, firstAvailableDrive = 0;

	if ( ( pFlags != NULL ) &&
		 ( ( *pFlags & FLAG_MOUNT_ON_SINGLE_LETTER ) != 0 )
		 )
	{
		// leave the drive letter as it was set
		return TRUE;
	}

	// Ckecks if the mount point is not mounted.
	DISKINFO_BUFFER_V8_0 dBuf;
	if ( !GetVDiskInfo( &dBuf, mountPoint, true ) )
		return true;

	// Otherwise it gets first available logical drive as old versions did

	curLogicalDisks = _GetLogicalDrives();

	//if (((*FavoriteDrive) > 0) && ((*FavoriteDrive) < 27))
	//{
	//	if ((curLogicalDisks & (1 << (*FavoriteDrive))) == 0)
	//	{
	//		return TRUE; // FavoriteDrive has a reasonable setting and is not busied by other devices
	//	}
	//}

	for (i=2; i<26; i++ )
	{
		if ((curLogicalDisks & (1 << i)) == 0)
		{
			firstAvailableDrive = i;
			break;
		}
	}

	if (!firstAvailableDrive) return FALSE;

	wcscpy( mountPoint, L"A:\\" );
	mountPoint[0] = L'A' + (int)firstAvailableDrive;

	return TRUE;
}

DWORD GetKeyBlockParamsWithVerification_Advanced(
		HWND   hWndParent,
		const wchar_t *text,	// Text, containing filename of the container
		const wchar_t *caption,	// Caption of the "get password" dialog window
		cKeyBlockParams *kbParams,
		DWORD  *pFlags,
		DWORD maxFreeSize,
		bool simple // = false
	)
{
	ODSFuncS( text );

	wchar_t fileName[ MAX_PATH ] = L"";
	wchar_t location[ MAX_PATH ] = L"";

	convertTextToFilenamePath( fileName, location, text );

	cDlgKBParams dlg( kbParams, caption, fileName, location  );

	dlg.m_useSecureTyping	= ((*pFlags & FLAG_ADVANCED_SECURE_TYPING) > 0);
	dlg.m_keyGenFlags		= *pFlags;

	INT_PTR res = dlg.DoModal( hMainInst, simple ? IDD_PasswdDlgV : IDD_KBParamsAdvanced, hWndParent );

//	DWORD res = dlg.DoModal( hMainInst, IDD_KBParamsAdvanced, hWndParent );
//	DWORD res = dlg.DoModal( hMainInst, IDD_KBParamsPKE, hWndParent );
//	DWORD res = dlg.DoModal( hMainInst, IDD_KBParamsPBE, hWndParent );

	*pFlags = dlg.m_keyGenFlags;

	if (res == -1) // system error
	{
		assert( false );
		return ERROR_INTERNAL_PROBLEM;
	}

	return (DWORD)res;
}

DWORD GetKeyBlockParamsWithVerification(
		HWND hWnd,
		const wchar_t	*text,
		const wchar_t	*caption,
		cKeyBlockParams *kbParams,
		DWORD *pFlags,
		DWORD maxFreeSize // = -1, -1 means max available
	)
{
	ODSFuncS( text );

	if ( maxFreeSize == -1 )
		maxFreeSize = DB_maxKeyQ; // sizeof ( DATA_BLOCK::keyMap );

	kbParams->kbType = kbtPBE;

//	kbParams->kbSSS.Init( maxFreeSize, kbParams->algInfo.m_algId, kbParams->algInfo.m_encModeId, kbParams->algInfo., 2);
	kbParams->kbSSS.Init( maxFreeSize, kbParams->kbPBE.m_algId, kbParams->kbPBE.m_encModeId, kbParams->kbPBE.m_hashAlgId );

	cPassword tmpPwd;
	if ( GetPasswordFromCommandLine( tmpPwd.wcBuffer, pFlags ) )
	{
		kbParams->kbPBE.m_pwd.ReadFromBuffer( tmpPwd.wcBuffer );
		return ERROR_NO;
	}

//	DWORD dwRes = GetPasswordWithVerification( hWnd, text, caption, kbParams->kbPBE.m_pwd.buffer, pFlags );
	DWORD dwRes = GetKeyBlockParamsWithVerification_Advanced( hWnd, text, caption, kbParams, pFlags, maxFreeSize, true );

	if ( dwRes == ERROR_AdvancedDlgRequired )
	{
		dwRes = GetKeyBlockParamsWithVerification_Advanced( hWnd, text, caption, kbParams, pFlags, maxFreeSize, false );
	}

	return dwRes;
}

DWORD GetPassword( 
		HWND   hWnd,
		const wchar_t *text,
		const wchar_t *caption,
		cPassword *pwd,
		DWORD  *pFlags,		// = NULL,
//		bool *pIsEncrypted, // = NULL,
		wchar_t *mountPoint// = NULL
	)
{
	cPassword tmpPwd;
	if ( GetPasswordFromCommandLine( tmpPwd.wcBuffer, pFlags, mountPoint ))
	{
		pwd->ReadFromBuffer( tmpPwd.wcBuffer );
		return ERROR_NO;
	}

	wchar_t fileName[ MAX_PATH ] = L"";
	wchar_t location[ MAX_PATH ] = L"";

	bool converted = convertTextToFilenamePath( fileName, location, text );

	cDlgGetPasswordAndMount dlg( caption, converted ? fileName : text, converted ? location : L"" );

	if ( pFlags != NULL )
	{
		dlg.m_useSecureTyping	= ((*pFlags & FLAG_ADVANCED_SECURE_TYPING) > 0);
		dlg.m_keyGenFlags		= *pFlags;
		dlg.m_bTravellerMode	= ((*pFlags & FLAG_TRAVELER_MODE ) != 0 );
	}

//	if ( pIsEncrypted != NULL )
//		dlg.m_headerIsEncrypted	= *pIsEncrypted;

	dlg.m_mountPoint		= mountPoint;

	INT_PTR res;

	int dlgID = IDD_PasswdDlgSimple;
	
	if ( dlg.m_mountPoint != NULL )
		dlgID = IDD_PasswdDlg;
	else
	if ( converted ) // The text has no valid path and container location
		dlgID = IDD_PasswdDlgSimple;
	else
		dlgID = IDD_PasswdDlgText; // The dialog has text only

	res = dlg.DoModal( hMainInst, dlgID, hWnd );

	if ( pFlags != NULL )
		*pFlags = dlg.m_keyGenFlags;

//	if ( pIsEncrypted != NULL )
//		*pIsEncrypted = dlg.m_headerIsEncrypted;

	if (res == -1) // system error
	{
		assert( false );
		return ERROR_INTERNAL_PROBLEM;
	}

	pwd->ReadFromBuffer( dlg.m_pwd.wcBuffer, CP_ACP );

	return (DWORD)res;
}

DWORD GetKeyBlockParams(
		HWND hWnd,
		const wchar_t	*text,
		const wchar_t	*caption,
		cKeyBlockParams *kbParams,
		DWORD *pFlags,
//		bool *pIsEncrypted,
		wchar_t *mountPoint
	)
{
//	kbParams->kbType = kbtPBE;
	return GetPassword( hWnd, text, caption, &( kbParams->kbPBE.m_pwd ), pFlags, //pIsEncrypted, 
				mountPoint );
}

DWORD GetAdvancedFlags( DWORD *pFlags, HWND  hWndParent )
{
	//int result;

	//KeyGenFlags  = *pFlags;
	//UseSecureTyping = TRUE;

	//result = DialogBox( hMainInst, MAKEINTRESOURCE(IDD_SetFlags), 
	//					hWndParent, (DLGPROC)getAdvancedFlagsDlgProc );

	//if (result == -1) // system error
	//	return ERROR_INTERNAL_PROBLEM;

	//*pFlags = KeyGenFlags;
	//
	//return (DWORD)result;

	cDlgGetAdvancedFlags dlg;

	dlg.m_useSecureTyping	= TRUE;
	dlg.m_keyGenFlags		= *pFlags;

	INT_PTR res = dlg.DoModal( hMainInst, IDD_SetFlags, hWndParent );

	if (res == -1) // system error
	{
		assert( false );
		return ERROR_INTERNAL_PROBLEM;
	}

	*pFlags = dlg.m_keyGenFlags;

	return (DWORD)res;
}