#ifndef V8_SSS_PROCESSOR_H
#define V8_SSS_PROCESSOR_H

#include <list>

#include <common/class_defs.h>
#include <alg/alg.h>

#include "keyblock.h"


class cPassword;
class SSShares;


namespace kgghost
{

	struct sss_user_t;


	/**
	 * @brief	Shamir Secret Share key block processor
	 */
	class CSSSProcessor : public IKeyBlockProcessor
	{
	public:


		/**
		 * @brief	Initialize clean runtime state
		 */
		CSSSProcessor();


		/**
		 * @brief	Free all runtime state
		 */
		BC_INHERITED	~CSSSProcessor();


		/**
		 * @brief	Decode SSS key block into encryption key
		 */
		BC_INHERITED	bool decode(const bc_byte_t* kbEncrypted, unsigned int & kbSize, key_info_t& key, cPassword const & init_pwd);


		/**
		 * @brief	Encode encryption key into SSS key block
		 */
		BC_INHERITED 	bool encode(bc_byte_t* kbEncrypted, unsigned int & kbSize, key_info_t const & key, cPassword const & init_pwd);


	protected:


		/**
		 * @brief	Add user to runtime register.
		 */
		void store_user(sss_user_t const & user);


		/**
		 * @brief	Check that this password is different from all previously entered and accepted
		 */
		bool is_unique_user(sss_user_t const & user) const;


		/**
		 * @brief	Checks if all required shares are assigned.
		 */
		bool is_ready() const
		{
			return (m_shares_required == this->total_shares_ready());
		}


		/**
		 * @brief	Get number of ready shares
		 */
		unsigned int total_shares_ready() const;


		/**
		 * @brief	Reset runtime status
		 */
		void reset();


	private:


		/* sss users */
		typedef std::list<sss_user_t*> 		user_list_t;
		typedef user_list_t::iterator 		user_iter_t;
		typedef user_list_t::const_iterator 	user_citer_t;

		/* runtime sss user table */
		user_list_t 	m_users;

		/* runtime share collector */	
		SSShares*	m_sss_runtime;

		/* number of shares required to restore\encode key */
		unsigned int 	m_shares_required;

		/* total shares. can me more than required */
		unsigned int 	m_shares_total;

	};

} // namespace kgghost

#endif
