#include "KGGhost.h"
#include "SpaceMap.h"
#include "UnicodeFuncs.h"

extern "C"
	int DecryptTable(
		HWND		hWnd,
		wchar_t		*contFileName,
		KEY_HANDLE	keyHandle,
		DWORD		contAlgId,
		ULONG		*pItemQ,
		sSpaceUnit	*suBuff,
		DWORDLONG	dwlOffset,
		DWORDLONG	dwlLength
	)
{

	if ( dwlLength != maxSpaceMapBlockSize )
		return dt_invalidPointer;

	if ( dwlLength > 0x7fffFFFF )
		return dt_invalidPointer;

	cPlainTextBlock ptb;
	if ( ptb.Allocate( (int)dwlLength ) == NULL )
		return dt_invalidPointer;

	HANDLE hFile = CreateFileT(
						contFileName,
						GENERIC_READ,
						FILE_SHARE_READ | FILE_SHARE_WRITE,
						NULL,
						OPEN_EXISTING,
						FILE_ATTRIBUTE_NORMAL,
						NULL );

	if ( hFile == INVALID_HANDLE_VALUE )
		return dt_errorInFileOpening;

	ULARGE_INTEGER uli;
	uli.LowPart = GetFileSize( hFile, &uli.HighPart );

	if ( uli.QuadPart < dwlOffset + dwlLength )
		return dt_invalidPointer;

	uli.QuadPart = dwlOffset;

	if ( ( SetFilePointer( hFile, uli.LowPart, (PLONG)&uli.HighPart, FILE_BEGIN ) == INVALID_SET_FILE_POINTER )
		 && ( GetLastError() != NO_ERROR )
		)
	{
		CloseHandle( hFile );
		return dt_errorInFileOpening;
	}

	DWORD read;

	BOOL ok = ReadFile( hFile, ptb.buffer, ptb.bufferSize, &read, NULL );

	CloseHandle( hFile );

	if ( !ok || ( read != ptb.bufferSize ) )
		return dt_errorInFileOpening;

	cSpaceMap map;

	if ( !map.decryptFrom( ptb.buffer, ptb.bufferSize, contAlgId, keyHandle ) )
		return dt_invalidSignature;

	ULONG Q = map.GetRealQuantity();

	if ( ( suBuff == NULL ) || ( *pItemQ < Q ) )
	{
		*pItemQ = Q;
		return dt_bufferTooSmall;
	}

	for( ULONG i = 0; i < Q; i++ )
	{
		suBuff[i] = map.m_encBlock.m_body.T[i];
	}

	*pItemQ = Q;

	return dt_errNo;
}
