#pragma once

#define SKIP_RESERVED

#include "KBlock.h"

#pragma pack(1)

struct sSpaceUnit
{
	ULONGLONG offset;
	ULONGLONG length;
};

// 
//	Space map table 
//
const int sigSize = 15;
#define szEncTableSig "EncTableSig"
const int maxSpaceMapBlockSize = _4K;
const int maxItemQ = RoundDown( maxSpaceMapBlockSize - sizeof( EBTail ) - sigSize, DB_MaxEncBlockSize ) / sizeof( sSpaceUnit );
//#define SKIP_RESERVED

struct sFixedTable
{
	sSpaceUnit  T[ maxItemQ ];
	char		Signature[ 15 ];
	bool whiteServiceInfo( cPlainTextBlock *ptb ) { return false; }
};

class cSpaceMap
{
public:

	encBlock < sFixedTable > m_encBlock;

	bool Init( sSpaceUnit *T, int Q )
	{
		if ( Q > maxItemQ )
			Q = maxItemQ;

		while( Q-- )
			m_encBlock.m_body.T[Q] = T[Q];

		return true;
	}

	cSpaceMap( sSpaceUnit *T = NULL, int Q = 0 )
	{
		assert( sizeof( m_encBlock ) == maxSpaceMapBlockSize );
		memset( &m_encBlock, 0, sizeof( m_encBlock ) );

		strcpy( m_encBlock.m_body.Signature, szEncTableSig );
	
		Init( T, Q );
	}

	~cSpaceMap()
	{
		memset( &m_encBlock, 0, sizeof( m_encBlock ) );
	}

	bool encryptTo( BYTE *buff, int buffSize, sKeyInfo *keyInfo, BYTE *pool )
	{
		encBlock < sFixedTable > *ebOut = ( encBlock < sFixedTable > *)buff;

		if ( sizeof( *ebOut ) != buffSize )
		{
			assert( false );
			return false;
		}

		return m_encBlock.encryptTo( ebOut, keyInfo, pool );
	}

	bool decryptFrom( BYTE *buff, int buffSize, DWORD algId, KEY_HANDLE keyHandle )
	{
		encBlock < sFixedTable > *ebIn = ( encBlock < sFixedTable > *)buff;

		if ( sizeof( *ebIn ) != buffSize )
		{
			assert( false );
			return false;
		}

		if ( !m_encBlock.decryptFrom( ebIn, algId, keyHandle ) )
			return false;

		return strcmp( m_encBlock.m_body.Signature, szEncTableSig ) == 0;
	}

	bool decryptFrom( BYTE *buff, int buffSize, sKeyInfo *keyInfo, BYTE *pool )
	{
		encBlock < sFixedTable > *ebIn = ( encBlock < sFixedTable > *)buff;

		if ( sizeof( *ebIn ) != buffSize )
		{
			assert( false );
			return false;
		}

		if ( !m_encBlock.decryptFrom( ebIn, keyInfo, pool ) )
			return false;

		return strcmp( m_encBlock.m_body.Signature, szEncTableSig ) == 0;
	}

	int GetRealQuantity()
	{
		int i = maxItemQ;

		while( i-- )
		{
			if ( m_encBlock.m_body.T[i].offset != 0 )
				break;
		}

		return i + 1;
	}

	ULONGLONG GetCommonLength()
	{
		int i = maxItemQ;

		ULONGLONG commonSize = 0;

		while( i-- )
		{
			commonSize +=  m_encBlock.m_body.T[i].length;
		}

		return commonSize;
	}

	//
	//	The real space may be changed
	//	The proc zeroes required units at tail
	//
	bool TruncateSpace( ULONGLONG newLength )
	{
		ULONGLONG length = 0;

		bool enough = false;

		for( int i = 0; i < maxItemQ; i++ )
		{
			if ( enough )
			{
				m_encBlock.m_body.T[i].length = 0;
				m_encBlock.m_body.T[i].offset = 0;
				continue;
			}

			ULONGLONG suLength = m_encBlock.m_body.T[i].length;
			if ( length + suLength < newLength )
			{
				length += suLength;
				continue;
			}

			enough = true;

			// turncates current if common length is more, does nothing if it is equal
			if ( length + suLength > newLength )
				m_encBlock.m_body.T[i].length = newLength - length;
		}

		return enough;
	}

	//
	//	GetEncryptedSpace
	//		Truncates space from first unit for the encrypted table
	//
	bool GetEncryptedSpace( sSpaceUnit *suHP )
	{
		if ( m_encBlock.m_body.T[0].length <= maxSpaceMapBlockSize )
			return false;

		suHP->offset = m_encBlock.m_body.T[0].offset;
		m_encBlock.m_body.T[0].length -= maxSpaceMapBlockSize;

		bool useTail = true;

		if ( useTail )
		{
			suHP->offset += m_encBlock.m_body.T[0].length; // adds truncated length
		}
		else
		{
			m_encBlock.m_body.T[0].offset += maxSpaceMapBlockSize; // moves forward
		}

		suHP->length = maxSpaceMapBlockSize;

		return true;
	}
};

//#undef SKIP_RESERVED

#pragma pack()