#include <stdlib.h>
#include <time.h>
#include <wchar.h>

#include "alg_utils.h"
#include "layout.h" // DB_POOL_SIZE
#include "kg_errors.h"

#include <common/core.h>

#include "CertificateLib/hashAlgs/hashAlgs.h"
#include "CertificateLib/SMemory/cPassword.h"
#include "CertificateLib/x509pkts/pkcs12_funcs.h"

using namespace bdk;

enum_hashes_t::enum_hashes_t()
{
	ha_getIDsOfAvailableAlgAlgorithms((int*)&m_total, &m_ids);
}


enum_hashes_t::~enum_hashes_t()
{
}


int alg_init_from_password(	bdk::BCAlgHandle& alg, int alg_id, int mode_id, char* pool,
				const cPassword* pwd, char* salt, unsigned int salt_size, int hash_id, int iteration)
{
	// check algorithm and mode
	BCAlgService* serv = bc_core()->alg_manager()->find_algorithm(alg_id);

	if( !serv )
	{
		BC_DEBUG_PRINT(("algorithm 0x%x does not exist\n", alg_id));
		return ERROR_INVALID_ALGORITHM;
	}

	if( !serv->is_mode_supported(mode_id) )
	{
		return ERROR_INVALID_ALGORITHM;
	}


	// generate encryption key
	unsigned int key_size_bytes = serv->mode_key_size(mode_id);
	key_size_bytes = key_size_bytes / 8;

	cPlainTextBlock key;
	BYTE* pwdkey = key.Allocate(key_size_bytes);

	if( !pwdkey ) {
		BC_DEBUG_PRINT(("failed allocating buffer of size %u\n", key_size_bytes));
		return ERROR_INTERNAL_PROBLEM;
	}

	int res = P12_producePseudoRandomBytes(
					pwdkey, key_size_bytes,
					(BYTE*)salt, salt_size,
					iteration,
					pwd->unicodeBuffer, (strlen(pwd->buffer) + 1) * 2,
					P12_prgm4key,
					(BYTE)hash_id
				);
	if( res == hae_invalidAlgorithmID )
	{
		BC_DEBUG_PRINT(("incorrect hash algorithm\n"));
		return ERROR_INVALID_ALGORITHM;
	}
	else if( res != x509e_ok )
	{
		BC_DEBUG_PRINT(("failed generating key from password\n"));
		return ERROR_INTERNAL_PROBLEM;
	}

	// allocate handle
	char zero_pool[512] = {0};
	if( pool == NULL )
	{
		pool = zero_pool;
	}

	alg = BCAlg::init(alg_id, mode_id, key.buffer, (unsigned char*)pool);
	if(!alg)
	{
		BC_DEBUG_PRINT(("failed generating alg handle\n"));
		return ERROR_INTERNAL_PROBLEM;
	}

	return ERROR_NO;
}


void ShredData(bc_byte_t* data, unsigned int dataLength)
{
	unsigned int i;

	if ( (!data) || (dataLength < 0) )
	{
		assert( false );
		return;
	}

	srand( (DWORD)time( NULL ) ^ (DWORD)clock() );

	for( i = 0; i < dataLength; i++ )
	{
		data[i] = (BYTE)rand();
	}
}

