/*******************************************************************\
*	Copyright (c) 2009 Jetico, Inc., Finland
*	All rights reserved.
*
*	File: alg_enums.h
*	Revision: refactored from ALG.H
*	Created: 26.05.2009
*	Description: Enumerations of hash and block cipher algorithms
*
\*******************************************************************/


#ifndef V8_ALG_REGISTERS_H
#define V8_ALG_REGISTERS_H

#include <alg/alg.h>
	
struct enum_ids_base_t
{
protected:

	int *m_ids;		// Table with IDs
	unsigned int m_total;	// IDs quantity

public:


	unsigned int total() { return m_total; }

	int operator [] (unsigned int id) { BC_ASSERT(id < this->total()); return m_ids[id]; }
};


/**
 * @brief	Enumerates all available block cipher algorithms
 */
struct enum_algs_t : public enum_ids_base_t
{
	/**
	 * @brief	Create cipher algorithms enumeration table
	 */
	enum_algs_t();


	/**
	 * @brief	Release cipher alglorithms enumeration table
	 */
	~enum_algs_t();
};


/**
 * @brief	Enumerates all available block cipher algorithms
 */
struct enum_modes_t : public enum_ids_base_t
{
	/**
	 * @brief	Create cipher algorithms enumeration table
	 */
	enum_modes_t();


	/**
	 * @brief	Release cipher alglorithms enumeration table
	 */
	~enum_modes_t();
};


/**
 * @brief	Enumerates all available hash algorithms
 */
struct enum_hashes_t : public enum_ids_base_t
{
	/**
	 * @brief	Create cipher algorithms enumeration table
	 */
	enum_hashes_t();


	/**
	 * @brief	Release cipher alglorithms enumeration table
	 */
	~enum_hashes_t();
};


// forward decl
class cPassword;


/**
 * @brief		Initialize algorithm by generating key from password
 *
 * @p alg		Algorithm handle to initialize
 * @p alg_id		Algorithm id
 * @p mode_id		Algorithm mode id
 * @p pool		Random data pool
 * @p pwd		Password buffer
 * @p salt		Salt buffer
 * @p salt_size 	Salt buffer size
 * @p hash_id		Hash algorithm id to generate password hash with
 * @p iteration 	Hash algorithm iteration number to use
 *
 * @return 		ERROR_NO and initialized handle,
 *			ERROR_INVALID_ALGORITHM and undefined handle if input is incorrect,
 *			ERROR_INTERNAL_PROBLEM and undefined handle otherwise.
 */
int alg_init_from_password(	bdk::BCAlgHandle& alg, int alg_id, int mode_id, char* pool, 
				const cPassword* pwd, char* salt, unsigned int salt_size, int hash_id, int iteration);


/**
 * @brief	Fill memory by random data
 *
 * @p data	Buffer to fill
 * @p length	Buffer length
 */
void ShredData( bc_byte_t* data, unsigned int length );


#endif

