#include "cDlgEncHeaderParse.h"
#include "resource.h"
#include "AutoDeallocate.h"
#include "..\..\..\CertificateLib\hashAlgs\hashAlgs.h"
#include "BC_ioctl.h"
#include "ODS.h"

BOOL cProgressDlg_EncHeaderParse::InitDialog()
{
	m_hProgress = GetDlgItem( m_hDlg, IDC_PROGRESS );

	DWORD threadId;
	HANDLE hThread = CreateThread( NULL, 0, threadFunc, this, 0, &threadId );

	if ( hThread == NULL )
	{
		MessageBoxT( m_hDlg, L"Error in thread creating", LoadStringOnceT( IDS_KeyGen ), MB_ICONSTOP );
		EndDialog( m_hDlg, -1 );
		return false;
	}

	CloseHandle( hThread );

	return TRUE;
}

void cProgressDlg_EncHeaderParse::SetAlgTxt( int idc, const char *algName, int num, int Q )
{
	cWCStr wcs( algName );
	SetAlgTxt( idc, wcs.m_szBody, num, Q );
}

void cProgressDlg_EncHeaderParse::SetAlgTxt( int idc, const wchar_t *algName, int num, int Q )
{
	wchar_t txt[ 128 ];
	::FormatT( txt, maxLen( txt ), L"(%d/%d) - %s", num + 1, Q, algName );
	SetDlgItemTextT( idc, txt );
}

// static 
DWORD WINAPI cProgressDlg_EncHeaderParse::threadFunc( void *ptr )
{
	cProgressDlg_EncHeaderParse *pThis = ( cProgressDlg_EncHeaderParse * )ptr;

	pThis->Run();

	PostMessage( pThis->m_hDlg, WM_COMMAND, IDOK, 0 );

	return 0;
}

LRESULT cProgressDlg_EncHeaderParse::RealDlgProc( UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	switch (uMsg) 
	{ 
		case WM_INITDIALOG:
			return InitDialog();

	    case WM_COMMAND: 
		{
			switch(wParam) 
			{ 
				case IDOK: 
					EndDialog( m_hDlg, m_res );
					// ??? Mistic feature, but app is crashes in Vista without the wating.
					// Cancel of decoding when container with encrypted header is been mounting.
					Sleep( 100 );
					return TRUE; 
				case IDCANCEL:
				{
					int res = MessageBoxT(
								m_hDlg,
								LoadStringOnceT ( IDS_Q_StopKeyEncodingProcess ),
								LoadStringTwiceT( IDS_KeyGen ),
								MB_ICONQUESTION | MB_YESNO | MB_DEFBUTTON2 );

					if ( res != IDYES )
							return 0;

					m_cancelled = true;
					EnableWindow( GetDlgItem( m_hDlg, IDCANCEL ), false );
					return TRUE;
				}
				default:
					return 0; 
			}
		}
		default:
			return 0; 
	}

	return 0;
}

int cProgressDlg_EncHeaderParse::Run()
{
	cEnumAlgIDs algs;
	cEnumHashAlgIDs hashAlgs;
	cEnumEncModes encModes;

	cSymAlgorithm cAlg;

	cADBlock < dbHeader > adbDecHeader;
	
	if ( adbDecHeader.m_T == NULL )
	{
		assert( false );
		*m_pErrCode = ERROR_INTERNAL_PROBLEM;
		return 0;
	}

	assert( m_pKBParams->kbType == kbtEmpty );

	SendMessage( m_hProgress, PBM_SETRANGE, 0, MAKELPARAM( 0, algs.Q * encModes.Q * hashAlgs.Q ) );
	SendMessage( m_hProgress, PBM_SETSTEP, 1, 0 );

	for( int i = 0; i < algs.Q; i++ )
	{
		m_pKBParams->kbPBE.m_algId = algs.T[i];

		for ( int m =  0; m < encModes.Q; m++ )
		{
			m_pKBParams->kbPBE.m_encModeId = encModes.T[m].id;

			//
			//	Checks algorithm values in registry
			//
			if ( !cAlg.Init( algs.T[i], encModes.T[m].id ) )
				continue;

			//
			//	Checks real installed algorithm
			//
			if ( !cAlg.supportMode( encModes.T[m].id ) )
				continue;

			//
			//	Checks real installed algorithm
			//
			//if ( ( encModes.T[m].id == EX_ENCRYPT_LRW_MODE_FLAG ) && ( !cAlg.supportLRWMode() ) )
			//	continue;

			wchar_t bf[ 100 ];
			SetAlgTxt( IDC_AlgTxt, cAlg.getAlgName( bf, maxLen( bf ) ), i, algs.Q );

			for( int j = 0; j < hashAlgs.Q; j++ )
			{
				m_pKBParams->kbPBE.m_hashAlgId = (BYTE)hashAlgs.T[j];
				SetAlgTxt( IDC_HashAlgTxt, ha_getHashAlgName( (BYTE)hashAlgs.T[j] ), j, hashAlgs.Q );

				m_pKeyMapInfo->setUndefined();

				//ODS_Enable( true );
				//ODSF( L"Algorithm %s %s", bf, getWCStr( ha_getHashAlgName( (BYTE)hashAlgs.T[j] ) ) );
				//ODS_Enable( ( m_pKBParams->kbPBE.m_hashAlgId == 128 ) && ( m_pKBParams->kbPBE.m_algId == 240 ) && ( encModes.T[m].id == EX_ENCRYPT_LRW_MODE_FLAG ) );

				*m_pErrCode =
					DataBlockVerifyKeyBlockAndGetKey(
						m_dataBlock,
						m_dbSize,
						m_pKBParams,
						m_pKeyMapInfo,
						m_pKeyHandle,
						m_pKeyInfoCopy,
						adbDecHeader.m_T, // m_pDecryptedHeader,
						NULL, // m_pDecryptedContInfo,
						NULL, // m_hProgress,
						&m_cancelled
					);

				SendMessage( m_hProgress, PBM_STEPIT, 0, 0 );

				if ( m_cancelled )
				{
				//	ODS( L"Error cancelled" );

					*m_pErrCode = ERROR_CANCELLED_BY_USER;
					return 0;
				}

				if ( *m_pErrCode == ERROR_NO )
				{
					//ODS( L"Error NO" );
					break; // breaks from Hash algorithm enumeration loop if no error
				}
			}

			if ( *m_pErrCode == ERROR_NO )
			{
#ifdef _UseCBCModeOnly_
				//
				//	If the flag is assigned all encryption modes will give valid result.
				//	So we have to check mode decrypted 
				//
				if ( adbDecHeader.m_T->encModeId != m_pKBParams->kbPBE.m_encModeId )
				{
					if ( ( m_pKeyHandle != NULL ) && ( *m_pKeyHandle != NULL ) )
					{
						Alg_FreeKeyHandler( cAlg.m_algServiceName, *m_pKeyHandle );
						*m_pKeyHandle = NULL;
					}
					
					*m_pErrCode = ERROR_INVALID_ALGORITHM;

					// m_pKBParams->kbType = kbtEmpty;

					continue;
				}
#endif
				copyEx( m_pDecryptedHeader, adbDecHeader.m_T );

				break; // breaks from encryption mode enumeration loop if no error
			}
		}

		if ( *m_pErrCode == ERROR_NO )
			break; // breaks from Container algorithm enumeration loop if no error
	}

	return 0;
}
