#include "cDlgKBParams.h"
#include "resource.h"
#include <windowsx.h>
#include "diskDriver.h"
#include "AutoDeallocate.h"

//// The problem with **** ||||| in password dialog is fixed 
//// by changing font name in resource file.
//const char *szPwdFontName = "MS Shell Dlg";
//const bool changeFont = false;


static void DrawFileIcon( HWND hDlg, HDC dc, HICON hIcon, int id )
{
	if ( !hIcon )
		return;

	HWND hWnd = GetDlgItem( hDlg, id );

	if ( hWnd == NULL )
		return;

	ShowWindow( hWnd, SW_HIDE );

	RECT r;
	GetWindowRect( hWnd, &r );
	//POINT p = { r.left, r.top };
	//ScreenToClient( hDlg, &p );
	//DrawIcon( dc, p.x, p.y, hIcon );
	MapWindowPoints( NULL, hDlg, (POINT*)&r, 2 );
	DrawIcon( dc, r.left, r.top, hIcon );
}

//
//	processContIcon
//		preprocesses dlg window procedure and exchange container icon to really extracted 
//		document from main exe instance.
//
//		use it just before main message switch
//
static void processContIcon( HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	static HICON hContIcon = NULL;

	switch (uMsg) 
	{
        case WM_INITDIALOG:
			wchar_t path[ MAX_PATH ];
			GetModuleFileNameT( NULL, path, maxLen( path ) );
			hContIcon = ExtractIconT( (HINSTANCE)GetWindowLongPtrT( hwnd, GWLP_HINSTANCE ), path, 1 );
			break;

		case WM_PAINT :
		{
			HDC dc;
			PAINTSTRUCT ps;

			if ((HDC) wParam != NULL)
				dc = (HDC) wParam;
			else
				dc = BeginPaint(hwnd,&ps);

			DrawFileIcon( hwnd, dc, hContIcon, IDC_Icon );

			if ((HDC) wParam == NULL)
				EndPaint(hwnd,&ps);

			break;
		}

		case WM_DESTROY:

			if ( hContIcon != NULL )
			{
				DestroyIcon( hContIcon );
				hContIcon = NULL;
			}

			break;

		default :
			break;
	}
}

static bool getRootDir( wchar_t *root, const wchar_t *path )
{
	// 'C:\qqq\"
	// \\PAV\c\qqq\

	if ( path[1] == L':' )
	{
		wcsncpy( root, path, 3 );
		root[3] = L'\0';
		return true;
	}
	else
	if ( path[0] == L'\\' )
	{
		int q = 4;
		int i = 0;

		for( ; path[i] != 0; )
		{
			root[i] = path[i];
			if ( path[i++] == L'\\' )
			{
				if ( --q == 0 )
					break;
			}
		}

		root[i] = L'\0';
		return true;
	}

	return false;
}

static bool isNTFS( const wchar_t *path, int *pErrIDS )
{
	wchar_t volume[ MAX_PATH ];
	DWORD dwSerNum, dwMaxComLen, dwFSFlags;
	wchar_t fileSystem[ 64 ] = L"";

	*pErrIDS = IDS_FolderInvalidPathName;

	wchar_t root[ MAX_PATH ];
	if ( !getRootDir( root, path ) )
		return false;

	if ( !GetVolumeInformationT(
			root,
			volume, maxLen( volume ),
			&dwSerNum,
			&dwMaxComLen,
			&dwFSFlags,
			fileSystem,
			maxLen( fileSystem ) )
		)
		return false;

	if ( wcsncmp( fileSystem, L"NTFS", 4 ) == 0 )
		return true;

	*pErrIDS = IDS_FolderNotNTFS;

	return false;
}

//
//	Checks if the folder may be used as mount point.
//

static bool checkFolder( wchar_t *folderName, int *pErrIDS )
{
	int errIDS;
	if ( pErrIDS == NULL )
		pErrIDS = &errIDS;

	*pErrIDS = IDS_FolderNameIsInvalid;

	wchar_t path[ MAX_PATH ];

	if ( !GetRootDirT( path, folderName ) )
		return false;

	UINT drvType = GetDriveTypeT( path );

	if ( drvType == DRIVE_UNKNOWN )
	{
		*pErrIDS = IDS_DriveTypeIsUnknown;
		return false;
	}

	if ( drvType == DRIVE_NO_ROOT_DIR )
	{
		*pErrIDS = IDS_FolderIsNotRootDir;
		return false;
	}

	if ( drvType == DRIVE_REMOTE )
	{
		*pErrIDS = IDS_IsNetworkDrive;
		return false;
	}

	int len = (int)wcslen( folderName );

	if ( len > MAX_PATH )
		return false;

	if ( folderName[ len - 1 ] == L'\\' )
		len--;

	wchar_t oldName[ MAX_PATH ];
	wcsncpy( oldName, folderName, len );
	oldName[ len ] = L'\0';

	if ( !FileExistsT( oldName ) )
	{
		if ( CreateDirectoryT( oldName, NULL ) )
			return true;
	}

	*pErrIDS = IDS_FolderInvalidPathName;

	if ( len > MAX_PATH - 3 )
		return false;

	if ( !isNTFS( folderName, pErrIDS ) )
		return false;

	wcscat( wcscpy( path, folderName ), L"\\*" );

	WIN32_FIND_DATAW findData;

	HANDLE hFind = FindFirstFileT( path, &findData );

	*pErrIDS = IDS_FolderDoesNotExistOrAccessIsDenied;

	if ( hFind == INVALID_HANDLE_VALUE )
		return false;

	*pErrIDS = IDS_NoError;

	bool isEmpty = true;

	do
	{
		if ( wcscmp( findData.cFileName, L".." ) == 0 ) continue;
		if ( wcscmp( findData.cFileName, L"." ) == 0 ) continue;

		*pErrIDS = IDS_FolderIsNotEmpty;

		isEmpty = false;

		break;
	}
	while( FindNextFileT( hFind, &findData ) );

	FindClose( hFind );

	if ( !isEmpty )
		return false;

	return true;

	//// If explorer has open view to this empty folder so the mpunt poin cannot be properly mounted.
	//// We tyr to rename it befre and retun name back

	//*pErrIDS = IDS_FolderIsBusied;

	//RemoveDirectoryT( oldName, false );

	//if ( FileExistsT( oldName ) )
	//	return false;

	//bool ok = CreateDirectoryT( oldName, NULL );

	//return ok;

	// Exloprer simple rename current opened view...

	//wchar_t newName[ MAX_PATH ];
	//wcsncpy( newName, folderName, len );

	//do
	//{
	//	int cnt = 1;
	//	FormatT( newName + len, addLen, L" (%d)", cnt++ );
	//}
	//while( FileExistsT( newName ) );

	//if ( !MoveFileExT( oldName, newName, 0 ) ) 
	//	return false;

	//if ( !FileExistsT( newName ) )
	//	return false;

	//bool ok = !FileExistsT( oldName );

	//// Return name back
	//MoveFileExT( newName, oldName, 0 );

	//if ( FileExistsT( newName ) )
	//	DeleteFileT( newName );

	//return ok;
}

LRESULT cDlgGetAdvancedFlags::RealDlgProc( UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	switch (uMsg) 
	{ 
        case WM_INITDIALOG:
		{
			Button_SetCheck( GetDlgItem( m_hDlg, IDC_STYPING),  ( m_keyGenFlags & FLAG_ADVANCED_SECURE_TYPING) > 0);
			Button_SetCheck( GetDlgItem( m_hDlg, IDC_SHOWCHARS), ( m_keyGenFlags & FLAG_ADVANCED_SHOW_PWD_CHARS) > 0);

			//
			//	Set the flag, otherwise kbdEdit does not show valid characters
			//	They are read as buffer if OK is pressed.
			//
			m_keyGenFlags |= FLAG_ADVANCED_TEST; // FLAG_ADVANCED_SHOW_PWD_CHARS;

			m_useSecureTyping = TRUE;
			BOOL res = kbd.GenerateEncodingTable( m_useSecureTyping );

			Edit_SetPasswordChar( GetDlgItem( m_hDlg, IDC_EDITSTRING ), 0 );

			m_edt.Attach( GetDlgItem( m_hDlg, IDC_EDITSTRING ) );

			SetFocus( m_edt.m_hEdit );

			return res;
		}
		case WM_DESTROY:
		{
			m_edt.Detach();
			kbd.FreeTables( TRUE );

			break;
		}
		case WM_COMMAND: 
		{
			int cmd = LOWORD( wParam );

			switch( cmd ) 
			{ 
				//case IDC_STYPING:
				//{
				//	ShowWindow( GetDlgItem( m_hDlg, IDC_LINE ), Button_GetCheck( GetDlgItem( m_hDlg, IDC_STYPING ) ) == BST_CHECKED );
				//	break;
				//}
				case IDC_SHOWCHARS:
			        if ( Button_GetCheck( GetDlgItem( m_hDlg, IDC_SHOWCHARS ) ) )
						 m_keyGenFlags |= FLAG_ADVANCED_SHOW_PWD_CHARS;
					else m_keyGenFlags &= ~FLAG_ADVANCED_SHOW_PWD_CHARS;

					break;
				case IDOK: 
 
			        if ( Button_GetCheck( GetDlgItem( m_hDlg, IDC_STYPING ) ) )
						 m_keyGenFlags |= FLAG_ADVANCED_SECURE_TYPING;
					else m_keyGenFlags &= ~FLAG_ADVANCED_SECURE_TYPING;

			        if ( Button_GetCheck( GetDlgItem( m_hDlg, IDC_SHOWCHARS ) ) )
						 m_keyGenFlags |= FLAG_ADVANCED_SHOW_PWD_CHARS;
					else m_keyGenFlags &= ~FLAG_ADVANCED_SHOW_PWD_CHARS;

					m_keyGenFlags &= ~FLAG_ADVANCED_TEST;

					EndDialog( m_hDlg, ERROR_NO);
					return TRUE; 
 
				case IDCANCEL: 
	
					m_keyGenFlags &= ~FLAG_ADVANCED_TEST;

					EndDialog( m_hDlg, (int)ERROR_CANCELLED_BY_USER); 
					return TRUE; 
			} 

            return 0; 
		}
    } 
    return FALSE; 

	UNREFERENCED_PARAMETER(lParam);
}

bool cDlgGetPasswordAndMount::checkEmptyFolder()
{
	int errIDS;

	if ( checkFolder( m_mountPoint, &errIDS ) )
		return true;

	wchar_t msg[ 1024 ];
	
	FormatT_S( msg, maxLen( msg ),IDS_InvalidMountPoint_S, m_mountPoint );
	wcscat( msg, L"\n\n" );
	wcscat( msg, LoadStringOnceT( errIDS ) );

	DlgMessageBoxT( msg, MB_ICONSTOP, LoadStringOnceT( IDS_ERROR ) );

	return false;
}

bool cDlgGetPasswordAndMount::OnBrowseForMountPoint( HWND hCtrl )
{
	if ( m_mountPoint == NULL )
		return false;

	HWND cmbHwnd = GetDlgItem( m_hDlg, IDC_DRIVES );

	if ( hCtrl != cmbHwnd )
		return false;

	int curSel = ComboBox_GetCurSel( cmbHwnd );

	if ( curSel != m_posBrowseForMountPoint )
		return false;

	while( true ) 
	{
		UINT uiFlags = BIF_RETURNONLYFSDIRS | BIF_NEWDIALOGSTYLE;

		if ( !BrowseForFolderT( m_hInst, IDS_ChooseFolderForMounting, m_hDlg, m_mountPoint, uiFlags  ) )
		{
			ComboBox_SetCurSel( cmbHwnd, 0 );
			return false;
		}
		
		int len = (int)wcslen( m_mountPoint );
		if ( len == 0 )
			return false;

		if ( m_mountPoint[ len - 1 ] != L'\\' )
			wcscpy( m_mountPoint + len, L"\\" );

		if ( checkEmptyFolder() )
			break;
	} 

	SelectMountPoint( cmbHwnd );
	return true;
}

bool isDriveLetter( const wchar_t *mountPoint, int *pNum = NULL )
{
	if ( wcslen( mountPoint ) != 3 )
		return false;

	if ( wcscmp( mountPoint + 1, L":\\" ) != 0 )
		return false;

	if ( towupper( mountPoint[0] ) < L'A' )
		return false;

	if ( towupper( mountPoint[0] ) > L'Z' )
		return false;

	if ( pNum )
		*pNum = ( towupper( mountPoint[0] ) - L'A' );

	return true;
}

//
//	ComboBoxEx32 is exluded now ( it doe not work for windows 98 ).
//	The combobox control works now perfectly for unicode texts
//
#define myComboBoxEx_GetItemTextT ComboBox_GetLBTextT
#define myComboBoxEx_InsertStringT ComboBox_InsertStringT
#define myComboBoxEx_SetItemTextT ComboBoxEx_SetItemTextT
#define myComboBoxEx_GetTextT GetWindowTextT

int cDlgGetPasswordAndMount::SelectMountPoint( HWND hCmb )
{
	if ( m_mountPoint == NULL )
		return 0;

	int count = ComboBox_GetCount( hCmb );

	// 1. Finds mount point name at available drives
	for (; count--; )
	{
		wchar_t text[ MAX_PATH ];
		myComboBoxEx_GetItemTextT( hCmb, count, text, maxLen( text ) );

		if ( wcscmp( text, m_mountPoint ) == 0 )
		{
			ComboBox_SetCurSel(  hCmb, count );
			return count;
		}
	}

	// 2. Checks if the mount point is a busied drive. If so it selects first available drive incombobox.

	if ( isDriveLetter( m_mountPoint ) )
	{
		ComboBox_SetCurSel( hCmb, 0 );
		return 0;
	}


	// 3. Adds mount point name  at the combobox tail and selects it
	int mpPos = m_posBrowseForMountPoint + 1;

//	myComboBoxEx_SetItemTextT( hCmb, mpPos, m_mountPoint );
	ComboBox_DeleteString( hCmb, mpPos );
	mpPos = myComboBoxEx_InsertStringT( hCmb, mpPos, m_mountPoint );
	return ComboBox_SetCurSel(  hCmb, mpPos );
}

bool cDlgGetPasswordAndMount::GetMountParams()
{
	if ( m_mountPoint == NULL )
		return true;

	int curSel = ComboBox_GetCurSel( GetDlgItem(m_hDlg, IDC_DRIVES) );
	if (  curSel == m_posBrowseForMountPoint )
	{
		assert( false );
		return false;
	}

	bool checked = Button_GetCheck( GetDlgItem(m_hDlg, IDC_AUTOMOUNT) ) == BST_CHECKED;

	//if ( m_headerIsEncrypted )
	//{
	//	m_headerIsEncrypted = !checked;
	//	m_keyGenFlags &= ~FLAG_CONTAINER_AUTO_MOUNT;
	//}
	//else
	if ( checked )	m_keyGenFlags |= FLAG_CONTAINER_AUTO_MOUNT;
	else			m_keyGenFlags &= ~FLAG_CONTAINER_AUTO_MOUNT;

	if ( Button_GetCheck( GetDlgItem( m_hDlg, IDC_READONLY ) ) )
			m_keyGenFlags |= FLAG_CONTAINER_READ_ONLY;
	else	m_keyGenFlags &= ~FLAG_CONTAINER_READ_ONLY;

	

	//
	//	Strange behaivioure from Petter Lumbye 23.09.2007
	//	First test 26.09.2007 shown that GetWindowTextT does not read the text from combobox properly.
	//

	if ( m_hDlg == NULL )
	{
		assert( false );
		wcscpy_s( m_mountPoint, MAX_PATH, L"W:\\" );
		return true;
	}

	HWND cmbHwnd = GetDlgItem( m_hDlg, IDC_DRIVES );

	if ( cmbHwnd == NULL )
	{
		assert( false );
		wcscpy_s( m_mountPoint, MAX_PATH, L"Y:\\" );
		return true;
	}

	m_mountPoint[0] = L'\0';

	//ComboBoxEx_GetTextT( cmbHwnd, m_mountPoint, MAX_PATH );

	//if ( wcslen( m_mountPoint ) != 0 )
	//	return true;

	myComboBoxEx_GetTextT( cmbHwnd, m_mountPoint, MAX_PATH );

	if ( wcslen( m_mountPoint ) != 0 )
		return true;

	assert( false );
	//curSel = ComboBox_GetCurSel( cmbHwnd );

	ComboBox_GetLBTextT( cmbHwnd, curSel, m_mountPoint, MAX_PATH ); 

	if ( wcslen( m_mountPoint ) != 0 )
		return true;

	cAutoDeallocate < char > adStr;

	if ( adStr.Allocate( MAX_PATH ) == NULL )
	{
		wcscpy_s( m_mountPoint, MAX_PATH, L"R:\\" );
		return true;
	}

	ComboBox_GetText( cmbHwnd, adStr.m_ptr, adStr.m_len );

	if ( strlen( adStr.m_ptr ) == 0 )
	{
		if ( curSel == -1 )
			wcscpy_s( m_mountPoint, MAX_PATH, L"O:\\" );
		else
			wcscpy_s( m_mountPoint, MAX_PATH, L"P:\\" );

		return true;
	}

	_mbstowcs( m_mountPoint, MAX_PATH, adStr.m_ptr, -1, false, CP_ACP );

	return true;
}

struct sWinVer : public OSVERSIONINFO
{
	sWinVer()
	{
		dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
		GetVersionEx( this );
	}

	bool is2K()
	{
		return ( ( dwPlatformId == VER_PLATFORM_WIN32_NT ) && ( dwMajorVersion >= 5 ) );
	}
};

void cDlgGetPasswordAndMount::SetMountParams()
{
	HWND hChk = GetDlgItem(m_hDlg, IDC_AUTOMOUNT);
	Button_SetCheck( GetDlgItem(m_hDlg, IDC_AUTOMOUNT), ( m_keyGenFlags & FLAG_CONTAINER_AUTO_MOUNT) > 0);

//	if ( m_headerIsEncrypted )
//	{
////		SetWindowText( hChk, "Decrypt header" );
////		Button_SetCheck( hChk, false );
//		EnableWindow( hChk, false );
//	}

	Button_SetCheck( GetDlgItem(m_hDlg, IDC_READONLY),  ( m_keyGenFlags & FLAG_CONTAINER_READ_ONLY) > 0);

	if ( m_keyGenFlags & FLAG_FORCE_READ_ONLY )
	{
		Button_SetCheck( GetDlgItem(m_hDlg, IDC_READONLY),  TRUE  );
		Button_Enable  ( GetDlgItem(m_hDlg, IDC_READONLY),  FALSE );
	}

	HWND cmbHwnd = GetDlgItem(m_hDlg, IDC_DRIVES);

	wchar_t  driveName[] = L"A:\\";

	if ( ( m_keyGenFlags & FLAG_MOUNT_ON_SINGLE_LETTER ) != 0 )
	{
		//
		// NT4 rudiment.
		//	if cont was not completely dismounted it will be mounted on the drive it was mounted before
		//

		assert( wcslen( m_mountPoint ) != 0 );

		// Spec hidden part creation function uses same flag for mounting before free cluster chain reading.
		EnableWindow( GetDlgItem( m_hDlg, IDC_AUTOMOUNT ), false );
		EnableWindow( GetDlgItem( m_hDlg, IDC_READONLY ), false );
		m_posBrowseForMountPoint = -1;
	}
	else
	{
		DWORD curLogicalDisks = _GetLogicalDrives();

		for ( int i=2; i<26; i++ )
		{
			if ((curLogicalDisks & (1 << i)) == 0)
			{
				driveName[0] = L'A' + i;
				myComboBoxEx_InsertStringT( cmbHwnd, -1, driveName );
			}
		}

		static sWinVer winVer;
		if ( !m_bTravellerMode && winVer.is2K() )
			m_posBrowseForMountPoint =
				myComboBoxEx_InsertStringT( cmbHwnd, -1, LoadStringOnceT( IDS_BrowseForMountPoint ) );
		else
			m_posBrowseForMountPoint = -1;
	}

	int len = (int)wcslen( m_mountPoint );
	if ( len != 0 )
		SelectMountPoint( cmbHwnd );
	else
		ComboBox_SetCurSel( cmbHwnd, 0 );
}

static bool isFreeDrive( int drv )
{
	DWORD dwMask = GetLogicalDrives();
	return ( ( dwMask & ( 1 << drv ) ) == 0 );
}

bool cDlgGetPasswordAndMount::CheckValidMountPoint()
{
	int drv;

	if ( isDriveLetter( m_mountPoint, &drv ) )
	{
		if ( !isFreeDrive( drv ) )
		{
			DlgMessageBoxT(
				FormatOnceT_S( IDS_Drive_S_isBusied, m_mountPoint ),
				MB_ICONEXCLAMATION,
				LoadStringOnceT( IDS_WARNING ) );

			return false;
		}
	}
	else
	if ( !checkEmptyFolder() )
		return false;

	return true;
}

//
//	cDlgGetPasswordAndMount::RealDlgProc
//		has been generated from getPasswordDlgProc and adopted for getPasswordDlgProcSimple ( without m_mountPoint )
//
LRESULT cDlgGetPasswordAndMount::RealDlgProc( UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	processContIcon( m_hDlg, uMsg, wParam, lParam );

	switch (uMsg) 
	{ 
		case WM_INITDIALOG:
		{
			if ( m_caption  != NULL ) SetWindowTextT( m_hDlg, m_caption );
			if ( m_fileName != NULL ) SetWindowTextT( GetDlgItem( m_hDlg, IDC_TextName), m_fileName );
			if ( m_pathName != NULL ) SetWindowTextT( GetDlgItem( m_hDlg, IDC_TextPath), m_pathName );

			if ( !m_useSecureTyping ) ShowWindow( GetDlgItem(m_hDlg, IDC_LINE), SW_HIDE );

			// Shows password chars explicitly
			if ((m_keyGenFlags & FLAG_ADVANCED_SHOW_PWD_CHARS) > 0)
				Edit_SetPasswordChar(GetDlgItem(m_hDlg, IDC_PASS), 0);

			// Sets the default push button to "Cancel"
			SendMessage(m_hDlg, DM_SETDEFID, (WPARAM) IDCANCEL, (LPARAM) 0); 

			kbd.GenerateEncodingTable( m_useSecureTyping );

			m_edt.Attach( GetDlgItem( m_hDlg, IDC_PASS ) );

			//
			//	If the dialogue is used as mount dialog
			//
			if ( m_mountPoint != NULL )
				SetMountParams();

			return true;
		}
		case WM_DESTROY:
		{
			m_edt.Detach();
			kbd.FreeTables( m_useSecureTyping );
			break;
		}
		case WM_COMMAND: 
		{
			// Set the default push button to "OK" when the user enters text
			if((HIWORD (wParam) == EN_CHANGE) && (LOWORD(wParam) == IDC_PASS)) 
				SendMessage(m_hDlg, DM_SETDEFID, (WPARAM) IDOK, (LPARAM) 0); 
	
			if ( HIWORD(wParam) == CBN_SELCHANGE )
			{
				OnBrowseForMountPoint( (HWND)lParam );
				break;
			}
			
			switch( LOWORD( wParam ) ) 
			{ 
				case IDOK:
				{
					//
					//	Gets mount point and other mount attributes for mount dlg part.
					//
					if ( m_mountPoint != NULL )
					{
						if ( !GetMountParams() )
							return false;

						if ( !CheckValidMountPoint() )
						{
							SetFocus( GetDlgItem( m_hDlg, IDC_DRIVES ) );
							return false;
						}
					}

					m_edt.CheckAndGetPassword( &m_pwd, &m_edt, true );
					if ( m_pwd.IsEmpty() )
					{
						MessageBoxT( m_edt.dlg->m_hDlg,
						m_edt.dlg->LoadStringOnceT( IDS_PASSWORD_EMPTY ),
						m_edt.dlg->LoadStringTwiceT( IDS_INVALID_PWD ),
						MB_OK | MB_ICONWARNING );
						SetFocus(  m_edt.m_hEdit );
						return false;
					}

					EndDialog(m_hDlg, ERROR_NO);
					return TRUE; 
				}
				case IDCANCEL: 
				{
					GetMountParams();

					//if ( Button_GetCheck( GetDlgItem(m_hDlg, IDC_AUTOMOUNT)))
					//		{ m_keyGenFlags |= FLAG_CONTAINER_AUTO_MOUNT; }
					//else	{ m_keyGenFlags &= ~FLAG_CONTAINER_AUTO_MOUNT; }

					//if ( Button_GetCheck( GetDlgItem(m_hDlg, IDC_READONLY)))
					//	m_keyGenFlags |= FLAG_CONTAINER_READ_ONLY;
					//else m_keyGenFlags &= ~FLAG_CONTAINER_READ_ONLY;
	
					EndDialog(m_hDlg, (int)ERROR_CANCELLED_BY_USER); 
					return TRUE; 
				}
			} 
			return 0; 
		}
	} 

    return FALSE; 

	UNREFERENCED_PARAMETER(lParam);
}