#include "cDlgKBParams.h"
#include "resource.h" 
#include <windowsx.h>

bool cPageKBParams::Init( cDlgKBParams *dlg )
{
	m_dlg = dlg;
	if ( dlg == NULL )
	{
		assert( false );
		return false;
	}

	m_hDlg = m_dlg->m_hDlg;
	m_useSecureTyping = m_dlg->m_useSecureTyping;

	return InitV();
}

cPageKBParams *cDlgKBParams::CurPage()
{
	switch( m_kbParams->kbType )
	{
		case kbtPBE: return &PBE;
		case kbtPKE: return &PKE;
		case kbtSSS: return &SSS;
		default:
			assert( false );
			return &PBE;
	}
}

void cDlgKBParams::ShowItems()
{
	PBE.ShowItems( m_kbParams->kbType == kbtPBE );
	PKE.ShowItems( m_kbParams->kbType == kbtPKE );
	SSS.ShowItems( m_kbParams->kbType == kbtSSS );
}

int cDlgKBParams::OnNotify( NMHDR *phdr )
{
	int res = SSS.Notify( phdr );

	if ( res != 0 )
		return res;

	return 0;
}

#define WM_ShowItems ( WM_USER + 2 )

bool cDlgKBParams::OnInitDialog()
{
	if ( m_caption  != NULL ) SetWindowTextT( m_hDlg, m_caption );
	if ( m_fileName != NULL ) SetWindowTextT( GetDlgItem( m_hDlg, IDC_TextName), m_fileName );
	if ( m_pathName != NULL ) SetWindowTextT( GetDlgItem( m_hDlg, IDC_TextPath), m_pathName );

	kbd.GenerateEncodingTable( m_useSecureTyping );

	HWND hCmbType = GetDlgItem( m_hDlg, IDC_cmbType );

	// We have to attach both kbdEdits anyway.
	PBE.Init( this );

	HWND hFirstEdit = GetDlgItem( m_hDlg, IDC_pwdPBEA );
	SetFocus( hFirstEdit );

	bool bTravellerMode = ( ( m_keyGenFlags & FLAG_TRAVELER_MODE ) != 0 );
	EnableWindow( GetDlgItem( m_hDlg, IDC_btnAdvanced ), !bTravellerMode );

	if ( hCmbType == NULL ) // It is not advanced dialog
	{
		return true;
	}

	PKE.Init( this );
	SSS.Init( this );


	// Init key block type
	ComboBox_AddStringT( hCmbType, LoadStringOnceT( IDS_szPBE ) );
	ComboBox_AddStringT( hCmbType, LoadStringOnceT( IDS_szPKE ) );
	ComboBox_AddStringT( hCmbType, LoadStringOnceT( IDS_szSSS ) );

	ComboBox_SetCurSel( hCmbType, m_kbParams->kbType - 1 );

	if ( m_caption  != NULL ) SetWindowTextT( m_hDlg, m_caption );
	if ( m_fileName != NULL ) SetWindowTextT( GetDlgItem( m_hDlg, IDC_TextName), m_fileName );
	if ( m_pathName != NULL ) SetWindowTextT( GetDlgItem( m_hDlg, IDC_TextPath), m_pathName );


	// Set the default push button to "Cancel" 
	SendMessage( m_hDlg, DM_SETDEFID, (WPARAM) IDCANCEL, (LPARAM) 0); 

	// It allows to set required focus after the dialog initialisation
	PostMessage( m_hDlg, WM_ShowItems, 0, 0 );
//	ShowItems();

	return true;
}

HWND FindSpinBuddy( HWND hDlg, HWND hEdit )
{
	HWND hItem = GetNextDlgGroupItem( hDlg, hEdit, false );

	if ( (HWND)SendMessage( hItem, UDM_GETBUDDY, 0, 0) == hEdit )
		return hItem;

	return NULL;
}

void UpdateSpinData( HWND hDlg, HWND hEdit )
{
	HWND hSpin = FindSpinBuddy( hDlg, hEdit );
	if ( hSpin == NULL )
		return;

	short min = 0,max = 0;
	cDlg::Spin_GetRange( hSpin, &min, &max );

	if ( !min && !max )
		return;

	char text[ 100 ];

	GetWindowText( hEdit, text, sizeof( text ) );
	int pos = atoi( text );

	if ( pos > max )
		pos = max;
	else
	if ( pos < min )
		pos = min;
	else
		return;

	cDlg::Spin_SetPos( hSpin, pos );
	SetFocus( hEdit );
}

BOOL cDlgKBParams::OnCommand( WPARAM wParam, LPARAM lParam )
{
	if ( LOWORD( wParam ) == IDC_btnAdvanced )
	{
		EndDialog( m_hDlg, ERROR_AdvancedDlgRequired );
		return true;
	}

	if ( HIWORD (wParam) == EN_KILLFOCUS )
		UpdateSpinData( m_hDlg, (HWND)lParam );


	if ( HIWORD (wParam) == EN_SETFOCUS )
	{
		HWND hEdit = (HWND)lParam;
	}

	if ( HIWORD (wParam) == CBN_SELCHANGE ) 
	{
		if ( LOWORD( wParam ) == IDC_cmbType )
		{
			m_kbParams->kbType = (ekbType)( ComboBox_GetCurSel( GetDlgItem( m_hDlg, IDC_cmbType )) + 1 );
			ShowItems();
		}
	}

	if ( wParam == IDCANCEL )
	{
		EndDialog( m_hDlg, (int)ERROR_CANCELLED_BY_USER); 
		return TRUE;
	}

	cPageKBParams *curPage = CurPage();
	
	return curPage->OnCommand( wParam );
}

LRESULT cDlgKBParams::RealDlgProc( UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	switch (uMsg) 
	{ 
		case WM_ShowItems:
			ShowItems();
			break;

        case WM_INITDIALOG:
			return OnInitDialog();

		case WM_userUpdateRanges:
			SSS.UpdateRanges();
			break;

		case WM_NOTIFY:
		{
			int res = OnNotify( (LPNMHDR)lParam );
			SetWindowLong( m_hDlg, DWL_MSGRESULT, res );
			return res;
		}
		case WM_DESTROY:

			PBE.Destroy();
			PKE.Destroy();
			SSS.Destroy();
			kbd.FreeTables( m_useSecureTyping );
			break;

        case WM_COMMAND: 
			return OnCommand( wParam, lParam );

		case WM_HELP:
			if ( ( CurPage() != NULL ) && ( CurPage()->m_MenuHelpIDs != NULL ) )
				OnHelpInfo( (HELPINFO*)lParam, CurPage()->m_MenuHelpIDs, false );
			break;

		default:
			return 0;
    }

	return 0;
}

