#pragma once

#include "cDlg.h"
#include "cKeyBlockParams.h"
#include "..\..\COMM_DEF\kbd.h"
#include "kbdEdit.h"
#include <assert.h>

class cDlgKBParams;

class cPageKBParams
{
	virtual bool InitV()		{ assert( false ); return false; }

public:
	cDlgKBParams *m_dlg;

	DWORD *m_MenuHelpIDs;

	HWND m_hDlg;
	bool m_useSecureTyping;

	cPageKBParams()
	{
		m_MenuHelpIDs = NULL;
		m_dlg = NULL;
		m_hDlg = NULL;
		m_useSecureTyping = false;
	}

	bool Init( cDlgKBParams *dlg );

	virtual void ShowItems( bool show ) { assert( false ); }
	virtual void Destroy()	{ assert( false ); }
	virtual bool OnCommand( WPARAM wParam )	{ assert( false ); return false; }
};

// PBE function are implemented at cDlgKBParamsPBE

class cPagePBE : public cPageKBParams
{
	cKbdEdit m_edtA;
	cKbdEdit m_edtB;

	bool InitV();
public:

	void ShowItems( bool show );
	void Destroy();
	bool OnCommand( WPARAM wParam );
};

// PKE function are implemented at cDlgKBParamsPKE

class cPagePKE : public cPageKBParams
{
	cWCStr m_searchedUserID;
	bool InitV();
public:
	void ShowItems( bool show );
	void Destroy();
	bool OnCommand( WPARAM wParam );

	bool OnBrowse();
	bool OnSearch();
	bool UpdateButtons();
};

// SSS function are implemented at cDlgKBParamsSSS
class cPageSSS : public cPageKBParams
{
	cKbdEdit m_edtA;
	cKbdEdit m_edtB;

	bool InitV();
public:

	HWND m_hWndSave;

	void ShowItems( bool show );
	void Destroy();
	bool OnCommand( WPARAM wParam );
	void UpdateRanges( bool init = false );
	void UpdateRange( int id, int min, int pos, bool init );
	int  Notify( NMHDR *phdr );
	bool AddShare();
	bool ResetAll();
	bool ViewShares();
};

class cDlgGetPassword : public cDlg
{
public:

	const wchar_t *m_caption;
	const wchar_t *m_fileName;
	const wchar_t *m_pathName;

	// Key board params 
	kbdParams kbd;
//	bool m_headerIsEncrypted;

	cDlgGetPassword( 
			const wchar_t *caption = NULL,
			const wchar_t *fileName = NULL,
			const wchar_t *pathName = NULL
		)
	{
		m_caption	= caption;
		m_fileName	= fileName;
		m_pathName	= pathName;
		m_useSecureTyping = false;
		m_keyGenFlags = 0;
//		m_headerIsEncrypted = false;
	}

	DWORD m_keyGenFlags;
	bool m_useSecureTyping;
};


class cDlgGetAdvancedFlags : public cDlgGetPassword
{
public:

	cKbdEdit m_edt;

	cDlgGetAdvancedFlags( ) : cDlgGetPassword()
	{
	}

	LRESULT RealDlgProc( UINT uMsg, WPARAM wParam, LPARAM lParam );
};

class cDlgGetPasswordAndMount : public cDlgGetPassword
{
public:

	cKbdEdit m_edt;
	cPassword m_pwd;

	// If the dialogue is used for mounting
	// the m_mountPoint is not null
	wchar_t *m_mountPoint;
	int m_posBrowseForMountPoint;
	bool m_bTravellerMode;

	cDlgGetPasswordAndMount( 
			const wchar_t *caption = NULL,
			const wchar_t *fileName = NULL,
			const wchar_t *pathName = NULL
			) : cDlgGetPassword( caption, fileName, pathName )
	{
		m_bTravellerMode = false;
		m_mountPoint = NULL;
		m_posBrowseForMountPoint = 0;
	}

	LRESULT RealDlgProc( UINT uMsg, WPARAM wParam, LPARAM lParam );

	void SetMountParams();
	bool GetMountParams();
	int SelectMountPoint( HWND hCmb );
	bool OnBrowseForMountPoint( HWND hCtrl );
	bool CheckValidMountPoint();
	bool checkEmptyFolder();
};

class cDlgKBParams : public cDlgGetPassword
{
public:

	cKeyBlockParams *m_kbParams;

	cDlgKBParams( 
			cKeyBlockParams *kbParams,
			const wchar_t *caption = NULL,
			const wchar_t *fileName = NULL,
			const wchar_t *pathName = NULL
		) : cDlgGetPassword( caption, fileName, pathName )
	{
		m_kbParams = kbParams;
	}

	LRESULT RealDlgProc( UINT uMsg, WPARAM wParam, LPARAM lParam );
	bool OnInitDialog();
	int OnNotify(  NMHDR *phdr );
	void ShowItems();
	BOOL OnCommand( WPARAM wParam, LPARAM lParam );

	cPagePBE PBE;
	cPagePKE PKE;
	cPageSSS SSS;

	cPageKBParams *CurPage();
};

#define WM_userUpdateRanges ( WM_USER + 10 )