#include "cDlgKBParams.h"
#include "resource.h" 
#include <windowsx.h>
#include "..\..\..\CertificateLib\publicKeyAlgs\publicKeyAlgs.h"
#include "..\..\..\CertificateLib\PGPPkts\Packets.h"
#include "..\..\..\BCArchive_V2\KM_Funcs.h"
#include "..\..\COMM_DEF\dllMain.h"
#include "commands.h"
//#include "HELP\HelpIds.h"

////////////////////////////
//
//	PKE Functions 
//

static int PKE_ItemTab[] = {
	IDC_stPKE_1,
	IDC_stPKE_2,
	IDC_stPKE_3,
	IDC_edtPKE_userID,
	IDC_edtPKE_keyID,
	IDC_edtPKE_algorithm,
	IDC_btnPKE_Browse,
	IDC_btnPKE_NetSearch
};

static int MenuHelpIDsT[] = {
	IDC_cmbType,				IDH_KeyBlockTypesPKE,
	IDC_grpType,				IDH_KeyBlockTypesPKE,
	IDC_edtPKE_userID,			IDH_edtPKE_userID,
	IDC_stPKE_1,				IDH_edtPKE_userID,
	IDC_edtPKE_keyID,			IDH_edtPKE_keyID,
	IDC_stPKE_2,				IDH_edtPKE_keyID,
	IDC_edtPKE_algorithm,		IDH_edtPKE_algorithm,
	IDC_stPKE_3,				IDH_edtPKE_algorithm,
	IDC_btnPKE_Browse,			IDH_btnPKE_Browse,
	IDC_btnPKE_NetSearch,		IDH_btnPKE_NetSearch,
	0,0
};

void cPagePKE::ShowItems( bool show )
{
	for ( int i = 0; i < sizeof( PKE_ItemTab )/sizeof( int ); i++ )
	{
		ShowWindow( GetDlgItem( m_hDlg, PKE_ItemTab[i] ), show ? SW_SHOW : SW_HIDE );
	}

	if ( show )
		UpdateButtons();
}

bool cPagePKE::InitV()
{
	m_MenuHelpIDs = (DWORD*)MenuHelpIDsT;

//	UpdateButtons();
	return true;
}

void cPagePKE::Destroy()
{
}

bool cPagePKE::OnCommand( WPARAM wParam )
{
	switch(wParam) 
	{
		case IDC_btnPKE_Browse:
			return OnBrowse();
		case IDC_btnPKE_NetSearch:
			return OnSearch();
		case IDOK: 
			EndDialog( m_hDlg, ERROR_NO );
			return true;
		default:
			return 0; 
	} 
}

bool cPagePKE::OnBrowse()
{
	m_searchedUserID.Free();

	if ( !KM_BrowseForPKBlock( m_hDlg, NULL, false, &m_dlg->m_kbParams->kbPKE.ptbKey, &m_dlg->m_kbParams->kbPKE.kff, NULL ) )
		return false;

	return UpdateButtons();
}

bool cPagePKE::OnSearch()
{
	m_searchedUserID.Free();

	if ( KM_SearchPKUser( m_hDlg, &m_dlg->m_kbParams->kbPKE.ptbKey, NULL, &m_searchedUserID ) != ppe_ok )
		return false;

	m_dlg->m_kbParams->kbPKE.kff = kffPGP;

	return UpdateButtons();
}

static bool ParsePublicKey(
	HWND hWnd,
	cPlainTextBlock *ptbKey,
	type_kff kff,
	HPKA *pPKA,
	cWCStr *wcsUserID,
	cPlainTextBlock *ptbKeyID )
{
	if ( *pPKA != NULL )
	{
		pka_free( *pPKA );
		*pPKA = NULL;
	}

	if ( ptbKey->bufferSize == 0 )
		return false;

	int userIDSize;
	int keyIDSize;

	HPKA pka = NULL;

	if ( kff == kffPGP )
	{
		//
		// Internal PGP public key or public subkey packets are used here 
		//	without main header 0x99.
		//

		sPublicKey *pk = (sPublicKey *)ptbKey->buffer;
		int pkSize = ptbKey->bufferSize;

		// Gets key ID
		keyIDSize = pgp_keyIDSize;
		if ( ptbKeyID->Allocate( keyIDSize ) == NULL )
			return false;

		if ( PKT_GetPublicKeyIDInternal( pk, pkSize, ptbKeyID->buffer )
			!= ppe_ok )
		{
			return false;
		}

		// Checks integrity of key packet.

		BYTE algID = -1;
		int publicMpiSize;

		int res = PKT_PublicKey_Parse( pk, pkSize, &publicMpiSize, &algID );
		if ( res != ppe_ok ) return false;

		//
		//	Create key handle.
		//
		bool old = pk->version < 4;
		int pkAlgID = old ? ((sPublicKeyV3 *)pk)->algID : pk->algID;
		BYTE *publicMpi = old ? ((sPublicKeyV3 *)pk)->mpi : pk->mpi;

		res = pka_createAlgorithm( &pka, algID, publicMpi, publicMpiSize );
		if ( res != ppe_ok ) return false;

		//
		//	Gets complete pgp user ID from BCDB, because the public key may be a subpacket of main key
		//	( PGP subpacket does not have user ID )
		//
		cPlainTextBlock tmpPtbKey;
		type_kff tmpKFF;
		if ( !KM_GetPKBlockByID( hWnd, ptbKeyID->buffer, ptbKeyID->bufferSize, false, &tmpPtbKey, &tmpKFF, wcsUserID ) )
			wcsUserID->Add( LoadStringOnceT( hMainInst, IDS_szUndefined ) );
	}
	else
	if ( kff == kffX509 )
	{
		//	Parses certificate public key.
		x509_certificate crt;
		int res = crt.parse( ptbKey->buffer, ptbKey->bufferSize );

		if ( res != x509e_ok )
			return false;

		// Generates user ID from subject RND 

		userIDSize = 0;
		res = crt.tbsCertificate.subjectName.GenerateName( NULL, &userIDSize );

		if ( res != x509e_bufferTooSmall )
			return false;

		cPlainTextBlock ptb;
		if ( ptb.Allocate( userIDSize ) == NULL )
			return false;

		res = crt.tbsCertificate.subjectName.GenerateName( (char *)ptb.buffer, &userIDSize );
		wcsUserID->Add( ptb.buffer, ptb.bufferSize );

		if ( res != x509e_ok )
			return false;


		//	Create public key handler ( use pka_free functions to free the handle ).

		res = pka_createAlgorithm_X509( &pka, &crt.tbsCertificate.subjectPKInfo );

		if ( res != x509e_ok )
			return false;

		// Generates local key ID

		keyIDSize = 0;
		res = pka_generateLocalKeyID( pka, NULL, &keyIDSize );

		if ( res != x509e_bufferTooSmall )
		{
			pka_free( pka );
			return false;
		}

		if ( ptbKeyID->Allocate( keyIDSize ) == NULL )
		{
			pka_free( pka );
			return false;
		}

		res = pka_generateLocalKeyID( pka, ptbKeyID->buffer, &keyIDSize );
		if ( res != x509e_ok )
		{
			pka_free( pka );
			return false;
		}
	}
	else
		return false;

	*pPKA = pka;

	return true;
}

bool cPagePKE::UpdateButtons()
{
	cPlainTextBlock  ptbKeyId;
	cWCStr wcsUserId;

	bool ok = false;

	if ( m_dlg->m_kbParams->kbPKE.ptbKey.bufferSize != 0 )
	{
		ok = ParsePublicKey(
			m_hDlg,
			&(m_dlg->m_kbParams->kbPKE.ptbKey),
			m_dlg->m_kbParams->kbPKE.kff,
			&(m_dlg->m_kbParams->kbPKE.pka),
			&wcsUserId,
			&ptbKeyId );

		if ( ok )
			m_dlg->m_kbParams->kbPKE.ptbKeyID.WriteBytes( ptbKeyId.buffer, ptbKeyId.bufferSize, true );
	}

	EnableWindow( GetDlgItem( m_dlg->m_hDlg, IDOK ), ok );

	wchar_t buff[ 256 ] = L"";
	if ( ok )
	{
		int algID = pka_getAlgID( m_dlg->m_kbParams->kbPKE.pka );

		// DSA algorithm may be used for signature only
		if ( !isPKEncryptionAlgorithm( algID ) ) 
		{
			m_dlg->DlgMessageBoxT( IDS_SignatureAlgorithmCannotBeUsedForEncryption, MB_ICONSTOP );
			ok = false;
		}

		int bitSize = pka_getModuleSize( m_dlg->m_kbParams->kbPKE.pka ) * 8;
		FormatT( buff, maxLen( buff ), LoadStringOnceT( hMainInst, IDS_szS_D_bits ), getWCStr( PKT_GetPublicKeyAlgName( algID ) ), bitSize );
	}

	SetDlgItemTextT( m_dlg->m_hDlg, IDC_edtPKE_algorithm, buff );

	bytes2string( buff, maxLen( buff ), ptbKeyId.buffer, ptbKeyId.bufferSize );
	SetDlgItemTextT( m_hDlg, IDC_edtPKE_keyID, buff );

	wchar_t *userID = L"";

	if ( !m_searchedUserID.isEmpty() ) // m_ptbSearchedUserID.bufferSize != 0 )
		userID = m_searchedUserID.m_szBody; // (char *)m_ptbSearchedUserID.buffer;
	else
		userID = wcsUserId.m_szBody; // (char *)ptbUserId.buffer;

	SetDlgItemTextT( m_hDlg, IDC_edtPKE_userID, userID );

	return ok;
}

//
//	End of PKE functions
//
//////////////////////////
