/*******************************************************************\
*
*	Copyright (c) 2006 Jetico, Inc., Finland
*	All rights reserved.
*
*	File: kg_defs.h
*	Revision: 
*	Created:
*	Description: declaration of error codes and flags
*		that are used to work with key generator DLLs
*
\*******************************************************************/

#ifndef BCMOD_V8_KG_DEFS_H
#define BCMOD_V8_KG_DEFS_H

#include "kg_errors.h"


#define PASSWORD_MINIMUM_LENGTH		8
#define PASSWORD_MAXIMUM_LENGTH		256
#define CONTAINER_ID_LENGTH			4
#define IVECTOR_LENGTH				8
#define IVECTOR_LENGTH_DB			DB_MAX_ENC_BLOCK_SIZE

#define SEED_LENGTH    128


/*******************************************************
 *
 *	Create flags that specify what the CreateKeyHandle()
 *	procedure has to do.
 *
 *******************************************************/

// new container is created, one password 
// will be used for opening it
#define CFLAG_CREATE_WITH_SINGLE_PASSWORD		0x00000001
#define CFLAG_CREATE_NEW_DATA_BLOCK				0x00000001

// The CreateKeyHandle procedure gets Data block that
// contains information allowing to verify password 
// entered by user. If the password is correct, load 
// encryption key and return Key Handle
#define CFLAG_VERIFY_AND_LOAD_KEY				0x00000002

// User wishes to change his password. Ask him to enter 
// old and new passwords, then correct Data block 
// according to the new password
#define CFLAG_CHANGE_PASSWORD					0x00000003


#define CFLAG_ADD_NEW_PASSWORD					0x00000004

#define CFLAG_OLD_DATA_REENCRYPTED_CONTAINER    0x00000005

#define CFLAG_REMOVE_ADDITIONAL_PASSWORD		0x00000006

#define CFLAG_ShowKeyBlockInfo					0x00000007

#define CFLAG_EncryptKeyBlock					0x00000008

#define CFLAG_SupportMode_LRW					0x00000009

#define CFLAG_SupportMode_XTS					0x0000000A

// Miscelleneous definitions
#define CFLAG_LEAST_AVAILABLE	CFLAG_CREATE_WITH_SINGLE_PASSWORD

#define CFLAG_MOST_AVAILABLE	CFLAG_SupportMode_XTS


#define CORRECT_CFLAG( cFlag ) (	(cFlag >= CFLAG_LEAST_AVAILABLE) && \
									(cFlag <= CFLAG_MOST_AVAILABLE) )


/*******************************************************
 *
 *	Advanced flags
 *
 *******************************************************/


#define	FLAG_CONTAINER_AUTO_MOUNT      0x00000001
#define	FLAG_CONTAINER_READ_ONLY       0x00000002
#define	FLAG_MOUNT_ON_SINGLE_LETTER    0x00000004

#define FLAG_ADVANCED_SECURE_TYPING    0x00010000
#define FLAG_ADVANCED_SHOW_PWD_CHARS   0x00020000

// If treal period finished then ReadOnly flag is 
// to be switche on and disabled.
#define FLAG_FORCE_READ_ONLY           0x00040000

#define FLAG_CREATE_WITH_NEW_SIGNATURE 0x00080000

//
//	Key generator may modify data block and assign the flag.
//	If the flag was assigned data block has to be updated
//
#define FLAG_DATA_BLOCK_WAS_MODIFIED    0x00100000
#define FLAG_DATA_BLOCK_4K_WAS_MODIFIED 0x00200000

//
//	Traveler mode driver does not allow mounting at mount point.
//	It mount to drive letter only.
//	The flag does not allow to add <Mount as subfolder ... > item in drivs combobox. 
//
//#define FLAG_DISABLE_MOUNT_POINT		0x00400000
#define FLAG_TRAVELER_MODE				0x00400000

//
//	Test mode for cDlgGetAdvancedFlags
//
#define FLAG_ADVANCED_TEST				0x80000000

// Several flags that indicates what function BestCrypt 
// wants to perform after receiving Key Handle from 
// the CreateKeyHandleEx() procedure.
//
// Key Generator module must clear the flag it gets 
// if the function is not allowed (for example, for 
// given user)

#define FLAG_ALLOW_MOUNT_CONTAINER    0x00000010
#define FLAG_ALLOW_CREATE_CONTAINER   0x00000020
#define FLAG_ALLOW_DELETE_CONTAINER   0x00000040
#define FLAG_ALLOW_PROPERTY_CONTAINER 0x00000080
#define FLAG_ALLOW_CHANGE_PASSWORD    0x00000100
#define FLAG_ALLOW_ADD_PASSWORD       0x00000200
#define FLAG_ALLOW_REMOVE_ADDITIONAL_PASSWORD 0x00000400


/*******************************************************
 *
 *	Flags that BestCrypt sends to Key Generator in 
 *  GetAdditionalCommandNames() function
 *
 *******************************************************/

#define FLAG_EVALUATION_VERSION          0x00000001
#define FLAG_EVALUATION_VERSION_EXPIRED  0x00000002


/*******************************************************
 *
 *	Flags that BestCrypt gets from the Key Generator when 
 *  the RunAdditionalCommand() function returns
 *
 *******************************************************/

// if the flag is set, BestCrypt Control Panel assumes that
// Data Block of the container was modified, so the program
// will update (re-write) it inside container file
#define FLAG_UPDATE_DATA_BLOCK          0x00000004

//
//	RunAdditionalCommand functions sends pointer to flag
//	If the flag has the bit it means that special sKGCmdParams structure follows 
//	directly after the DWORD flag, if the params was succesfully updated
//	state is stUpdated
//
#define FLAG_EXTENDED_PARAMS			0x00000008

//
//	The flag is used by additional commands 'Mount with separate key block'
//	the key block may be read from backuped key block or from container backup copy
//
#define FLAG_MOUNT_NOW					0x00000010

#endif
