/**
 *
 *	Copyright (c) 2009 Jetico, Inc., Finland
 *	All rights reserved.
 *
 *	file: container.h
 *	scope: v8 format
 *	author: evgeny yakovlev
 *
 *	description:
 *	Definition of container-related interfaces.
 *
 *	history:
 *
 */

#ifndef V8_CONTAINER_H
#define V8_CONTAINER_H

#include <stdlib.h>
#include <string.h>

#include "datablock.h"
#include "kgghost.h"

#include <common/build.h>
#include <common/types.h>

#include <alg/alg.h>

#include <container/format.h>
#include <container/factory.h>


namespace kgghost
{

	/**
	 * @brief 	v8 format adaptor
	 */
	class V8Format : public bdk::IBCContainerFormat
	{
	public:


		V8Format(kgghost::DATA_BLOCK const & data, bc_ulong_t total_size) :
			m_db(data),
			m_total_size(total_size)
		{
		};


		BC_INHERITED	std::string name() const
		{
			return "BCv8";
		}


        	BC_INHERITED 	bc_ulong_t size_bytes () const
		{
       			return m_db.header()->layout.length;
		}


		BC_INHERITED	void set_size_bytes(bc_ulong_t size)
		{
       			m_db.header()->layout.length = size;
		}


       		BC_INHERITED 	std::string description () const
		{
       			return std::string((char*)m_db.header()->description);
		}


		BC_INHERITED	void set_description(std::string const & desc)
		{
			memset(m_db.header()->description, 0, CONTAINER_DESC_LENGTH * sizeof(*m_db.header()->description));
			strncpy((char*)m_db.header()->description, desc.c_str(), CONTAINER_DESC_LENGTH);
		}


       		BC_INHERITED 	bc_alg_id_t algorithm_id () const
		{
       			return m_db.header()->contAlgId;
		}


		BC_INHERITED 	void set_algorithm_id(bc_alg_id_t id)
		{
       			m_db.header()->contAlgId = id;
		}


		BC_INHERITED 	bc_alg_mode_id_t algorithm_mode_id() const
		{
			return m_db.header()->encModeId;
		}


		BC_INHERITED 	void set_algorithm_mode_id(bc_alg_mode_id_t mod)
		{
			m_db.header()->encModeId = mod;
		}


	        BC_INHERITED 	bool is_locked () const
		{
        		return (m_db.header()->busiedFlag[0] == 'B');
		}


		BC_INHERITED 	void lock()
		{
			m_db.header()->busiedFlag[0] = 'B';
		}


		BC_INHERITED 	void unlock()
		{
			m_db.header()->busiedFlag[0] = 0xEB;
		}


		BC_INHERITED	void serialize(bdk::IBCContainerIO* dest) const;

		BC_INHERITED	bdk::BCContainerViewHandle create_view();


		BC_INHERITED	bdk::BCContainerViewHandle create_hidden_view(bdk::IBCContainerIO* io, bc_ulong_t size);


		BC_INHERITED 	bdk::BCContainerViewHandle get_view(bdk::IBCContainerIO* io);


		BC_INHERITED	bdk::BCContainerViewHandle update_view(bdk::IBCContainerIO* io, bdk::bc_view_change_t op);


	protected:


		bdk::BCContainerViewHandle init_view( bdk::IBCContainerIO* io, key_info_t const & kinfo, bdk::BCAlgHandle alg );


	private:


		/* full ghost data block */
		kgghost::DATA_BLOCK	m_db;

		/* real deserealized size */
		unsigned int		m_total_size;
	};


	/**
	 * @brief	V8 container factory
	 */
	class V8Factory : public bdk::IBCContainerFactory
	{
	public:


		BC_INHERITED 	bool parse(bdk::IBCContainerIO* src);


		BC_INHERITED	bdk::IBCContainerFormat* load(bdk::IBCContainerIO* src);


		BC_INHERITED	bdk::IBCContainerFormat* make(bdk::bc_container_info_t const & info);

	};

}



#endif

