#include "datablock.h"
#include "kg_errors.h"
#include "keyblock.h"

#include <alg/alg.h>

using namespace kgghost;
using namespace bdk;

/*
static unsigned int keysize_to_keyblocks(DWORD size)
{
	return (size + DB_MIN_ENCRYPTED_KEY_BLOCK_SIZE - 1) / DB_MIN_ENCRYPTED_KEY_BLOCK_SIZE;
}
*/

static keyblock_descriptor_t empty_descriptor()
{
	return keyblock_descriptor_t(0, 0, KEYBLOCK_UNDEFINED);
}


static keyblock_descriptor_t undefined_descriptor(unsigned int base, unsigned int total)
{
	return keyblock_descriptor_t(base, total, KEYBLOCK_UNDEFINED);
}


keyblock_descriptor_t::keyblock_descriptor_t() :
	base_slot(0),
	total_slots(0),
	type(KEYBLOCK_UNDEFINED),
	hidden(false)
{

}


keyblock_descriptor_t::keyblock_descriptor_t(unsigned int base, unsigned int total, bcu16_t type, bool hidden /*= false*/) :
	base_slot(base),
	total_slots(total),
	type(type),
	hidden(hidden)
{

}


keyblock_descriptor_t keyblock_map_t::parse_keyblock_at(unsigned int kb)
{
	bcu16_t type = map[kb].type;

	// empty region 
	if( type == SLOT_EMPTY ) 
	{
		return empty_descriptor();
	}
	// key body, find head and rerun
	else if( type == SLOT_TAIL) 
	{
		while(map[kb--].type == SLOT_TAIL)
			;
		
		return this->parse_keyblock_at(kb);
	}
	// key head
	else
	{
		// count tail length
		unsigned int tail_length = 0;
		unsigned int max_tail = this->capacity() - kb;
		while(tail_length < max_tail)
		{
			if(map[kb + tail_length + 1].type != SLOT_TAIL) 
				break;

			++tail_length;
		}

		// create descriptor
		return keyblock_descriptor_t(kb, tail_length + 1, type);
	}
}


keyblock_descriptor_t keyblock_map_t::get_free_keyblock(unsigned int size)
{
	unsigned int cur_slot = 0;
	keyblock_descriptor_t kd_free;

	while(cur_slot < this->capacity())
	{
		if(map[cur_slot].type == SLOT_EMPTY)
		{
			kd_free = this->get_free_keyblock_from(cur_slot);

			if(kd_free.total_slots >= size) return kd_free;
			else cur_slot += kd_free.total_slots;
		}
	}

	return kd_free;
}


keyblock_descriptor_t keyblock_map_t::get_max_free_keyblock()
{
	unsigned int cur_slot = 0;
	keyblock_descriptor_t kd, kd_max;

	while(cur_slot < this->capacity())
	{
		if(map[cur_slot].type == SLOT_EMPTY)
		{
			kd = this->get_free_keyblock_from(cur_slot);
			cur_slot += kd.total_slots;

			if(kd.total_slots >= kd_max.total_slots) kd_max = kd;
		}
	}

	return kd_max;
}


keyblock_descriptor_t keyblock_map_t::get_free_keyblock_from(unsigned int pos)
{
	unsigned int end = pos;

	while(end < this->capacity())
	{
		if(map[end].type != SLOT_EMPTY)
			break;

		++end;
	}

	return undefined_descriptor(pos, end - pos);
}


bool keyblock_map_t::check_free_keyblock(unsigned int kb_num)
{
	return (this->get_max_free_keyblock().total_slots >= kb_num);
}


void db_header_t::normalize()
{
	memset( busiedFlag, 0, sizeof( busiedFlag ) );
	memcpy( volumeLabel, DB_HEADER_VOLUME_LABEL, sizeof( volumeLabel ) );
	memcpy( signature, DB_HEADER_SIGNATURE, sizeof( signature ) );
	memset( busiedName, 0, sizeof( busiedName ) );
}


namespace kgghost 
{

/*
	BOOL DataBlock_Allocate(DATA_BLOCK** ppDataBlock, DWORD* pDataSize)
	{

		if ( *pDataSize < sizeof(DATA_BLOCK) ) 
			*pDataSize = sizeof(DATA_BLOCK);

		*pDataSize = ROUND( *pDataSize, _4K );

		if ( *pDataSize > DB_MAX_SIZE )
			*pDataSize = DB_MAX_SIZE;
	
		*ppDataBlock = (DATA_BLOCK*) malloc(*pDataSize);
	
		if (!(*ppDataBlock)) return FALSE;

		ShredData((bc_byte_t*)(*ppDataBlock), *pDataSize );

		return TRUE;
	}


	BOOL DataBlock_ReAllocate(DATA_BLOCK** ppDB, DWORD oldSize, DWORD* pNewSize )
	{
		*pNewSize = ROUND( *pNewSize, _4K );

		if ( oldSize > *pNewSize )
			ShredData((bc_byte_t*)*ppDB  + *pNewSize, oldSize - *pNewSize );

		DATA_BLOCK	*pDBNew;

		if ( !DataBlockAllocate( &pDBNew, pNewSize ) )
			return FALSE;
	
		memcpy( pDBNew, *ppDB, oldSize );

		DataBlockFree( ppDB, &oldSize );
	
		*ppDB = pDBNew;
	
		return TRUE;
	}


	BOOL DataBlock_Free( DATA_BLOCK** ppDataBlock, DWORD *pDataSize )
	{
		if ( pDataSize != 0 )
		{
			ShredData( (bc_byte_t*)*ppDataBlock, *pDataSize );
		}

		free(*ppDataBlock);
		*ppDataBlock = NULL;

		return TRUE;
	}


	BOOL DataBlock_Init(DATA_BLOCK* db, DWORD dbSize, DWORD algId, DWORD encModeId, DWORD hashAlgId,	space_unit_t* pContInfo)
	{
		ShredData((bc_byte_t*)db, dbSize); // fills DataBlock by random garbage
	
		db_header_t* hdr = db->header();

		hdr->normalize(); // fill unstable fields by default values

		hdr->version 	= DB_FORMAT_VERSION;
		hdr->keyGenId 	= KEYGEN_ID;
		hdr->contAlgId 	= algId;
		hdr->encModeId 	= encModeId;
		hdr->hashAlgId 	= hashAlgId;
		hdr->layout 	= *pContInfo;

		memset( hdr->description, 0, sizeof(hdr->description) );

		return TRUE;
	}
*/

	DWORD DataBlock_Check(DATA_BLOCK *pDataBlock, DWORD dbSize)
	{
		BC_ASSERT(pDataBlock);
		BC_ASSERT((dbSize >= DB_MIN_SIZE) && (dbSize <= DB_MAX_SIZE));

		if ((dbSize < DB_MIN_SIZE) || (dbSize > DB_MAX_SIZE)) {
			return ERROR_DAMAGED_DATA_BLOCK;
		}
	
		if(pDataBlock->header()->maybe_encrypted())
			return ERROR_NO; // can't check anything more than a size

		if(strncmp((const char*)pDataBlock->header()->signature, DB_HEADER_SIGNATURE, strlen(DB_HEADER_SIGNATURE)) != 0)
		{
			BC_DEBUG_PRINT(("Debug: v8: data block check failed at signature (%s)\n", pDataBlock->header()->signature));
			return ERROR_DAMAGED_DATA_BLOCK;
		}

		if ( pDataBlock->header()->version != DB_FORMAT_VERSION )
			return ERROR_INCORRECT_FORMAT_VERSION;

		return ERROR_NO;
	}


	keyblock_descriptor_t DataBlock_ParseKeyblock( DATA_BLOCK* db, unsigned int slot_num, unsigned int total_slots)
	{
		keyblock_descriptor_t kd;

		if( !db->header()->maybe_encrypted() )
		{
			kd = db->header()->keyMap.parse_keyblock_at(slot_num);
	
			// no defined keyblock at this spot? set it to point at free space remaining
			if(kd.is_empty())
			{
				kd = db->header()->keyMap.get_free_keyblock_from(slot_num);
			}
		}
		// if header is encrypted we can't read key map correctly. construct descriptor manually.
		else 
		{
			kd = undefined_descriptor(slot_num, total_slots - slot_num);
		}

		return kd;
	}


	unsigned int DataBlock_KeyblockCapacity(DWORD dbSize)
	{
		BC_ASSERT((dbSize >= DB_MIN_SIZE) && (dbSize <= DB_MAX_SIZE));
		dbSize -= DB_MIN_SIZE;

		return dbSize / DB_KEYBLOCK_SLOT_SIZE;
	}


	DWORD DataBlock_VerifyKeyblock(DATA_BLOCK* db, DWORD dbSize, const cPassword& pwd, key_info_t& in_out_key)
	{
		keyblock_descriptor_t kb_desc;

		unsigned int total_slots = DataBlock_KeyblockCapacity(dbSize);
		unsigned int cur_slot = 0;

		// iterate all possible slots until we hit correct decoding
		while( cur_slot < total_slots  )
		{
			if( !db->header()->maybe_encrypted() )
			{
				kb_desc = db->header()->keyMap.parse_keyblock_at(cur_slot);
	
				// no defined keyblock at this spot? set it to point at free space remaining
				if(kb_desc.is_empty())
				{
					kb_desc = db->header()->keyMap.get_free_keyblock_from(cur_slot);
					if(kb_desc.total_slots > (total_slots - cur_slot))
						kb_desc.total_slots = total_slots - cur_slot;
				}
			}
			// if header is encrypted we can't read key map correctly. construct descriptor manually.
			else 
			{
				kb_desc = undefined_descriptor(cur_slot, total_slots - cur_slot);
			}


			BC_DEBUG_PRINT(("Debug: v8: Processing keyblock %d\\%d: type 0x%x, number of slots %d\n", 
					cur_slot + 1, total_slots,
					kb_desc.type, 
					kb_desc.total_slots
			));


			// try to decode current key block
			CKeyBlock keyblock((bc_byte_t*)&db->key_tab[cur_slot], kb_desc.total_slots * DB_KEYBLOCK_SLOT_SIZE, kb_desc.type);
			if( keyblock.decode(in_out_key, pwd))
				return ERROR_NO;

			// failed, move on
			if( (kb_desc.type == KEYBLOCK_UNDEFINED) ) {
				cur_slot++;
			} else {
				cur_slot += kb_desc.total_slots;
			}
		}

		// nothing was found
		return ERROR_INCORRECT_PASSWORD;
	}


	DWORD DataBlock_GetKeyHandle(
			DATA_BLOCK		*db,
			DWORD			kblock_size,
			cPassword const &	pwd,
			BCAlgHandle&		out_handle,	
			key_info_t&		kinfo
		)
	{
		DWORD rc;

		keyblock_descriptor_t kd;

		// verify params and get key info
		rc = DataBlock_VerifyKeyblock(db, kblock_size, pwd, kinfo);
		if( rc != ERROR_NO ) 
			return rc;


		// initialize algorithm from key info
		if ( !kinfo.is_for_hidden_part() )
		{
			out_handle = BCAlg::init(kinfo.alg_id, kinfo.mode_id, (unsigned char*)kinfo.key, db->pool);
			
			// TODO:
			// add verification here!
			//

			ShredData((bc_byte_t*)kinfo.key, sizeof(kinfo.key));
		} 
		else 
		{ // this is a hidden part key

/*
			CEncBlock < sVerificationBlock > *verifyBlock = ( CEncBlock < sVerificationBlock > * )( cur_keyblock + kbSize );

			int verifyBlockSize = sizeof( *verifyBlock );

			if ( kbSize + verifyBlockSize > kbEncryptedSize )
			{
				assert( false );
				continue;
			}

			CBCAlg_Handle alg;
			if( !bc_init_alg( alg, pKeyInfo->keyAlgId, pKeyInfo->encModeId, pKeyInfo->key, dataBlock->pool ) )
				return ERROR_INVALID_ALGORITHM;


			if ( !verifyBlock->verify(alg) )
				return ERROR_INVALID_ALGORITHM;
			

			kbSize += verifyBlockSize;
			
			copyEx < dbHeader > ( pDecryptedHeader, NULL );
			copyEx < sKeyInfo > ( pKeyInfoCopy, pKeyInfo );
			copyEx < dbContInfo > ( pContInfo, &(pKeyInfo->contInfo) );

			if ( keyMapInfo != NULL )
			{
				keyMapInfo->setInfo( foundNum, 0, kbParams->kbType, kbSize );
			}
*/
		}
		
		return ERROR_NO;	
	}	

} // namespace kgghost

#if 0

DWORD DataBlockWriteToKeyTable(
		DATA_BLOCK	**ppDB,
		DWORD		*pDBSize,
		int		num,
		BYTE		*kb,
		DWORD		kbSize
	)
{
	bool isEncrypted = (*ppDB)->header.m_body.IsEncrypted();

	if ( !isEncrypted )
	{
		DWORD sizeRequired = (*ppDB)->header.m_body.getRealSize(); //(BYTE*)((*ppDB)->keyTab[ num ][ 0 ]) - (BYTE*)(*ppDB)

		if ( sizeRequired == DWORD(-1) )
		{
			assert( false );
			return ERROR_INTERNAL_PROBLEM;
		}
		
		sizeRequired += kbSize;

		if ( sizeRequired > *pDBSize )
		{
			if ( !DataBlockReAllocate( ppDB, *pDBSize, &sizeRequired ) )
			{
				assert( false );
				return ERROR_INTERNAL_PROBLEM;
			}

			*pDBSize = sizeRequired;
		}
	}

	memcpy( (BYTE*)(&((*ppDB)->keyTab[ num ][ 0 ])), kb, kbSize );

	return ERROR_NO;
}


DWORD DataBlockWriteKey(DATA_BLOCK**		ppDB,
			DWORD*			pDBSize,
			key_info_t*		key,
		
			// optional
			key_descriptor_t*	key_descriptor,
			kbproc_params_t*	kbParams,
			keyblock_map_t*		keyMapT,
			bool			bEncryptIt,
			db_header_t*		decHeader
	)
{
	DATA_BLOCK *db = *ppDB;
	

	// key should be ready
	BC_ASSERT(key == NULL);
	if( key == NULL ) return ERROR_INTERNAL_PROBLEM;


	key_descriptor_t kdTmp;
	if ( key_descriptor == NULL ) {
		key_descriptor = &kdTmp;
	}


	// Allocates place for decrypted and temporary data.

	cPlainTextBlock ptb;

	int encodedKeySize = 0;
	
	if ( kbParams != NULL )
		encodedKeySize = kbParams->getEncodedKeySize( keyInfo->size );

	//
	//	Allocates maximum size datablock
	//
	DWORD dbSize = DB_MaxSize;

	if ( ptb.Allocate(
			dbSize + 
			encodedKeySize +
			sizeof( key_t ) +
			0 ) == NULL )
	{
		assert( false );
		return ERROR_INTERNAL_PROBLEM;
	}

	int pos = 0;

	DATA_BLOCK* 	dbTmp 		= (DATA_BLOCK *)ptb.buffer;
	BYTE* 		kbEncoded 	= (ptb.buffer + dbSize);
	key_t* 		keyTmp 		= (key_t *)(ptb.buffer + dbSize + encodedKeySize);

	// Decrypts cont info and copies it to temporary buffer

	memcpy( (BYTE *)dbTmp + sizeof( dbTmp->header ), (BYTE *)db + sizeof( db->header ), *pDBSize - sizeof( db->header ) );
	ShredData( (BYTE *)dbTmp + *pDBSize, dbSize - *pDBSize );
	ShredData( kbEncoded, encodedKeySize );

	DWORD dwRes = decryptAndVerifyHeader( db, keyInfo, &(dbTmp->header) );
	
	if ( ( dwRes != ERROR_NO ) &&
		( dwRes != ERROR_INCORRECT_BLOCK_SIGNATURE ) )
	{
		return dwRes;
	}



	if( key_descriptor != NULL ) 
	{
		dbTmp->remove_key(key_descriptor); // delete existing key, if any
	}
	else
	{
		dbTmp->create_key_vacant(key_descriptor); // create new key in vacant space
	}



	// Frees existing key block ( shreds datablock and marks empty key map table )
	if ( keyMapInfo->position != -1 )
	{
//		dbHeader::MarkEmpty( dbTmp->header.m_body.keyMap, keyMapInfo->position, keyMapInfo->quantity );
		dbHeader::freeKeyBlockChain( dbTmp, dbSize, dbTmp->header.m_body.keyMap, keyMapInfo );
		keyMapInfo->setUndefined();
	}

	if ( kbParams != NULL )
	{
		if ( key_map == NULL )
			key_map = dbTmp->key_map();

		if( !key_map->check_vacant_space(req) ) 
			return ERROR_NOT_ENOUGH_SPACE_FOR_KEY;


		if ( !kbParams->encode_key(kbEncoded, encodedKeySize, key, sizeof(*key)) )
		{
			return ERROR_INTERNAL_PROBLEM;
		}

/*
#ifdef BC_DEBUG
		bool res = kbParams->kbType == kbtPKE; // public key cannot verify it's own encryption
		while( !res )
		{
			res = kbParams->decodeKey( keyInfoTmp, sizeof( sKeyInfo ), kbEncoded, encodedKeySize );
			if ( res )
				res = ( memcmp( keyInfo, keyInfoTmp, keyInfo->size ) == 0 );

			assert( res );
		}
#endif
*/

		// Verify 
		if ( kbParams->kbType != kbtPKE ) // public key cannot verify it's own encryption
		{
			if ( !kbParams->decodeKey( keyTmp, sizeof(key_t), kbEncoded, encodedKeySize ) )
				return ERROR_INTERNAL_PROBLEM;

			if ( memcmp( key, keyTmp, key->size ) != 0 )
				return ERROR_INTERNAL_PROBLEM;
		}

		BYTE *kb = dbTmp->keyTab[ num ];

		// Encode keyInfo by kbParams directly to the place in data block was found.

		if ( kb + encodedKeySize > ( BYTE* )dbTmp + dbSize )
		{
			assert( false );
			return false;
		}

		memcpy( kb, kbEncoded, encodedKeySize );

		key->setInfo( num, 1, kbParams->kbType, encodedKeySize );

		dbHeader::MarkBusied( dbTmp->header.m_body.keyMap, keyMapInfo );
	}

	// Gets really busied size
	DWORD realSize = dbTmp->header.m_body.getRealSize();

	if ( realSize == DWORD(-1) )
	{
		assert( false );
		return ERROR_INTERNAL_PROBLEM;
	}

	DWORD rounded_4K_Size = Round( realSize, _4K );

	if ( rounded_4K_Size > dbSize )
	{
		assert( false );
		return ERROR_INTERNAL_PROBLEM;
	}

	// Checks if new block has to be extended
	if ( rounded_4K_Size > *pDBSize )
	{
		DataBlockFree( ppDB, pDBSize );

#ifdef _UseSimpleAndExetendedBlockOnly_
		rounded_4K_Size = DB_MaxSize;
#endif

		*pDBSize = rounded_4K_Size;

		if ( !DataBlockAllocate( ppDB, pDBSize ) ||
			( *pDBSize != rounded_4K_Size ) )
		{
			assert( false );
			return ERROR_INTERNAL_PROBLEM;
		}
	}
	else
	if ( ( rounded_4K_Size < *pDBSize )
#ifdef _UseSimpleAndExetendedBlockOnly_
			&& ( rounded_4K_Size == _4K )
#endif
		)
	{
/*
		wchar_t bf[2][100];
		const wchar_t *message = 
			IH.FormatOnceT(
					IDS_HPTruncate_SSQ, 
					GetSizeTextT( bf[0], maxLen( bf[0] ), rounded_4K_Size ),
					GetSizeTextT( bf[1], maxLen( bf[1] ), *pDBSize ) );

		int res = MessageBoxT( 0, message, IH.LoadStringOnceT( IDS_KeyGen ), MB_YESNO | MB_ICONQUESTION | MB_DEFBUTTON2 );
*/
/*
		if ( res == IDYES )
		{
			// Shreds and truncates tail of existing.
			ShredData( (BYTE *)*ppDB + rounded_4K_Size, *pDBSize - rounded_4K_Size );
			*pDBSize = rounded_4K_Size;
		}
*/
	}

	// Copies tmp data block to new or to old

	memcpy( *ppDB, dbTmp, *pDBSize );
	copyEx( decHeader, &(dbTmp->header.m_body ) );

	// Encrypts header or updates it's tail digest

	if ( bEncryptIt )
		return encryptHeader( *ppDB, dbTmp, keyInfo );
	else
	{
		if ( ! (*ppDB)->header.updateTail( keyInfo, (*ppDB)->pool ) )
		{
			assert( false );
			return ERROR_INTERNAL_PROBLEM;
		}

		return ERROR_NO;
	}
}


//
//	decryptAndVerifyHeader
//
DWORD decryptAndVerifyHeader(DATA_BLOCK* dataBlock, key_t* keyInfo, CEncBlock< dbHeader > * decryptedHeader)
{
	CBCAlg_Handle alg;
	if( !bc_init_alg(alg, keyInfo->keyAlgId, keyInfo->encModeId, keyInfo->key, dataBlock->pool) ) {
		return ERROR_INVALID_ALGORITHM;
	}

	DWORD err;

	DWORD hashAlgId = keyInfo->hashAlgId;

	if ( !decryptHeaderEx( dataBlock, &hashAlgId, alg, decryptedHeader, &err ) ) //, decryptedContInfo, mainKeyInfo ) )
		return err;

/*
	if ( pDelegateKeyHandle != NULL )
		cAlg.delegateHandle( pDelegateKeyHandle );
*/

	return ERROR_NO;
}
//
//	encryptHeaderEx
//		encrypts entire header or it's tail with digest by a key handle for usual part.
//		Note, if the key handle is for hidden part the usual part key is derived from decrypted hidden part.
//
//		1. If dataBlock is encrypted ( see DATA_BLOCK::header.isEncrypted )
//			1.1 it checks if key handle is valid for the data block by decryptHeaderEx.
//			1.2 If is valid the function copies dataBlock to output buffer and returns true.
//			If is not valid the function returns false
//		2. Encrypts header by the key handle for usual part
//
bool
	encryptHeaderEx(
		DATA_BLOCK*			dataBlock,
		CBCAlg_Handle			handle,
		CEncBlock <db_header_t>*	dbOut,			// Entire encrypted block with tail
		DWORD*				pErrorCode,	// if the function returns FALSE, look on the Error code
		bool				bEntire			// = true, Defines if entire or tail only encryption is used
	)
{
	// *pErrorCode = verifyKeyHandle( dataBlock, keyHandle, algId, encModeId );

	//if ( *pErrorCode != ERROR_NO )
	//	return false;


	cPlainTextBlock ptb;
	if ( ptb.Allocate(sizeof(DATA_BLOCK)) == NULL )
		return false;

	DATA_BLOCK *dbTmp = (DATA_BLOCK *)ptb.buffer;
	memcpy( dbTmp, dataBlock, sizeof(DATA_BLOCK) );


	// 1.

	*pErrorCode = ERROR_INCORRECT_BLOCK_SIGNATURE;

	if ( dataBlock->header()->is_encrypted() )
	{
		// 1.1
		if ( !decryptHeaderEx( dataBlock, handle, &dbTmp->header, pErrorCode ) )
			return false;

		// 1.2
		if ( bEntire )
		{
			if ( dbOut != &(dataBlock->header) )
				copyEx( dbOut, &(dataBlock->header) ); // , sizeof( *dbOut ) );
		}
		else
		{
			if ( !dbTmp->header.updateTail( hashAlgId, algId, encModeId, keyHandle ) )
				return false;

			copyEx< encBlock< dbHeader> >( dbOut, &(dbTmp->header) );
		//	memCopyEx( dbOut, dbTmp, sizeof( *dbOut ) );
		}

		*pErrorCode = ERROR_NO;
		return true;
	}

	// 2. 

	*pErrorCode = ERROR_INCORRECT_PASSWORD; // ERROR_INVALID_ALGORITHM;

	if ( bEntire )
	{
		if( !dbTmp->eblock_header.encrypt_from(dataBlock->header()) )
			return false;
	}
	else
	{
		if ( !dbTmp->header.updateTail( hashAlgId, algId, encModeId, keyHandle ) )
			return false;
	}

	*pErrorCode = ERROR_NO;
	copyEx< encBlock< dbHeader> >( dbOut, &(dbTmp->header) );

#ifdef _DEBUG
	if ( bEntire )
		return dbOut->decryptBodyTo( &(dbTmp->header.m_body), algId, encModeId, keyHandle );
	else
		return dbOut->verifyTail( algId, encModeId, keyHandle );
#endif

	return true;
}


//
//	decryptHeaderEx
//		decrypts header by a key handle, the key handle may be for usual part only
//			( hidden part key consists of both hidden and main parts keys ) see DataBlockVerifyKeyBlockAndGetKey
//	
//		1. If dataBlock is not encrypted ( see DATA_BLOCK::header.isEncrypted ) it verifies header
//			and copies dataBlock to output buffer and returns true.
//		2. Tries to decrypt dataBlock::contInfo with the algoritm id and key handle.
//			If the contInfo is properly decrypted, the function decrypts header with keyMap
//			with same key handler, copies decrypted result to out buff and returns true.
//		3. Otherwise returns false.
//
static DWORD decryptHeaderEx( 	
		DATA_BLOCK*		db,
		CBCAlg_Handle		handle,
		DWORD*			pHashAlgId,	// = NULL, [Out] pointer to hash algorithm decrypted from encBlock tail
		CEncBlock <dbHeader>*	headerOut,	// = NULL, Decrypted header, caller procedure is responsible for secure erasing of the decrypted data
)
{
	bool isEncrypted = db->header()->is_encrypted();

	DWORD hashAlgId = -1;

	if ( pHashAlgId == NULL )
		pHashAlgId = &hashAlgId;

	if ( !isEncrypted )
	{
		bool verified = db->header.verifyTail( handle, pHashAlgId );

		if ( verified )
		{
			copyEx < encBlock < dbHeader >  >( headerOut, &dataBlock->header );

			return ERROR_NO;
		}
		else
		{
			// May be the key handle for hidden part
			// we can verify it later.
		}
	}

	// 2.
	cPlainTextBlock ptbDecrypted;
	if ( ptbDecrypted.Allocate( sizeof( DATA_BLOCK ) ) == NULL )
	{
		assert( false );
		*pErrorCode = ERROR_INTERNAL_PROBLEM;
		return false;
	}

	DATA_BLOCK *dbDecrypted = ( DATA_BLOCK *)ptbDecrypted.buffer;
	memcpy( dbDecrypted, db, sizeof( DATA_BLOCK ) );

	if ( !dbDecrypted->header.decryptFrom( &db->header, contAlgId, encModeId, keyHandle ) )
	{
		*pErrorCode = ERROR_INCORRECT_BLOCK_SIGNATURE;

		return false;
	}

	//memCopyEx( headerOut, dbDecrypted, sizeof( *headerOut ) );
	copyEx( headerOut, &(dbDecrypted->header) );

	*pErrorCode = ERROR_NO;
	return true;
}



bool EncryptHeader(
		const wchar_t	*contPath,
		encBlock< dbHeader > *db,
		DWORD		dbSize,
		DWORD		*pErrCode,
		DWORD		*pFlags,
		sCKHParams	*ps
		//dbContInfo	*mainContInfo,
		//dbContInfo	*contInfo,
		//DWORD		*pContAlgId,	// [out], if container is encrypted the contAlgId value is -1,
		//							// but we have to return real algorithm id back
		//DWORD		*pEncModeId,	// [out], returns here container encryption mode id ( LRW or CBC )
		//DWORD		*pHashAlgId	// [out], returns here id of the hash algorithm is used for key block verification,
	)
{
	CBCAlg_Handle handle;
	
	sCKHParams	psTmp;
	if ( ps == NULL )
		ps = &psTmp;

	bool isEncrypted = db->m_body.IsEncrypted();

	bool res = 
		CreateHandle_MainPart( 
			hWnd,
			contPath,
			LoadStringOnceT( hMainInst, isEncrypted ? IDS_ENTER_PASSWORD_FOR_DECRYPTION : IDS_ENTER_PASSWORD_FOR_ENCRYPTION ),
			(DATA_BLOCK **)&db,
			&dbSize,
			NULL,
			&keyHandle,
			ps );

	if ( !res )
		return false;

	if ( isEncrypted )
	{
		res = decryptHeaderEx( 
				(DATA_BLOCK *)db,
				ps->contAlgId, ps->encModeId, &(ps->hashAlgId), keyHandle, 
				(encBlock< dbHeader > *)db,
				pErrCode );

//		*pHashAlgId = hashAlgId;
	}
	else
		res = encryptHeaderEx( 
				(DATA_BLOCK *)db,
				ps->contAlgId, ps->encModeId, ps->hashAlgId, keyHandle, 
				(encBlock< dbHeader > *)db,
				pErrCode,
				true );
	
	return res;
}



//
//	verifyKeyHandle
//		Any create key handle function has a ready key handle.
//		The function allows to verify the handle before using or bata block updating.
//
DWORD verifyHandle( DATA_BLOCK *db, CBCAlg_Handle handle, dbHeader *decHeaderOut )
{
	cPlainTextBlock ptb;

	encBlock < dbHeader > *decHeader = (encBlock < dbHeader > *)ptb.Allocate( sizeof( encBlock < dbHeader > ) );

	if ( decHeader == NULL )
		return ERROR_BUFFER_TOO_SMALL;

	DWORD err = ERROR_NO;

	if ( !decryptHeaderEx( db, algId, encModeId, NULL, keyHandle, decHeader, &err ) )
		return err;

	copyEx( decHeaderOut, &(decHeader->m_body ) );

	return ERROR_NO;
}

//
//	DataBlockWriteKey_Hidden
//
//		1. Writes hidden key info encrypted by kbParams
//		2. Writes simple verification block at the key tail encrypted by hidden key info
//
//	How the key read, see DataBlockVerifyKeyBlockAndGetKey
//
//	Note 1. The function does not extend key block for hidden part key. 
//			It will security nonsens if block is extended but key map reports that no one key presents in extended part.
//
//	Note 2. It shreds old key block information if keyMapInfo specified ( is not NULL or position != -1 )
//
DWORD 
	DataBlockWriteKey_Hidden(
		DATA_BLOCK			**ppDB,
		DWORD				*pDBSize,
		sKeyInfo			*hiddenKeyInfo,	//	= NULL	if null, so it does not required to save encoded key info
											//	the existing hidden part from started position is simple shreded
		cKeyBlockParams		*kbParams,		//	= NULL

		sKeyMapInfo			*keyMapInfo,	// = NULL,	See Note 2.

		dbKeyId				*keyMap			// the map is used for detecting busied items.
											// Ifnormation about existing hidden parts positions may be accumulated by and sent to the
											// function create new hidden part. See CreateHiddenPart
	)
{
	if ( keyMap == NULL )
	{
		if ( hiddenKeyInfo != NULL )
		{	assert( false );
			return ERROR_INTERNAL_PROBLEM;
		}
	}

	int hiddenPartKBSize = DB_MinEncryptedKeyBlockSize;
	int verifyBlockSize = sizeof( encBlock < sVerificationBlock > );
	
	if ( kbParams != NULL )
	{
		hiddenPartKBSize = kbParams->getEncodedKeySize( hiddenKeyInfo->size );
	}

	int kbSize = hiddenPartKBSize + verifyBlockSize;

	// allocaters space for temporary plain text data
	cPlainTextBlock ptb;

	if ( ptb.Allocate(
			kbSize +
//			sizeof( sKeyInfo ) +
			sizeof( sKeyInfo ) +
//			sizeof( encBlock< dbHeader > ) +
			0 ) == NULL )
	{
		assert( false );
		return ERROR_INTERNAL_PROBLEM;
	}

	int pos = 0;

	// Hidden part ket block position.
	BYTE *hpKB = ptb.buffer + pos; pos += hiddenPartKBSize;

	// Main part key block position ( will be encrypted by hidden part key ).
	encBlock< sVerificationBlock > *verifyBlock = ( encBlock< sVerificationBlock > *)(ptb.buffer + pos); pos += verifyBlockSize;

	sKeyInfo *keyInfoTmp	  = (sKeyInfo *)(ptb.buffer + pos); pos += sizeof( sKeyInfo );
//	encBlock< dbHeader > *decryptedHeader = (encBlock< dbHeader > *)(ptb.buffer + pos); pos += sizeof( *decryptedHeader );
	assert( ptb.bufferSize == pos );

	// Clears current busied map item ( it will be shredded later )

	if ( ( keyMapInfo->position != -1 ) && ( keyMap != NULL ) )
		dbHeader::MarkEmpty( keyMap, keyMapInfo->position, keyMapInfo->quantity );

	int blocksRequired = dbKeyId::GetRequiredQ( kbSize );

	int curKeyMapQ = DATA_BLOCK::getKeyQ( *pDBSize );

	// Find space inside current block size 
	// See Note 1.
	int newPosNum = dbHeader::findEmptySpace( keyMap, blocksRequired, 0, curKeyMapQ, true );

	if ( newPosNum == -1 )
		return ERROR_NOT_ENOUGH_SPACE_FOR_KEY; 
    
	// Prepares encoded key
	if ( ( kbParams != NULL ) && ( hiddenKeyInfo != NULL  ) )
	{
		if ( !kbParams->encodeKey( hpKB, hiddenPartKBSize, hiddenKeyInfo, sizeof( *hiddenKeyInfo ) ) ) // hiddenKeyInfo->size ) )
		{
			assert( false );
			return ERROR_INTERNAL_PROBLEM;
		}

		if ( kbParams->kbType != kbtPKE )
			if ( !kbParams->decodeKey( keyInfoTmp, sizeof( sKeyInfo ), hpKB, hiddenPartKBSize ) )
			{
				assert( false );
				return ERROR_INTERNAL_PROBLEM;
			}

		//
		//	Encodes and checks the encoding of verification block
		//
		sVerificationBlock vb;
		vb.hashAlgId	= hiddenKeyInfo->hashAlgId;
		vb.keyAlgId		= hiddenKeyInfo->keyAlgId;
		vb.encModeId	= hiddenKeyInfo->encModeId;

		if ( !verifyBlock->encryptFromBody( &vb, hiddenKeyInfo, (*ppDB)->pool ) ||
			 !verifyBlock->decryptBodyTo( &vb, hiddenKeyInfo, (*ppDB)->pool ) 
				 )
		{
			assert( false );
			return ERROR_INTERNAL_PROBLEM;
		}

		memset( &vb, 0, sizeof( vb ) );
	}

	// Shreads old hidden part key blocks 
	if ( ( keyMapInfo != NULL ) && ( keyMapInfo->position != -1 ) )
	{
		dbHeader::freeKeyBlockChain( *ppDB, *pDBSize, keyMap, keyMapInfo );
	}

	// Set new HP busied marks
	if ( keyMapInfo != NULL )
		keyMapInfo->setInfo( newPosNum, 0, kbParams->kbType, kbSize ); // kbtHP

	dbHeader::MarkBusied( keyMap, newPosNum, blocksRequired, kbParams->kbType, kbSize ); // kbtHP
	
	return DataBlockWriteToKeyTable( ppDB, pDBSize, newPosNum, hpKB, kbSize );
}


//
// DataBlockGetContainerId()
//		Container ID is initially randomized for main and hidden contInfo.
//		The info is not changed even if main contInfo is encrypted.
//
BOOL DataBlockGetContainerId( DATA_BLOCK *DataBlock, BYTE *ContainerId )
{
	memcpy( ContainerId, DataBlock->header.m_body.containerID, CONTAINER_ID_LENGTH );
	return true;
}


/*

bool dbContInfo::isForHiddenPart()
{
	return length == maxHPBlockSize;
}

bool dbContInfo::isValid()
{
	return length >= maxHPBlockSize;
}


//
//	encryptHeader
//		1. Checks if header is encrypted
//			1.1 if it is encrypted with the key simple copies eencrypted db to encrypted Buffer
//			1.2 if the key does not decrypts the "encrypted" db returns false
//		2. Encrypts the plain text db to enc buffer
//
//	returns error code
//
static DWORD encryptHeader( DATA_BLOCK *dbOut, DATA_BLOCK *dbIn, sKeyInfo *keyInfo )
{
	if ( dbIn->header.m_body.IsEncrypted() )
	{
		// Simple check decryption by the key info
		if ( !dbIn->header.decryptBodyTo( NULL, keyInfo, dbIn->pool ) )
			return ERROR_DAMAGED_DATA_BLOCK;

		if ( dbOut != dbIn )
			memcpy( dbOut, dbIn, sizeof( dbOut->header ) );

		return ERROR_NO;
	}

	cPlainTextBlock ptb;
	if ( ptb.Allocate( sizeof( encBlock< dbHeader > ) ) == NULL )
	{
		assert( false );
		return ERROR_INTERNAL_PROBLEM;
	}

	if ( !dbIn->header.encryptTo( ( encBlock< dbHeader > *)ptb.buffer, keyInfo, dbIn->pool ) )
	{
		assert( false );
		return ERROR_INVALID_ALGORITHM;
	}

	memcpy( dbOut, ptb.buffer, ptb.bufferSize );

	return ERROR_NO;
}






//
//	isValid
//		simple checks size, hashAlgId and keyAlgId
//
bool sKeyInfo::isValid()
{
	bool validSize = ( size <= sizeof( *this ) ) && ( size >= ( sizeof( *this ) - sizeof ( key ) ) );

	if ( !validSize )
		return false;

	int hashSize = 0;
	if ( ha_getDigestSize( &hashSize, hashAlgId ) != hae_ok )
		return false;

	if ( !Alg_GetAlgInfoByID(  keyAlgId, encModeId ) )
		return false;

	return true;
}

int sKeyInfo::getBufferSize( int keySize )
{
	return ( sizeWithoutKey + keySize );
}

int sKeyInfo::getAlgKeySizeBytes( int contAlgId, DWORD encModeId )
{
	DWORD keyLengthInBites = -1;

	if ( !Alg_GetAlgInfoByID( contAlgId, encModeId, NULL, 0, &keyLengthInBites ) )
		return -1;

	return ( keyLengthInBites + 7 ) / 8;
}

int sKeyInfo::getAlgKeySizeBytes()
{
	return getAlgKeySizeBytes( keyAlgId, encModeId );
}

int sKeyInfo::getKeySizeBytes()
{
	return size - sizeWithoutKey;
}


									

DWORD GenerateKey(
		BYTE *key,
		int   keyLength,
		BYTE *salt,
		int	  saltSize,
		BYTE *seed,
		int   seedSize,
		BYTE  hashAlgID,
		bool isKey // determintes is kenereted key or initial vector
	)
{
	int err = P12_producePseudoRandomBytes(
				key,  keyLength,
				salt, saltSize,
				1,
				seed, seedSize,
				isKey ? P12_prgm4key : P12_prgm4IV,
				hashAlgID
			);

	if ( err == x509e_ok )
		return ERROR_NO;

	if ( err == hae_invalidAlgorithmID )
		return ERROR_INCORRECT_HASH_ALGORITHM;

	//assert( false );
	return ERROR_INTERNAL_PROBLEM;

}



bool updateKeyParams( sCKHParams *ps, sKeyInfo *keyInfo, dbHeader *decHeader )
{
	bool equ = true;

	ps->contInfo = keyInfo->contInfo;

	if ( ( decHeader != NULL ) && !keyInfo->contInfo.isForHiddenPart() )
	{
		copyEx < dbHeader > ( &(ps->decryptedHeader), decHeader );
		assert( keyInfo->hashAlgId == decHeader->hashAlgId );
	}

	if ( keyInfo->keyAlgId  != ps->contAlgId ) { assert( ps->contAlgId == -1 ); ps->contAlgId = keyInfo->keyAlgId;  equ = false; }
	if ( keyInfo->encModeId != ps->encModeId ) { assert( ps->encModeId == 0 ); ps->encModeId = keyInfo->encModeId; equ = false; }
	if ( keyInfo->hashAlgId != ps->hashAlgId ) { assert( ps->hashAlgId == -1 ); ps->hashAlgId = keyInfo->hashAlgId; equ = false; }

	return equ;
}


//
//	CreateHandle_MainPart
//		Main part password required.
//
bool CreateHandle_MainPart(
		//HWND hWnd,
		const wchar_t *text,
		const wchar_t *caption,

		DATA_BLOCK			**ppDB,
		DWORD				*pDBSize,
		sKeyInfo			*keyInfo, // = NULL
		KEY_HANDLE			*pKeyHandle, // = NULL
		sCKHParams			*ps,		// = NULL
		dbKeyId				*keyMapT	// = NULL
	)
{

	if ( caption == NULL )
		caption = LoadStringOnceT( hMainInst, IDS_ENTER_EXISTING_PASSWORD );
	if ( text == NULL )
		text = LoadStringTwiceT( hMainInst, IDS_KeyBlockUpdating );


	CBCAlg_Handle handle;

	sCKHParams tmp;
	if ( ps == NULL )
		ps = &tmp;

	for(;;)
	{
		DWORD flags = 0;
		DWORD errCode = 0;
		DWORD	rights = RIGHT_DO_NOT_BACKUP_KEYS;

		//KEY_HANDLE keyHandle;


		if ( !GetKeyHandle( 
				hWnd, text, caption,
				ppDB, pDBSize,
				&errCode, &flags,
				&keyHandle,
				NULL,
				rights,
				ps,
				keyInfo )
			)
				return false;


		if ( !ps->contInfo.isForHiddenPart() )
		{

			//
			// Delegates key handle.
			//
			if ( pKeyHandle != NULL )
				*pKeyHandle = keyHandle;
			else
				FreeKeyHandle( ps->contAlgId, keyHandle );

			return true;

			
		}

		if ( keyMapT != NULL )
			dbHeader::MarkBusied( keyMapT, &(ps->keyMapInfo) );

		FreeKeyHandle( ps->contAlgId, keyHandle );


		MessageBoxT(
			hWnd,
			LoadStringOnceT( hMainInst, IDS_HP_PWD_CannotBeUsedForKeyBlockUpdating ),
			LoadStringTwiceT( hMainInst, IDS_WARNING ),
			MB_ICONEXCLAMATION  );

	}
}


//
//	FindSamePassword
//		Tryes to decode every key block in the data block with different algorithms
//		Returns true if at least one key block is compatible, otherwise return false.
//
bool FindSamePassword(
		DATA_BLOCK	*pDB,
		DWORD		dbSize,
		cPassword	*pwd,
		ekbType		*pKBType, // = NULL
		sEncIDs		*encIds // = NULL
	)
{
	if ( pKBType != NULL ) *pKBType = kbtEmpty;

	cKeyBlockParams kbParams;

	DWORD errorCode = 0;
	DWORD flags = 0;

	kbParams.kbPBE.m_pwd.Set( pwd );

	cEnumAlgIDs algIDs;
	cEnumEncModes encModes;
	cEnumHashAlgIDs hashAlgs;

	for( int i = algIDs.Q; i--; )
	{
		DWORD algId = algIDs.T[i];

		for ( int j = encModes.Q; j--; )
		{
			DWORD encModeId = encModes.T[j].id;

			for ( int k = hashAlgs.Q; k--; )
			{
				DWORD hashAlgId = hashAlgs.T[i];

				// The tricks allow to use same code if algorithm id is defined ( the data block header is not encrypted )
				if ( encIds != NULL )
				{
					i = j = k = 0;
					algId		= encIds->m_algId;
					encModeId	= encIds->m_encModeId;
					hashAlgId	= encIds->m_hashAlgId;
				}
				else
				if ( !pDB->header.m_body.IsEncrypted() )
				{
					i = j = k = 0;
					algId		= pDB->header.m_body.contAlgId;
					encModeId	= pDB->header.m_body.encModeId;
					hashAlgId	= pDB->header.m_body.hashAlgId;
				}

				kbParams.Init( NULL, 0, algId, encModeId, hashAlgId, &errorCode, &flags, 0 );

				errorCode = DataBlockVerifyKeyBlockAndGetKey( pDB, dbSize, &kbParams );

				if ( errorCode == ERROR_NO )
				{
					if ( pKBType != NULL ) *pKBType = kbParams.kbType;
					return true;
				}
			}
		}
	}

	return false;
}


*/



#endif // 0

