#include "UnicodeFuncs.h"
#include "kbdEdit.h"
#include "cDlgKBParams.h"
#include "cKeyBlockParams.h"
#include "resource.h"
#include <stdlib.h>
#include <windowsx.h>

//
// Funcs from pwdlg.cpp
//
//void cKbdEdit::InsertCharToBuffer( wchar_t Letter, bool IsLetter )
void cKbdEdit::InsertCharToBuffer( char mbLetter, bool IsLetter )
{
	wchar_t Letter = ( wchar_t )mbLetter;

	DWORD bufferSize = maxLen( m_buffer );

	// Send the EM_GETSEL message to the edit control.
	// The low-order word of the return value is the character
	// position of the caret relative to the first character in the
	// edit control.
	DWORD caretPos = (WORD)Edit_GetSel( m_hEdit );

	if ( caretPos >= bufferSize )
	   return;

	// caretPos is numbered from 1, so we have to decrement it
	if ( ( Letter != VK_BACK ) && (( Letter != VK_DELETE ) || IsLetter ) )
	   caretPos--;

	DWORD pwdLength = Edit_LineLength( m_hEdit, 0 ); 
	if ( pwdLength >= bufferSize )
	   return;

	if ( ( Letter == VK_BACK ) ||
		( Letter == VK_DELETE ) && ( !IsLetter ) )
	{
		for ( DWORD i = caretPos; i < pwdLength; i++ )
			m_buffer[i] = m_buffer[i+1];
	}
	else
	{
		if ( ( ( caretPos + 1 ) != pwdLength ) && ( pwdLength > 0 ) )
		{
			for( DWORD i = pwdLength - 1; i > caretPos ; i-- )
				m_buffer[i] = m_buffer[i-1];
		}

		m_buffer[ caretPos ] = Letter;
	}

	m_buffer[ pwdLength ] = 0;
}

char getRandomLetter()
{
	static int val = 0;
	if ( val == 0 )
	{
		srand( (UINT)time(0) );
		val = 1;
	}

    BYTE ticks = (BYTE)rand() & 0x3f;
    return (char)(0x20 + ticks);
}

LRESULT cKbdEdit::realEditProc( UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	LRESULT lpRes;
	char	letter;
	bool	isLetter  = false;
	UINT    virtualKey, scanCode;
	DWORD   retCharsNum;
	char    retChar1, retChar2;

	switch (uMsg)
	{
		case WM_INPUTLANGCHANGE:
		{
			if (dlg->m_useSecureTyping)
			{
				dlg->kbd.ResetEncodingTable( dlg->m_useSecureTyping );
				dlg->kbd.FreeTables( dlg->m_useSecureTyping );

				dlg->kbd.GenerateEncodingTable( dlg->m_useSecureTyping );
				dlg->kbd.SetEncodingTable( dlg->m_useSecureTyping );
			}

			break;
		}

		case WM_SETFOCUS:
			if ( !dlg->kbd.SetEncodingTable( dlg->m_useSecureTyping ) )
			{	// send NULL string to delete all tables, because problems occured 
				dlg->kbd.ResetEncodingTable( dlg->m_useSecureTyping );
				dlg->kbd.FreeTables( dlg->m_useSecureTyping );
				MessageBoxT( GetParent(m_hEdit), L"Error in setting encoding table", L"kbdEdit - editSubclassProc", MB_ICONSTOP );
				EndDialog( GetParent(m_hEdit), ERROR_INTERNAL_PROBLEM );
			}
			break;

		case WM_KILLFOCUS:
			if (!dlg->kbd.ResetEncodingTable( dlg->m_useSecureTyping ))
			{	// send NULL string to delete all tables, because problems occured
				dlg->kbd.FreeTables( dlg->m_useSecureTyping );
			}
			break;

		case WM_KEYDOWN:
			
			virtualKey = (UINT)wParam;
			scanCode = (UINT)((lParam >> 16) & 0xFF);

			dlg->kbd.Wm_KeyDown( virtualKey, dlg->m_useSecureTyping, scanCode );
			dlg->kbd.IncrementCharacterOrderCNTRL( (int)wParam, dlg->m_useSecureTyping );
			break;
		
		case WM_DEADCHAR:
			dlg->kbd.Wm_DeadChar();
			break;

		case 0x0109: // WM_UNICHAR:
		{
			assert( false );
			break;
		}

		case WM_CHAR:
		{
			letter = (char)wParam;

			wchar_t wcs[ 10 ] = L"";
			
			retCharsNum = maxLen( wcs );

			if (dlg->kbd.Wm_Char( letter, dlg->m_useSecureTyping, &retCharsNum, &retChar1, &retChar2, wcs ))
			{
				if ( ((dlg->m_keyGenFlags & FLAG_ADVANCED_SHOW_PWD_CHARS) != 0 ) ||
					 ((dlg->m_keyGenFlags & FLAG_ADVANCED_TEST) != 0 )
					)
				{
					if (retCharsNum == 2)
					{
						wParam = (WPARAM)retChar2;
						KBD_CallWindowProc( (WNDPROC)m_origWndProc, m_hEdit, uMsg, wParam, lParam);
						InsertCharToBuffer( retChar2 );
						wParam = (WPARAM)retChar1;
						lpRes = KBD_CallWindowProc( (WNDPROC)m_origWndProc, m_hEdit, uMsg, wParam, lParam);
						InsertCharToBuffer( retChar1 );
					}
					else
					{
						wParam = (WPARAM)retChar2;
						lpRes = KBD_CallWindowProc( (WNDPROC)m_origWndProc, m_hEdit, uMsg, wParam, lParam);
						InsertCharToBuffer( retChar2 );
					}
				}
				else if (dlg->m_useSecureTyping)
				{
					if (retCharsNum == 2)
					{
						KBD_CallWindowProc( (WNDPROC)m_origWndProc, m_hEdit, uMsg, wParam, lParam);
						InsertCharToBuffer( retChar2 );
						letter = getRandomLetter();
						wParam = (WPARAM)letter;
						lpRes = KBD_CallWindowProc( (WNDPROC)m_origWndProc, m_hEdit, uMsg, wParam, lParam);
						InsertCharToBuffer( retChar1 );
					}
					else
					{
						lpRes = KBD_CallWindowProc( (WNDPROC)m_origWndProc, m_hEdit, uMsg, wParam, lParam);
						InsertCharToBuffer( retChar2 );
					}

					//if ( (dlg->m_keyGenFlags & FLAG_ADVANCED_TEST ) > 0)
					//{
					//	WORD ss,sf;
					//	SendMessage( m_hEdit, EM_GETSEL, (WPARAM)(&ss), (LPARAM)(&sf) );
					//	SetWindowTextT( m_hEdit, m_buffer );
					//	SendMessage( m_hEdit, EM_SETSEL, (WPARAM)ss, (LPARAM)sf );
					//}
				}
				else
				{
					lpRes = KBD_CallWindowProc((WNDPROC)m_origWndProc, m_hEdit, uMsg, wParam, lParam);
					InsertCharToBuffer( retChar2 );
				}
			}
			else
			{
				lpRes = KBD_CallWindowProc((WNDPROC)m_origWndProc, m_hEdit, uMsg, wParam, lParam);
				InsertCharToBuffer( (char)wParam );
			}

			return lpRes;
		}

		case WM_KEYUP:
		{
			dlg->kbd.IncrementCharacterOrder( (int)wParam, dlg->m_useSecureTyping );
			lpRes = KBD_CallWindowProc((WNDPROC)m_origWndProc, m_hEdit, uMsg, wParam, lParam);

			if	(((dlg->m_keyGenFlags & FLAG_ADVANCED_SHOW_PWD_CHARS) == 0) &&
					( dlg->m_useSecureTyping && (wParam == VK_DELETE) )
				)
			{
				InsertCharToBuffer( VK_DELETE, false );
			}
			
			return lpRes;
		}
	}

	lpRes = KBD_CallWindowProc((WNDPROC)m_origWndProc, m_hEdit, uMsg, wParam, lParam);

	return lpRes;
}

bool cKbdEdit::CheckAndGetPassword( cPassword *pwd, cKbdEdit *edt, bool silent )
{
	if ( edt->m_hEdit == 0 )
	{
		assert( false );
		return false;
	}

	pwd->ClearBuffers();

	int pwdLength = Edit_LineLength( edt->m_hEdit, 0 );

	if ( pwdLength == 0 ) 
	{
		if ( !silent )
			MessageBoxT( edt->dlg->m_hDlg,
				edt->dlg->LoadStringOnceT( IDS_PASSWORD_EMPTY ),
				edt->dlg->LoadStringTwiceT( IDS_INVALID_PWD ),
				MB_OK | MB_ICONWARNING );

		SetFocus(  edt->m_hEdit );
		return false;
    } 

	if ( (( edt->dlg->m_keyGenFlags & FLAG_ADVANCED_SHOW_PWD_CHARS) != 0) || ( !edt->dlg->m_useSecureTyping ) )
		pwd->ReadFromEditBox( edt->m_hEdit );
	else
		pwd->ReadFromBuffer( edt->m_buffer );

    if ( pwdLength < PASSWORD_MINIMUM_LENGTH ) 
	{
		if ( !silent )
			MessageBoxT( edt->dlg->m_hDlg,
				edt->dlg->LoadStringOnceT ( IDS_PWD_TOO_SHORT ),
				edt->dlg->LoadStringTwiceT( IDS_INVALID_PWD ),
				MB_OK | MB_ICONWARNING );

		SetFocus( edt->m_hEdit );
		return false;
    } 

	return true;
}


//
//	CheckAndGetPassword
//		1. Checks password length, 
//		2. Verifies if edtB is not NULL,
//
bool cKbdEdit::CheckAndGetPassword( cPassword *pwd, cKbdEdit *edtA, cKbdEdit *edtB, bool silent )
{
	if ( edtA->m_hEdit == 0 )
	{
		assert( false );
		return false;
	}

	if ( !CheckAndGetPassword( pwd, edtA, silent ) )
		return false;

	if ( edtB == NULL )
		return true;

	int pwdConfirmLength = Edit_LineLength( edtB->m_hEdit, 0 );

	if ( pwdConfirmLength == 0 )
	{
		SetFocus( edtB->m_hEdit );
		return 0; // let user to type the second copy of password
	}

	cPassword pwdConfirm;

	if ( !CheckAndGetPassword( &pwdConfirm, edtB, true ) ||
		 ( wcscmp( pwd->wcBuffer, pwdConfirm.wcBuffer ) != 0 ) )
	{
		if ( !silent )
			MessageBoxT( edtB->dlg->m_hDlg,
				LoadStringOnceT ( edtB->dlg->m_hInst, IDS_PASSWORD_NOT_SAME ),
				LoadStringTwiceT( edtB->dlg->m_hInst, IDS_PASSWORD_MISMATCH ),
				MB_ICONEXCLAMATION );

		SetFocus( edtB->m_hEdit );
		return 0;
	}

	return true;
}

bool cKbdEdit::Attach( HWND hEdit )
{
	dlg = (cDlgKBParams *)cDlg::GetDlgClass( GetParent( hEdit ) );
	if ( dlg == NULL )
		return false;

	kbd = &dlg->kbd; // m_kbParams;

	if ((dlg->m_keyGenFlags & FLAG_ADVANCED_SHOW_PWD_CHARS) != 0)
		Edit_SetPasswordChar( hEdit, 0);

	m_hEdit = hEdit;

	SetWindowLongPtrT( m_hEdit, GWLP_USERDATA, (LONG_PTR)this );
	assert( this == (cKbdEdit *)GetWindowLongPtrT( m_hEdit, GWLP_USERDATA ) );

	//
	//	HookKeyboardProc from kbd.h uses this setting 
	//
	kbd->SetParams( GetParent( m_hEdit ), kbd );
	assert( kbd == kbd->GetParams( GetParent( m_hEdit ) ) );

	m_origWndProc = (LONG_PTR)KBD_SetWindowProc( m_hEdit, (LONG_PTR)editProc);

	return true;
}

void cKbdEdit::Detach()
{
	if ( m_hEdit == NULL )
		return;

	if ( m_origWndProc == NULL )
	{
		assert( false );
	}
	else
	{
		KBD_SetWindowProc( m_hEdit, (LONG_PTR)m_origWndProc );
		m_origWndProc = NULL;
	}

	SetWindowTextT( m_hEdit, L"          " );

	kbd->SetParams( GetParent( m_hEdit ), NULL );
//	SetWindowLongPtr( m_hEdit, GWLP_USERDATA, (LONG_PTR)NULL );
}
