#include "keyblock.h"
#include "datablock.h"
#include "PBE_block.h"
#include "SSS_block.h"
#include "PKE_block.h"


using namespace kgghost;



namespace kgghost
{

	class CUndefinedProcessor : public IKeyBlockProcessor
	{
	public:

		BC_INHERITED	bool decode(const bc_byte_t* kbEncrypted, unsigned int & kbSize, key_info_t& key, cPassword const & init_pwd);
		BC_INHERITED 	bool encode(bc_byte_t* kbEncrypted, unsigned int & kbSize, key_info_t const & key, cPassword const & init_pwd);

	private:

		CPBEProcessor m_pbe;
		//CSSSProcessor m_sss;
		//CPKEProcessor m_pke;
	};

}

bool CUndefinedProcessor::decode(const bc_byte_t* io_kbEncoded, unsigned int & io_kbSize, key_info_t& key, cPassword const & init_pwd)
{
	return 	m_pbe.decode(io_kbEncoded, io_kbSize, key, init_pwd);// ||
	 	//m_sss.decode(io_kbEncoded, io_kbSize, key, init_pwd) ||
	 	//m_pke.decode(io_kbEncoded, io_kbSize, key, init_pwd);

}


bool CUndefinedProcessor::encode(bc_byte_t* io_kbPlain, unsigned int & io_kbSize, key_info_t const & key, cPassword const & init_pwd)
{
	return 	m_pbe.encode(io_kbPlain, io_kbSize, key, init_pwd);// ||
	 	//m_sss.encode(io_kbPlain, io_kbSize, key, init_pwd) ||
	 	//m_pke.encode(io_kbPlain, io_kbSize, key, init_pwd);
}


CKeyBlock::CKeyBlock(bc_byte_t* data, unsigned int size, bcu16_t type) :
	m_data(data),
	m_size(size),
	m_type(type)
{
	m_proc = this->construct_processor();
	BC_ASSERT(m_proc); // check that type processor is defined
}


CKeyBlock::~CKeyBlock()
{
	delete m_proc;
}


bool CKeyBlock::decode(key_info_t & key, cPassword const & init_pwd)
{
	if( !m_proc ) return false;
	else return m_proc->decode(m_data, m_size, key, init_pwd);
}


bool CKeyBlock::encode(key_info_t const & key, cPassword const & init_pwd)
{
	if( !m_proc ) return false;
	else return m_proc->encode(m_data, m_size, key, init_pwd);
}


IKeyBlockProcessor* CKeyBlock::construct_processor()
{
	switch(m_type)
	{
	case KEYBLOCK_PBE:
		return new CPBEProcessor;

//	case KEYBLOCK_SSS:
//		return new CSSSProcessor;

//	case KEYBLOCK_PKE:
//		return new CPKEProcessor;

	default:
	        BC_DEBUG_PRINT(("Debug: got unknown type 0x%x\n", m_type));
	        return new CUndefinedProcessor;
	};
}

/*
CKeyBlockProcessor::CKeyBlockProcessor()
{
	m_procs.clear();
	m_procs.push_back( proc_entry_t(new CPBEProcessor, KEYBLOCK_PBE) );
	m_procs.push_back( proc_entry_t(new CSSSProcessor, KEYBLOCK_SSS) );
	//m_procs.push_back( proc_entry_t(new CPKEProcessor, KEYBLOCK_PKE) );

}


CKeyBlockProcessor::~CKeyBlockProcessor()
{
	proc_iter_t i = m_procs.begin();
	proc_iter_t end = m_procs.end();

	while(i != end)
	{
		delete i->proc;
		++i;
	}
}


bool CKeyBlockProcessor::decode(const bc_byte_t* kbEncoded, keyblock_descriptor_t& kb_desc, key_info_t& key, const cPassword* init_pwd)
{
	bcu16_t type = kb_desc.type;
	IKeyBlockProcessor* proc = this->get_processor_for_keyblock(type);
	if(!proc)
		return false;
	else
		return proc->decode(kbEncoded, kb_desc, key, init_pwd);
}


bool CKeyBlockProcessor::encode(bc_byte_t* kbPlain, keyblock_descriptor_t& kb_desc, key_info_t const & key, const cPassword* init_pwd)
{
	bcu16_t type = kb_desc.type;
	IKeyBlockProcessor* proc = this->get_processor_for_keyblock(type);
	if(!proc)
		return false;
	else
		return proc->encode(kbPlain, kb_desc, key, init_pwd);
}


IKeyBlockProcessor* CKeyBlockProcessor::get_processor_for_keyblock(bcu16_t kb_type)
{
	proc_iter_t i = m_procs.begin();
	proc_iter_t end = m_procs.end();

	while(i != end)
	{
		if(i->type == kb_type)
			return i->proc;
		++i;
	}
}
*/

#if 0


bool cKeyBlockParams::Init(
			DATA_BLOCK *dataBlockArray,
			DWORD dataBlockArraySizeInBytes,
			int		contAlgId,
			DWORD	encMode,
			int		contHashAlgId,
			DWORD	*pErrorCode,
			DWORD	*pFlags,
			HWND	hWndParent
		)
{
	ODSFunc;

	dataBlock = dataBlockArray;
	dataBlockSize = dataBlockArraySizeInBytes;

	// kbSSS may ask password for additional shares, so it needs in the parameters.
	kbSSS.pFlags = pFlags;
	kbSSS.pErrorCode = pErrorCode;
	kbSSS.hWnd = hWndParent;

	kbPBE.m_algId = contAlgId;
	kbPBE.m_hashAlgId = contHashAlgId;
	kbPBE.m_encModeId = encMode;

	return algInfo.Init( contAlgId, encMode );
}


bool CKeyBlockProcessor::decode_key(const char* kb_data, unsigned int kb_data_size, char* key_buf, unsigned int key_buf_size)
{
	bool res = false;

	keyblock_type_t saved = m_kb_type;

/*
	// Checks for all types
	if ( checkPasswordOnly )
		kbType = kbtEmpty;

	kbSSS.encodedSize = 0;
*/


	switch ( m_kb_type )
	{
		case KB_TYPE_PBE:
			//res = kbPBE.decodeKey( (BYTE *)keyBuff, keyBuffSize, kbEncoded, kbEncodedSize, algInfo.m_algBlockSize );
/*
#ifdef _DEBUG
#define _Michael_Burley_Problem
#endif
#ifdef _Michael_Burley_Problem
			//
			// Michael Burley encounter a problem.
			// The 0x00A7 symbol has been read as 0xFFA7 early
			// So old containers cannot be opened again.
			//
			// If decoding result is not true, we'll check the 0x00A7 symbol, change it 0xFFA7
			//	and try to decode again.
			//	The changed password will be restored.
			//
			if ( !res )
			{
				int len = (int)wcslen(kbPBE.m_pwd.wcBuffer);
				int pos = -1;
				bool found = false;
				bool foundB5 = false;
				int pos_0402 = -1;
				for ( pos = len; pos--; )
				{
					if ( kbPBE.m_pwd.wcBuffer[pos] == 0x00A7 )
					{
						found = true;
						kbPBE.m_pwd.wcBuffer[pos] = 0xFFA7;
					}
					else
					if ( kbPBE.m_pwd.wcBuffer[pos] == 0x00B5 )
					{
						foundB5 = true;
						kbPBE.m_pwd.wcBuffer[pos] = 0xFFB5;
					}
					else
					if ( kbPBE.m_pwd.wcBuffer[pos] == 0x0402 )
					{
						pos_0402 = pos;
					}
				}

				if ( found || foundB5 )
				{
					wchar_t path[ MAX_PATH ];

					wchar_t f1 = -1, f3 = -1;


					if ( ODS_GetFileName( path, maxLen( path ) ) )
					{
						ODS_CreateNewFile( false );

						// Read Last Values
						HANDLE hFile = CreateFileT( path, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL );
						if ( hFile != INVALID_HANDLE_VALUE )
						{
							wchar_t bf[ 100 ];
							int len = maxLen( bf );
							DWORD dwRead;

							SetFilePointer( hFile, -( ( len - 1 ) * sizeof( bf[0] ) ) , NULL, FILE_END );
							bool ok = ReadFile( hFile, (BYTE*)bf, (len - 1) * sizeof( bf[0] ), &dwRead, NULL );

							CloseHandle( hFile );

							if ( ok )
							{

								len = dwRead / sizeof( bf[0] );

								// 33:901 pwd[1] - 0000, pwd[3] - 00C9

								if ( len > 35 )
								{
									bf[ --len ] = L'\0';
									bf[ --len ] = L'\0';

									wchar_t *ptr = wcsrchr( bf, L':' );
									if ( ptr != NULL )
									{
										cMBStr mbs( ptr );
										DWORD t, dw1, dw3;
										sscanf( mbs.m_szBody, ":%d pwd[1] - %x, pwd[3] - %x", &t, &dw1, &dw3 );

										int res = sscanf( mbs.m_szBody, ":%d pwd[1] - %x, pwd[3] - %x", &t, &dw1, &dw3 );
										if ( res == 3 )
										{
											f1 = dw1;
											f3 = dw3;

											ODS( FormatOnceT( L"Found processed: pwd[1] - %04X, pwd[3] - %04X", f1, f3 ) );
										}
									}
								}
							}
						}
					}
					else
					{
						ODS_Enable();
						ODS( L"Found unicode symbols" );
					}

					// It tries to add FF to char buffer
					// @�~�|I.just love and adore Donna-she is so cute and.Cuddly\}][{


					if ( !foundB5 )
						res = kbPBE.decodeKey( (BYTE *)keyBuff, keyBuffSize, kbEncoded, kbEncodedSize, algInfo.m_algBlockSize );
					else
					{
						wchar_t wc1 = kbPBE.m_pwd.wcBuffer[1];
						wchar_t wc3 = kbPBE.m_pwd.wcBuffer[3];

						if ( ( f1 != -1 ) && ( f3 != -1 ) )
						{
							kbPBE.m_pwd.wcBuffer[1] = f1;
							kbPBE.m_pwd.wcBuffer[3] = f3;
						}

						for( int i = 0x10000; i--; )
						{
							ODS( FormatOnceT( L"pwd[1] - %04X, pwd[3] - %04X", (DWORD)kbPBE.m_pwd.wcBuffer[1] , (DWORD)kbPBE.m_pwd.wcBuffer[3] ) );

							for( int j = 0x10000; j--; )
							{
								res = kbPBE.decodeKey( (BYTE *)keyBuff, keyBuffSize, kbEncoded, kbEncodedSize, algInfo.m_algBlockSize );
								if ( res )
								{
									ODS( FormatOnceT( L"Found !!! pwd[1] - %04X, pwd[3] - %04X", (DWORD)kbPBE.m_pwd.wcBuffer[1] , (DWORD)kbPBE.m_pwd.wcBuffer[3] ) );
									break;
								}

								kbPBE.m_pwd.wcBuffer[3] += 0x1;

							}

							if ( res )
								break;

							kbPBE.m_pwd.wcBuffer[1] += 0x1;
						}

						kbPBE.m_pwd.wcBuffer[1] = wc1;
						kbPBE.m_pwd.wcBuffer[3] = wc3;
					}

					for ( pos = len; pos--; )
					{
						if ( kbPBE.m_pwd.wcBuffer[pos] == 0xFFA7 )
							kbPBE.m_pwd.wcBuffer[pos] = 0x00A7;
						else
						if ( kbPBE.m_pwd.wcBuffer[pos] == 0xFFB5 )
							kbPBE.m_pwd.wcBuffer[pos] = 0x00B5;
					}
				}
			}
#endif
*/
			break;

		case kbtPKE:
			res = false; // not supported

/*
			//	See 'Secret key notes' at DataBlockVerifyKeyBlockAndGetKey.

			if ( kbPKE.ska == NULL )
				kbPKE.openSecretKey( kbSSS.hWnd, &kbPBE.m_pwd, kbEncoded, true );

			if ( kbPKE.ska == NULL )
				res = false;
			else
				res = kbPKE.decodeKey( (BYTE *)keyBuff, keyBuffSize, kbEncoded, kbEncodedSize );
*/

			break;
		case kbtSSS:
			res = false; // not supported
/*
			kbSSS.SetEncIds( &kbPBE );

			kbSSS.version = -1;

			if ( kbSSS.itemQ != 0 )
				res = kbSSS.decodeKey( (BYTE *)keyBuff, keyBuffSize, kbEncoded, kbEncodedSize );
			else
			{
				kbSSS.encodedSize = kbEncodedSize;
				res = kbSSS.decodeKeyWithFirstPwd( &kbPBE.m_pwd, (BYTE *)keyBuff, keyBuffSize, kbEncoded, &(kbSSS.encodedSize), checkPasswordOnly );
			}
*/
			break;
		case kbtEmpty:
		{
			//
			//	Note, if kbParams type is undefined ( kbtEmpty ), so all available
			//	encoding shemes are implemented to DATA_BLOCK,
			//	see kbParams->decodeKey
			//

			res = kbPBE.decodeKey( (BYTE *)keyBuff, keyBuffSize, kbEncoded, kbEncodedSize, algInfo.m_algBlockSize );

			if ( res ) { saved = kbtPBE; break; }

/*
			if ( kbPKE.ska != NULL )
			{
				//
				//	For future improvemen, the check function for encrypted ( or hidden part ) key block enumeration
				//	may be moved to one loop above ( excluding PBE algorithm enumeration, including the key block enumertion only )
				//	In the case we have to change loops order key block enumeration must be first and algorithm verification second,
				//	but it may be slowly, because we have to load all algorithms for every block again.
				//
				res = kbPKE.decodeKey( (BYTE *)keyBuff, keyBuffSize, kbEncoded, kbEncodedSize );
				if ( res ) { saved = kbtPKE; break; }
			}

			kbSSS.SetEncIds( &kbPBE );
			kbSSS.version = -1;

			kbSSS.encodedSize = kbEncodedSize;
			res = kbSSS.decodeKeyWithFirstPwd( &kbPBE.m_pwd, (BYTE *)keyBuff, keyBuffSize, kbEncoded, &kbSSS.encodedSize, checkPasswordOnly );
			if ( res ) { saved = kbtSSS; break; }
*/
			break;
		}
		default:
			res = false;
	}

	if ( res && ! checkPasswordOnly )
	{
		res = keyBuff->isValid();
	}

	kbType = saved;

	// for current release check that we didn't miss unsupported types
	BC_ASSERT(kbType == kbtPBE);

	return res;
}

bool cKeyBlockParams::encodeKey( BYTE *kbEncoded, int kbEncodedSize, sKeyInfo *keyBuff, int keyBuffSize )
{
	bool res = false;

	assert( keyBuff->size <= keyBuffSize );

	switch ( kbType )
	{
		case kbtPBE:
			res = kbPBE.encodeKey( kbEncoded, kbEncodedSize, (BYTE *)keyBuff, keyBuffSize );
			break;

		case kbtPKE:
/*
			res = kbPKE.encodeKey( kbEncoded, kbEncodedSize, (BYTE *)keyBuff, keyBuffSize );
			break;
*/
			res = false;
			break;

		case kbtSSS:
/*
			res = kbSSS.encodeKey( kbEncoded, kbEncodedSize, (BYTE *)keyBuff, keyBuffSize );
			break;
*/
			res = false;
			break;

		default:
		{
			BC_ASSERT(false);
			return false;
		}
	}

	return res;
}

int cKeyBlockParams::getEncodedKeySize( int keyLength )
{
	switch ( kbType )
	{
		case kbtPBE: return kbPBE.getEncodedKeySize( keyLength );
		case kbtPKE: return false;//return kbPKE.getEncodedKeySize( keyLength );
		case kbtSSS: return false;//return kbSSS.getEncodedKeySize( keyLength );
		default:
		{
			BC_ASSERT(false);
			return false;
		}
	}
}

//
//	Tries to get a password from key block parameters.
//	The password is required as client network password for corporate version.
//
bool cKeyBlockParams::getPassword( char *pwdBuff )
{
	BC_ASSERT(false);
	return false;
}

#endif // 0
