/*******************************************************************\
*
*	Copyright (c) 2007-2009 Jetico, Inc., Finland
*	All rights reserved.
*
*	File: kgGhost.cpp
*	Revision:
*	Created:
*	Description: kgghost main interface implemetation
*
\*******************************************************************/

#include "kgghost.h"
#include "keyblock.h"
#include "user.h"
#include "comm_def/kg_defs.h"
#include "CertificateLib/x509pkts/pkcs12_funcs.h"

#include <common/core.h>

using namespace kgghost;
using namespace bdk;


#define DISPLAY_NAME_LENGTH 32
#define SERVICE_NAME_LENGTH 32
#define EYE_CATCH_STRING    "__BCKEYGEN__"


struct BC_KEYGEN_BLOCK
{
	char  eyeCatchString[16];
	DWORD keygenID;
	char  displayName[ DISPLAY_NAME_LENGTH ];
	char  serviceName[ SERVICE_NAME_LENGTH ];
	DWORD majorVersion;
	DWORD minorVersion;
};

BC_KEYGEN_BLOCK sig =
{
	EYE_CATCH_STRING,
	KEYGEN_ID,
	DISPLAY_NAME,
	SERVICE_NAME,
	MAJOR_VERSION,
	MINOR_VERSION
};



//
//	QueryFunction();
//		Checks if the function is available for the data block.
//
BOOL QueryFunction(const DWORD functionID)
{
	switch ( functionID )
	{
		case CFLAG_CREATE_WITH_SINGLE_PASSWORD:
		case CFLAG_VERIFY_AND_LOAD_KEY:
		case CFLAG_CHANGE_PASSWORD:
		case CFLAG_ADD_NEW_PASSWORD:
		case CFLAG_OLD_DATA_REENCRYPTED_CONTAINER:
		case CFLAG_REMOVE_ADDITIONAL_PASSWORD:
		case CFLAG_ShowKeyBlockInfo:
		case CFLAG_EncryptKeyBlock:
		case CFLAG_SupportMode_LRW:
		case CFLAG_SupportMode_XTS:
			return true;
		default :
			return false;
	}
}


/*******************************************************
 *
 *	GetNameAndVersion()
 *
 *******************************************************/
BOOL GetNameAndVersion(char* Name, UINT nameSize, DWORD* Major, DWORD* Minor)
{
	strncpy(Name, DISPLAY_NAME, nameSize - 1);
	Name[nameSize - 1] = 0;
	*Major = MAJOR_VERSION;
	*Minor = MINOR_VERSION;
	return TRUE;
}


////////////////////////////////////////////////
//
//	Main functions are called by CreateKeyHandleEx function
//		1. Call seed dialog to accumulates seed
//		2. Generates key to keyInfo->key
//	Note, the keyInfo's keyAlgId and hashAlgId have to be assigned before.
//
static BOOL GenerateKey(key_info_t* keyInfo, DWORD* pErrCode)
{
	int seedSize = SEED_LENGTH;

	cPlainTextBlock ptb;
	if ( ptb.Allocate( seedSize + 0 ) == NULL)
	{
		*pErrCode = ERROR_INTERNAL_PROBLEM;
		return FALSE;
	}

	char *seed = (char*)ptb.buffer;

	*pErrCode = User_GenerateSeed( seed, seedSize );
	if ( *pErrCode != ERROR_NO ) return FALSE;


	BCAlgService* alg = bc_core()->alg_manager()->find_algorithm(keyInfo->alg_id);
	if ( !alg ) {
		*pErrCode = ERROR_INCORRECT_ENCR_ALGORITHM;
		return FALSE;
	}

	unsigned int keySize = alg->key_size();


	keyInfo->calc_size(keySize);


	// Generate key
	int err = P12_producePseudoRandomBytes((BYTE*)keyInfo->key, keySize, NULL, 0, 1, (BYTE*)seed, seedSize, P12_prgm4key, keyInfo->hash_id);

	if ( err == x509e_ok )
		return ERROR_NO;

	if ( err == hae_invalidAlgorithmID )
		return ERROR_INCORRECT_HASH_ALGORITHM;

	return ERROR_INTERNAL_PROBLEM;
}



static void UpdateContext(kg_context_t* ps, key_info_t* keyInfo)
{
	ps->contInfo = keyInfo->cont_info;
	ps->contAlgId = keyInfo->alg_id;
	ps->encModeId = keyInfo->mode_id;
	ps->hashAlgId = keyInfo->hash_id;
}



BOOL
	GetKeyHandle(
		const char	*Caption,
		DATA_BLOCK	**ppDB,
		DWORD		*pDBSize,
		BCAlgHandle&	out_handle,
		kg_context_t	*ghost_ctx,
		key_info_t	*pKeyInfo,
		unsigned int	*pFlags,
		DWORD		rights,
		DWORD		*pErrCode
	)
{
	// check data block integrity
	*pErrCode = DataBlock_Check(*ppDB, *pDBSize);
	if ( *pErrCode != ERROR_NO ) {
		BC_DEBUG_PRINT(("Data Block Check failed: %d\n", *pErrCode));
		return FALSE;
	}


	// create local key info buffer
	cPlainTextBlock ptb;
	if ( ptb.Allocate(sizeof(key_info_t)) == NULL)
	{
		*pErrCode = ERROR_INTERNAL_PROBLEM;
		return FALSE;
	}

	memset( ptb.buffer, 0, ptb.bufferSize );
	key_info_t* keyInfo = (key_info_t*)(ptb.buffer);


	// init key info with the data we assume from input.
	keyInfo->alg_id = ghost_ctx->contAlgId;
	keyInfo->mode_id = ghost_ctx->encModeId;
	keyInfo->hash_id = ghost_ctx->hashAlgId;


	// ask for initial password
	cPassword pwd;
	*pErrCode = User_GetPassword(Caption, pwd);
	if ((*pErrCode) != ERROR_NO)
		return FALSE;



	bool bIsForUsual = false;

	if ( ghost_ctx->contAlgId != -1 )
	{
		*pErrCode = DataBlock_GetKeyHandle(*ppDB, *pDBSize, pwd, out_handle, *keyInfo);

		bIsForUsual = !keyInfo->is_for_hidden_part();
	}
	else
	{
	        /**
		std::vector<BCAlgService*> algs;
		bc_core()->alg_manager()->enum_algorithms(algs);

		for(size_t i = 0; i < algs.size(); ++i)
		{
		}
		*pErrCode = ERROR_INCORRECT_PASSWORD;

		cProgressDlg_EncHeaderParse dlg(
						pErrCode,
						*ppDB, *pDBSize,
						&kbParams,
						pKeyHandle,
						keyInfo,
						&(decHeader->m_body),
//						decContInfo,
//						mainKeyInfo,
						&(ps->keyMapInfo)
						);

		dlg.DoModal( hMainInst, IDD_WaitEncHeaderParseDlg, hWnd );

		bIsForUsual = !keyInfo->contInfo.isForHiddenPart(); // bodyOffset == decHeader->m_body.contInfo.bodyOffset;

		if ( ( *pErrCode == ERROR_NO ) && ( bIsForUsual ) )
		{
			//
			//	Data block header was encrypted, so key gen reads
			//	maximum available space from data block from container.
			//	Now we have properly decrypted header and can recover
			//	original data block size
			//
			if ( !UpdateAfterDecryption( ppDB, pDBSize, &(decHeader->m_body) ) )
			{
				assert( false );
				return ERROR_INTERNAL_PROBLEM;
			}

			memcpy( &(ps->decryptedHeader), decHeader, sizeof( dbHeader ) );
		}
*/
	}


	if ( (*pErrCode) != ERROR_NO ) {
		BC_DEBUG_PRINT(("Debug: v8: GetKeyHandle returned 0x%x\n", *pErrCode));
		return FALSE;
	}


	// now that we're back, correct any initial parameters and return

	//updateKeyParams( ps, keyInfo, &decHeader->m_body );

	if( pKeyInfo ) memcpy(pKeyInfo, keyInfo, sizeof(key_info_t));

	UpdateContext(ghost_ctx, keyInfo);

	return TRUE;
}



extern "C" BOOL CreateKeyHandle_V2(
		// in
		DWORD		command,
		const char*	caption,
		unsigned int*	flags,

		// in\out
		BYTE**		ioDataBlock,
		DWORD*		ioDBSize,
		kg_context_t*	ioContext,

		// out
		BCAlgHandle& 	oHandle,
		DWORD*		oErrCode
)
{
	// check in\out parameters present
	BC_ASSERT(ioContext);
	BC_ASSERT(ioDataBlock);
	BC_ASSERT(ioDBSize);

	// check command flag
	if (!(CORRECT_CFLAG(command)))
	{
		*oErrCode = ERROR_INCORRECT_CREATE_FLAG;
		return FALSE;
	}


	// check if given algorithm with given mode can be called
	BCAlgService* algserv = bc_core()->alg_manager()->find_algorithm(ioContext->contAlgId);
	if ( !algserv )
	{
		BC_DEBUG_PRINT(("algorithm 0x%x can not be called\n", ioContext->contAlgId));
		*oErrCode = ERROR_INVALID_ALGORITHM;
		return FALSE;
	}
	else if( !algserv->is_mode_supported(ioContext->encModeId) )
	{
		BC_DEBUG_PRINT(("algorithm 0x%x does not support mode 0x%x\n", ioContext->contAlgId, ioContext->encModeId));
		*oErrCode = ERROR_INVALID_ALGORITHM;
		return FALSE;
	}


	BOOL	bResult = FALSE;
	DWORD	rights; /// = RIGHT_DO_NOT_BACKUP_KEYS;
	DATA_BLOCK** ioDB = (DATA_BLOCK**)ioDataBlock;

	switch (command)
	{
		// **************  Creating new Data Block and return Key Handle ***************

		case CFLAG_CREATE_WITH_SINGLE_PASSWORD:
		{
/**
			if ( pParams->contInfo.length == 0 )
			{
				bResult = ERROR_INCORRECT_DATA_BLOCK_SIZE;
				break;
			}

			if ( pParams->contInfo.offset == 0 ) // Initial call, body size is defined here
			{
				pParams->contInfo.offset = DB_MIN_SIZE;
			}

			pParams->decryptedHeader.contInfo = pParams->contInfo;

			bResult = CreateNewDataBlock( text, caption, ppDB, pDBSize, out_handle, pErrCode, pFlags, rights, params );
*/
			bResult = ERROR_INCORRECT_CREATE_FLAG;
			break;
		}

		// **************  Verify password and return Key Handle ***************

		case CFLAG_VERIFY_AND_LOAD_KEY: // 1 - minimum number of keys inside the block
		{
			ioContext->bEncrypted = (*ioDB)->header()->maybe_encrypted();
			bResult = GetKeyHandle(caption, ioDB, ioDBSize, oHandle, ioContext, NULL, flags, rights, oErrCode);
			break;
		}
		case CFLAG_CHANGE_PASSWORD:
		{
			//bResult = ChangePassword( text, caption, ppDB, pDBSize, pErrCode, pFlags, params );
			break;
		}
		case CFLAG_ADD_NEW_PASSWORD:
		{
			//bResult = AddPassword( text, caption, ppDB, pDBSize, pErrCode, pFlags, params );
			break;
		}
		case CFLAG_REMOVE_ADDITIONAL_PASSWORD:
		{
			//bResult = RemovePassword( text, caption, ppDB, pDBSize, pErrCode, pFlags, params );
			break;
		}

		//
		//	KG-Ghost specific commands
		//


		case CFLAG_ShowKeyBlockInfo:
		{
			//bResult = ShowKeyBlockInfo( text, *ppDB, *pDBSize );
			break;
		}

		case CFLAG_EncryptKeyBlock:
		{
			//bResult = EncryptHeader( text, &(*ppDB)->header, *pDBSize, pErrCode, pFlags, params );
			break;
		}
		case CFLAG_OLD_DATA_REENCRYPTED_CONTAINER:
		{
			*oErrCode = ERROR_NO;
			bResult = TRUE;
			break;
		}
		default:
			BC_ASSERT(false); // this should have been checked before
			*oErrCode = ERROR_INCORRECT_CREATE_FLAG;
			bResult = FALSE;
			break;
	}
	return bResult;
}


/*******************************************************
 *
 *	FreeDataBlock() - frees the memory allocated when
 *  user calls CreateKeyHandle() procedure with
 *  CreateFlag equal to CFLAG_CREATE_WITH_SINGLE_PASSWORD
 *
 *******************************************************/

/*

DWORD FreeDataBlock( BYTE **vDataBlock )
{
	if (!DataBlockFree( (DATA_BLOCK **)vDataBlock, NULL ))
	{
		BC_ASSERT(false);
		return ERROR_INTERNAL_PROBLEM;
	}

	return ERROR_NO;
}

*/



//
//
//	Refactor below
//
//


#if 0
static BOOL
CreateNewDataBlock(
	const wchar_t	*text,		// Text, containing filename of the container
	const wchar_t	*caption,	// Caption of the "get password" dialog window
	DATA_BLOCK	**ppDB,
	DWORD		*pDBSize,
	kg_params_t*	ps,
	CBCAlg_Handle	handle,
	DWORD		*pFlags,
	DWORD		rights,
	DWORD		*pErrCode
	)
{
	*pErrCode = ERROR_INTERNAL_PROBLEM;

	if ( !DataBlockAllocate( ppDB, pDBSize ) )
	{
		assert( false );
		return FALSE;
	}

	if ( !DataBlockInit( *ppDB, *pDBSize, ps->contAlgId, ps->encModeId, ps->hashAlgId, &ps->contInfo ) )
	{
		DataBlockFree( ppDB, pDBSize );
		*pDBSize = 0;
		assert( false );
		return FALSE;
	}

	cKeyBlockParams kbParams;
	kbParams.Init( *ppDB, *pDBSize, ps->contAlgId, ps->encModeId, ps->hashAlgId, pErrCode, pFlags, hWnd );

	kbParams.m_allowChangeAlgorithms = true;

	DWORD maxKeyBuffSize = 0;
	*pErrCode = DataBlock_CheckFreeSpace( *ppDB, 0, &maxKeyBuffSize );
	if ( *pErrCode != ERROR_NO ) return FALSE;

	*pErrCode = GetKeyBlockParamsWithVerification( hWnd, text, caption, &kbParams, pFlags, maxKeyBuffSize );
	if ( *pErrCode != ERROR_NO ) return FALSE;

	ps->contAlgId = kbParams.kbPBE.m_algId;
	ps->encModeId = kbParams.kbPBE.m_encModeId;
	ps->hashAlgId = kbParams.kbPBE.m_hashAlgId;

	//
	//	Init once more, the parameters may be changed.
	//
	if ( kbParams.m_allowChangeAlgorithms )
		DataBlockInit( *ppDB, *pDBSize, ps->contAlgId, ps->encModeId, ps->hashAlgId, &ps->contInfo );

	cPlainTextBlock ptbKey;
	sKeyInfo *keyInfo = (sKeyInfo *)(ptbKey.Allocate( sizeof( sKeyInfo ) ) );
	if ( keyInfo == NULL )
	{
		*pErrCode = ERROR_INTERNAL_PROBLEM;
		assert( false );
		return FALSE;
	}

	keyInfo->contInfo = ps->contInfo;

//	BYTE *decryptedKeyIVandDigest;

	//
	//	Key block generation process may require additional seed, so we generate it here in common procedure.
	//
	BYTE *additionalSeed = NULL;
	int additionalSeedSize = 0;

	if ( kbParams.kbType == kbtSSS )
	{
		additionalSeedSize = kbParams.kbSSS.getSeedSize( sizeof( sKeyInfo ) ); // keyInfo->size );

		additionalSeed = kbParams.kbSSS.ptbSeed.Allocate( additionalSeedSize );
		if ( additionalSeed == NULL )
			return false;
	}

	keyInfo->keyAlgId  = kbParams.kbPBE.m_algId;
	keyInfo->encModeId = kbParams.kbPBE.m_encModeId;
	keyInfo->hashAlgId = (BYTE)kbParams.kbPBE.m_hashAlgId;

	if ( !GetSeedAndGenerateKey(
			hWnd,
			keyInfo,
			pErrCode,
			additionalSeed,
			additionalSeedSize )
		)
			return false;

	(*ppDB)->header.m_body.contInfo = ps->contInfo;
	(*ppDB)->header.m_body.contAlgId = keyInfo->keyAlgId;
	(*ppDB)->header.m_body.encModeId = keyInfo->encModeId;
	(*ppDB)->header.m_body.hashAlgId = keyInfo->hashAlgId;

	(*ppDB)->header.updateTail( keyInfo, (*ppDB)->pool );

	ps->keyMapInfo.setUndefined();

	if ( DataBlockWriteKey( ppDB, pDBSize, ps->bEncrypted, keyInfo, &kbParams, &(ps->keyMapInfo), NULL, NULL ) != ERROR_NO )
	{
		DataBlockFree( ppDB, pDBSize );
		*pDBSize = 0;
		return FALSE;
	}

	if ( kbParams.kbType == kbtPKE )
	{
		//  PKE cannot verify encryption, so it gets key handle only
		if ( pKeyHandle != NULL )
		{
			if ( !cSymAlgorithm::createKey(
					pKeyHandle,
					keyInfo->keyAlgId,
					keyInfo->encModeId,
					keyInfo->key,
					keyInfo->getKeySizeBytes(),
					(*ppDB)->pool )
				)
			{
				assert( false );
				*pErrCode = ERROR_INTERNAL_PROBLEM;
			}
		}
	}
	else
	{
		//  verify all that we did before and get key handle

		ps->keyMapInfo.setUndefined();

		*pErrCode =
			DataBlockVerifyKeyBlockAndGetKey(
					*ppDB,
					*pDBSize,
					&kbParams,
					&(ps->keyMapInfo),
					pKeyHandle,
					keyInfo,
					&(ps->decryptedHeader),
					&(ps->contInfo )
				);
	}

	if ( *pErrCode != ERROR_NO )
	{
		DataBlockFree( ppDB, pDBSize );
		return FALSE;
	}

	updateKeyParams( ps, keyInfo );

	*pErrCode = ERROR_NO;
	return TRUE;
}
#endif

#if 0
//
//	Data block header was encrypted, so key gen reads
//	maximum available space from data block from container.
//	Now we have properly decrypted header and can recover
//	original data block size
//
static bool
	UpdateAfterDecryption(
		DATA_BLOCK	**ppDB,
		DWORD		*pDBSize,
		dbHeader	*decHeader
	)
{
	// Gets realy busied size
	DWORD realSize = decHeader->getRealSize();

	if ( realSize == DWORD(-1) )
	{
		assert( false );
		return false;
	}

	DWORD rounded_4K_Size = Round( realSize, _4K );

	if ( rounded_4K_Size > *pDBSize )
	{
		assert( false );
		return false;
	}

	if ( *pDBSize > _4K )
	{
		// Data block has to be truncated
		// If some far keys have been deleted
		// So getRealSize returns size less then really busied
		// so really busied block may be lost
		// Key gen read encrypted key block
		//		[ header 4K ] .... [       maximum 16K           ]
		// Real placement
		//		[ header 4K ] ....   [ real keys ][ deleted keys ]
		// This updating does
		//		[ header 4K ] ....         [ real size from tail ]

		BYTE *dbPtr = (BYTE*)*ppDB;
		DWORD tailSize = rounded_4K_Size - _4K;

		memmove( dbPtr + _4K, dbPtr + *pDBSize - tailSize, tailSize );

		// Variants to avoid the ambiguity
		// 1. Do not allow encrypt header if data block size more then 4K
		// 2. Checks and truncates size of data block before encryption,
		//	and does not allow to modify encrypted key block
		// 3. Updates first 4K after decryption only.

		// The varionat 3 is implemented see flags
		//	FLAG_DATA_BLOCK_WAS_MODIFIED
		//	FLAG_DATA_BLOCK_4K_WAS_MODIFIED
	}

	*pDBSize = rounded_4K_Size;

	return true;
}
#endif



/***********************************************************
 *
 *	UpdateAdvancedSettings() - The procedure calls dialog
 *  which allows user to set options specific for
 *	the current Key Generation module. Other BestCrypt
 *	modules know nothing about the settings, they just
 *	use the settings (flags) when call other Key Generation
 *	functions.
 *
 ***********************************************************/

#if 0
//
//	ChangePassword
//
//	Note, it saves existing hidden parts.
//		1. If you enter new password but it is a password for hidden part
//		the function remembers position of the valid hidden part, it allows to save the valid HP when new key is added.
//		2. The key map at ps->decryptedHeader.keyMap is updated by known hidden parts
//
static BOOL
	ChangePassword(
		HWND		hWnd,
		const wchar_t	*Text,
		const wchar_t	*Caption,

		DATA_BLOCK	**ppDB,
		DWORD		*pDBSize,

		DWORD		*pErrCode,
		DWORD		*pFlags,

		sCKHParams	*ps
	)
{
	*pErrCode = DataBlockCheck(	*ppDB, *pDBSize, ps->contAlgId );

	if ( *pErrCode	!= ERROR_NO )
			return FALSE;

	cKeyBlockParams kbParams;
	cKeyBlockParams kbParamsNew;

	kbParams.Init( *ppDB, *pDBSize, ps->contAlgId, ps->encModeId, ps->hashAlgId, pErrCode, pFlags, hWnd );
	kbParamsNew.Init( *ppDB, *pDBSize, ps->contAlgId, ps->encModeId, ps->hashAlgId, pErrCode, pFlags, hWnd );

	if (((*pErrCode) =
			GetKeyBlockParams(
				hWnd, Text, LoadStringOnceT( hMainInst, IDS_EnterOldPassword ),
				&kbParams, pFlags ))
					!= ERROR_NO)
			return FALSE;

	bool bHidPart = FALSE;
	DWORD offset = 0;

	//
	// Allocates space for decrypted header, cont and key info.
	//
	cPlainTextBlock ptb;
	if ( ptb.Allocate(
		sizeof( sKeyInfo )	 +
		sizeof( sKeyInfo )	 +
		sizeof( sKeyInfo )	 +
		sizeof( dbHeader )	 +
		0 ) == NULL )
	{
		assert( false );
		*pErrCode = ERROR_INTERNAL_PROBLEM;
		return FALSE;
	}

	int pos = 0;
	dbHeader	*decHeader		= (dbHeader	  *)(ptb.buffer + pos); pos += sizeof( dbHeader   );
	sKeyInfo	*keyInfo		= (sKeyInfo   *)(ptb.buffer + pos); pos += sizeof( sKeyInfo   );
	sKeyInfo	*tmpKeyInfo		= (sKeyInfo   *)(ptb.buffer + pos); pos += sizeof( sKeyInfo   );
	sKeyInfo	*mainKeyInfo	= (sKeyInfo   *)(ptb.buffer + pos); pos += sizeof( sKeyInfo   );
	assert( pos == ptb.bufferSize );

	if (((*pErrCode) =
			DataBlockVerifyKeyBlockAndGetKey(
				*ppDB,
				*pDBSize,
				&kbParams,
				&(ps->keyMapInfo ),
				NULL, // pKeyHandle,
				keyInfo,
				decHeader
				))
					!= ERROR_NO )
			return FALSE;

	bool bIsForUsual = !keyInfo->contInfo.isForHiddenPart();

	//
	// Every valid password you enter here leaves traces at the keyMap ( actual for hidden part passwords )
	//	so DataBlockWriteKey will saves the known password.
	//
	dbKeyId keyMapT[ DB_maxKeyQ ];
	decHeader->updateKeyMap( keyMapT, false );
	ps->decryptedHeader.updateKeyMap( keyMapT, false );

	if ( bIsForUsual )
	{
		for(;;)
		{
			if (((*pErrCode) =
					GetKeyBlockParamsWithVerification(
						hWnd, Text, LoadStringOnceT( hMainInst, IDS_EnterNewPassword ),
						&kbParamsNew, pFlags ))
							!= ERROR_NO )
					return FALSE;

			// Check if new password is the compatible with any old password
			sKeyMapInfo kmInfo; // for valid hidden password places collecting

			if (((*pErrCode) = DataBlockVerifyKeyBlockAndGetKey(
									*ppDB,
									*pDBSize,
									&kbParamsNew,
									&kmInfo )
								) == ERROR_NO )
			{

				// Accumulates hidden keyMapT
				dbHeader::MarkBusied( keyMapT, &kmInfo );

				MessageBoxT(
						hWnd,
						LoadStringOnceT( hMainInst, IDS_PASSWORD_IN_USE ),
						LoadStringTwiceT( hMainInst, IDS_WARNING ),
						MB_OK | MB_ICONWARNING );

				continue;
			}

			// Overwrites at found position saved at ps->keyMapInfo.
			if (((*pErrCode) =
				DataBlockWriteKey( ppDB, pDBSize, (*ppDB)->header.m_body.IsEncrypted(), keyInfo, &kbParamsNew, &(ps->keyMapInfo ), keyMapT, decHeader ) )
						!= ERROR_NO )
					return FALSE;

			break;
		}
	}
	else
	{
		// We are going to change password for hidden part?
		int iRes = MessageBoxT(
				hWnd,
				LoadStringOnceT( hMainInst, IDS_HidPasswordChangeWarning ),
				LoadStringTwiceT( hMainInst, IDS_WARNING ),
				MB_YESNO | MB_ICONWARNING );

		if (iRes == IDNO)
		{
			*pErrCode = ERROR_CANCELLED_BY_USER;
			return FALSE;
		}

		// Check loop...
		for(;;)
		{
			if (((*pErrCode) = GetKeyBlockParamsWithVerification(
						hWnd, Text, LoadStringOnceT( hMainInst, IDS_EnterNewHiddenPassword ),
						&kbParamsNew, pFlags ))
							!= ERROR_NO)
				return FALSE;

			// Check if password for hidden part is the compatible with any old password
			sKeyMapInfo kmInfo; // for valid hidden password places collecting

			if (((*pErrCode) = DataBlockVerifyKeyBlockAndGetKey(
									*ppDB,
									*pDBSize,
									&kbParamsNew,
									&kmInfo,
									NULL,
									tmpKeyInfo )
								) == ERROR_NO )
			{
				// We have no yet main key info, so check the tmpKeyInfo
				// Accumulates hidden keyMapT and checks validity of main cont info
				if ( tmpKeyInfo->contInfo.isForHiddenPart() )
				{
					dbHeader::MarkBusied( keyMapT, &kmInfo );
				}
				else
				if ( !mainKeyInfo->isValid() )
				{
					memcpy( mainKeyInfo, tmpKeyInfo, keyInfo->size );
				}
				else
				{
					assert( memcmp( mainKeyInfo, keyInfo, keyInfo->size ) );
				}

				MessageBoxT(
					hWnd,
					LoadStringOnceT( hMainInst, IDS_PASSWORD_IN_USE ),
					LoadStringTwiceT( hMainInst, IDS_WARNING ),
					MB_OK | MB_ICONWARNING );
				continue;
			}

			*pErrCode =
				DataBlockWriteKey_Hidden(
						ppDB,
						pDBSize,
						keyInfo,
						&kbParamsNew,
						&(ps->keyMapInfo),
						keyMapT
					);

			// break from the check loop...
			// v.1.03 - 24.04.08 the break was added
			break;
		}
	}

	if ( *pErrCode != ERROR_NO )
		return FALSE;

	updateKeyParams( ps, keyInfo, decHeader );

	return TRUE;
}
#endif


#if 0
//
//	AddNewKey
//		Adds new key to data block.
// !!!	1. It has to every time ask password to decode the key info has to be saved wth additional password.
//			The password may be for hidden as well as for original parts.
//		2. It asks new key block parameters
//		3. Encodes key info by new key block parameters and writes encoded new key to data block
//
//	Notes.
//		1. If you enter new password but is is a password for hidden part
//		the function remembers position of the valid hidden part, it allows to save the valid HP when new key is added.
//		2. The key map at ps->decryptedHeader.keyMap is updated by known hidden parts
//
static BOOL AddPassword(const wchar_t* Text, const wchar_t* Caption, DATA_BLOCK** ppDB, DWORD* pDBSize, DWORD* pErrCode, DWORD* pFlags, kg_params_t* ps)
{
	if (((*pErrCode) = DataBlockCheck(*ppDB, *pDBSize, ps->contAlgId )) != ERROR_NO)
		return FALSE;

	DWORD maxFreeSize = 0;

	//bool isEncrypted = (*ppDB)->header.m_body.IsEncrypted();

	////
	////	For encrypted it will be ckecked below, after decrypting
	////
	//if ( !isEncrypted )
	//{
	//	*pErrCode = dbHeader::checkFreeSpace( (*ppDB)->header.m_body.keyMap, DB_MinEncryptedKeyBlockSize, &maxFreeSize );
	//	if( *pErrCode != ERROR_NO )
	//		return false;
	//}

	bool isEncrypted = ps->decryptedHeader.IsEncrypted();

	//
	//	For encrypted it will be ckecked below, after decrypting
	//
	if ( !isEncrypted )
	{
		*pErrCode = dbHeader::checkFreeSpace( ps->decryptedHeader.keyMap, DB_MinEncryptedKeyBlockSize, &maxFreeSize );
		if( *pErrCode != ERROR_NO )
			return false;
	}

	cKeyBlockParams kbParams;
	cKeyBlockParams kbParamsNew;

	kbParams.Init( *ppDB, *pDBSize, ps->contAlgId, ps->encModeId, ps->hashAlgId, pErrCode, pFlags, hWnd );
	kbParamsNew.Init( *ppDB, *pDBSize, ps->contAlgId, ps->encModeId, ps->hashAlgId, pErrCode, pFlags, hWnd );

	if (((*pErrCode) = GetKeyBlockParams(
				hWnd, Text, LoadStringOnceT( hMainInst, IDS_ENTER_EXISTING_PASSWORD ),
				&kbParams, pFlags )) != ERROR_NO )
		return FALSE;

	BOOL bHidPart = FALSE;

	cPlainTextBlock ptb;

	if ( ptb.Allocate(
			sizeof( sKeyInfo ) +
			sizeof( dbHeader ) +
			0 )
				== NULL
		)
	{
		assert( false );
		*pErrCode = ERROR_INTERNAL_PROBLEM;
		return FALSE;
	}

	memset( ptb.buffer, 0, ptb.bufferSize );

	int pos = 0;

	sKeyInfo *keyInfo		= ( sKeyInfo   * )(ptb.buffer + pos); pos += sizeof( sKeyInfo   );
	dbHeader *decHeader		= ( dbHeader   * )(ptb.buffer + pos); pos += sizeof( dbHeader   );

	assert( pos == ptb.bufferSize );

	*pErrCode =
			DataBlockVerifyKeyBlockAndGetKey(
					*ppDB,
					*pDBSize,
					&kbParams,
					&(ps->keyMapInfo),
					NULL,
					keyInfo,
					decHeader
					);

	if ( *pErrCode != ERROR_NO )
		return FALSE;

	dbKeyId	keyMapT[ DB_maxKeyQ ];
	decHeader->updateKeyMap( keyMapT, false );
	ps->decryptedHeader.updateKeyMap( keyMapT, false );

	bool bIsForUsual = !keyInfo->contInfo.isForHiddenPart();

	if ( !bIsForUsual )
	{
		int iRes = MessageBoxT( hWnd,
						LoadStringOnceT( hMainInst, IDS_NewHidPasswordWarning ),
						LoadStringTwiceT( hMainInst, IDS_WARNING ),
						MB_YESNO | MB_ICONWARNING );

		if (iRes == IDNO)
		{
			*pErrCode = ERROR_CANCELLED_BY_USER;
			return FALSE;
		}

		bHidPart = TRUE;
		dbHeader::MarkBusied( keyMapT, &ps->keyMapInfo );
	}

	//
	//	For not encrypted it was ckeck above, before password entering
	//
	if ( isEncrypted )
	{
		*pErrCode = dbHeader::checkFreeSpace( decHeader->keyMap, DB_MinEncryptedKeyBlockSize, &maxFreeSize );
		if( *pErrCode != ERROR_NO )
			return false;
	}

	kbParamsNew.kbType = kbtPBE; // use it as default for not advanced dialogue.

	wchar_t caption[ 256 ];
	LoadStringT( hMainInst, IDS_EnterAdditionalPassword, caption, maxLen( caption ) );

	for(;;)
	{
		if (((*pErrCode) =
					GetKeyBlockParamsWithVerification(
						hWnd,
						Text, caption,
						&kbParamsNew,
						pFlags,
						maxFreeSize )
							) != ERROR_NO )
		{
			break;
		}

		// Check if such new password was added earlier

		if ( ( kbParamsNew.kbType != kbtPKE ) && // PKE cannot check the key because secret key has to be activated first
			 ( kbParamsNew.kbType != kbtSSS )	// SSS checks every password at creating
		    )
		{

			sKeyMapInfo kmi;

			if(	DataBlockVerifyKeyBlockAndGetKey(
						*ppDB,
						*pDBSize,
						&kbParamsNew,
						&kmi )
					== ERROR_NO )
			{
				dbHeader::MarkBusied( keyMapT, &kmi );

				MessageBoxT( hWnd,
					LoadStringOnceT( hMainInst, IDS_PASSWORD_IN_USE ),
					LoadStringTwiceT( hMainInst, IDS_WARNING ),
					MB_OK | MB_ICONWARNING );

				continue;
			}
		}

		break;
	}

	if (//((*pErrCode) == ERROR_NO) ||              // such password was added earliar - nothing to do
		((*pErrCode) == ERROR_CANCELLED_BY_USER)) // process is cancelled
	{
		return FALSE; // TRUE;
	}

	// It may be required for GenerateSeed dialogue as main wnd
	kbParamsNew.kbSSS.hWnd = hWnd;

	// If key map info passed to DataBlockWriteKey or DataBlockWriteKey_Hidden is not undefined the functions shreads existing.
	ps->keyMapInfo.setUndefined();

	if (!bHidPart)
	{
		*pErrCode = DataBlockWriteKey( ppDB, pDBSize, (*ppDB)->header.m_body.IsEncrypted(), keyInfo, &kbParamsNew, &(ps->keyMapInfo), keyMapT, decHeader );
	}
	else
	{
		*pErrCode =
				DataBlockWriteKey_Hidden(
					ppDB,
					pDBSize,
					keyInfo,
					&kbParamsNew,
					&(ps->keyMapInfo),
					keyMapT );
	}

	if ((*pErrCode) == ERROR_NOT_ENOUGH_SPACE_FOR_KEY )
	{
		MessageBoxT( hWnd,
			LoadStringOnceT( hMainInst, IDS_NoFreeSpaceToAddPassword ),
			LoadStringTwiceT( hMainInst, IDS_ERROR ),
			MB_OK | MB_ICONWARNING );
		*pErrCode = ERROR_NO;
	}

	if ( *pErrCode != ERROR_NO )
		return FALSE;

	updateKeyParams( ps, keyInfo, decHeader );

	return TRUE;
}
#endif

#if 0
static BOOL RemovePassword( const wchar_t* Text, const wchar_t* Caption, DATA_BLOCK** ppDB, DWORD* pDBSize, DWORD* pErrCode, DWORD* pFlags, kg_params_t* ps)
{
	cKeyBlockParams kbParams;
	kbParams.Init( *ppDB, *pDBSize, ps->contAlgId, ps->encModeId, ps->hashAlgId, pErrCode, pFlags, hWnd );

	cPlainTextBlock ptb;

	if ( ptb.Allocate(
			sizeof( sKeyInfo ) +
			sizeof( dbHeader ) +
			0 )
				== NULL
		)
	{
		assert( false );
		*pErrCode = ERROR_INTERNAL_PROBLEM;
		return FALSE;
	}

	memset( ptb.buffer, 0, ptb.bufferSize );

	int pos = 0;

	sKeyInfo *keyInfo		= ( sKeyInfo   * )(ptb.buffer + pos); pos += sizeof( sKeyInfo   );
	dbHeader *decHeader		= ( dbHeader   * )(ptb.buffer + pos); pos += sizeof( dbHeader   );

	assert( pos == ptb.bufferSize );

	if (((*pErrCode) =
				DataBlockCheck(*ppDB, *pDBSize, ps->contAlgId ))
					!= ERROR_NO)
			return FALSE;

	for( int iAttempts = 10; iAttempts--;  )
	{
		if (((*pErrCode) = GetKeyBlockParams(
					hWnd,
					Text,
					LoadStringOnceT( hMainInst, IDS_EnterAdditionalPasswordToDelete ),
					&kbParams, pFlags ))
						!= ERROR_NO)
		{
			if ( *pErrCode != ERROR_CANCELLED_BY_USER )
				*pErrCode = ERROR_NO;

			return *pErrCode == ERROR_NO;
		}

		// Check if we have entered password for hidden part.
		// If so, refuse to delete it

		*pErrCode =
				DataBlockVerifyKeyBlockAndGetKey(
						*ppDB,
						*pDBSize,
						&kbParams,
						&(ps->keyMapInfo ),
						NULL,
						keyInfo,
						decHeader );

		if ((*pErrCode) == ERROR_INCORRECT_PASSWORD)
		{
			MessageBoxT( hWnd,
				LoadStringOnceT( hMainInst, IDS_INCORRECT_PASSWORD ),
				LoadStringTwiceT( hMainInst, IDS_WARNING ),
				MB_OK | MB_ICONWARNING );
			*pErrCode = ERROR_NO;
			continue;
		}
		else
		if ((*pErrCode) != ERROR_NO )
			return FALSE;

		bool bIsForUsual = !keyInfo->contInfo.isForHiddenPart();

		if ( !bIsForUsual )
		{
			//MessageBoxT( hWnd,
			//	LoadStringOnceT( hMainInst, IDS_CannotDeleteHiddenPassword ),
			//	LoadStringTwiceT( hMainInst, IDS_WARNING ),
			//	MB_OK | MB_ICONWARNING );

			//*pErrCode = ERROR_NO;
			//continue;

			int res = MessageBoxT( hWnd,
				LoadStringOnceT( hMainInst, IDS_DeleteHiddenPasswordQ ),
				LoadStringTwiceT( hMainInst, IDS_INFORMATION ),
				MB_YESNO | MB_ICONQUESTION );

			if ( res == IDNO )
			{
				*pErrCode = ERROR_NO;
				continue;
			}
		}

		sKeyMapInfo kmi;
		kmi.setInfo( &(ps->keyMapInfo) );

		if ( bIsForUsual )
		{
			//
			//	Note - v.8
			//	BC v.8 allows to backup and restore key block so even the master password may be deleted.
			//	BC will notify before real key block updating.
			//
			//if ( ps->keyMapInfo.position == 0 )// == ERROR_MASTER_PASSWORD_ENTERED)
			//{
			//	MessageBoxT( hWnd,
			//		LoadStringOnceT( hMainInst, IDS_CannotDeleteMasterPassword ),
			//		LoadStringTwiceT( hMainInst, IDS_WARNING ),
			//		MB_OK | MB_ICONWARNING );
			//
			//	*pErrCode = ERROR_NO;
			//	continue;
			//}

			//
			//	Simple shreds found position by ps->keyMapInfo field without overwritting because kbParams is NULL
			//
			*pErrCode = DataBlockWriteKey( ppDB, pDBSize, (*ppDB)->header.m_body.IsEncrypted(), keyInfo, NULL, &kmi, NULL, decHeader );
		}
		else
		{
			if ( dbHeader::shredKeyTab( *ppDB, *pDBSize, &kmi ) )
				*pErrCode = ERROR_NO;
			else
				*pErrCode = ERROR_INCORRECT_DATA_BLOCK_SIZE;
		}

		if ( (*pErrCode) != ERROR_NO )
			return FALSE;

		// BC v.8 will notify before key block updating. See Note - v.8 above
		//MessageBoxT( hWnd,
		//	LoadStringOnceT( hMainInst, IDS_SuccessInDeletePassword ),
		//	LoadStringTwiceT( hMainInst, IDS_INFORMATION ),
		//	MB_OK | MB_ICONWARNING );

		break;
	}

	if ( (*pErrCode) != ERROR_NO )
		return FALSE;

	updateKeyParams( ps, keyInfo, decHeader );

	return TRUE;
}
#endif
#if 0
extern "C" DWORD UpdateAdvancedSettings( DWORD *Flags, HWND  hWnd )
{
	return GetAdvancedFlags( Flags, hWnd );
}
#endif


//
//	CreateHiddenPartEx
//	It does:
//		1. Gets new key block params for hidden part
//			1.1 Checks if same password already exists.
//		// See ( Description 1.1 ) below...
//			1.2 If the kbParams is for hidden part, it asks confirmation to overwrite the existing.
//				So space at key map table may be freed.
//			1.3 It mart the field is busied and does not allow to overwrite when key key is written to data block.
//
//		2. Verifies all we did before and gets key handle
//
//	Rules:
//		1. It does not modify dbHeader so main part key handle is not required.
//		2. It uses keyMapT to detect empty key places, and writes new key directly to data block
//		3. It does not extend key block, so it has to treat as busied all key in map beyond real data block size.
//

#if 0
BOOL CreateHiddenPartEx(
		wchar_t		*contFileName,
		DATA_BLOCK	*pDB,			// block of data that is stored inside file-container in encrypted form
		DWORD		dbSize,			// size of the block of data
		CBCAlg_Handle	out_handle_hp,
		sCKHParams	*psHP,			// parameters for created hidden part

		dbKeyId		*keyMapT,		// the information about busied key blocks, new hidden part block will be created at empty key places

		DWORD		*pErrCode,		// if the function returns FALSE, look at the Error code

		DWORD		flags
	)
{
	*pErrCode = ERROR_INTERNAL_PROBLEM;

	int verifyBlockSize = sizeof( encBlock < sVerificationBlock > );

	if ( ( keyMapT == NULL ) || ( psHP == NULL ) || ( psHP->size < sizeof( sCKHParams ) ) )
	{
		assert( false );
		*pErrCode = ERROR_INCORRECT_FORMAT_VERSION;
		return FALSE;
	}

	cSymAlgorithm alg;

	if ( !alg.Init( psHP->contAlgId, psHP->encModeId ) )
	{
		assert( false );
		*pErrCode = ERROR_INVALID_ALGORITHM;
		return FALSE;
	}

	cKeyBlockParams kbParams;
	kbParams.Init( pDB, dbSize, psHP->contAlgId, psHP->encModeId, psHP->hashAlgId, pErrCode, &flags, hWnd );

	*pErrCode = ERROR_INTERNAL_PROBLEM;

	cPlainTextBlock ptbDecrypted;
	if ( ptbDecrypted.Allocate(
			sizeof( sKeyInfo ) +
			0 ) == NULL )
	{
		assert( false );
		return FALSE;
	}

	int pos = 0;
	sKeyInfo *keyInfo		= ( sKeyInfo *)			(ptbDecrypted.buffer + pos); pos += sizeof( sKeyInfo );
	assert( pos == ptbDecrypted.bufferSize );

	keyInfo->keyAlgId  = kbParams.kbPBE.m_algId;
	keyInfo->encModeId = kbParams.kbPBE.m_encModeId;
	keyInfo->hashAlgId = (BYTE)kbParams.kbPBE.m_hashAlgId;

	keyInfo->contInfo = psHP->contInfo;

	if ( !GetSeedAndGenerateKey( hWnd, keyInfo, pErrCode ) )
		return false;

	sKeyMapInfo kmiTmp;

	for(;;)
	{
		// 1. Gets new key block params for hidden part

		//DWORD maxFreeSize = DB_MinEncryptedKeyBlockSize;
		//dbHeader::checkFreeSpace( keyMap, DB_MinEncryptedKeyBlockSize, &maxFreeSize );

		//if ( maxFreeSize < DB_MinEncryptedKeyBlockSize ) // It may be freed see comments at 1.2
		//	maxFreeSize = DB_MinEncryptedKeyBlockSize;

		*pErrCode =
			GetKeyBlockParamsWithVerification(
					hWnd, contFileName,
					LoadStringOnceT( hMainInst, IDS_HIDDEN_PASSWORD ),
					&kbParams, &flags ); // , maxFreeSize );

		if ( *pErrCode != ERROR_NO )
			return FALSE;

		// 1.1 Checks if same password already exists.
		sKeyMapInfo kmTmp;

// #define _allowToOverwriteExistingHiddenPartHere_
		//	Description 1.1 + 1.2
		//	The block 1.1 checks is same password already exists.
		//	But also it checks if the existing password is for hidden part and allows to overwrite the existing one.
		//	So we have to completely decode keyInfo in the case. It is checked immediately for PBE, but SSS requires
		//	additional shared passwords ( note, we have been going to check password only ).
		//
		//	Description 1.1 + 1.3 without 1.2
		//	It does not check hidden or usual part has the password. It just allow to save the existing key as all create
		//	key functions ( Add or Change ) do.
		//
#ifdef _allowToOverwriteExistingHiddenPartHere_
		//dbContInfo contInfo;

		////
		////	Sets undefined key block type,
		////	it means that all available shemes PBE, PKE and SSS will be implemented to the DATA_BLOCK.
		////	See implementaion of the idea at DataBlockVerifyKeyBlockAndGetKey.
		////
		//ekbType kbTypeSave = kbParams.kbType;
		//kbParams.kbType = kbtEmpty;

		//*pErrCode = DataBlockVerifyKeyBlockAndGetKey( pDB, dbSize, &kbParams, &kmTmp, NULL, NULL, NULL, &contInfo );
		//
		//kbParams.kbType = kbTypeSave;

		//if ( *pErrCode == ERROR_NO )
		//{
		//	bool bIsForUsual = !contInfo.isForHiddenPart(); // bodyOffset == decHeader->contInfo.bodyOffset;

		//	if ( bIsForUsual )
		//	{
		//		MessageBoxT( hWnd,
		//			LoadStringOnceT( hMainInst, IDS_PASSWORDS_EQUAL ),
		//			LoadStringTwiceT( hMainInst, IDS_INVALID_PWD ),
		//			MB_OK | MB_ICONWARNING );

		//		continue;
		//	}
		//	else
		//	{
		//		// 1.2 The kbParams is for hidden part.
		//		int res = MessageBoxT( hWnd,
		//				LoadStringOnceT( hMainInst, IDS_HidPasswordOverwrite ),
		//				LoadStringTwiceT( hMainInst, IDS_INFORMATION ),
		//				MB_YESNO | MB_ICONQUESTION | MB_DEFBUTTON2 );

		//		if ( res != IDYES )
		//		{
		//			// 1.3 Marks the hideen part place as busied
		//			dbHeader::MarkBusied( keyMapT, &kmTmp );
		//			continue;
		//		}

		//		if ( kmTmp.position == -1 )
		//		{
		//			assert( false );
		//		}
		//		else
		//		{
		//			// Overwrites key table place were hidden part was found and frees keyMapT.
		//			//DataBlockWriteKey( &pDB, &dbSize, pDB->header.m_body.IsEncrypted(), NULL, NULL, &kmTmp, keyMapT );

		//			// Sheades key table only..
		//			// The hidden part creation may shred the key tab without header modification
		//			dbHeader::shredKeyTab( pDB, dbSize, &kmTmp );
		//			if ( keyMapT != NULL )
		//				dbHeader::MarkEmpty( keyMapT, kmTmp.position, kmTmp.quantity );

		//			kmTmp.setUndefined();
		//		}
		//	}
		//}
#else
		*pErrCode = DataBlockVerifyKeyBlockAndGetKey( pDB, dbSize, &kbParams, &kmTmp ); //, NULL, NULL, NULL, NULL );

		if ( *pErrCode == ERROR_NO )
		{
			// 1.3 Accumulates hidden keyMapT
			dbHeader::MarkBusied( keyMapT, &kmTmp );

			MessageBoxT(
					hWnd,
					LoadStringOnceT( hMainInst, IDS_PASSWORD_IN_USE ),
					LoadStringTwiceT( hMainInst, IDS_WARNING ),
					MB_OK | MB_ICONWARNING );

			continue;
		}
#endif

		// 2. Writes hidden part key info to data block

		*pErrCode = DataBlockWriteKey_Hidden( &pDB, &dbSize, keyInfo, &kbParams, &kmiTmp, keyMapT );

		if ( *pErrCode != ERROR_NO )
		{
			if ( (*pErrCode) == ERROR_NOT_ENOUGH_SPACE_FOR_KEY )
			{
				int res = MessageBoxT(
					hWnd,
					LoadStringOnceT( hMainInst, IDS_NoFreeSpaceToAddPassword ),
					LoadStringTwiceT( hMainInst, IDS_ERROR ),
					MB_ICONSTOP | MB_OKCANCEL );

				// The free space may be freed if an existing hidden part was overwritten see p 1.2
				if ( res == IDOK )
					continue;

			}

			if ( *pErrCode != ERROR_CANCELLED_BY_USER )
			{
				assert( false );
				*pErrCode = ERROR_CANCELLED_BY_USER;
			}

			return FALSE;
		}

		break;
	}

	// 2. verifies all that we did before and gets key handle
	if (((*pErrCode) =
			DataBlockVerifyKeyBlockAndGetKey(
					pDB, dbSize,
					&kbParams, &(psHP->keyMapInfo ),
					pKeyHandleHP,
					keyInfo,
					NULL,
					&(psHP->contInfo ) )
				) != ERROR_NO )
			return FALSE;

	updateKeyParams( psHP, keyInfo );

	if ( !psHP->contInfo.isForHiddenPart() ||
		( kmiTmp.position != psHP->keyMapInfo.position ) ||
		( kmiTmp.quantity != psHP->keyMapInfo.quantity ) )

	{
		assert( false );

		*pErrCode = ERROR_INTERNAL_PROBLEM;

		if ( pKeyHandleHP != NULL )
			FreeKeyHandle( psHP->contAlgId, *pKeyHandleHP );

		return FALSE;
	}

	*pErrCode = ERROR_NO;
	return TRUE;
}
#endif

/***********************************************************************************************
 *
 * The function is needed if the Key Generator is developed for using with
 * some of the latest BestCrypt versions - because of some reason.
 *
 ***********************************************************************************************/

#if 0
BOOL CheckBestCryptVersion(DWORD MajorVersion, DWORD MinorVersion)
{
	if ( MajorVersion < 8 ) return FALSE;
	return TRUE;
}
#endif

//
//	UpdateHeader
//		Note, inBuff may be same as outBuff
//		If data block is encrypted so the header, contInfo and keyMap is encrypted
//		The function decryptes/encrypts data block to outBuff
//

#if 0
BOOL UpdateHeader(
	HWND	hWnd,		// parent Window for any dialogs drawn by the DLL
	BYTE	*dataBlock,
	DWORD	dataBlockSize,
	DWORD	algId,			// Encryption Algorithm Identificator
	DWORD	encModeId,		// Encryption mode
	DWORD	keyHandle,		// Key Handle
	BYTE	*outBuff,		// Caller procedure is responsible for secure erasing of the decrypted data
	DWORD	buffSize,
	DWORD	*pErrCode,		// if the function returns FALSE, look on the Error code
	sUBParam	*param		// = NULL
)
{
	DWORD errCode;
	if ( pErrCode == NULL )
		pErrCode = &errCode;

	//*pErrCode = verifyKeyHandle( (DATA_BLOCK *)dataBlock, keyHandle, algId, encModeId );
	//
	//if ( *pErrCode != ERROR_NO )
	//	return false;

	if ( param != NULL )
	{
		DWORD hashAlgId = 0;//bchaSHA256;

		sParamEncrypt *sp = (sParamEncrypt *)param;
		if ( param->size >= sizeof( sParamEncrypt ) )
			hashAlgId = sp->hashAlgId;

		if ( hashAlgId == 0 )
			hashAlgId = bchaSHA256;

		if (  ( param->flags == UBFlag_Encrypt ) ||
		      ( param->flags == UBFlag_Encrypt_Digest_Only ) )
		{
			if ( ( outBuff != NULL ) && ( buffSize < sizeof( encBlock< dbHeader > ) ) )
			{
				*pErrCode = ERROR_BUFFER_TOO_SMALL;
				return false;
			}

			bool bEntire = ( param->flags == UBFlag_Encrypt );

			return
				encryptHeaderEx(
						(DATA_BLOCK *)dataBlock,
						algId, encModeId, hashAlgId, keyHandle,
						(encBlock< dbHeader > *)outBuff,
						pErrCode,
						bEntire );
		}
		else
		if ( param->flags == UBFlag_Decrypt )
			return decryptHeaderEx(
						(DATA_BLOCK *)dataBlock,
						algId, encModeId, &sp->hashAlgId, keyHandle,
						(encBlock< dbHeader > *)outBuff,
						pErrCode );

		return false;
	}
	else
		return decryptHeaderEx(
						(DATA_BLOCK *)dataBlock,
						algId, encModeId, NULL, keyHandle,
						(encBlock< dbHeader > *)outBuff,
						pErrCode );
}
#endif

