/*******************************************************************
 *
 *	Copyright (c) 2005-2009 Jetico, Inc., Finland
 *	All rights reserved.
 *
 *	File: kgghost.h
 *	Revision: 
 *	Created:
 *	Description: External kgghost interface.
 *
 *******************************************************************/

#ifndef V8_KGGHOST_H
#define V8_KGGHOST_H

#include "layout.h"
#include "datablock.h"
#include "comm_def/kg_defs.h"

#include <common/build.h>

#define KEYGEN_ID    0x00000004
#define DISPLAY_NAME "KG-Ghost"
#define SERVICE_NAME "kgGhost.dll"

#define MAJOR_VERSION 0x00000001
#define MINOR_VERSION 0x00000000


/**
 * @brief	An in\out set of context for kgghost operations.
 */
struct kg_context_t
{ 
	kg_context_t() { memset( this, 0, sizeof (*this)); }


	/*	In: 	metric of the new container or hidden part
		Out:	metric of authenticated container or hidden part */
	space_unit_t 	contInfo;

	/*	In:	encryption algorithm id of the new container
		Out:	real encryption algorithm of the container */
	DWORD 		contAlgId;

	
	/*	In:	encryption mode of the new container
		Out:	real encryption mode of the container */
	DWORD 		encModeId;


	/*	In:	hash algorithm for new keyblock
		Out:	real hash algorithm of the keyblock */
	DWORD 		hashAlgId;


	/*	In: 	header is encrypted
		Out:	verified if header is encrypted */
	bool bEncrypted;


	/*	In:	explicit descrypted header 
		Out:	decrypted header on request */
	kgghost::db_header_t decryptedHeader;


	~kg_context_t() { memset( this, 0, sizeof (*this)); }
};



/**
 * @brief
 */
BC_EXTERN
BOOL GetNameAndVersion( 
		char  *Name, 
		UINT  nameSize, 
		DWORD *Major, 
		DWORD *Minor 
);


/**
 * @brief
 */
extern "C"
BOOL CreateKeyHandle_V2(
		// in
		DWORD		command,
		const char*	caption,
		unsigned int*	flags, 

		// in\out
		BYTE**		ioDataBlock,
		DWORD*		ioDBSize,
		kg_context_t*	ioParams,

		// out
		bdk::BCAlgHandle& 	oHandle,
		DWORD*		oErrCode
);


/**
 * @brief
 */
BC_EXTERN
BOOL CreateKeyHandle(
	char		*AlgServiceName,// what Encryption Algorithm will use the encryption key
	DWORD		AlgId,			// Encryption Algorithm Identificator
	DWORD		AlgKeyLength,	// encryption key length for the Encryption Algorithm
	char		*Text,			// Any information text (optional parameter)
	char		*Caption,		// Caption of the "get password" dialog window (optional parameter)
	DWORD		CreateFlag,		// whether to create new key or open existing
	BYTE		**vDataBlock,	// block of data that is stored inside file-container in encrypted form
	DWORD		*DataSize,		// size of the block of data
	DWORD		*ErrorCode		// if the function returns FALSE, look at the Error code
);


/**
 * @brief
 */
BC_EXTERN
BOOL CreateKeyHandleEx(
	char		*AlgServiceName,	// what Encryption Algorithm will use the encryption key
	DWORD		AlgId,			// Encryption Algorithm Identificator
	DWORD		AlgKeyLength,	// encryption key length for the Encryption Algorithm
	char		*Text,			// Text, containing filename of the container
	char		*Caption,		// Caption of the "get password" dialog window
	DWORD		CreateFlag,		// whether to create new key or open existing
	BYTE		**vDataBlock,	// block of data that is stored inside file-container in encrypted form
	DWORD		*DataSize,		// size of the block of data
///	KEY_HANDLE	*KeyHandle,		// returned Key Handle
	DWORD		*ErrorCode,		// if the function returns FALSE, look on the Error code
///	HWND		hWndParent,		// parent Window for any dialogs drawn by the DLL
	DWORD		*FavoriteDrive, // if 0 <= *Favorite drive < 26, user will be asked to enter drive letter
	DWORD		*Flags          // see the KG_DEFS.H file for the flags (FLAG_...) that may be set/get
);


/**
 * @brief
 */
BC_EXTERN
DWORD UpdateAdvancedSettings( DWORD *Flags /**, HWND hWndParent*/ );


/**
 * @brief
 */
BC_EXTERN
DWORD FreeDataBlock( BYTE **vDataBlock );


/**
 * @brief
 *
BC_EXTERN
BOOL CreateHiddenPartEx(
///		HWND		hWnd,
		wchar_t			*contFileName,
		kgghost::DATA_BLOCK	*pDB,			// block of data that is stored inside file-container in encrypted form
		DWORD		dbSize,			// size of the block of data
///		KEY_HANDLE	*pKeyHandleHP,	// hidden part key handle 
		kg_params_t*	psHP,			// parameters for created hidden part
		kgghost::key_map_t*		keyMapT,		// the information about busied key blocks, new hidden part block will be created at empty key places
		DWORD		*pErrCode,		// if the function returns FALSE, look at the Error code
		DWORD		flags
	);
*/
//
//	Real space unit table may be stored at specified address.
//	The DecryptTable tries to decrypt it.
//
enum { dt_errNo = 0, dt_bufferTooSmall = 1, dt_invalidSignature, dt_errorInFileOpening, dt_invalidPointer, dt_errorInFileWritting };


BC_EXTERN
BOOL GetAdditionalCommandNames(
	wchar_t  *CommandNamesBuffer,   // buffer where the function returns strings of commands, like "comm1\0comm2\coom3\0\0"
	int   *BufferSize,           // size of the buffer for the command names
	DWORD *ErrorCode,            // if the function returns FALSE, ErrorCode is valid, see kg_defs.h
	DWORD *Flags                 // some flags BestCrypt tells to the Key Generator, see kg_defs.h
);


BC_EXTERN
BOOL RunAdditionalCommand(
///	HWND    hWnd,
	wchar_t    *CommandName,   
	wchar_t    *ContainerPath,
	BYTE    *DataBlock,
	int     DataBlockSize,
	void   *params           // see the kg_defs.h file
);


BC_EXTERN
BOOL CheckBestCryptVersion(
///	HWND    hWnd,
	DWORD   MajorVersion,
	DWORD   MinorVersion 
);


BC_EXTERN
BOOL QueryFunction(
	const BYTE	**vDataBlock,	// block of data that is stored inside file-container in encrypted form
	const DWORD	dataSize,		// size of the block of data
	const DWORD	functionID		// identefier of function used as CerateKeyHandle(.., CreateFlag, ... )
);


enum {
	UBFlag_Decrypt = 0,
	UBFlag_Encrypt = 1,
	UBFlag_Encrypt_Digest_Only = 2
};

struct sUBParam
{
	DWORD	size;
	DWORD	flags;
	
	sUBParam()
	{
		memset( this, 0, sizeof( *this ) );
		size = sizeof( *this );
	}
};

struct sParamEncrypt : public sUBParam
{
	DWORD hashAlgId;
};



BC_EXTERN
BOOL UpdateHeader(
///	HWND	hWnd,		// parent Window for any dialogs drawn by the DLL
	BYTE	*dataBlock,
	DWORD	dataBlockSize,
	DWORD	algId,			// Encryption Algorithm Identificator
	DWORD	encModeId,		// Encryption mode
///	DWORD	keyHandle,		// Key Handle
	BYTE	*outBuff,		// Caller procedure is responsible for secure erasing of the decrypted data
	DWORD	buffSize,
	DWORD	*pErrCode,		// if the function returns FALSE, look on the Error code
	sUBParam	*param		// = NULL
	);

/*
BC_EXTERN
int ShowMessage( HWND hWnd, kg_params_t* params );


BC_EXTERN
bool GetAlarmCrashHotkey( DWORD *pDWValue );


BC_EXTERN
bool SetAlarmCrashHotkey( DWORD dwValue );
*/

#endif

