/*******************************************************************\
*
*	Copyright (c) 2009 Jetico, Inc., Finland
*	All rights reserved.
*
*	File: layout.h
*	Revision: 
*	Created:
*	Description: declaration of DATA_BLOCK layout sizes and offsets
*
\*******************************************************************/

#ifndef V8_LAYOUT_H
#define V8_LAYOUT_H

#include <common/types.h>

#define _4KB 	4096

#define ROUND_UP( value, blockSize ) 	( ((value) +  (blockSize) - 1) & ~((blockSize) - 1) )
#define ROUND_DOWN( value, blockSize ) 	( (value) & ~((blockSize) - 1) )
#define ROUND( value, blockSize ) 	ROUND_UP( value, blockSize )
#define ROUND_TAIL( value, blockSize ) 	ROUND( (value), blockSize ) - (value)


//#define DB_MIN_SIZE				_4K
#define DB_KEY_MAP_SIZE				512
#define DB_HEADER_SIZE_WITH_KEY_MAP		( 512 + DB_KEY_MAP_SIZE )



// ( 4k - 1024 - 512 ) / 256 = 10 keys may be stored at minimal key block
#define DB_MIN_SLOTS_Q 				((DB_MIN_SIZE - DB_HEADER_SIZE_WITH_KEY_MAP - DB_POOL_SIZE ) / DB_KEYBLOCK_SLOT_SIZE) 

#define DB_MAP_ENTRY_SIZE			8

// 512 / 8 == 64 keys may be added to container
#define DB_EXTENDED_SLOTS_Q 			( DB_KEY_MAP_SIZE / DB_MAP_ENTRY_SIZE ) 

// Total maximum size by all DATA_BLOCK structures
// The remainder from all requred fields is occupied by key map table 
//#define DB_MAX_SIZE				ROUND( DB_EXTENDED_SLOTS_Q * DB_KEYBLOCK_SLOT_SIZE + DB_HEADER_SIZE_WITH_KEY_MAP + DB_POOL_SIZE, _4K )

//
//	We have to emulate the initial part of previose hidden sector
//	because:
//	- Guard utility detects the OEMid ( 'LOCOS94 ' ) field to work with conatiner
//	- Old versions of BestCrypt have to identify this as  bc container but a newer unsupported one to avoid data corruption 
//	volume label field will be changed form 'CRYPTED_DSK' ( 11 symbols ) to key gen name 'BC_KeyGenID'
//
// sizeof( BPB ) + 7 is space between signature and volume label in real BootRecord
//							   25         + 7  - 4 == 28
#define DB_OCCUPIED_NAME_SIZE 			( sizeof( BPB ) + 7 -  CONTAINER_ID_LENGTH ) 
#define	MAXIMUM_KEY_SIZE_BYTES			64

#define DB_MAX_ENC_BLOCK_SIZE			16	// Max encryption block size

#define	MAXIMUM_DIGEST_SIZE_BYTES		32
#define SALT_SIZE				8


/**
 * @brief	Space unit span definition
 */
struct space_unit_t
{
	bcu64_t offset;
	bcu64_t length;

	bool is_hidden() const
	{
		return (length == _4KB);
	}
};

#endif
