/**

	Copyright (c) 2009 Jetico, Inc., Finland
	All rights reserved.

*/

#ifndef BDK_ALG_MANAGER_H
#define BDK_ALG_MANAGER_H

#include <list>

#include <common/class_defs.h>
#include <common/smartptr.h>

#include <alg/service.h>


namespace bdk
{

        /** @brief List of supported algorithms */
        typedef std::list<BCAlgService*> BCAlgList;

	/**
	 * @brief	Manages loaded alg services
	 */
	class BCAlgManager
	{
	BC_FORBID_COPY(BCAlgManager)

	public:


		/**
		 * @brief	Construct manager
		 */
		BCAlgManager();


		/**
		 * @brief 	Destroy manager and deallocate any hanging algorithms' resources
		 */
		~BCAlgManager() throw();


		/**
		 * @brief 	Initialize algorithm
		 *
		 * @p info	Algorithm info
		 *
		 * @return	true if algorithm can be loaded, false otherwise
		 */
		BCAlgService* 	load_algorithm( const bc_alg_info_t& info );


		/**
		 * @brief	Release algorithm service and kill all it's handles
		 *
		 * @p alg	Algorithm to release
		 */
		void 		release_algorithm( BCAlgService* alg );


		/**
		 * @brief	Find algorithm service by id
		 *
		 * @p id	Algorithm unique id
		 *
		 * @return	Algorithm service or null, if not found
		 */
		BCAlgService* 	find_algorithm( int id );


		/**
		 * @brief	Find algorithm service by name
		 *
		 * @p name	Algorithm unique name
		 *
		 * @return	Algorithm service or null, if not found
		 */
		BCAlgService* 	find_algorithm( std::string const & name );


		/**
		 * @brief	List all registered algorithms.
		 */
		void list_supported_algorithms(BCAlgList & e);


	private:


		typedef BCSharedPtr<BCAlgService>	serptr_t;
		typedef std::list < serptr_t > 		entries_t;

		/* services storage */
		entries_t	m_entries;
	};


}

#endif

