/*
 *	Copyright (c) 2009 Jetico, Inc., Finland
 *	All rights reserved.
 */

#ifndef BDK_ALG_MODE_H
#define BDK_ALG_MODE_H


namespace bdk
{
        /**
         * @brief       Algorithm mode definition
         */
        struct bc_alg_mode_t
        {
                /* mode tag */
                bc_alg_mode_id_t        id;

                /* is this mode supported ? */
                bool                    is_supported;

                /* total key size for encryption mode */
                size_t                  key_size;
        };


        /**
         * @brief       Encryption mode tag.
         */



        class BCAlgMode
        {
        public:

                enum mode_tag_t
                {
                        AM_LRW = 0xbc000001,     ///< LRW tweakable narrow-block encryption
                        AM_CBC = 0xbc000002,     ///< Cypher Block Chaining mode
                        AM_XTS = 0xbc000004,     ///< XTS tweakable narrow-block encryption
                };


                BCAlgMode(bcu32_t id, size_t key_size) :
                        m_tag(id),
                        m_key_size(key_size)
                {
                }

                bcu32_t id() const
                {
                        return m_id;
                }

                std::string name() const
                {
                        return name(m_tag);
                }


                static std::string name(mode_tag_t tag)
                {
                        switch(id)
                        {
                                case AM_CBC:    return "CBC";
                                case AM_LRW:    return "LRW";
                                case AM_XTS:    return "XTS";
                                default: BC_ASSERT(false);
                        }
                }

        private:

                mode_tag_t m_tag;

                size_t m_key_size;

        };
}


#endif /* BDK_ALG_MODE_H
