/**
 *	Copyright (c) 2009 Jetico, Inc., Finland
 *	All rights reserved.
 */

#ifndef BDK_BACKEND_SYNC_H
#define BDK_BACKEND_SYNC_H

#include <common/types.h>

namespace bdk
{
        namespace backend
        {

                /**
                 * @brief	Mutex handle
                 */
                typedef struct bc_os_mutex_t* bc_mutex_handle_t;


                /**
                 * @brief	Get system mutex handle.
                 */
                bc_mutex_handle_t bc_mutex_get();

                /**
                 * @brief	Put system mutex handle
                 */
                void bc_mutex_put(bc_mutex_handle_t& handle);

                /**
                 * @brief	Lock system mutex
                 */
                void bc_mutex_lock(bc_mutex_handle_t handle);

                /**
                 * @brief	Unlock system mutex
                 */
                void bc_mutex_unlock(bc_mutex_handle_t handle);

                /**
                 * @brief     Thread handle
                 */
                typedef struct bc_os_thread_t* bc_thread_handle_t;

                /**
                 * @brief     Thread entry function pointer
                 */
                typedef void*(*bc_thread_func_t)(void*);

                /**
                 * @brief     Thread entry parameter
                 */
                typedef void* bc_thread_par_t;

                /**
                 * @brief     Get new system thread handle.
                 */
                bc_thread_handle_t bc_thread_get();

                /**
                 * @brief     Release system thread handle.
                 */
                void bc_thread_put(bc_thread_handle_t handle);

                /**
                 * @brief     Start thread.
                 */
                void bc_thread_start(bc_thread_handle_t handle, bc_thread_func_t entry,
                                     bc_thread_par_t par);
        }
}

#endif
