#ifndef BDK_COMMON_BUILD_H
#define BDK_COMMON_BUILD_H


/*
 *	Version
 */


#define BCCORE_BUILD_MAJOR              3
#define BCCORE_BUILD_MINOR             	0
#define BCCORE_BUILD_PATCH             	0


/*
 *	Copiler defs
 */


#if defined(_MSC_VER)

	#define BC_INLINE		inline
        #define BC_PACKED(def)          #pragma pack(push,1) \
                                        def \
                                        #pragma pack(pop)

        #ifdef BCCORE_MODULE

                #define MODULE          __declspec(dllexport)

        #else

                #define MODULE          __declspec(dllimport)

        #endif

        #define BC_MODULE_EXPORT           __declspec(dllexport)
        #define BC_MODULE_IMPORT           __declspec(dllimport)
	#define BC_EXTERN		   BC_MODULE_EXPORT

#elif defined(__GNUC__)

        //#error The only supported compilers are MSVC and GCC.
	#define BC_INLINE 		inline
        #define BC_PACKED(def)          def __attribute__((packed))

        #ifdef BCCORE_MODULE

                #define MODULE          __attribute__((visibility("default")))

        #else

                #define MODULE

        #endif

        #define BC_MODULE_EXPORT           __attribute__((visibility("default")))
        #define BC_MODULE_IMPORT
	#define BC_EXTERN		   BC_MODULE_EXPORT


	#define ATOMIC_INC(value)  	\
	__asm__	__volatile__		\
        (				\
            	"lock\n\t"		\
            	"incl %0"		\
		:"+m"( (value) )	\
		:			\
            	:"cc"			\
        );				\


	#define ATOMIC_DEC(value)	\
	__asm__				\
	(				\
		"lock\n\t"		\
		"decl %0"		\
		:"+m"( (value) )	\
		:			\
		:"cc"			\
	);
#else

        #error The only supported compilers are MSVC and GCC.

#endif


/*
 *	Debug assert
 */


#if defined(BC_DEBUG)

	#include <assert.h>
	#define BC_ASSERT(exp)		assert((exp))

	#include <common/log.h>
	#include <stdio.h>
	#define BC_DEBUG_PRINT(str)	printf str

	#include <common/class_defs.h>
	#define BC_DELETE(type, value) 	bc_checked_delete<type>( (value) )
#else

	#define BC_ASSERT(exp)
	#define BC_DEBUG_PRINT(str)
	#define BC_DELETE(type, value) 	delete (value)

#endif


#endif
