#ifndef BDK_COMMON_CLASS_DEFS_H
#define BDK_COMMON_CLASS_DEFS_H

/**
 * @brief 	Declare non-copyable class
 */
#define BC_FORBID_COPY(classname)			\
private:									\
	classname ( const classname & rhv ) {};	\
	classname & operator = ( const classname & rhv ) { return *this; }


/**
 * @brief	Declare interface class
 */
#define BC_INTERFACE(classname)				\
protected:									\
	classname () {};						\
public:										\
	virtual ~ classname () {};


/**
 * @brief 	Declare no-throw new
 */
#include <new>
#define NOTHROW_NEW 	new(std::nothrow)


/*
 * @brief	Tag to comment inherited functionality
 */
#define BC_INHERITED	virtual


namespace bdk
{

	/**
	 * @brief 	Checked delete of incomplete type
	 */
	template < class T >
	void bc_checked_delete( T* value )
	{
		typedef T this_type_must_be_complete_at_destruction_point;
		(void)sizeof(this_type_must_be_complete_at_destruction_point);
		delete value;
	}


	/**
	 * @brief 	Storage of type's data in a buffer rounded to specific value
	 */
	template
	<
		class T,
		unsigned int boundary
	>
	struct bc_padded_type_t
	{
		/* sizeof(T) rounded to boundary value. if size is already rounded up, will create a buffer of sizeof(T) size */
		unsigned char m_buffer[
			(sizeof(T) + boundary - 1) & 	// sizeof type overflowed to next rounding boundary
			~(boundary - 1) 		// zero out overflowed rounding bits
		];


		/**
		 * @brief	Get stored type
		 */
		T* type() { return reinterpret_cast<T*>(&m_buffer); }


		/**
		 * @brief	Get stored type
		 */
		const T* type() const { return reinterpret_cast<const T*>(&m_buffer); }
	};

}

#endif


