#ifndef BC_COMMAND_H
#define BC_COMMAND_H

#include <string>
#include <map>

#include <common/class_defs.h>

namespace bdk 
{

        /**
         * @brief BestCrypt command implementation interface
         */
        class BCCommandImpl {
        public:
                /**
                 * @brief Execute command
                 */
                //virtual bool execute(unsigned int argc, char** argv) = 0;
		virtual bool execute() = 0;

                virtual ~BCCommandImpl() {};
        };
        


	/**
	 * @brief	Instance that manages registered commands.
	 * @more	Can be extended by modules to add specific command processing as well as overload existing commands.
	 */
	class BCCommandManager
	{
	BC_FORBID_COPY(BCCommandManager)

	public:


		BCCommandManager();

		~BCCommandManager();

		void register_command( std::string const & name, BCCommandImpl* implementor );
		
		void unregister_command( std::string const & name );

		bool find_command(std::string const & name);



	private:


		typedef std::map< std::string, BCCommandImpl* > commands_t;
		typedef commands_t::iterator	commands_iter_t;

		commands_t m_commands;


	protected:


		commands_iter_t _find_command_iter(std::string const & name);

	};
}


#endif

