/**
 *
 *	Copyright (c) 2009 Jetico, Inc., Finland
 *	All rights reserved.
 *
 *	file: core.h
 *	scope: bdk\common
 *	author: evgeny yakovlev
 *
 *	description:
 *	Core services entry-point.
 *
 */

#ifndef BDK_COMMON_CORE_H
#define BDK_COMMON_CORE_H

#include <mod/manager.h>
#include <alg/manager.h>
#include <container/manager.h>

#include <container/factory.h>
#include <container/format.h>

namespace bdk
{


	/**
	 * @brief	Core service entry point
	 * @more	A single-entity hub from where all other services can be accessed.
	 *		An instance of this class must be created before any other services can be reached.
	 *		Once it has been created it can be accessed through singleton derived methods (see CBCSingleton).
	 */
	class BCCore
	{
	BC_FORBID_COPY(BCCore)
	public:


		/**
		 * @brief	Core singleton instance ref.
		 */
		static BCCore& instance();


		/**
		 * @brief	Core singleton instance ptr.
		 */
		static BCCore* p_instance();


		/**
		 * @brief		Get core container factory
		 */
		BCContainerManager* 	container_manager()
		{
			return &m_cont_manager;
		}


		/**
		 * @brief		Get core module manager
		 */
		BCModuleManager*	module_manager()
		{
			return &m_mod_manager;
		}


		/**
		 * @brief		Get core algorithm manager
		 */
		BCAlgManager*		alg_manager()
		{
			return &m_alg_manager;
		}


		/**
		 * @brief		Load algorithm provider support
		 */
		bool 			load_algservice( const bc_alg_info_t& alginfo )
		{
			return alg_manager()->load_algorithm(alginfo);
		}


		/**
		 * @brief		Release algorithm service
		 */
		void			release_algservice( int id )
		{
			//
		}


		/**
		 * @brief	Get core alg service by alg id
		 */
		BCAlgService* get_alg_service(int algid)
		{
			return alg_manager()->find_algorithm(algid);
		}


		/**
		 * @brief	Get core alg service by alg name
		 */
		BCAlgService* get_alg_service(std::string const & algname)
		{
			return alg_manager()->find_algorithm(algname);
		}


		BCAlgHandle	init_alg( BCAlgService* service, int mode, bc_byte_t* key, bc_byte_t* pool  )
		{
			return bc_make_handle(service->allocate_alg( mode, key, pool ));
		}



	private:


		/**
		 * @brief	Forbid explicit construction
		 */
		BCCore() {};

		/* container factory root */
		BCContainerManager	m_cont_manager;

		/* loadable module manager */
		BCModuleManager	m_mod_manager;

		/* algorithm service manager */
		BCAlgManager		m_alg_manager;

		/* command manager */
		//CBCCommand_Manager	m_command_manager;


		/* log manager */
//		CBCLog_Manager		m_log_manager;

		/* container link manager */
//		CBCLink_Manager		m_link_manager;


	};


	BCCore* bc_core();

}

#endif
