
#ifndef BCC_COMMON_DSO_H
#define BCC_COMMON_DSO_H

#include <stdlib.h>


#include <backend/dso.h>
#include <common/exception.h>
#include <common/class_defs.h>


namespace bdk
{

	class BCDSO
	{
	BC_FORBID_COPY(BCDSO)
	public:


		/**
		 * @brief 	Initializes dso reference
		 * @throws 	CBCException on failed init. System message embedded in exception.
		 */
		BCDSO( const char* dsoname, bc_dso_resolve_t rflag, bc_dso_visibility_t vflag );


		/**
		 * @brief 	Releases dso reference
		 */
		~BCDSO() throw();


		/**
		 * @brief	Return filesystem path to dso object
		 */
		const char* path()
		{
			return backend::bc_dso_path( m_handle );
		}


		/**
		 * @brief	Find requested symbol in dso object
		 *
		 * @p sym	Store symbol handle here
		 * @p symname	Symbol name to look for
		 *
		 * @return	System result with detailed platform error description.
	 	 */
		void* load_symbol_type( const char* symname)
		{
			return backend::bc_dso_find( m_handle, symname );
			//return reinterpret_cast < T > (dsosym);
		}


	private:

		/** backend defined dso handle */
		backend::bc_dso_handle_t m_handle;
	};

}


#endif

