#ifndef BDK_COMMON_EXCEPTION_H
#define BDK_COMMON_EXCEPTION_H

#include <string>
#include <exception>

#include <common/types.h>
#include <common/class_defs.h>

namespace bdk
{

	/**
	 * @brief Basic BC exception.
	 *
	 * Provides common functionality for general errors
	 * Specific errors are derived
	 */
	class BCException : public std::exception
	{
	public:


		/**
		 * @brief	Construct empty exception.
		 */
		BCException();


		/**
		 * @brief	Construct exception with a message
		 */
		explicit BCException(std::string const & msg);


		/**
		 * @brief	Destroy excpetion.
		 */
		~BCException() throw() {};


		/**
		 * @brief	Get exception message.
		 */
		BC_INHERITED	const char* what() const throw()
		{
			return m_msg.c_str();
		}


	protected:

		/* message string */
		std::string m_msg;

	};


	/**
	 * @brief	System exception.
	 * @more	Thrown on system failures, mainly from backend implementations
	 */
	class BCSystemException : public BCException
	{
	public:

		/**
		 * @brief	Construct system exception with attached system-specific error definition.
		 */
		BCSystemException(std::string const & msg) : BCException(msg) {};

		BCSystemException() {};

	};


	class BCDublicateException : public BCException
	{
	public:
		BCDublicateException(std::string const & msg) : BCException(msg) {};

	};

}



#endif

