/*
 *	Copyright (c) 2009 Jetico, Inc., Finland
 *	All rights reserved.
 */

#ifndef BDK_COMMON_MEMORY_H
#define BDK_COMMON_MEMORY_H

#include <common/types.h>
#include <common/class_defs.h>
#include <common/build.h>

namespace bdk
{
        /**
         * @brief       Shred memory region
         */
        void bc_shred(bc_byte_t* mem, unsigned int size);

        /**
         * @brief       Static buffer object allocated on stack.
         */
        template < unsigned int size >
        class BCStaticBuffer
        {
        public:

                BCStaticBuffer()
                {
                        memset(m_buf, '0', size);
                };

                ~BCStaticBuffer()
                {
                        bc_shred(m_buf, size);
                };

                bc_byte_t operator[] (unsigned int idx)
                {
                        BC_ASSERT(idx < size);
                        return m_buf[idx];
                }

                bc_byte_t* ptr(unsigned int idx)
                {
                        BC_ASSERT(idx < size);
                        return &m_buf[idx];
                }

                operator bc_byte_t*()
                {
                        return ptr(0);
                }

                template < class T >
                operator T*()
                {
                        return reinterpret_cast<T*>(ptr(0));
                }

        protected:

                bc_byte_t m_buf[size];
        };

#if 0
        /**
         * @brief       Dynamic buffer.
         */
        class BCBuffer
        {
        public:

                BCBuffer() : m_buf(NULL), m_size(0) {}

                BCBuffer(bc_ulong_t size)
                {
                        this->reserve(size);
                }

                BCBuffer(BCBuffer const & rhv)
                {
                        this->reserve(rhv.size());
                        memcpy(m_buf, rhv.m_buf);
                }

                BCBuffer& operator = (BCBuffer const & rhv)
                {
                        this->reserve(rhv.size());
                        memcpy(m_buf, rhv.m_buf);
                }


                ~BCBuffer();

                void reserve(bc_ulong_t size);

                void free();

                bc_ulong_t size() const
                {
                        return m_size;
                }

                bc_byte_t operator[] (unsigned int idx)
                {
                        BC_ASSERT(idx < m_size);
                        return m_buf[idx];
                }

                bc_byte_t* ptr(unsigned int idx)
                {
                        BC_ASSERT(idx < m_size);
                        return &m_buf[idx];
                }

                operator bc_byte_t*()
                {
                        return ptr(0);
                }

                template < class T >
                operator T*()
                {
                        return reinterpret_cast<T*>(ptr(0));
                }

        private:

                bc_byte_t* m_buf;

                bc_ulong_t m_size;
        }
        };
#endif
}

#endif /* BDK_COMMON_MEMORY_H */
