/*
 *	Copyright (c) 2009 Jetico, Inc., Finland
 *	All rights reserved.
 */

#ifndef BDK_CONTAINER_CREATOR_H
#define BDK_CONTAINER_CREATOR_H

namespace bdk
{

        /**
         * @brief       Container creation.
         * @more        Overload protected event handlers to add process tracking.
         */
        class BCContainerCreator
        {
        public:

                BCContainerCreator();

                ~BCContainerCreator();

                /**
                 * @brief       Begin process.
                 *
                 * @p io        Container io object.
                 * @p view      Loaded container view.
                 * @p quick     Do not randomize view.
                 */
                void            begin(ContainerFile* file, BCContainerViewHandle view, bool quick);

                /**
                 * @brief       Abort process.
                 */
                void            abort();

                /**
                 * @brief       Get current progress.
                 */
                bc_ulong_t      get_progress() const
                {
                        return m_sectors_done;
                }

                /**
                 * @brief       Was process aborted?
                 */
                bool            is_aborted() const
                {
                        return m_aborted;
                }

        protected:

                /**
                 * @brief       Called when all initialization is done and the process is about to begin.
                 */
                virtual void    on_begin()
                {
                }

                /**
                 * @brief       Called when all process is about to end.
                 */
                virtual void    on_end()
                {
                }

                /**
                 * @brief       Called before each new container sector write.
                 */
                virtual void    on_sector_write()
                {
                }

        private:

                /* worker thread */
                BCThread m_creation_thread;

                /* aborted flag */
                volatile bool m_aborted;

                /* current number of sectors created */
                volatile bc_ulong_t m_sectors_done;
        };
}

#endif /* BDK_CONTAINER_CREATOR_H
