/**
 *
 *	Copyright (c) 2009 Jetico, Inc., Finland
 *	All rights reserved.
 *
 *	file: factory.h
 *	scope: bdk\container
 *	author: evgeny yakovlev
 *
 *	description: 
 *	BC module-extended container factory
 *
 */

#ifndef BDK_CONTAINER_FACTORY_H
#define BDK_CONTAINER_FACTORY_H

#include <container/type.h>


namespace bdk
{
	struct bc_container_info_t;
}

namespace bdk
{

	
	/**
	 * @brief	Interface for container factory module
	 * @more	Modules that define new container formats should subclass this interface and register it during on_load hook call.
	 *		There are two basic types of products required from factory implementation:
	 *		1) Copy of the container info parsed from existing container data source.
	 *		This one is not always accessible for containers that hide their meta info.
	 *
	 *		2) Ready for modification version.
	 */
	class IBCContainerFactory
	{
	BC_FORBID_COPY(IBCContainerFactory)
	BC_INTERFACE(IBCContainerFactory)
	public: 
		
		/**
		 * @brief	Parse container format and fill out meta info structure
		 *
		 * @p src	Initialized container media source.
		 *
		 * @return	true if container format was recognized and metadata was loaded, false otherwise
		 */
		virtual 	bool parse(IBCContainerIO* data) = 0;


		/**
		 * @brief	Return access to authenticating methods before loading a changeable implementation.
		 * @more	If you don't need a changeable version and only need to take a look, use parse_info.
		 */	
		virtual		IBCContainerFormat* load(IBCContainerIO* data) = 0;


		/**
		 * @brief 	Produce new container that this factory implements based on supplied info.
		 *
		 * @return 	Loaded container implementor or NULL if there was an error
		 */
		virtual 	IBCContainerFormat* make(bc_container_info_t const & info) = 0;

	};
}

#endif


