#ifndef BDK_CONTAINER_SOURCE_H
#define BDK_CONTAINER_SOURCE_H

#include <common/class_defs.h>
#include <common/types.h>

#include <alg/alg.h>

namespace bdk
{

	/**
	 * @brief	Virtual io container interface.
	 * @more	Used for container format serialization and parsing.
	 *		Extended to support container media.
	 */
	class IBCContainerIO
	{
	BC_INTERFACE(IBCContainerIO)
	public:


		/**
		 * @brief	Read bytes from io container.
		 *
		 * @p buf	Destination buffer to read to.
		 * @p nbytes	Size of the destination buffer.
		 * @p offset	Virtual buffer position to read from.
		 *
		 * @return	Number of bytes read.
		 */
		virtual 	bc_ulong_t read_bytes(bc_byte_t* buf, bc_ulong_t nbytes, bc_ulong_t offset) const = 0;


		/**
		 * @brief	Write bytes to io container.
		 *
		 * @p buf	Source buffer to write from.
		 * @p nbytes	Size of the source buffer.
		 * @p offset	Virtual buffer position to write to.
		 *
		 * @return	Number of bytes written.
		 */
		virtual 	bc_ulong_t write_bytes(const bc_byte_t* buf, bc_ulong_t nbytes, bc_ulong_t offset)  = 0;


		/**
		 * @brief	Current buffer size
		 */
		virtual 	bc_ulong_t size() const = 0;


		virtual 	void bind(bdk::backend::bc_crypto_dev_handle_t dev, bc_key_handle_t key_handle, bdk::BCAlgHandle alg, bcu32_t flags, 
					const bc_space_unit_t& layout, unsigned int start_sector ) = 0;
		

		
	};

}

	
#endif
