#ifndef BDK_MODULE_EXPORT_H
#define BDK_MODULE_EXPORT_H


#include <common/types.h>
#include <common/class_defs.h>
#include <common/log.h>


#define BC_EXPORT_MODULE(export_class_name)			\
extern "C" bdk::IBCModuleDelegate* bc_module_export()	\
{								\
        export_class_name * the_export = new export_class_name;		\
        return the_export;				\
}			


namespace bdk
{

	/**
	 * @brief	Core's representative inside the module library 
	 *
	 * @more	Delegates some work specific to module's implementation details
	 */
	class IBCModuleDelegate
	{
	BC_FORBID_COPY(IBCModuleDelegate)
	BC_INTERFACE(IBCModuleDelegate)

	public: //	Mandatory implementation
		

		/**
		 * @brief 	Module name
		 */
		virtual const char* name() const = 0;
		

		/**
		 * @brief 	Module id
		 */
		virtual bc_mod_id_t id() const = 0;


	public: //	Optional implementation

		
		/**
		 * @brief 	Module optional load hook
		 *
		 * @more	Will be called last in module loading sequence
		 *
		 * @return 	Module can report load failure that the core will consider and abort loading
		 */
		virtual bool on_load() 
		{
			return true;
		}


		/**
		 * @brief 	Module optional release hook
		 *
		 * @more	Will be called last in module release sequence
		 *
		 * @return 	Module can report release failure that the core will consider
		 */
		virtual bool on_release() 
		{
			return true;
		}


		/**
		 * @brief 	Optional module description
		 */
		virtual const char* description() const
		{
			return "";
		}

	};
}


#endif
