/**
 *
 *	Copyright (c) 2009 Jetico, Inc., Finland
 *	All rights reserved.
 *
 *	file: alg.cpp
 *	scope: bdk\alg
 *	author: evgeny yakovlev
 *
 *	description: 
 *	Algorithm module facade implementation.
 *
 *	history:
 *
 */

#include <alg/alg.h>
#include <alg/manager.h>

#include <common/log.h>
#include <common/exception.h>
#include <common/core.h>


using namespace bdk;


BCAlg::BCAlg( BCAlgService* service, int mode_id, bc_byte_t* key, bc_byte_t* pool ) : 
	m_mode_id(mode_id),
	m_service(service)
{
	bc_alg_mode_t mode;

	m_service->mode_info(mode, mode_id);
	m_key_handle = backend::bc_crypto_store_key ( this->id(), key, mode.key_size, pool );
}


BCAlgHandle BCAlg::init(int id, int mode, bc_byte_t* key, bc_byte_t* pool)
{
	BCAlgService* service = bc_core()->alg_manager()->find_algorithm(id);

	if( !service ) 
	{
		return BCAlgHandle(NULL);
	}

	return BCAlgHandle(service->allocate_alg(mode, key, pool), bc_alg_deleter());
}


BCAlgHandle BCAlg::init(std::string const & name, int mode, bc_byte_t* key, bc_byte_t* pool)
{
	BCAlgService* service = bc_core()->alg_manager()->find_algorithm(name);

	if( !service ) 
	{
		return BCAlgHandle(NULL);
	}

	return BCAlgHandle(service->allocate_alg(mode, key, pool), bc_alg_deleter());
}


BCAlg::~BCAlg() throw()
{
	// release driver resources
		
	try {
		backend::bc_crypto_release_key(this->id(), m_key_handle );
	} catch ( const BCException& e ) {
	}
}


void BCAlg::encrypt( const bc_byte_t* iv, bc_byte_t* buffer, size_t bufsiz ) const
{
	backend::bc_crypto_encrypt( this->id(), m_key_handle, m_mode_id, iv, buffer, bufsiz );
}


void BCAlg::decrypt( const bc_byte_t* iv, bc_byte_t* buffer, size_t bufsiz ) const
{
	backend::bc_crypto_decrypt( this->id(), m_key_handle, m_mode_id, iv, buffer, bufsiz );
}


bool BCAlg::is_valid() const 
{
	return m_valid;
}


void BCAlg::release()
{
	if( !m_valid ) return;

	// release driver resources
		
	backend::bc_crypto_release_key( this->id(), m_key_handle );
	
	// invalidate us
	m_valid = false;
}


void bc_alg_deleter::operator() (BCAlg* alg)
{
	BC_DEBUG_PRINT(("Debug: releasing shared alg handle\n"));
	// locate service
	BCAlgService* serv = bc_core()->alg_manager()->find_algorithm(alg->id());

	// if service is alive, ask it to unregister instance
	if( serv != NULL ) 
	{
		serv->release_alg(alg);
	}
	// if service is somehow dead, just delete an object by myself
	else
	{
		BC_DELETE(BCAlg, alg);
	}
	
}

