#include <string.h>
#include <sys/param.h>

#include <alg/device.h>
#include <common/exception.h>
#include <common/build.h>
#include <common/log.h>


using namespace bdk;


BCDevice::BCDevice( const char* device )
{
	m_dev_handle = backend::bc_crypto_dev_get( device );
}


BCDevice::BCDevice( backend::bc_crypto_dev_handle_t sysdev )
{
	m_dev_handle = sysdev;
}


BCDevice::~BCDevice() throw()
{
	try {
		backend::bc_crypto_dev_put( m_dev_handle );
	} catch (const BCException& e) {
	}
}


BCDeviceHandle BCDevice::get(std::string const & name)
{
	return bc_make_handle(new BCDevice(name.c_str()));
}



BCDeviceHandle BCDevice::get_free()
{
	backend::bc_crypto_dev_handle_t sysdev = backend::bc_crypto_dev_get_free();
	if(!sysdev)
		throw BCException("no free device");

	return bc_make_handle(new BCDevice(sysdev));
}


BCDeviceHandle BCDevice::invalid_device()
{
	return BCDeviceHandle(NULL);
}

bool BCDevice::backing_file_path(std::string& path)
{
	path.clear();

	char pathbuf[MAXPATHLEN] = {0};
	bool rc = bc_crypto_stat_dev(m_dev_handle, pathbuf, sizeof(pathbuf) - 1);
	if(!rc || (0 == strlen(pathbuf))) {
		return false;
	}

	path.assign(pathbuf);
	return true;
}
