/**
 *
 *	Copyright (c) 2009 Jetico, Inc., Finland
 *	All rights reserved.
 *
 *	file: alg_service.cpp
 *	scope: bdk\alg
 *	author: evgeny yakovlev
 *
 *	description:
 *	Algorithm service provider wrapper implementation.
 *
 *	history:
 *
 */

#include <string.h>
#include <algorithm>

#include <common/core.h>
#include <common/exception.h>
#include <common/log.h>

#include <alg/alg.h>
#include <alg/service.h>

using namespace bdk;


BCAlgService::BCAlgService( const bc_alg_info_t& info ) :
	m_name(info.name),
	m_provider_name(info.prvname),
	m_id(info.id),
	m_key_size(info.key_size),
	m_block_size(info.block_size),
	m_modes(info.mode_map)
{
	bool rc = backend::bc_crypto_verify(m_id);
	if(!rc)
		throw BCException("algorithm service provider " + m_provider_name + " failed verification");
}


BCAlgService::~BCAlgService() throw()
{
	owners_iter i = m_owners.begin();
	owners_iter end = m_owners.end();

#ifdef BC_DEBUG
	if( i != end )
	{
		BC_DEBUG_PRINT(("Debug: service %s has %d live handles at destruction point\n", this->name(), m_owners.size()));
	}
#endif

	while(i != end)
	{
		delete (*i);
		i++;
	}
}


BCAlg* BCAlgService::allocate_alg(int mode, bc_byte_t* key, bc_byte_t* pool)
{
	// check requested mode

	if( !this->is_mode_supported( mode ) ) {
		return false;
	}


	// construct handle
	BCAlg* alg = NOTHROW_NEW BCAlg(this, mode, key, pool);
	if( !alg ) {
		return NULL;
	}


	BC_DEBUG_PRINT(("Debug: Allocating algorithm handle\n"));
	m_owners.push_back(alg);
	return alg;
}


void BCAlgService::release_alg( BCAlg* alg )
{
	owners_iter i = std::find(m_owners.begin(), m_owners.end(), alg);
	BC_ASSERT( i != m_owners.end() );

	m_owners.erase(i);
	delete alg;
}


bool BCAlgService::verify()
{
	return backend::bc_crypto_verify(m_id);
}


