
#include <string.h>

#include <common/entype.h>
#include <common/exception.h>

#include <CertificateLib/hashAlgs/hashAlgs.h>


namespace bdk
{

	void calculate_digest( int hash_id, bc_byte_t* digest, const bc_byte_t* text, unsigned int text_size )
	{
		HHA ha;
		int res = ha_createAlgorithm(&ha, (BYTE)hash_id);
		if(res != hae_ok)
			throw BCException("Invalid hash algorithm id");

		ha_init( ha );
		ha_process( ha, (BYTE*)text, text_size );
		ha_final( ha, digest );
		ha_free( ha );
	}


	bool verify_digest( int hash_id, const bc_byte_t* digest, bc_byte_t* text, unsigned int text_size )
	{
		HHA ha;
		int res = ha_createAlgorithm(&ha, (BYTE)hash_id);
		if(res != hae_ok)
			throw BCException("Invalid hash algorithm id");

		int size = ha_getDigestSize(ha);

		bc_byte_t new_digest[DIGEST_BYTES];
		calculate_digest(hash_id, new_digest, text, text_size);
/*
		#ifdef BC_DEBUG
			BC_DEBUG_PRINT(("Debug: verifying digest:\nbody (size %d):\n", text_size));

			for(unsigned int i = 0; i < text_size; ++i)
			{
				BC_DEBUG_PRINT(("0x%x ", text[i]));
			}

			BC_DEBUG_PRINT(("\noriginal:\n"));

			for(unsigned int i = 0; i < DIGEST_BYTES; ++i)
			{
				BC_DEBUG_PRINT(("0x%x ", digest[i]));
			}

			BC_DEBUG_PRINT(("\ncalculated:\n"));

			for(unsigned int i = 0; i < DIGEST_BYTES; ++i)
			{
				BC_DEBUG_PRINT(("0x%x ", new_digest[i]));
			}

			BC_DEBUG_PRINT(("\n"));

		#endif
*/
		bool rc = (memcmp(digest, new_digest, size) == 0);
		return rc;
	}
}

