/*
 * bc_file.cpp
 *
 *  Created on: Sep 9, 2009
 *      Author: inso
 */

#include <common/file.h>
#include <common/fs.h>

using namespace bdk;

BCFile::BCFile(std::string const & fname, bc_word_t flags, bc_word_t mode/* = BC_PERM_DEFAULT */)
                throw(BCSystemException)
{
        m_fd = backend::bc_file_open(fname.c_str(), flags, mode); // can throw system exception from backend here
}

BCFileHandle BCFile::open(std::string const & fname, bc_word_t flags) throw(BCSystemException)
{
        return bc_make_handle(new BCFile(fname, flags));
}

BCFileHandle BCFile::create(std::string const & fname, bc_word_t flags, bc_word_t mode /* = BC_PERM_DEFAULT */)
                throw(BCSystemException)
{
        return bc_make_handle(new BCFile(fname, flags, mode));
}

BCFile::~BCFile() throw()
{
        try
        {
                backend::bc_file_close(m_fd);
        }
        catch (const BCException& e)
        {
                // silently catch
        }
}

bc_ulong_t BCFile::read(void *dst, size_t size, size_t len) throw(BCSystemException)
{
        return backend::bc_file_read(m_fd, (unsigned char*) dst, size * len);
}

bc_ulong_t BCFile::write(const void* src, size_t size, size_t len) throw(BCSystemException)
{
        return backend::bc_file_write(m_fd, (const unsigned char*) src, size * len);
}

bc_ulong_t BCFile::pread(void* dst, size_t size, size_t len, bc_ulong_t offset)
                throw(BCSystemException)
{
        return backend::bc_file_pread(m_fd, (unsigned char*) dst, size * len, offset);
}

bc_ulong_t BCFile::pwrite(const void* src, size_t size, size_t len, bc_ulong_t offset)
                throw(BCSystemException)
{
        return backend::bc_file_pwrite(m_fd, (const unsigned char*) src, size * len, offset);
}

bc_ulong_t BCFile::tell() const throw(BCSystemException)
{
        return backend::bc_file_tell(m_fd);
}

bc_ulong_t BCFile::seek(bc_ulong_t offset, bc_file_seek_t where) throw(BCSystemException)
{
        return backend::bc_file_seek(m_fd, where, offset);
}

bool BCFile::is_eof() const
{
        return backend::bc_file_is_eof(m_fd);
}

std::string BCFile::system_path() const
{
        return backend::bc_file_name(m_fd);
}
