/**
 *
 *	Copyright (c) 2009 Jetico, Inc., Finland
 *	All rights reserved.
 *
 *	file: factory.cpp
 *	scope: core\common
 *	author: evgeny yakovlev
 *
 *	description: 
 *	BC module-extended container factory
 *
 *	history:
 *	- experimental state.
 *
 */

#include <algorithm>

#include <container/manager.h>
#include <container/format.h>
#include <container/factory.h>

using namespace bdk;


void BCContainerManager::register_type(std::string const & name, IBCContainerFactory* factory, bool support_unrecognized)
{
	products_iter_t i = _product_iter(name);
	
	if(i != m_products.end()) {
		BC_DEBUG_PRINT(("Debug: factory for product type %s registered\n", name.c_str()));
		return;
	}
	else
	{
		bc_type_entry_t type;
		type.type_factory = factory;
		type.type_name = name;
		type.type_unrecognized = support_unrecognized;

		m_products.push_back(type);

		BC_DEBUG_PRINT(("Debug: registered product %s\n", name.c_str()));
	}
}


void BCContainerManager::unregister_type(std::string const & name)
{
	products_iter_t i = _product_iter(name);

	if( (i == m_products.end()) )
		return;

	m_products.erase(i);
}


bool BCContainerManager::query_type_by_name(std::string const & name, bc_type_entry_t& type)
{
	products_iter_t i = _product_iter(name);
	if(i == m_products.end())
		return false;

	type = (*i);
	return true;
}


bool BCContainerManager::query_type_by_data(IBCContainerIO* data, bc_type_entry_t& type)
{
	products_iter_t i = m_products.begin();
	products_iter_t end = m_products.end();
	
	for(;i != end; ++i)
	{
		
		BC_DEBUG_PRINT(("Trying next type %s\n", i->type_name.c_str()));
		// try to recognize source
		bool rc = i->type_factory->parse(data);
		if(rc)
		{
			BC_DEBUG_PRINT(("Debug: container was parsed as a %s type\n", i->type_name.c_str()));
			type = (*i);
			return true;
		}
	}

	return false;
}


