#include <container/type.h>
#include <common/core.h>

using namespace bdk;

BCContainerHandle BCContainer::init(IBCContainerFormat* fmt, std::string const & type, bool recognized)
{
	return bc_make_handle(new BCContainer(fmt, type, recognized));
}


BCContainerHandle BCContainer::load_by_type(IBCContainerIO* data, std::string const & type)
{
	bc_type_entry_t type_info;
	bool recognized = true;

	// get implicit type first
	if(!bc_core()->container_manager()->query_type_by_data(data, type_info) || (type_info.type_name != type))
	{
		recognized = false;
	}

	// if implicit type differs from explicit we will go on only if explicit type supports unrecognized formats.
	if(!recognized)
	{
		if(!bc_core()->container_manager()->query_type_by_name(type, type_info) || !type_info.type_unrecognized)
		{
			return BCContainerHandle();
		}
	}


	IBCContainerFormat* fmt = type_info.type_factory->load(data);
	if( !fmt )
		return BCContainerHandle(NULL);

	return BCContainer::init(fmt, type_info.type_name, recognized);
}


BCContainerHandle BCContainer::load_by_data(IBCContainerIO* data)
{
	bc_type_entry_t type_info;

	// parse type from data
	if( !bc_core()->container_manager()->query_type_by_data(data, type_info) )
		return BCContainerHandle(NULL);

	IBCContainerFormat* fmt = type_info.type_factory->load(data);
	if( !fmt )
		return BCContainerHandle();

	return BCContainer::init(fmt, type_info.type_name, true);
}


BCContainerHandle BCContainer::make( std::string const & type, int algid, int algmode, bc_size_t size, std::string const & desc )
{
	bc_type_entry_t type_info;
	if( !bc_core()->container_manager()->query_type_by_name(type, type_info) )
		return BCContainerHandle();

	bc_container_info_t info;
	info.cont_alg_id = algid;
	info.cont_mode_id = algmode;
	info.cont_size = size;
	info.cont_description = desc;

	IBCContainerFormat* fmt = type_info.type_factory->make(info);
	if( !fmt )
		return BCContainerHandle();

	return BCContainer::init(fmt, type, true);
}

