/**
 *
 *	Copyright (c) 2009 Jetico, Inc., Finland
 *	All rights reserved.
 *
 *	file: bind.h
 *	scope: bdk\alg
 *	author: evgeny yakovlev
 *
 *	description: 
 *	BC crypto loop binding
 *
 *	history:
 *	- experimental state.
 *
 */

#include <string.h>
#include <limits.h>

#include <alg/bind.h>
#include <common/build.h>

using namespace bdk;




// check supplied sector buffer for partition table, fill info and return last non-extended partiotion sector offset
static bool parse_partition_table(const bc_byte_t* buf, part_info_t* out_info)
{

	struct partition 
	{
		unsigned char active;
    		unsigned char start_head;
    		unsigned char start_sec;
    		unsigned char start_cyl;
    		unsigned char os_type;
    		unsigned char end_head;
    		unsigned char end_sec;
    		unsigned char end_cyl;
    		unsigned int  lba_start_sec;
    		unsigned int  lba_nr_sec;
	} __attribute__((packed));

	memset(out_info, '\0', sizeof(part_info_t));

    	// check msdos partition signature
    	if (0x55 != buf[0x1FE] || 0xAA != buf[0x1FF])
		return 0;
	
	// read partiotion table
    	struct partition* prt 	= (struct partition *)(buf+0x1BE);
	bool found = false;

	for (unsigned int i = 0; i < 4; ++i, ++prt) 
	{
		//x[i] = 0;
		if (0 == prt->lba_nr_sec)
		    	continue; // no partition in this slot

		// found one
		switch (prt->os_type) 
		{
		case 0x05:	// DOS extended
		case 0x0F:	// Windows 95 extended
		case 0x85:	// Linux extended
		{
			// found extended partition
			found = true;
	    		out_info->ext_part_count++;
	    		break;
		}
		default:
		{
			found = true;
			out_info->part_count++;
			out_info->table[i].offset = prt->lba_start_sec;
			out_info->table[i].length = prt->lba_nr_sec;
		}
		};
    	}

    	return found;
}


unsigned int BCLoopBinder::partition_info(bc_sector_t const & first, part_info_t& info) const
{
	unsigned int start_sector = INT_MAX;

	memset(&info, 0, sizeof(part_info_t));
	if(parse_partition_table(first.sec_data, &info))
	{
#ifdef BC_DEBUG
		BC_DEBUG_PRINT(("Loop Binder: partition table info:\nPrimary count: %d\nExtended count: %d\n", info.part_count, info.ext_part_count));
		for(unsigned int i = 0; i < 4; ++i)
		{
			BC_DEBUG_PRINT(("Entry %d: start sector %d, total sectors %d\n", i, info.table[i].offset, info.table[i].length));
		}
#endif
		bc_get_log(BC_WARN).msg("Warning: %d (%d extended) partitions detected inside container\n", info.part_count, info.ext_part_count);
		
		// find earliest primary partiotion
		for(unsigned int i = 0; i < info.part_count; ++i)
		{
			if( start_sector > info.table[i].offset )
				start_sector = info.table[i].offset;
		}

	}
	else 
	{
		start_sector = 0;
	}

	return start_sector;
}

