#include <mod/module.h>

#include <common/dso.h>
#include <common/log.h>
#include <common/exception.h>
#include <common/build.h>
#include <common/class_defs.h>


using namespace bdk;


BCModule::BCModule( const char* libname ) :
	m_refcount(0),
	m_status(BC_MOD_LOAD)
{
	try
	{
		BCDSO* dptr = new BCDSO(libname, BC_DSO_RES_NOW, BC_DSO_VIS_LOCAL );

		m_dynamic.reset(dptr);

		// load export
		typedef IBCModuleDelegate*(*export_fptr_t)(void);
		export_fptr_t fptr = (export_fptr_t)m_dynamic->load_symbol_type ("bc_module_export");

		if( !fptr )
			throw ("module export symbol not found");

		m_export.reset((*fptr)());

		// call optional load hook
		m_status = BC_MOD_INST;
		if(!m_export->on_load())
			throw ("module load hook reported failure");

		m_status = BC_MOD_READY;
	}
	catch ( const BCException& e )
	{
		throw;	// rethrow above
	}
}


BCModule::~BCModule() throw()
{
	m_export->on_release();
}


void BCModule::get_reference()
{
	++m_refcount;
}


void BCModule::put_reference()
{
	--m_refcount;
}


const char* BCModule::name() const
{
	return m_export->name();
}


bc_mod_id_t BCModule::id() const
{
	return m_export->id();
}


const char* BCModule::description() const
{
	return m_export->description();
}


bcu32_t BCModule::refcount() const
{
	return m_refcount;
}


const char* BCModule::dso_path() const
{
	return m_dynamic->path();
}


