#include <string.h>

#include <mod/manager.h>
#include <mod/module.h>
#include <common/exception.h>
#include <common/build.h>


using namespace bdk;


BCModuleManager::BCModuleManager()
{
}

BCModuleManager::~BCModuleManager()
{
	modules_t::iterator i = m_entries.begin();
	modules_t::iterator end = m_entries.end();

	while(i != end)
	{
		delete (*i);
		++i;
	}
}


BCModule* BCModuleManager::load_module( const char* modpath )
{
	try
	{
		// create new module

		BCModule* mod = NOTHROW_NEW BCModule(modpath);
		if( !mod ) return NULL;

		BC_DEBUG_PRINT(("module manager: loaded module: %s, %d\n", mod->name(), mod->id()));

		// check for existing entry

		BC_ASSERT( !this->find_module(mod->id()) );

		// add and return

		m_entries.push_back(mod);
		return mod;
	}
	catch (const BCException& e)
	{
		return NULL;
	}
}


void BCModuleManager::release_module( BCModule* mod )
{
	BC_ASSERT( this->find_module(mod->id()) );

	// remove entry
	m_entries.remove(mod);
}


bool BCModuleManager::module_loaded( const BCModule* mod )
{
	modules_t::iterator i = m_entries.begin();
	modules_t::iterator end = m_entries.end();

	while( i != end )
	{
		if( (*i)->id() == mod->id() )
			return true;
		++i;
	}

	return false;
}


BCModule* BCModuleManager::find_module( bcu32_t id )
{
	modules_t::iterator i = m_entries.begin();
	modules_t::iterator end = m_entries.end();

	while( i != end )
	{
		if( (*i)->id() == id )
		{
			return (*i);
		}

		++i;
	}

	return NULL;
}


BCModule* BCModuleManager::find_module( std::string const & name )
{
	modules_t::iterator i = m_entries.begin();
	modules_t::iterator end = m_entries.end();

	while( i != end )
	{
		if( (*i)->name() == name )
		{
			return (*i);
		}

		++i;
	}

	return NULL;
}



