/**
 *
 *	Copyright (c) 2009 Jetico, Inc., Finland
 *	All rights reserved.
 *
 *	file: io.cpp
 *	scope: linux backend
 *	author: evgeny yakovlev
 *
 *	description:
 *	file io linux backend implementation
 *
 *	history:
 *
 */

#include <string.h>
#include <errno.h>

#include "os_sync.h"
#include <backend/sync.h>
#include <common/exception.h>


using namespace bdk;
using namespace bdk::backend;


class SyncException : public BCSystemException
{
public:
	SyncException(std::string const & action, std::string const & syserr)
	{
		m_msg = action + ":" + syserr;
	}
};

bc_mutex_handle_t get_mutex()
{
	pthread_mutexattr_t attrib;
	pthread_mutex_t	mutex;

	int rc = pthread_mutexattr_init(&attrib);
	if(rc != 0)
		throw SyncException("mutexattr init failed", strerror(rc));

	rc = pthread_mutex_init(&mutex, &attrib);
	if(rc != 0)
		throw SyncException("mutex init failed", strerror(rc));

	bc_mutex_handle_t handle = new bc_os_mutex_t;
	handle->sysmutex = mutex;

	return handle;
}


void put_mutex(bc_mutex_handle_t& handle)
{
	int rc = pthread_mutex_destroy(&handle->sysmutex);
	delete handle;
	handle = NULL;

	if( !rc )
		throw SyncException("mutex destroy failed", strerror(rc));
}

bc_thread_handle_t get_thread()
{
        bc_thread_handle_t handle = new bc_os_thread_t;
        return handle;
}

void put_thread(bc_thread_handle_t& handle)
{
        delete handle;
        handle = NULL;
}


namespace bdk
{
namespace backend
{

	bc_mutex_handle_t bc_mutex_get()
	{
		return get_mutex();
	}

	void bc_mutex_put(bc_mutex_handle_t& handle)
	{
		put_mutex(handle);
	}

	void bc_mutex_lock(bc_mutex_handle_t handle)
	{
		int rc = pthread_mutex_lock(&handle->sysmutex);
		if(rc != 0 )
			throw SyncException("mutex lock failed", strerror(rc));
	}

	void bc_mutex_unlock(bc_mutex_handle_t handle)
	{
		int rc = pthread_mutex_unlock(&handle->sysmutex);
		if(rc != 0 )
			throw SyncException("mutex unlock failed", strerror(rc));
	}

	bc_thread_handle_t bc_thread_get()
	{
                return get_thread();
	}

	void bc_thread_put(bc_thread_handle_t handle)
	{
	        put_thread(handle);
	}

        void bc_thread_start(bc_thread_handle_t handle, bc_thread_func_t entry, bc_thread_par_t par)
        {
                int rc = pthread_create(&handle->systhread, NULL, entry, par);
                if(rc != 0)
                        throw SyncException("pthread create failed", strerror(rc));
        }
}
}


