/*******************************************************************
 *	Copyright (c) 1994-1998 Jetico, Inc., Finland
 *	All rights reserved.
 *
 *	File:		driver/bc_mgr.h
 *
 *	Description:	algorithm manager data structures
 *
 *	Scope:		BestCrypt pseudo-device driver
 *
 *	Platforms:	Linux
 * 
 *	Author:		Nail R. Kaipov
 *
 *	Created:	10-Nov-1998
 *
 *	Revision:	
 *
 *******************************************************************/
 
#ifndef __BC_MGR_H__
#define __BC_MGR_H__

#include "bc_alg.h"

#define BC_ALGO_NAME_LEN	32
#define MODULE_NAMELEN		64



/**
 * @brief	Algorithm runtime plugin entry
 */
struct bc_alg_mod 
{
	struct bc_algorithm	alg;
	struct bc_alg_mod	*next;
	char 			mod_name[MODULE_NAMELEN+1];
	u_int			mod_flags;
	u_int			mod_refcount;

};


int bc_mgr_init(void);


/**
 * @brief	Register new algorithm.
 *
 * @p plugin	Algorithm description structure.
 */
int bc_mgr_register_alg (struct bc_valg* valg);


/**
 * @brief	Unregister loaded algorithm.
 *
 * @p plugin	Algorithm id.
 */
int bc_mgr_unregister_alg (bc_alg_id id);


/**
 * @brief	Get registered algorithm description.
 *
 * @p plugin	Algorithm id.
 */
struct bc_algorithm* bc_mgr_get_alg (bc_alg_id id);


int bc_mgr_proc_conf(char *page, char **start, off_t off, int count, int *eof, void *data);



#endif /* __BC_MGR_H__ */

