/*******************************************************************
 *	Copyright (c) 2009 Jetico, Inc., Finland
 *	All rights reserved.
 *
 *	File:		mod/bc_mod.c
 *
 *	Description:	common entry point and registration factory for bc algorithm plugins
 *
 *	Scope:		BestCrypt algorithms drivers
 *
 *	Platforms:	Linux
 * 
 *	Author:		Evgeny Yakovlev, Nail R. Kaipov
 *
 *	Created:	25-Jun-2009
 *
 *******************************************************************/

#include <linux/module.h>
#include <linux/init.h>
#include <linux/version.h>

#include "algorithm.h"


	MODULE_LICENSE("Jetico, Inc.");
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
	#include <linux/vermagic.h>
	MODULE_INFO(vermagic, VERMAGIC_STRING);

	#ifdef _NEED_THIS_MODULE
	struct module __this_module
	__attribute__((section(".gnu.linkonce.this_module"))) = {
		#ifdef _KBUILD_QUOTES
		 .name = KBUILD_MODNAME,
	 	#else
		 .name = __stringify(KBUILD_MODNAME),
		#endif
		 .init = init_module,
		#ifdef CONFIG_MODULE_UNLOAD
		 .exit = cleanup_module,
		#endif
	};

	static const char __module_depends[]
		#ifdef __attribute_used__
		__attribute_used__
		#else
		__used
		#endif
		__attribute__((section(".modinfo"))) =
		"depends=bc";
	#endif

#else 

EXPORT_NO_SYMBOLS;

#endif


#include <bc_alg.h>
#include <bc_mgr.h>

static struct bc_valg algo;

int __init init_module(void)
{
	algo.id      		= ALGORITHM_ID;	
	algo.name     		= DISPLAY_NAME;
	algo.key_size_bits   	= KEY_LENGTH;
	algo.ext_key_size_bytes = KEY_LENGTH_EXTENDED;
	algo.block_size_bytes	= BLOCK_LENGTH;
	algo.owner	 	= THIS_MODULE;

	algo.encrypt_cbc	= Encrypt;
	algo.decrypt_cbc	= Decrypt;
	algo.encrypt_ecb	= Encrypt_ECB;
	algo.decrypt_ecb	= Decrypt_ECB;
	algo.key_extend		= KeyExtend;


	return bc_mgr_register_alg(&algo);	
}


void __exit cleanup_module(void) 
{
	bc_mgr_unregister_alg(ALGORITHM_ID);
}


