#!/bin/bash

if [ $# -ne 3 ] ; then
	echo "Invalid number of arguments. release.sh major minor patch.";
	exit 1;
fi

VER_MAJOR=$1
VER_MINOR=$2
VER_PATCH=$3
VER=$VER_MAJOR.$VER_MINOR-$VER_PATCH

PROJECT_ROOT=../
BUILD_DIR=$PROJECT_ROOT/build
RPM_DIR=~/rpmbuild
TAG_DIR=$PROJECT_ROOT/tags/BCRYPT-$VER_MAJOR\_$VER_MINOR-$VER_PATCH

echo "Building release for version $VER"

# prepare temp directory to build dist
[ -d $BUILD_DIR ] && rm -rf $BUILD_DIR
mkdir $BUILD_DIR
svn export . $BUILD_DIR/bcrypt
cd $BUILD_DIR

# create dist archives
echo "Creating source archives"
tar -zcf bcrypt-$VER.tar.gz bcrypt
mv bcrypt bcrypt-$VER  
tar -zcf BestCrypt-$VER.tar.gz bcrypt-$VER  

# create dist srpm
echo "Creating rmps"
sudo mv bcrypt-$VER.tar.gz $RPM_DIR/SOURCES
sudo cp bcrypt-$VER/bcrypt.spec $RPM_DIR/SPECS
sudo rpmbuild -ba $RPM_DIR/SPECS/bcrypt.spec
cp $RPM_DIR/SRPMS/BestCrypt-$VER.src.rpm ./

# create release tag in repo
echo "Tagging release"
cd $PROJECT_ROOT/trunk
if [ -d $TAG_DIR ] ; then
	echo "Tag dir already exists, skipping";
else
	svn export ./ $TAG_DIR;
	svn add $TAG_DIR;
	#svn ci -m "release $VER_MAJOR.$VER_MINOR-$VER_PATCH";
fi
