/*
 *	Copyright (c) 2009 Jetico, Inc., Finland
 *	All rights reserved.
 *
 *	Container interface.
 */

#ifndef BC_CONTAINER_H
#define BC_CONTAINER_H

#include "v7/hidden_sector.h"
#include "v8/header.h"

namespace bestcrypt {

	struct bc_volume;

	typedef enum 
	{
		kBCFormat_Unknown = 0,
		kBCFormat_V7 = 7,
		kBCFormat_V8 = 8,
	} bc_format_version;
 

	struct bc_container
	{
		bc_format_version	version;	// parsed version, unknown if ghost
		bc_keygen_module* 	kg;		// loaded keygen

		// untyped format specific data
		void* 			format_data;
		size_t			datasize;

		// function table. some of those can be null if container lacks some capability.
		bool (*store) (const char* file);
		bool (*decode_volume) (bc_volume* out_vol);
		bool (*keygen_command) (int argc, char** argv);
	};


	/// Load container from file.
	bool bc_load_container(const char* fname, bc_container* out_cont);

}

#endif

