#include "context.h"


bool parse_command_line(char** argv, int argc, bc_cmd_context_t& ctx)
{
	
    	while ( (c=getopt(argc,argv,"AVvfhprt:a:o:s:d:m:u:g:"))!=-1 )
	{
        	switch ( (char)c )
	        {
        	case 'V':
                	bc_get_log(BC_INFO).msg("BestCrypt version %d.%d-%d\nCopyright Jetico, Inc. See LICENSE for details.\n", VERSION_MAJOR, VERSION_MINOR, VERSION_PATCH);
                	return false;
            	case 'v':
                	ctx.verbose = true;
	                break;
        	case 'A':
                	ctx.umount_all = true;
                	break;
            	case 'p':
                	ctx.random_password = true;
                	break;          
            	case 'r':
                	ctx.ro = true;
	                break;          
            	case 'f':
                	ctx.force = true;
	                break;          
            	case 'h':
                	usage(stdout);
	                return false;
            	case 't':
                	ctx.fstype.assign(optarg);
	                break;
            	case 'a':
                    	ctx.alg.assign(optarg);
                	break;
            	case 'o':
                	ctx.options.assign(optarg);
                	break;
            	case 'd':
                	ctx.description.assign(optarg);
 	               	break;
		case 'c':
			ctx.type.assign(optarg);
			break;
            	case 's':
			ctx.size = bc_atol(optarg);
                	if ( ctx.size == 0 )
                	{
                    		bc_get_log(BC_ERR).msg("Invalid container size\n");
                    		return false;
                	}
                	break;
            	case 'm':
                	ctx.mode = strtol(optarg, NULL, 8) & 07777;
                	if (0 == ctx.mode)
                	{
                    		bc_get_log(BC_ERR).msg("Invalid mode. Using 0700\n");
                    		ctx.mode = 0700;
                	}
                	break;
            	case 'u':
                	if (0 != ruid) 
                	{
                    		bc_get_log(BC_ERR).msg("Only root can specify mountpoint's owner\n");
                    		return false;
                	}
                	ctx.uid = get_uid_by_name(optarg);
                	if (-1 == ctx.uid)
                	{
                    		bc_get_log(BC_ERR).msg("Invalid username '%s'\n",optarg);
                    		return false;
                	}
                	break;
            	case 'g':
                	ctx.gid = get_gid_by_name(optarg);
                	if (-1 == ctx.gid)
                	{
                    		bc_get_log(BC_ERR).msg("Invalid groupname '%s'\n",optarg);
                    		return false;
                	}
                	break;
            	case '?': /* fall thru */
            	default:
                	usage(stderr);
                	return false;
        	} /* end of switch */

    	} /* end of while */

	

}


