/**
 *
 *	Copyright (c) 2009 Jetico, Inc., Finland
 *	All rights reserved.
 *
 *	file: format.cpp
 *	scope: linux client
 *	author: evgeny yakovlev
 *
 *	description:
 *	Implementation of linux client 'format' command.
 *
 *	history:
 *
 */

#include <string.h>

#include "format.h"
#include "misc.h"
#include "iofile.h"
#include "fs.h"

#include <alg/device.h>
#include <common/core.h>
#include <common/log.h>


using namespace bdk;


bool bcformat( int argc, char** argv, const char* fstype )
{
	if( argc < 2 ) {
		bc_get_log(BC_MSG_CRIT) << "Please specify container name";
		return false;
	} else if ( argc > 2 ) {
		bc_get_log(BC_MSG_CRIT) << "Too many parameters";
		return false;
	}


	const char* fname = argv[1];

	BC_DEBUG_PRINT(("format command: container %s, fstype %s\n", fname, fstype));

	if( !fstype ) {
		bc_get_log(BC_MSG_WARN) << "File system not specified, falling back to msdos";
		fstype = "msdos";
	}

	FSHandle fs = NativeFS::get_by_name(fstype);
	if(!fs)
	{
		bc_get_log(BC_MSG_CRIT) << "File system " << fstype << " not recognized";
		return false;
	}


	// check file existance and write access
	BCFSObject cont_path(fname);

	if( !cont_path.is_valid() ) {
		bc_get_log(BC_MSG_CRIT) << "File " << cont_path << " does not exist";
		return false;
	}

	if( !cont_path.access(BC_ACCESS_READ|BC_ACCESS_WRITE) ) {
		bc_get_log(BC_MSG_CRIT) << "You do not have write access for file " << cont_path;
		return false;
	}

	//
	//	Load container
	//


	// load container file
	ContainerFile cont_file(fname, BC_FILE_LARGE|BC_FILE_READ|BC_FILE_WRITE);
	BCContainerHandle cont = load_container_file(cont_file);

	if( !cont )
	{
		bc_get_log(BC_MSG_CRIT) << "Could not parse container from file " << cont_path;
		return false;
	}


	BCContainerViewHandle view = cont->format()->get_view(&cont_file);
	if( !view )
	{
		bc_get_log(BC_MSG_CRIT) << "failed authentication";
		return false;
	}


	// get free device
	bdk::BCDeviceHandle dev = bdk::BCDevice::get_free();
	BCContainerAutoBinding binding = view->create_binding(&cont_file, dev, false);

	// format
	fs->format(dev);

	BC_DEBUG_PRINT(("Debug: formatting done\n"));

	// we're done
	return true;
}

