/**
 *
 *	Copyright (c) 2009 Jetico, Inc., Finland
 *	All rights reserved.
 *
 *	file: fsck.h
 *	scope: linux client
 *	author: evgeny yakovlev
 *
 *	description:
 *	linux client 'fsck' command - file system checking.
 */

#include <string.h>

#include "fsck.h"
#include "iofile.h"
#include "misc.h"
#include "fs.h"

#include <common/log.h>


using namespace bdk;


bool bcfsck(int argc, char** argv, const char* fstype)
{
	if( argc < 2 ) {
		bc_get_log(BC_MSG_CRIT) << "Please specify container name";
		return false;
	} else if (argc > 2) {
		bc_get_log(BC_MSG_CRIT) << "Too many parameters";
		return false;
	}

	BCFSObject cpath(argv[1]);

	// check file existance and write access
	if( !cpath.is_valid() )
	{
		bc_get_log(BC_MSG_CRIT) << "File " << cpath << " does not exist";
		return false;
	}

	if( !cpath.access(BC_ACCESS_WRITE) )
	{
		bc_get_log(BC_MSG_CRIT) << "You do not have write permissions for file " << cpath;
		return false;
	}

	// load container file
	ContainerFile cont_file(cpath, BC_FILE_LARGE|BC_FILE_READ|BC_FILE_WRITE);
	BCContainerHandle cont = load_container_file(cont_file);

	if( !cont )
	{
		bc_get_log(BC_MSG_CRIT) << "Could not parse container from file " << cpath;
		return false;
	}


	BCContainerViewHandle view = cont->format()->get_view(&cont_file);
	if(!view)
	{
		bc_get_log(BC_MSG_CRIT) << "failed authentication";
		return false;
	}


	// check fstype
	FSHandle fs;

	// if user did not specify fstype try to use implicit
	if ( fstype == NULL )
		fs = NativeFS::get_by_id(FS_TAG_UNDEFINED);
	// if user specified fsname, check that it is consistent with implicit but go with specified
	else
	{
		fs = NativeFS::get_by_name(fstype);
		if(!fs)
		{
		        bc_get_log(BC_MSG_CRIT) << "Error: unknown file system type " << fstype;
		        return false;
		}
	}


	BCDeviceHandle dev = BCDevice::get_free();
	BCContainerAutoBinding binding = view->create_binding(&cont_file, dev, false);

	fs->check(dev);
	return true;
}
