/**
*
*	Copyright (c) 2009 Jetico, Inc., Finland
*	All rights reserved.
*
*	file: info.cpp
*	scope: linux client
*	author: evgeny yakovlev
*
*	description:
*	linux client 'info' command.
*/

#include <string.h>

#include "info.h"
#include "misc.h"
#include "iofile.h"
#include "fs.h"

#include <common/log.h>
#include <common/build.h>
#include <common/exception.h>
#include <common/core.h>

#include <mod/module.h>


using namespace bdk;


bool bcinfo( int argc, char** argv )
{
	if( argc < 2 ) {
		bc_get_log(BC_MSG_CRIT) << "Please specify container name";
		return false;
	} else if ( argc > 2 ) {
		bc_get_log(BC_MSG_CRIT) << "Too many parameters";
		return false;
	}

	BCFSObject fpath(argv[1]);

	if( !fpath.is_valid() )
	{
		bc_get_log(BC_MSG_CRIT) << "Error: file " << fpath << " does not exist";
		return false;
	}

	if(!fpath.access(BC_ACCESS_READ))
	{
		bc_get_log(BC_MSG_CRIT) << "Error: you do not have read access for file " << fpath;
		return false;
	}


	// open container file
	ContainerFile cont_file(fpath, BC_FILE_READ|BC_FILE_LARGE);
	BCContainerHandle cont = BCContainer::load_by_data(&cont_file);
	if( !cont || !cont->is_recognized() )
	{
		bc_get_log(BC_MSG_CRIT) << "Error: could not load container from file " << fpath;
		return false;
	}


	// print info
	BCAlgService* serv = bc_core()->get_alg_service(cont->format()->algorithm_id());
	if( !serv )
	{
		bc_get_log(BC_MSG_CRIT) << "Error: unknown container encryption algorithm";
		return false;
	}


	fprintf(stdout, "Container %s information:\n"\
		" Algorithm:       %s\n"\
		" Encryption mode: %s\n"\
		" Description:     %s\n"\
		" Format:          %s\n"\
		" Status:          %s\n",
		fpath.path().c_str(),
		serv->name(),
		serv->mode_name(cont->format()->algorithm_mode_id()),
		cont->format()->description().c_str(),
		cont->format()->name().c_str(),
		(cont->format()->is_locked()) ? "locked" : "unlocked"
	);

	return true;
}

