/**
 *
 *	Copyright (c) 2009 Jetico, Inc., Finland
 *	All rights reserved.
 *
 *	file: file.h
 *	scope: BestCrypt linux app
 *	author: evgeny yakovlev
 *
 *	description:
 *	Container file binding and loading extentions
 *
 *	history:
 *
 */

#ifndef BC_FILE_H
#define BC_FILE_H

#include <common/file.h>
#include <container/io.h>
#include <container/type.h>



/**
 * @brief	File as a container data and binding source
 */
class ContainerFile : public bdk::IBCContainerIO
{
//---------------------------------------------------------------
//	Inherited as a container io source
public:


	BC_INHERITED	bc_ulong_t size() const
	{
		bc_ulong_t cur = m_file->tell();
		bc_ulong_t size = m_file->seek(0, BC_SEEK_END);
		m_file->seek(cur, BC_SEEK_SET);
		return size;
	}


	BC_INHERITED	bc_ulong_t read_bytes(bc_byte_t* buffer, bc_ulong_t nbytes, bc_ulong_t offset) const
	{
		return m_file->pread(buffer, nbytes, sizeof(bc_byte_t), offset);
	}


	BC_INHERITED	bc_ulong_t write_bytes(const bc_byte_t* buffer, bc_ulong_t nbytes, bc_ulong_t offset)
	{
		return m_file->pwrite(buffer, nbytes, sizeof(bc_byte_t), offset);
	}


	BC_INHERITED 	void bind(bdk::backend::bc_crypto_dev_handle_t dev, bc_key_handle_t key_handle, bdk::BCAlgHandle alg, bcu32_t flags,
					const bc_space_unit_t& layout, unsigned int start_sector )
	{
		bdk::backend::bc_crypto_set_file(dev, m_file->system_handle(), key_handle, alg->id(), alg->mode(), flags, &layout, start_sector);
	}


	BC_INHERITED	~ContainerFile()
	{
	}


//---------------------------------------------------------------
//	BestCrypt container file ops
public:


	ContainerFile()
	{
	}


	explicit ContainerFile(bdk::BCFileHandle file) :
		m_file(file)
	{
	}


	ContainerFile( std::string const & fname, bc_word_t flags, bc_word_t mode = BC_PERM_DEFAULT ) :
		m_file(bdk::BCFile::create(fname, flags, mode))
	{
	}


	ContainerFile(bdk::BCFSObject const & fpath, bc_word_t flags, bc_word_t mode = BC_PERM_DEFAULT ) :
		m_file(bdk::BCFile::create(fpath.path(), flags, mode))
	{
	}


	bdk::BCFileHandle	get_file()
	{
		return m_file;
	}


	void set_file(bdk::BCFileHandle file)
	{
		m_file = file;
	}


	/**
	 * @brief	Link container file to device
	 */
	//bool link(bdk::bc_data_context_t& ctx, bdk::BCDeviceHandle& dev);


	bool is_file() const
	{
	        return m_file->fs_object().is_file();
	}


	bool is_device() const
	{
	        return m_file->fs_object().is_special();
	}

/*
	bc_ulong_t get_device_sector_size() const
	{

	}


	bc_ulong_t get_total_size() const
	{
	}
*/

private:


	bdk::BCFileHandle m_file;
};


/**
 * @brief	File object as a loop\binding source
 */
struct bc_file_loop_source_t
{
	bdk::BCFileHandle m_file;

	explicit bc_file_loop_source_t(bdk::BCFileHandle file) : m_file(file) {};


	/**
	 * @brief	Adapt read_sector to file call
	 */
	bc_ulong_t read_sector(bc_sector_t & sec, bc_ulong_t secnum)
	{
		return m_file->pread(&sec.sec_data, sizeof(sec.sec_data), sizeof(*sec.sec_data), secnum * 512);
	}


	/**
	 * @brief	Adapt write sector to file call
	 */
	bc_ulong_t write_sector(bc_sector_t const & sec, bc_ulong_t secnum)
	{
		return m_file->pwrite(&sec.sec_data, sizeof(sec.sec_data), sizeof(*sec.sec_data), secnum * 512);
	}


	void bind(bdk::backend::bc_crypto_dev_handle_t dev, bc_key_handle_t key_handle, bdk::BCAlgHandle alg, bcu32_t flags, const bc_space_unit_t& layout, unsigned int start_sector )
	{
		bdk::backend::bc_crypto_set_file(dev, m_file->system_handle(), key_handle, alg->id(), alg->mode(), flags, &layout, start_sector);
	}
};


bdk::BCContainerHandle load_container_file(ContainerFile & file);

#endif

