/*
 *	list.cpp
 *	bctool list command implementation.
 *	lists all configured bc devices, their backing container files and mount points.
 */

#include <stdio.h>
#include <string.h>
#include <mntent.h>
#include <sys/param.h>

#include <alg/device.h>
#include <common/log.h>

#include "config.h"

using namespace bdk;

bool bclist(void) 
{
	bc_dev_conf_t conf = get_dev_config();
	
	char devname[0xff] = {0};
	backend::bc_crypto_dev_handle_t dev;

	char pathbuf[MAXPATHLEN] = {0};

	for(unsigned i = 0; i < conf.total_devs; ++i) {
		memset(devname, 0, sizeof(devname));
		snprintf(devname, sizeof(devname) - 1, "/dev/bcrypt%d", i);
		
		dev = backend::bc_crypto_dev_get(devname);
		if(!dev) {
			bc_get_log(BC_MSG_CRIT) << "Error: failed getting device " << devname;
			return false;
		}

		bool configured = backend::bc_crypto_stat_dev(dev, pathbuf, sizeof(pathbuf) - 1);
		backend::bc_crypto_dev_put(dev);

		if(configured) {
			printf("%s \t%s \t", devname, pathbuf);

			// look for device mount point.
			// configured devices may not be actually mounted.
			FILE* mtab = setmntent("/etc/mtab", "r");
			if( NULL == mtab ) {
				bc_get_log(BC_MSG_CRIT) << "Error: failed opening mtab";
				return false;
			}

			struct mntent* pmnt;

			while((pmnt = getmntent(mtab)) && !feof(mtab)) {
				if(0 == strcmp(devname, pmnt->mnt_fsname)) {
					printf("%s\n", pmnt->mnt_dir);
					break;
				}
			}

			endmntent(mtab);
		}
	}

	return true;
}


