#ifndef BC_LINUX_MISC_H
#define BC_LINUX_MISC_H

#include <signal.h>
#include <stdio.h>

#include <string>

#include <common/types.h>
#include <common/log.h>
#include <common/fs.h>

/**
 * @brief 	bc_ask answers
 */
enum bc_answer_t
{
	BC_ANSWER_NO	= 0,
	BC_ANSWER_YES	= 1
};


/**
 * @brief	Ask y/n qestion.
 *
 * @p string	Question string
 * @p def_ans	Default answer.
 *
 * @return	User choise
 */
bc_answer_t bc_ask(const char* string, bc_answer_t def_ans);


/**
 * @brief	Get size of block device
 *
 * @p size	Size to store
 * @p bdevname	Name of block device file
 */
bool bc_get_bdev_size(bc_size_t& size, const char* bdevname);


/**
 * @brief	Find mounted best crypt device at mount point
 */
bool find_mounted_device( const char* mpoint, char* buf, size_t bufsiz );


/**
 * @brief	Log locked container warning
 */
void bc_locked_warning(const char* cont_name);


/**
 * @brief		Replace file by another file
 *
 * @p orig_name		File original name that will be replaced
 * @p new_name		File new name
 * @p backup_name	File backup name
 *
 * @return		pass\fail
 */
bool bc_replace_file( const char* orig_name, const char* new_name, const char* backup_name );


/**
 * @brief	Set term signals handler
 */
void set_sighandler( sighandler_t handler );


/**
 * @brief	Get user id by name
 */
int get_uid_by_name(const char* uname);


/**
 * @brief	Get user group id by user name
 */
int get_gid_by_name(const char* uname);

/**
 * @brief atol wrapper
 */
long long bc_atol ( const char *str );


/**
 * @brief Convert all characters in a string to lower case, inplace
 */
char* bc_str_to_lower ( char *string );


/**
 * @brief Convert all characters in a string to lower case and write to dst buffer
 */
char* bc_str_to_lower ( const char* src, char* dst, size_t bufsiz );



/**
 * @brief	Console log printer
 */
class ConsoleLog : public bdk::BCLogListener
{
public:
	
	void log_message(std::string const & msg, bdk::bc_msg_level lvl)
	{
		fprintf(stderr, "%s\n", msg.c_str());
	}
};


#endif
