/*
 *	Copyright (c) 2009 Jetico, Inc., Finland
 *	All rights reserved.
 *
 *	Keygen module handling.
 */

#include "module.h"

#include <dlfcn.h>


typedef std::list<bestcrypt::bc_keygen_module> modlist_t;
typedef modlist_t::iterator moditer_t;

static modlist_t g_list;

namespace bestcrypt {


	bc_keygen_module* bc_load_keygen_module(const char* path)
	{
		bc_keygen_module mod;

		void* dso = dlopen(path, RTLD_LAZY | RTLD_LOCAL);
		if(!dso) {
			throw BCModuleException("Unexpected error during keygen module loading", dlerror());
		}

		module_load_handler* loader = (module_load_handler) dlsym(dso, "module_load_handler");
		BC_ASSERT(loader);

		
		if(!loader(&mod.info)) {
			return NULL;
		}

		mod.name = path;
		mod.dso = dso;

		g_list.insert(mod);

		return &mod;
	}


	bc_keygen_module* bc_lookup_keygen(int id) 
	{
		moditer_t cur = g_list.begin();
		moditer_t end = g_list.end();

		while(cur != end) {
			if(id == cur->info.id) {
				return &(*cur);
			}

			++cur;
		}

		return NULL;
	}


	bc_keygen_module* bc_lookup_keygen(const char* name)
	{
		moditer_t cur = g_list.begin();
		moditer_t end = g_list.end();

		while(cur != end) {
			if(0 == strcmp(name, cur->info.name)) {
				return &(*cur);
			}

			++cur;
		}

		return NULL;
	}


	bc_keygen_module* bc_lookup_default_ghost_keygen()
	{
		return NULL; // stub
	}

}


