/*
 *	Copyright (c) 2009 Jetico, Inc., Finland
 *	All rights reserved.
 *
 *	Keygen module handling.
 */

#ifndef BC_MODULE_H
#define BC_MODULE_H

#include "exception.h"

namespace bestcrypt {


	/// Exception thrown on system error.
	class BCModuleException : public BCException {
	public:

		BCModuleException(const char* str, const char* dl_err) : BCException(str), m_dlerr(dl_err) {};

		const char* dlerr() const throw() {
			return m_dlerr;
		}


	private:

		const char* m_dlerr;
	};
	

	/// Keygen info, initialized on loading.
	struct bc_keygen_info
	{
		int		id;
		const char* 	tag;
	} bc_keygen_info;

	
	/// Keygen library handle
	typedef struct bc_keygen_module
	{
		const char* 	path;
		void*		dso;
		unsigned int	refcount;
		bc_keygen_info	info;
	} bc_keygen_module;


	/// Keygen module linker export
	typedef bool (*load_module_handler) (bc_keygen_info* out_info);


	/// Load keygen library by path.
	bc_keygen_module* bc_load_keygen_module(const char* path);

	/// Lookup keygen library by id.
	bc_keygen_module* bc_lookup_keygen(int id);

	/// Lookup keygen library by name tag.
	bc_keygen_module* bc_lookup_keygen(const char* name);

	/// Lookup default keygen for ghost containers.
	bc_keygen_module* bc_lookup_default_ghost_keygen();
}

#endif

