/**
 *
 *	Copyright (c) 2009 Jetico, Inc., Finland
 *	All rights reserved.
 *
 *	file: passwd.cpp
 *	scope: linux client
 *	author: evgeny yakovlev
 *
 *	description: 
 *	linux client 'passwd', 'add_passwd', 'del_passwd' commands.
 */

#include "passwd.h"
#include "misc.h"
#include "iofile.h"

#include <common/log.h>

using namespace bdk;


bool bcpasswd(const char* fname, bc_view_change_t op)
{
	BCFSObject cont_path(fname);
	
	if( !cont_path.is_valid() ) {
		bc_get_log(BC_MSG_CRIT) << "File " << cont_path << " does not exist";
		return false;
	}

	if( !cont_path.access(BC_ACCESS_WRITE) ) {
		bc_get_log(BC_MSG_CRIT) << "You do not have write permissions for file " << cont_path;
		return false;
	}

	
	ContainerFile cont_file(cont_path.path(), BC_FILE_LARGE|BC_FILE_READ|BC_FILE_WRITE);
	BCContainerHandle cont = load_container_file(cont_file);

	if( !cont ) {
		bc_get_log(BC_MSG_CRIT) << "Could not parse container from file " << cont_path; 
		return false;
	}

	if( cont->format()->is_locked() ) {
		bc_locked_warning(fname);
		return false;
	}

	
	BCContainerViewHandle view = cont->format()->update_view(&cont_file, op);
	if(!view) 
	{
		bc_get_log(BC_MSG_CRIT) << "Error: failed container password modification";
		return false;
	}


	cont->format()->serialize(&cont_file);

	bc_get_log(BC_MSG_INFO) << "Password successfuly modified";
	return true;
}


bool bcpasswd_add(int argc, char** argv)
{
	if( argc < 2 ) {
		bc_get_log(BC_MSG_CRIT) << "Error: Please specify container name";
		return false;
	} else if (argc > 2) {
		bc_get_log(BC_MSG_CRIT) << "Error: Too many parameters";
		return false;
	}

	const char* fname = argv[1];
	return bcpasswd(fname, VIEW_ADD_AUTH);
}


bool bcpasswd_del(int argc, char** argv)
{
	if( argc < 2 ) {
		bc_get_log(BC_MSG_CRIT) << "Error: Please specify container name";
		return false;
	} else if (argc > 2) {
		bc_get_log(BC_MSG_CRIT) << "Error: Too many parameters";
		return false;
	}

	const char* fname = argv[1];
	return bcpasswd(fname, VIEW_REMOVE_AUTH);
}


bool bcpasswd_change(int argc, char** argv)
{
	if( argc < 2 ) {
		bc_get_log(BC_MSG_CRIT) << "Error: Please specify container name";
		return false;
	} else if (argc > 2) {
		bc_get_log(BC_MSG_CRIT) << "Error: Too many parameters";
		return false;
	}

	const char* fname = argv[1];
	return bcpasswd(fname, VIEW_CHANGE_AUTH);
}


