
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>

#include "process.h"

#include <common/build.h>
#include <common/exception.h>

using namespace bdk;


static void do_spawn( const std::string path,  const std::vector < std::string > & args )
{
	char* cargs[1024] = {NULL};
	BC_ASSERT((sizeof(cargs) * sizeof(*cargs)) > args.size());

	cargs[0] = (char*)path.c_str();
	char** pargs = &cargs[1];

	std::vector < std::string >::const_iterator i = args.begin();
	std::vector < std::string >::const_iterator end = args.end();

	while(i != end) {
		*pargs++ = (char*)i->c_str();
		++i;
	}

	*pargs = NULL;

	const char* env[] = {"PATH=/sbin:/usr/sbin:/bin:/usr/bin", NULL};

	execve(path.c_str(), (char* const*)cargs, (char* const*)env);
		
	// not reached if ok
	throw BCSystemException("execve failed");
}


int Process::exec( const std::vector < std::string > & args )
{
	pid_t pid = fork();
	if(pid < 0) {
		throw BCSystemException("Process fork failed");
	}
	
	if( 0 == pid ) {
		do_spawn(m_name, args);
	}

	// parent
	// catch child exit status
	int status;
	waitpid( pid, &status, 0);
	return WEXITSTATUS(status);
}


int Process::exec( const std::vector < std::string > & args, std::string& output )
{
	int fds[2] = {0};
	if(pipe(fds) < 0) {
		throw BCSystemException("Pipe failed");
	}

	int read_fd = fds[0];
	int write_fd = fds[1];

	pid_t pid = fork();
	if(pid < 0) {
		throw BCSystemException("Process fork failed");
	}
	
	if( 0 == pid ) 
	{
		// redirect stdout
		close(read_fd);
		dup2(write_fd, STDOUT_FILENO);
	
		do_spawn(m_name, args);
	}

	// parent
	// read child stdout through pipe
	close(write_fd);
	ssize_t nread = 0;
	char buf[1024] = {0};
	output.clear();

	do {
		nread = read(read_fd, buf, sizeof(buf));
		if(nread < 0) {
			throw BCSystemException("Fai;ed reading process output");
		}

		output.append(buf, nread);
	} while(nread);
	
	close(read_fd);

	int status;
	waitpid( pid, &status, 0);
	return WEXITSTATUS(status);
}

