#include "test_lrw_aes.h"
#include <string.h>

#include <alg/alg.h>
#include <common/log.h>


using namespace bdk;

bool bc_test_lrw_aes()
{
	unsigned char key[512] =
	{
		// AES 256 bit key1
		0xf8, 0xd4, 0x76, 0xff, 0xd6, 0x46, 0xee, 0x6c, 0x23, 0x84, 0xcb, 0x1c, 0x77, 0xd6, 0x19, 0x5d,
		0xfe, 0xf1, 0xa9, 0xf3, 0x7b, 0xbc, 0x8d, 0x21, 0xa7, 0x9c, 0x21, 0xf8, 0xcb, 0x90, 0x02, 0x89,

		// AES 128 bit tweak key2
		0xa8, 0x45, 0x34, 0x8e, 0xc8, 0xc5, 0xb5, 0xf1, 0x26, 0xf5, 0x0e, 0x76, 0xfe, 0xfd, 0x1b, 0x1e
	};

	unsigned char plaintext[16] =
	{
		0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46
	};

	unsigned char iv[8] =
	{
		0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
	};

	// C-text is different from IEEE draft because our IV is multiplied by 512
	unsigned char cyphertext[16] =
	{
		0x2b, 0x37, 0x9c, 0xcc, 0xa0, 0x40, 0x4b, 0xf7, 0xf3, 0xcd, 0x6a, 0x98, 0x35, 0x17, 0x97, 0x36
	};

	unsigned char buf[16] = {0};

	memcpy(buf, plaintext, sizeof(plaintext));

	unsigned char pool[512] = {0};

	BCAlgHandle alg = BCAlg::init("rijndael", AM_LRW, key, pool);
	if(!alg)
	{
		bc_get_log(BC_MSG_CRIT) << "Error: failed to initialize algorithm";
		return false;
	}

	alg->encrypt(iv, buf, sizeof(buf));

	if( 0 != memcmp(cyphertext, buf, sizeof(buf)) ) {
		bc_get_log(BC_MSG_CRIT) << "Encryption test failed";
		return false;
	}

	alg->decrypt(iv, buf, sizeof(buf));

	if( 0 != memcmp(plaintext, buf, sizeof(buf)) ) {
		bc_get_log(BC_MSG_CRIT) << "Decryption test failed";
		return false;
	}

	bc_get_log(BC_MSG_INFO) << "Test ok.";
	return true;
}
