
/**
 *
 *	Copyright (c) 2009 Jetico, Inc., Finland
 *	All rights reserved.
 *
 *	file: unlock.cpp
 *	scope: linux client
 *	author: evgeny yakovlev
 *
 *	description: 
 *	linux client 'unlock' command.
 */

#include "unlock.h"
#include "misc.h"
#include "iofile.h"

#include <common/log.h>

using namespace bdk;


bool bcunlock(int argc, char** argv)
{
	if( argc < 2 ) {
		bc_get_log(BC_MSG_CRIT) << "Error: Please specify container name";
		return false;
	} else if (argc > 2) {
		bc_get_log(BC_MSG_CRIT) << "Error: Too many parameters";
		return false;
	}

	BCFSObject cont_path(argv[1]);
	
	if( !cont_path.is_valid() ) {
		bc_get_log(BC_MSG_CRIT) << "File " << cont_path << " does not exist";
		return false;
	}

	if( !cont_path.access(BC_ACCESS_WRITE) ) {
		bc_get_log(BC_MSG_CRIT) << "You do not have write permissions for file " << cont_path;
		return false;
	}

	
	ContainerFile cont_file(cont_path, BC_FILE_LARGE|BC_FILE_READ|BC_FILE_WRITE);
	BCContainerHandle cont = load_container_file(cont_file);

	if( !cont ) {
		bc_get_log(BC_MSG_CRIT) << "Could not parse container from file " << cont_path; 
		return false;
	}


	if( !cont->is_recognized() ) {
		bc_get_log(BC_MSG_WARN) << "Container " << cont_path << " was not recognized.\n" 
					<< "Lock masks can not be reset for unrecognized containers";
		return false;
	}

	// check lock mask status
	if( !cont->format()->is_locked() ) {
		bc_get_log(BC_MSG_WARN) << "Warning: container " << cont_path << " is not locked.";
		return true;
	}

	// reset lock mask
	cont->format()->unlock();

	// update file
	cont->format()->serialize(&cont_file);
	return true;
}


